<?php


if ( file_exists( dirname( __FILE__ ) . '/cmb2/init.php' ) ) {
	require_once dirname( __FILE__ ) . '/cmb2/init.php';
} elseif ( file_exists( dirname( __FILE__ ) . '/CMB2/init.php' ) ) {
	require_once dirname( __FILE__ ) . '/CMB2/init.php';
}

/**
 * Conditionally displays a metabox when used as a callback in the 'show_on_cb' cmb2_box parameter
 *
 * @param  CMB2 object $cmb CMB2 object
 *
 * @return bool             True if metabox should show
 */
function h5vp_yourprefix_show_if_front_page( $cmb ) {
	// Don't show this metabox if it's not the front page template
	if ( $cmb->object_id !== get_option( 'page_on_front' ) ) {
		return false;
	}
	return true;
}

/**
 * Conditionally displays a field when used as a callback in the 'show_on_cb' field parameter
 *
 * @param  CMB2_Field object $field Field object
 *
 * @return bool                     True if metabox should show
 */
function h5vp_yourprefix_hide_if_no_cats( $field ) {
	// Don't show this field if not in the cats category
	if ( ! has_tag( 'cats', $field->object_id ) ) {
		return false;
	}
	return true;
}

/**
 * Conditionally displays a message if the $post_id is 2
 *
 * @param  array             $field_args Array of field parameters
 * @param  CMB2_Field object $field      Field object
 */
function h5vp_yourprefix_before_row_if_2( $field_args, $field ) {
	if ( 2 == $field->object_id ) {
		echo '<p>Testing <b>"before_row"</b> parameter (on $post_id 2)</p>';
	} else {
		echo '<p>Testing <b>"before_row"</b> parameter (<b>NOT</b> on $post_id 2)</p>';
	}
}

add_action( 'cmb2_admin_init', 'h5vp_yourprefix_register_demo_metabox' );
/**
 * Hook in and add a demo metabox. Can only happen on the 'cmb2_admin_init' or 'cmb2_init' hook.
 */
function h5vp_yourprefix_register_demo_metabox() {

	// Start with an underscore to hide fields from custom fields list
	$prefix = '_ahp_';

	/**
	 * Sample metabox to demonstrate each field type included
	 */
	$cmb_demo = new_cmb2_box( array(
		'id'            => $prefix . 'metabox2',
		'title'         => __( 'Configure your video player' ),
		'object_types'  => array( 'videoplayer', ), // Post type
		// 'show_on_cb' => 'yourprefix_show_if_front_page', // function should return a bool value
		'context'    => 'normal',
		// 'priority'   => 'high',
		// 'show_names' => true, // Show field names on the left
		// 'cmb_styles' => false, // false to disable the CMB stylesheet
		// 'closed'     => true, // true to keep the metabox closed by default
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Video', 'cmb2' ),
		'desc' => __( 'Select an Mp4 or Ogg video file. Or Paste a external video file link', 'cmb2' ),
		'id'   => $prefix . 'video-file',
		'type' => 'file',
		'repeatable' => true,	
		'text' => array(
		'add_row_text' => 'Add Another Special Row',
		),		
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Poster Image', 'cmb2' ),
		'desc' => __( 'Specifies an image to be shown while the video is downloading, or until the user hits the play button', 'cmb2' ),
		'id'   => $prefix . 'video-poster',
		'type' => 'file',
	) );	
	$cmb_demo->add_field( array(
		'name'             => __( 'Repeat', 'cmb2' ),
		'desc'             => __( '', 'cmb2' ),
		'id'               => $prefix . 'video-repeat',
		'type'             => 'radio_inline',
		'options'          => array(
			'once' => __( 'Repeat Once', '' ),
			'loop'   => __( 'Repeat again and again', 'loop' ),
		),
		'default'             => 'once',
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Muted', 'cmb2' ),
		'desc' => __( 'Check if you want the video output should be muted', 'cmb2' ),
		'id'   => $prefix . 'video-muted',
		'type' => 'checkbox',
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Auto Play', 'cmb2' ),
		'desc' => __( 'Check if you want video will start playing as soon as it is ready', 'cmb2' ),
		'id'   => $prefix . 'video-autoplay',
		'type' => 'checkbox',
	) );

	$cmb_demo->add_field( array(
		'name' => __( 'Player Width In pixel', 'cmb2' ),
		'desc' => __( 'Sets the player width. Height will be calculate base on the value. Left blank for Responsive Player', 'cmb2' ),
		'id'   => $prefix . 'video-size',
		'type' => 'text_number',
	
	) );
	
	$cmb_demo->add_field( array(
		'name' => __( 'Seek Time', 'cmb2' ),
		'desc' => __( 'The time, in seconds, to seek when a user hits fast forward or rewind. Deafult value is 10 Sec.', 'cmb2' ),
		'id'   => $prefix . 'seek_time',
		'default'   =>'10',
		'type' => 'text_number',
	) );	

	$cmb_demo->add_field( array(
		'name' => __( 'Auto Hide Control', 'cmb2' ),
		'desc' => __( 'Check if you want the controls (such as a play/pause button etc) hide automatically.', 'cmb2' ),
		'id'   => $prefix . 'video-control',
		'type' => 'checkbox',
	) );
	
	$cmb_demo->add_field( array(
		'name'    => 'Preload',
		'desc' => __( 'Specify how the Video file should be loaded when the page loads.', 'cmb2' ),
		'id'      => $prefix . 'preload',
		'type'    => 'radio_inline',
		'options' => array(
			'auto' => __( 'Auto - Browser should load the entire file when the page loads.', 'cmb2' ),
			'metadata'   => __( 'Metadata - Browser should load only metadata when the page loads.', 'cmb2' ),
			'none'     => __( 'None - Browser should NOT load the file when the page loads.', 'cmb2' ),
		),
		'default' => 'auto',
	) );
	$blog_group_id = $cmb_demo->add_field( array(
		'id'          => 'blog_group',
		'type'        => 'group',
		'repeatable'  => true,
		'desc' => "Enable video quality switcher By Putting different qualities of same video, Leave blank if you don't want the quality switcher in the player.",
		
		'options'     => array(
			'group_title'   => 'Quality {#}',
			'add_button'    => 'Add new Quality',
			'remove_button' => 'Remove Quality',
			'closed'        => true,  // Repeater fields closed by default - neat & compact.
			'sortable'      => true,  // Allow changing the order of repeated groups.
		),
	) );
	$cmb_demo->add_group_field( $blog_group_id, array(
		'name' => 'Video',
		'desc' => 'Select an Mp4 or Ogg video file Or Paste a external video file link',
		'id'   => 'video_file',
		'type' => 'file',
	) );
	$cmb_demo->add_group_field( $blog_group_id, array(
		'name' => 'Size',
		'desc' => 'Enter the Video Size, eg: 240 , 360 , 480 , 720, 1080, 2160 .',
		'id'   => 'size',
		'type' => 'text_number',
		'attributes'  => array(
			'placeholder' => '1080',
				)		
	) );	
	$captions_group_id = $cmb_demo->add_field( array(
		'id'          => 'caption_group',
		'type'        => 'group',
		'repeatable'  => true,
		'desc' => "You can Set single or mulitple Subtitle, Leave blank if you don't want to use subtitle.",
		
		'options'     => array(
			'group_title'   => 'Subtitle {#}',
			'add_button'    => 'Add new Subtitle',
			'remove_button' => 'Remove Subtitle',
			'closed'        => true,  // Repeater fields closed by default - neat & compact.
			'sortable'      => true,  // Allow changing the order of repeated groups.
		),
	) );
	$cmb_demo->add_group_field( $captions_group_id, array(
		'name' => 'Subtitle File',
		'desc' => '.vtt file only, select a .vtt file or Paste .vtt file link.',
		'id'   => 'caption_file',
		'type' => 'file',
	) );
	$cmb_demo->add_group_field( $captions_group_id, array(
		'name' => 'Size',
		'desc' => 'Eg: English, French etc. This will be used as a identifire in the subtitle switcher when multiple subtitle will be set.',
		'id'   => 'label',
		'type' => 'text',
		'attributes'  => array(
		'placeholder' => 'English',
	)		
	) );		
}
add_action( 'cmb2_admin_init', 'h5vp_controls_metabox' );
function h5vp_controls_metabox() {

	// Start with an underscore to hide fields from custom fields list
	$prefix = '_ahp_';

	/**
	 * Sample metabox to demonstrate each field type included
	 */
	$cmb_demo = new_cmb2_box( array(
		'id'            => $prefix . 'controls',
		'title'         => __( 'Controls' ),
		'object_types'  => array( 'videoplayer', ), // Post type
		// 'show_on_cb' => 'yourprefix_show_if_front_page', // function should return a bool value
		'context'    => 'side',
		'priority'   => 'low',
		 'show_names' => false, // Show field names on the left
		// 'cmb_styles' => false, // false to disable the CMB stylesheet
		// 'closed'     => true, // true to keep the metabox closed by default
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Hide Large Play Button', 'cmb2' ),	
		'desc' => __( 'Hide Large Play Button', 'cmb2' ),
		'id'   => $prefix . 'hide_large_play',
		'type' => 'checkbox',
		//'default' => 'on',		
	) );	

	$cmb_demo->add_field( array(
		'name' => __( 'Hide Restart Button', 'cmb2' ),	
		'desc' => __( 'Hide Restart Button', 'cmb2' ),
		'id'   => $prefix . 'restart_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );		
	$cmb_demo->add_field( array(
		'name' => __( 'Hide Play Button', 'cmb2' ),
		'desc' => __( 'Hide Play button', 'cmb2' ),
		'id'   => $prefix . 'play_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );	
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Rewind button', 'cmb2' ),
		'desc' => __( 'Hide Rewind Button.', 'cmb2' ),
		'id'   => $prefix . 'rewind_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Fast forward button', 'cmb2' ),
		'desc' => __( 'Hide Fast forward Button', 'cmb2' ),
		'id'   => $prefix . 'fast_forward',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
	
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Progress Bar', 'cmb2' ),
		'desc' => __( 'Hide Video progress bar.', 'cmb2' ),
		'id'   => $prefix . 'progress',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
	$cmb_demo->add_field( array(
		'name' => __( 'Hide Current Time', 'cmb2' ),
		'desc' => __( 'Hide Current Time ', 'cmb2' ),
		'id'   => $prefix . 'current_time',
		'type' => 'checkbox',
		//'default' => 'on',
	) );	
		$cmb_demo->add_field( array(
		'name' => __( 'Hide video Duration', 'cmb2' ),
		'desc' => __( 'Hide Video Duration', 'cmb2' ),
		'id'   => $prefix . 'duration',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide the Mute button', 'cmb2' ),
		'desc' => __( 'Hide Mute Button', 'cmb2' ),
		'id'   => $prefix . 'mute_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Volume Control', 'cmb2' ),
		'desc' => __( 'Hide Volume Control', 'cmb2' ),
		'id'   => $prefix . 'volume_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Settings Button', 'cmb2' ),
		'desc' => __( 'Hide Settings Button', 'cmb2' ),
		'id'   => $prefix . 'setting_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide PIP Button', 'cmb2' ),
		'desc' => __( 'Hide PIP Button', 'cmb2' ),
		'id'   => $prefix . 'pip_button',
		'type' => 'checkbox',
		//'default' => 'on',
	) );
	
		$cmb_demo->add_field( array(
		'name' => __( 'Hide AirPlay Button', 'cmb2' ),
		'desc' => __( 'Hide AirPlay Button', 'cmb2' ),
		'id'   => $prefix . 'airplay_button',
		'type' => 'checkbox',
	) );	
		$cmb_demo->add_field( array(
		'name' => __( 'Hide FullScreen button', 'cmb2' ),
		'desc' => __( 'Hide FullScreen Button', 'cmb2' ),
		'id'   => $prefix . 'fullscreen_button',
		'type' => 'checkbox',
		//'default'=> 'on',
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Download Button', 'cmb2' ),
		'desc' => __( 'Hide Download Button', 'cmb2' ),
		'id'   => $prefix . 'download_button',
		'type' => 'checkbox',
		//'default'=> 'on'	
		
	) );
		$cmb_demo->add_field( array(
		'name' => __( 'Hide Controls Shadow', 'cmb2' ),
		'desc' => __( 'Hide Controls Shadow', 'cmb2' ),
		'id'   => $prefix . 'hide_shadow',
		'type' => 'checkbox',
		//'default'=> 'on'	
		
	) );	
}

// Add Number type
// render numbers
add_action( 'cmb2_render_text_number', 'h5vp_cmb_render_text_number', 10, 5 );
function h5vp_cmb_render_text_number( $field, $escaped_value, $object_id, $object_type, $field_type_object ) {
    echo $field_type_object->input( array( 'class' => 'cmb2-text-small', 'type' => 'number' ) );
}

// sanitize the field
add_filter( 'cmb2_sanitize_text_number', 'h5vp_cmb2_sanitize_text_number', 10, 2 );
function h5vp_cmb2_sanitize_text_number( $null, $new ) {
    $new = preg_replace( "/[^0-9]/", "", $new );

    return $new;
}