<?php

if ( !class_exists('Html5Video_Settings_API_Test' ) ):
class Html5Video_Settings_API_Test {

    private $settings_api;

    function __construct() {
        $this->settings_api = new Html5Video_Settings_API;

        add_action( 'admin_init', array($this, 'admin_init') );
        add_action( 'admin_menu', array($this, 'admin_menu') );
    }

    function admin_init() {

        //set the settings
        $this->settings_api->set_sections( $this->get_settings_sections() );
        $this->settings_api->set_fields( $this->get_settings_fields() );

        //initialize settings
        $this->settings_api->admin_init();
    }

    function admin_menu() {
        add_options_page( 'Html5 Video Player Settings', 'Html5 Video Player Settings', 'delete_posts', 'html5vp_settings', array($this, 'plugin_page') );
    }

    function get_settings_sections() {
        $sections = array(
            array(
                'id' => 'h5vp_quick',
                'title' => __( 'Preset for Quick Players', 'wedevs' )
            ),		
            array(
                'id' => 'h5vp_basics',
                'title' => __( 'Color Settings', 'wedevs' )
            ),
			array(
                'id' => 'h5vp_style',
                'title' => __( 'Custom Style', 'wedevs' )
            ),
			array(
                'id' => 'h5vp_translations',
                'title' => __( 'Player Translatios', 'wedevs' )
            )			
        );
        return $sections;
    }

    /**
     * Returns all the settings fields
     *
     * @return array settings fields
     */
    function get_settings_fields() {
        $settings_fields = array(
            'h5vp_basics' => array(
                array(
                    'name'    => 'color_one',
                    'label'   => __( 'Player Primary Color', 'wedevs' ),
                    'desc'    => __( 'Change the primary color of the player.', 'wedevs' ),
                    'type'    => 'color',
                    'default' => '#dd3333'
                ),
                array(
                    'name'    => 'color_two',
                    'label'   => __( 'Secondery Color ( Hover color )', 'wedevs' ),
                    'desc'    => __( 'Change the secondery color of the player.', 'wedevs' ),
                    'type'    => 'color',
                    'default' => '#1aafff'
                ),

            ),
			'h5vp_quick' => array(			
				array(
                    'name'    => 'repeat',
                    'label'   => __( 'Repeat', 'wedevs' ),
                    'desc'    => __( 'Select Loop to specify that the video will start over again, every time it is finished', 'wedevs' ),
                    'type'    => 'radio',
                    'default' => 'once',					
                    'options' => array(
                        'once'  => 'Repeat Once',
						'loop' => 'Loop'
                        
                    )
                ),
				array(
                    'name'  => 'muted',
                    'label' => __( 'Muted', 'wedevs' ),
                    'desc'  => __( 'Check if you want the video output should be muted', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
				array(
                    'name'  => 'autoplay',
                    'label' => __( 'Autoplay', 'wedevs' ),
                    'desc'  => __( ' Check if you want video will start playing as soon as it is ready', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
                array(
                    'name'              => 'width',
                    'label'             => __( 'Player Width', 'wedevs' ),
                    'desc'              => __( 'Define the width of the player, Enter 0 if you want the player width will cover the parents container.', 'wedevs' ),
                    'placeholder'       => __( '0', 'wedevs' ),
                    'min'               => 0,
                    'max'               => 50000,
                    'step'              => '1',
                    'type'              => 'number',
                    'default'           => '0',
                    'sanitize_callback' => 'intval'
                ),
                array(
                    'name'              => 'seek_time',
                    'label'             => __( 'Seek Time', 'wedevs' ),
                    'desc'              => __( 'The time, in seconds, to seek when a user hits fast forward or rewind. Deafult value is 10 Sec', 'wedevs' ),
                    'placeholder'       => __( '10', 'wedevs' ),
                    'min'               => 1,
                    'max'               => 50000,
                    'step'              => '1',
                    'type'              => 'number',
                    'default'           => '10',
                    'sanitize_callback' => 'intval'
                ),

				array(
                    'name'  => 'hide_control',
                    'label' => __( 'Auto Hide Controls', 'wedevs' ),
                    'desc'  => __( 'Check if you want the controls (such as a play/pause button etc) hide automatically.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	
				
				array(
                    'name'    => 'preload',
                    'label'   => __( 'Preload', 'wedevs' ),
                    'desc'    => __( 'Specify how the video file should be loaded when the page loads.', 'wedevs' ),
                    'type'    => 'radio',
					'default' => 'auto',
                    'options' => array(
                        'auto'  => 'Auto - Browser should load the entire file when the page loads.',
						'metadata' => 'Metadata - Browser should load only metadata when the page loads.',
						'none' => 'None - Browser should NOT load the file when the page loads.',
                        
                    )
                ),

				array(
                    'name'  => 'large_play',
                    'label' => __( 'Hide Large Play Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the large play button from the center of the player.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
				array(
                    'name'  => 'restart_button',
                    'label' => __( 'Hide Restart Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the Restart button.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
				array(
                    'name'  => 'play_button',
                    'label' => __( 'Hide Play Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the play button from the playe.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),				
				
				array(
                    'name'  => 'rewind_button',
                    'label' => __( 'Hide Rewind Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the rewind button.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
				array(
                    'name'  => 'fast_forward_button',
                    'label' => __( 'Hide Fast Forward Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the Fast forward button from the player.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),

				array(
                    'name'  => 'progress_bar',
                    'label' => __( 'Disable Progressbar', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the progressbar from the player.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),
				array(
                    'name'  => 'current_time',
                    'label' => __( 'Hide Current Time', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the current time.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),					
				array(
                    'name'  => 'duration',
                    'label' => __( 'Hide duration', 'wedevs' ),
                    'desc'  => __( ' Check if you want to hide the video duration.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	

				array(
                    'name'  => 'mute_button',
                    'label' => __( 'Hide Mute Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the mute button', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	
				array(
                    'name'  => 'volume_hide',
                    'label' => __( 'Hide volume Control', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide volume control from the player', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	
				array(
                    'name'  => 'setting',
                    'label' => __( 'Hide Setting Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the settings or the gear icon from the player.', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	
				
				array(
                    'name'  => 'pip_button',
                    'label' => __( 'Hide PIP Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the PIP button', 'wedevs' ),
                    'type'  => 'checkbox'
                ),	
				array(
                    'name'  => 'fullscreen',
                    'label' => __( 'Hide fullScreen button', 'wedevs' ),
                    'desc'  => __( ' Check if you want to hide the FullScreen button', 'wedevs' ),
                    'type'  => 'checkbox'
                ),					
				
				
				
				array(
                    'name'  => 'download',
                    'label' => __( 'Hide Download Button', 'wedevs' ),
                    'desc'  => __( 'Check if you want to hide the download button', 'wedevs' ),
                    'type'  => 'checkbox'
                ),				

            ),	
            'h5vp_style' => array(
                array(
                    'name'    => 'custom_css',
                    'label'   => __( 'Custom CSS', 'wedevs' ),
                    'desc'    => __( 'Enter your custom CSS code.', 'wedevs' ),
                    'type'    => 'textarea',
                )

            ),
			'h5vp_translations' => array(
                array(
                    'name'    => 'password',
                    'label'   => __( 'Translation Guideline:', 'wedevs' ),
                    'default' => 'Please don\'t touch any text between round brackets {}, those are dynamic variable eg: {currentTime}',
                    'type'    => 'password',					
                ),			
                array(
                    'name'    => 'restart',
                    'label'   => __( 'Restart', 'wedevs' ),
                    'default' => 'Restart',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'rewind ',
                    'label'   => __( 'Rewind {seektime}s', 'wedevs' ),
                    'default' => 'Rewind {seektime}s',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'play',
                    'label'   => __( 'Play', 'wedevs' ),
                    'default' => 'Play',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'pause',
                    'label'   => __( 'Pause', 'wedevs' ),
                    'default' => 'Pause',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'fastForward',
                    'label'   => __( 'Forward {seektime}s', 'wedevs' ),
                    'default' => 'Forward {seektime}s',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'seek',
                    'label'   => __( 'Seek', 'wedevs' ),
                    'default' => 'Seek',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'seekLabel',
                    'label'   => __( '{currentTime} of {duration}', 'wedevs' ),
                    'default' => '{currentTime} of {duration}',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'played',
                    'label'   => __( 'Played', 'wedevs' ),
                    'default' => 'Played',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'buffered',
                    'label'   => __( 'Buffered', 'wedevs' ),
                    'default' => 'Buffered',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'currentTime',
                    'label'   => __( 'Current time', 'wedevs' ),
                    'default' => 'Current time',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'duration',
                    'label'   => __( 'Duration', 'wedevs' ),
                    'default' => 'Duration',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'volume',
                    'label'   => __( 'Volume', 'wedevs' ),
                    'default' => 'Volume',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'mute',
                    'label'   => __( 'Mute', 'wedevs' ),
                    'default' => 'Mute',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'unmute',
                    'label'   => __( 'Unmute', 'wedevs' ),
                    'default' => 'Unmute',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'enableCaptions',
                    'label'   => __( 'Enable captions', 'wedevs' ),
                    'default' => 'Enable captions',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'disableCaptions',
                    'label'   => __( 'Disable captions', 'wedevs' ),
                    'default' => 'Disable captions',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'download',
                    'label'   => __( 'Download', 'wedevs' ),
                    'default' => 'Download',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'enterFullscreen',
                    'label'   => __( 'Enter fullscreen', 'wedevs' ),
                    'default' => 'Enter fullscreen',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'exitFullscreen',
                    'label'   => __( 'Exit fullscreen', 'wedevs' ),
                    'default' => 'Exit fullscreen',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'frameTitle',
                    'label'   => __( 'Player for {title}', 'wedevs' ),
                    'default' => 'Player for {title}',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'captions',
                    'label'   => __( 'Captions', 'wedevs' ),
                    'default' => 'Captions',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'settings',
                    'label'   => __( 'Settings', 'wedevs' ),
                    'default' => 'Settings',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'pip',
                    'label'   => __( 'PIP', 'wedevs' ),
                    'default' => 'PIP',
                    'type'    => 'text',					
                ),
                array(
                    'name'    => 'menuBack',
                    'label'   => __( 'Go back to previous menu', 'wedevs' ),
                    'default' => 'Go back to previous menu',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'speed',
                    'label'   => __( 'Speed', 'wedevs' ),
                    'default' => 'Speed',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'normal',
                    'label'   => __( 'Normal', 'wedevs' ),
                    'default' => 'Normal',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'quality',
                    'label'   => __( 'Quality', 'wedevs' ),
                    'default' => 'Quality',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'loop',
                    'label'   => __( 'Loop', 'wedevs' ),
                    'default' => 'Loop',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'start',
                    'label'   => __( 'Start', 'wedevs' ),
                    'default' => 'Start',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'End',
                    'label'   => __( 'End', 'wedevs' ),
                    'default' => 'End',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'all',
                    'label'   => __( 'All', 'wedevs' ),
                    'default' => 'All',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'reset',
                    'label'   => __( 'Reset', 'wedevs' ),
                    'default' => 'Reset',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'disabled',
                    'label'   => __( 'Disabled', 'wedevs' ),
                    'default' => 'Disabled',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'enabled',
                    'label'   => __( 'Enabled', 'wedevs' ),
                    'default' => 'Enabled',
                    'type'    => 'text',					
                ),	
                array(
                    'name'    => 'ad',
                    'label'   => __( 'Ad', 'wedevs' ),
                    'default' => 'Ad',
                    'type'    => 'text',					
                ),	
		)		
        );

        return $settings_fields;
    }

    function plugin_page() {
        echo '<div class="wrap"><h2>Html5 Video Player Settings</h2>';

        $this->settings_api->show_navigation();
        $this->settings_api->show_forms();

        echo '</div>';
    }

    /**
     * Get all the pages
     *
     * @return array page names with key value pairs
     */
    function get_pages() {
        $pages = get_pages();
        $pages_options = array();
        if ( $pages ) {
            foreach ($pages as $page) {
                $pages_options[$page->ID] = $page->post_title;
            }
        }

        return $pages_options;
    }

}
endif;
new Html5Video_Settings_API_Test();