<?php
/**
 * Admin functions and actions.
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	video-vibe/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Video_Vibe_Admin{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Video_Vibe_Admin();
        return self::$instance;
    }

	private function __construct(){
		if( !defined('VIDEO_VIBE_OPTION')){
		    define('VIDEO_VIBE_OPTION','video-vibe');
		}
		add_action('init',function(){
			$this->init_video_vibe();
		});
		$this->wistia_media_ids = array();
	}


	/**
	 * Register generally helpful constants.
	 *
	 * @since 1.0
	 *
	 * @access private
	 * @return void
	 */
	private function get_vv_vars() {

		$this->albums = '';
		$this->albums_per_page = apply_filters('vv_vimeo_albums_per_page',5);
		$this->videos_per_page = apply_filters('vv_vimeo_videos_per_page',20);
		$this->settings = get_option(VIDEO_VIBE_OPTION);
		$this->vimeo_privacy = $this->settings['vimeo_privacy'];
		$this->wistia_privacy = $this->settings['wistia_privacy'];
	}

	


	/**
	 * Run action and filter hooks.
	 *
	 * @since 1.0
	 *
	 * @access private
	 * @return void
	 */
	private function  init_video_vibe() {
		if(!defined('VIMEO_ROOT_ENDPOINT')){
			define('VIMEO_ROOT_ENDPOINT','https://api.vimeo.com');
		}
		$this->get_vv_vars();
		if(is_user_logged_in() && current_user_can('edit_posts')){
			
			
			//Adds Media Tab
			add_filter( 'media_upload_tabs'       , array( $this, 'video_vibe_tabs_vimeo' ) );
			
			add_action( 'media_upload_vv_vimeo_library' , array( $this, 'vv_vimeo_library_iframe' ) );

			add_action( 'media_upload_vv_wistia_library' , array( $this, 'vv_wistia_library_iframe' ) );

			add_action('wp_ajax_vv_viemeo_plupload',array($this,'vv_viemeo_plupload'));
			add_action('wp_ajax_get_vimeo_upload_ticket',array($this,'get_vimeo_upload_ticket'));
			add_action('wp_ajax_get_vimeo_albums',array($this,'get_vimeo_albums'));
			add_action('wp_ajax_get_vimeo_album_videos',array($this,'get_vimeo_album_videos'));
			add_action('wp_ajax_update_uploaded_vimeo_video',array($this,'update_uploaded_vimeo_video'));
			add_action('wp_ajax_delete_user_album',array($this,'delete_user_album'));
			
			add_action('wp_ajax_get_vimeo_me_info',array($this,'get_vimeo_me_info'));



			//wistia
			add_action('wp_ajax_create_user_wistia_project',array($this,'create_user_wistia_project'));
			//add_action('wp_footer',array($this,'print_wistia_media_ids'));
			add_filter('the_content',array($this,'add_the_scripts_once')); 
			
		}
		//shortcode
		$this->vimeo_access_token = get_option('vimeo_access_token');
		$this->vv_vimeo_front_script_loaded = 0;
		$this->vv_wistia_front_script_loaded = 0;
		add_shortcode('wplms_vimeo',array( $this,'wplms_vimeo_shortcode'));
		add_shortcode('wplms_wistia',array( $this,'wplms_wistia_shortcode'));
	}

	function delete_user_album(){
		if(!is_user_logged_in())
			die();
		$user_id = get_current_user_id();
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_albums'.$user_id) || !is_user_logged_in()){
			die();
		}else{
			$yes = delete_user_meta($user_id,'vv_vimeo_user_album');
			if($yes){
				echo 'success';
			}else{
				echo 'not';
			}
		}
		die();
	}
	
	function wplms_vimeo_shortcode($atts,$content=null){
		extract(shortcode_atts(array(
		  'file'   => '',
        ), $atts));
        $args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				    	),
				);
		$response = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/videos/'.$file,$args);
		
		$response = json_decode(wp_remote_retrieve_body( $response ),true);
		
		$return = '';
		if(!empty($response['embed']['html'])){
			if(!$this->vv_vimeo_front_script_loaded){
				$this->vv_vimeo_front_script_loaded = 1;
				$return .= '<script type="text/javascript" src="'.plugin_dir_url( __FILE__ ) . '../assets/vimeo-player/player.min.js' .'"></script>';
				$return .= '<script type="text/javascript" src="'.plugin_dir_url( __FILE__ ) . '../assets/js/video-vibe-vimeo-front.js' .'"></script>';
			}
			
			$return .= '<div class="vv-vimeo-wrapper '.(!empty($this->settings['vimeo_seek_lock'])?'seek_lock':'not_seek_lock').' '.(!empty($this->settings['vimeo_auto_mark_complete'])?'auto_mark_complete':'not_auto_mark_complete').'">'.do_shortcode('[iframevideo]'.$response['embed']['html'].'[/iframevideo]').'</div>';
		}
		
		return $return;
	}

	function wplms_wistia_shortcode($atts,$content=null){
		
		extract(shortcode_atts(array(
		  'file'   => '',
        ), $atts));
        //http://fast.wistia.net/oembed?url=https://support.wistia.com/medias/26sk4lmiix
        $color = '#54bbff';
        if(function_exists('bp_wplms_get_theme_color') && !empty(bp_wplms_get_theme_color())){
        	$color = bp_wplms_get_theme_color();
        }
        $color = str_replace('#', '', $color);

        $color = apply_filters('vv_wistia_player_color',$color,$atts);
		$response = wp_remote_get( 'http://fast.wistia.net/oembed?url=https://support.wistia.com/medias/'.$file.'&embedType=async&videoFoam=1&playerColor='.$color,$args);
		$response = json_decode(wp_remote_retrieve_body( $response ),true);
		
		$return = '';
		if(!empty($response['html'])){
						//also send the video id json data
			if(!$this->vv_wistia_front_script_loaded){
				$this->vv_wistia_front_script_loaded=1;
				echo '<script>var wistia_media_ids=[];</script>';
			}

			$return .= '<div class="vv-wistia-wrapper '.(!empty($this->settings['wistia_seek_lock'])?'seek_lock':'not_seek_lock').' '.(!empty($this->settings['wistia_auto_mark_complete'])?'auto_mark_complete':'not_auto_mark_complete').'">'.do_shortcode('[iframevideo]'.$response['html'].'[/iframevideo]').'</div>';
			$return .= '<script>if(wistia_media_ids){wistia_media_ids.push("'.$file.'");}</script>';
			$this->wistia_media_ids[] =  $file;
		}
		
		return $return;
	}

	function print_wistia_media_ids(){
		if(!empty($this->wistia_media_ids)){
			echo '<script>var wistia_media_ids='.json_encode($this->wistia_media_ids).';</script>';
		}
		if(!empty($this->settings)){
			$settings = array(
				'wistia_seek_lock' => empty($this->settings['wistia_seek_lock'])?0:1,
				'wistia_privacy' => empty($this->settings['wistia_privacy'])?0:1,
				'wistia_auto_mark_complete' => empty($this->settings['wistia_auto_mark_complete'])?0:1
			);
			echo '<script>var wistia_vv_settings='.json_encode($settings).';</script>';
		}
	}

	function add_the_scripts_once($content){
		if(has_shortcode( $content, 'wplms_wistia' )){
			ob_start();
			$this->print_wistia_media_ids();
			$en = ob_get_clean();
			$enqueue .= '<script src="//fast.wistia.com/assets/external/E-v1.js" async></script>';
			$enqueue .= '<script type="text/javascript" src="'.plugin_dir_url( __FILE__ ) . '../assets/js/video-vibe-wistia-front.js' .'"></script>';

			$content = $content.$en.$enqueue;
		}
		return $content;
	}

	function get_vimeo_me_info(){
		if(!is_user_logged_in())
			die();
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_me_vimeo') || !is_user_logged_in()){
			die();
		}else{
			$args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				  	));
			$request = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/me',$args );
			echo wp_remote_retrieve_body( $request );
		}
		die();
	}

	function update_uploaded_vimeo_video(){
		/*
		to delete domain video:
		 
		DELETE	https://api.vimeo.com/videos/{video_id}/privacy/domains/{domain}
		eg  https://api.vimeo.com/videos/225254854/privacy/domains/vibethemes.com


		to add domain to video:

		PUT https://api.vimeo.com/videos/{video_id}/privacy/domains/{domain}
		eg  https://api.vimeo.com/videos/225254854/privacy/domains/vibethemes.com
		*/
		if(!is_user_logged_in())
			die();
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_albums'.get_current_user_id())){
			die();
		}else{
			$args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				  	));
			$video_file = sanitize_text_field($_POST['video']);
			//fetch all video data to modify
			$request1 = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/videos/'.$video_file,$args);
			$video_json = json_decode(wp_remote_retrieve_body( $request1 ),true);
			if(!empty($video_json)){
				
				
				$video_json['name'] = $_POST['name'];
				$video_json['privacy']['view'] = $_POST['privacy'];
				if(!empty($_POST['vimeo_domain_privacy_ed'])){
					$video_json['privacy']['embed'] = 'whitelist';
				}
				if(!empty($_POST['password']) && $_POST['privacy']=='password'){
					$video_json['password'] = $_POST['password'];
				}
				/*if(!empty($this->settings['vimeo_seek_lock'])){
					$video_json['embed']['playbar'] = false;
				}*/

				if(!empty($video_json['type'])){
					unset($video_json['type']);
				}
				if(!empty($video_json['user'])){
					unset($video_json['user']);
				}
				if(!empty($video_json['embed']['logos'])){
					unset($video_json['embed']['logos']);
				}
				
				$args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				    	'Content-type' => 'application/json',
				  	),
				  	'method' => 'PATCH',
				  	'timeout' => 30,
				   	'body' => json_encode($video_json),
					);

				
				//edit video call
				$request2 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/videos/'.$_POST['video'],$args);
				$request2_body = json_decode(wp_remote_retrieve_body( $request2 ),true);
				print_r(	$request2_body);
				if(!empty($request2_body['uri']) ){
					$args5 = array(
					    	'headers' => array(
					    		'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					  		),
					  		'method' => 'PUT',
					  		'timeout' => 200,
				    		
						);

			   		if(!empty($_POST['vimeo_domain_privacy_ed'])){
			   			$url = site_url();
						$parse = parse_url($url);
						if(!empty($parse)){
							$site_domain =  $parse['host'];
							if(!empty($site_domain)){
								preg_match("/[a-z0-9\-]{1,63}\.[a-z\.]{2,6}$/", $site_domain, $_domain_tld);
							}
						}
						
			   			if(!empty($_domain_tld[0])){
			   				$args6 = array(
						    	'headers' => array(
						    		'Authorization' => 'Bearer ' . $this->vimeo_access_token,
						  		),
						  		'method' => 'PUT',
						  		'timeout' => 200,
							);
			   				//add site domain to video call
				   			$response6 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/videos/'.$_POST['video'].'/privacy/domains/'.$_domain_tld[0],$args6);
				   			if(!empty($response6) && !empty($response6['response']['code']) && $response6['response']['code'] == '204'){
				   				echo '<p>'._x("Site domain added to allowed domain for this video",'','video-vibe').'</p>';
				   			}
				   		}
				   			
			   			
			   		}
			   		
			   		//adding video to album request
			   		$response5 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/users/'.$_POST['user'].'/albums/'.$_POST['album'].'/videos/'.$_POST['video'],$args5);
			   		if(!empty($response5) && !empty($response5['response']['code']) && $response5['response']['code'] == '204'){
			   			echo 'success';	
			   		}
		   		}
			}
		}
		die();
	}

	function get_vimeo_album_videos(){
		if(!is_user_logged_in())
			die();
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_albums'.get_current_user_id())){
			die();
		}else{
			$args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				    	),
				);
			$link = $_POST['link'];
			if(!empty($link)){
				$request =  wp_remote_get(VIMEO_ROOT_ENDPOINT.$link,$args);
				if(!empty($request))
				echo wp_remote_retrieve_body( $request );
			}else{
				$request = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/me/albums/'.$_POST['album'].'/videos/?per_page='.$this->videos_per_page,$args);
				echo wp_remote_retrieve_body( $request );
			}
			
		}
		die();
	}
	
	function get_vimeo_albums(){
		if(!is_user_logged_in())
			die();
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_albums'.get_current_user_id())){
			die();
		}else{
			$args = array(
		  	'headers' => array(
		    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
		    	),
		  	);
			$link = (!empty($_POST['link'])?$_POST['link']:'');
			if(!empty($link)){
				$request =  wp_remote_get(VIMEO_ROOT_ENDPOINT.$link,$args);
				if(!empty($request))
				echo wp_remote_retrieve_body( $request );
			}else{
				$request =  wp_remote_get(VIMEO_ROOT_ENDPOINT.'/me/albums?per_page='.$this->albums_per_page,$args);
				if(!empty($request))
				echo wp_remote_retrieve_body( $request );
			}
			
			
		}
		die();
	}

	function get_vimeo_upload_ticket(){
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'get_vv_vimeo_upload_ticket') || !is_user_logged_in()){
			return false;
		}else{

			$user_id = get_current_user_id();
			$upload_dir_base = wp_upload_dir();
			$filePath = $upload_dir_base['basedir']."/temp_folder_vv_vimeo/".$user_id."/".$_POST['name'];
			if(!in_array($this->_mime_content_type_vimeo( $filePath),get_allowed_mime_types())){
				echo '0';
				unlink($filePath);
				die();
			}else{

				$upload = $this->upload_file_vimeo( $filePath );
			   	if(!empty($upload['new_video_id'])){
			   			
		
			   		//set name privacy of the uploaded video
			   		//edit video call : PATCH https://api.vimeo.com/videos/{video_id
					echo '<div class="wplms_vimeo_errors"><p class="wplms_success"><h5>' . sprintf(__('File "%s" uploaded to album "%s"','video-vibe'), $_POST["name"],$_POST['album_name']). '<h5></p>
						<p>'._x('Please name your file and set its privacy','uploaded heading asking to set privacy and all ','video-vibe').'</p>
						<p>'._x('Name','name in form that shows after video upload','video-vibe').'<input type="text" name="vimeo_video_file_name" class="vimeo_video_file_name"></p>
						<p>'._x('Privacy','privacy in form that shows after video upload','video-vibe').'
						<select class="vimeo_video_file_privacy" name="vimeo_video_file_privacy">
						  <option value="">Choose one</option>
						  <option value="anybody">'._x('Anybody','vimeo uploaded file dropdown','video-vibe').'</option>
						  <option value="nobody">'._x('Private','vimeo uploaded file dropdown','video-vibe').'</option>
						  <option value="password">'._x('Password','vimeo uploaded file dropdown','video-vibe').'</option>
						  <option value="disable">'._x('Hide from vimeo.com','vimeo uploaded file dropdown','video-vibe').'</option>
						  
						</select>
						</p>

					<p><input type="password" name="vimeo_video_password" class="vimeo_video_password" style="display:none"></p>';
					if(!empty($this->settings['vimeo_domain_privacy_ed'])){
						echo '<p>'._x('Allow this video for this site domain only.','','video-vibe').'<input type="checkbox" value="1" name="vimeo_domain_privacy_ed" class="vimeo_domain_privacy_ed"></p>';
					}
					

					echo '<p>
						<a class="update_vimeo_video_file button button-primary" data-user-id="'.$upload['vimeo_user_id'].'" data-album-id="'.$_POST['album_id'].'" data-new-vimeo-file="'.$upload['new_video_id'].'">Update</a>
					
						<a class="video_vibe_insert_plupload button disabled" data-new-vimeo-file="'.$upload['new_video_id'].'">'.__('Insert','video-vibe').'</a>
					</p>
					</div>';	
			   	}else{
			   		echo '<div class="wplms_vimeo_errors"><p class="wplms_error error warning"><h5>'._x('Something went wrong!!','somwthing wrong warning','video-vibe').'</h5></p></div>';
			   	}
			   		
			}
			die();
		}
		die();
	}

	public function video_vibe_tabs_vimeo( $tabs ) {
		if(!empty($this->vimeo_access_token)){
			$tabs['vv_vimeo_library'] = __( 'Vimeo Library', 'video-vibe' );
			
		}
		if(!empty($this->settings) && !empty($this->settings['wistia_access_token'])){
			$tabs['vv_wistia_library'] = __( 'Wistia Library', 'video-vibe' );
		}
		return $tabs;
	}

	function vv_viemeo_plupload(){

		check_ajax_referer('vv_viemeo_plupload');
		if(!is_user_logged_in())
			die(__('user not logged in','video-vibe'));
		$user_id = get_current_user_id();
		if (empty($_FILES) || $_FILES['file']['error']) {
		  die('{"OK": 0, "info": "Failed to move uploaded file."}');
		}

		$chunk = isset($_REQUEST["chunk"]) ? intval($_REQUEST["chunk"]) : 0;
		$chunks = isset($_REQUEST["chunks"]) ? intval($_REQUEST["chunks"]) : 0;
		 
		$fileName = isset($_REQUEST["name"]) ? $_REQUEST["name"] : $_FILES["file"]["name"];
		$upload_dir_base = wp_upload_dir();
		if(function_exists('is_dir') && !is_dir($upload_dir_base['basedir']."/temp_folder_vv_vimeo/".$user_id)){
			if(function_exists('mkdir')) 
				mkdir($upload_dir_base['basedir']."/temp_folder_vv_vimeo/".$user_id, 0755, true) || chmod($upload_dir_base['basedir']."/temp_folder_vv_vimeo/".$user_id, 0755);
		}
		$filePath = $upload_dir_base['basedir']."/temp_folder_vv_vimeo/".$user_id."/".$fileName;
		 
		// Open temp file
		if($chunk == 0) $perm = "wb" ;
		else $perm = "ab";

		$out = @fopen("{$filePath}.part",$perm );

		if ($out) {
		  // Read binary input stream and append it to temp file
		  $in = @fopen($_FILES['file']['tmp_name'], "rb");
		 
		  if ($in) {
		    while ($buff = fread($in, 4096)){
		      fwrite($out, $buff);
		    }
		  } else
		    die('{"OK": 0, "info": "Failed to open input stream."}');
		 
		  @fclose($in);
		  @fclose($out);
		 
		  @unlink($_FILES['file']['tmp_name']);
		} else
		  die('{"OK": 0, "info": "Failed to open output stream."}');
		 
		 
		// Check if file has been uploaded
		if (!$chunks || $chunk == $chunks - 1) {
		  // Strip the temp .part suffix off
		  rename("{$filePath}.part", $filePath);
		}
		die('{"OK": 1, "info": "Upload successful."}');
		exit;
	}

	function _mime_content_type_vimeo($filename) {

	    /**
	    *    mimetype
	    *    Returns a file mimetype. Note that it is a true mimetype fetch, using php and OS methods. It will NOT
	    *    revert to a guessed mimetype based on the file extension if it can't find the type.
	    *    In that case, it will return false
	    **/
	    if (!file_exists($filename) || !is_readable($filename)) return false;
	    if(class_exists('finfo')){
	        $result = new finfo();
	        if (is_resource($result) === true) {
	            return $result->file($filename, FILEINFO_MIME_TYPE);
	        }
	    }
	    
	     // Trying finfo
	     if (function_exists('finfo_open')) {
	       $finfo = finfo_open(FILEINFO_MIME);
	       $mimeType = finfo_file($finfo, $filename);
	       finfo_close($finfo);
	       // Mimetype can come in text/plain; charset=us-ascii form
	       if (strpos($mimeType, ';')) list($mimeType,) = explode(';', $mimeType);
	       return $mimeType;
	     }
	    
	     // Trying mime_content_type
	     if (function_exists('mime_content_type')) {
	       return mime_content_type($filename);
	     }
	    

	     // Trying to get mimetype from images
	     $imageData = getimagesize($filename);
	     if (!empty($imageData['mime'])) {
	       return $imageData['mime'];
	     }
	     // Trying exec
	     if (function_exists('exec')) {
	       $mimeType = exec("/usr/bin/file -i -b $filename");
	       if(strpos($mimeType,';')){
	         $mimeTypes = explode(';',$mimeType);
	         return $mimeTypes[0];
	       }
	       if (!empty($mimeType)) return $mimeType;
	     }
	    return false;
    }
    
	public function vv_vimeo_library_iframe() {
		if ( ! empty( $_POST ) ) {
			$return = media_upload_form_handler();
			if ( is_string( $return ) )
				return $return;
		}
		wp_iframe( array( $this, 'vv_vimeo_library_tab' ) );
	}

	public function create_user_wistia_project(){
		if(!is_user_logged_in())
			die();
		$user_id = get_current_user_id();
		$user = get_userdata($user_id);
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'vibe_wistia_security')){
			die();
		}
		$return = array();
        $projectdata = array(
        		'name'=>$user->user_email,
        		'adminEmail' => $user->user_email,

        	);
        $wistiaapi = VibeWistiaApi::init();
        $project = $wistiaapi->projectCreate($projectdata);
        $project = (array)$project;
        if(!empty($project) && !empty($project['publicId'])){
        	update_user_meta($user_id,'wistia_project_id',$project['publicId']);
        	echo json_encode(array('projectid'=>$project['publicId']));
        }
        die();
	}

	public function vv_wistia_library_iframe() {
		if ( ! empty( $_POST ) ) {
			$return = media_upload_form_handler();
			if ( is_string( $return ) )
				return $return;
		}
		wp_iframe( array( $this, 'vv_wistia_library_tab' ) );
	}
	
	public function vv_wistia_library_tab( $type = 'file', $errors = null, $id = null ) {
		if(empty($this->wistia_token)){
            $this->wistia_token = '';
            if(!empty($this->settings) && !empty($this->settings['wistia_access_token'])){
                $this->wistia_token = $this->settings['wistia_access_token'];
            }

        }
        //disabling to check for admin
        $this->wistia_privacy = (is_user_logged_in() && current_user_can('manage_options'))?0:$this->wistia_privacy;
		echo '<div id="vibe_wistia_library"></div>';
		$fcm_token_set = array();
        $user_id = 0;
        $loggedin = false;
        if(is_user_logged_in()){
            $loggedin = true;
            $user_id = get_current_user_id();
            
        }
       
        $data = array(
                'user'=>array(
                    'id'=> $user_id,
                    'ip'=>$_SERVER["REMOTE_ADDR"],
                ),
                'loggedin'=>$loggedin,
                'settings'=>array(
                    'security'=>vibe_get_option('security'),
                    'security_nonce'=>wp_create_nonce( 'vibe_wistia_security' ),
                    'wp_rest'   => wp_create_nonce('wp_rest'),
                    'ajaxurl' => admin_url('admin-ajax.php')
                ),
                'videovibe_api_url'=>home_url().'/wp-json/'.VIDEOVIBE_API_NAMESPACE.'/',
                'wistia_privacy' => (!empty($this->wistia_privacy))?1:0,
                'wistia_token' => $this->wistia_token,
                'instructor_project_id' => get_user_meta($user_id,'wistia_project_id',true),
                'projects_per_page' => apply_filters('videovibe_wistia_projects_per_page',15),
                'medias_per_page' => apply_filters('videovibe_wistia_medias_per_page',15),
                
                'translations'=>array(
                    'leave_page_check'=>_x('Are you sure you want to leave the page ?','','video-vibe'),
                    'yes'=>_x('Yes','','video-vibe'),
                    'cancel'=>_x('Cancel','','video-vibe'),
                    'insert'=>_x('Insert','','video-vibe'),
                    'uploaded_videos'=>_x('Videos Uploaded Just Now','','video-vibe'),
                    'no_albums'=>_x('Sorry, there were no projects found.Contact Admin to create an album in your wistia account','','video-vibe'),
                    'project_name'=>_x('Project Name','','video-vibe'),
                    'actions' => _x('Actions','','video-vibe'), 
                    'load_medias'=>_x('Load Medias','','video-vibe'),
                    'library'=>_x('Library','media tab','video-vibe'),
                    'upload'=>_x('Upload','media tab','video-vibe'),
                    'no_medias'=>_x('Sorry, there were no media found.Upload some videos to this project!','','video-vibe'),
                    'backtoprojects'=>_x('Back to Projects','media tab','video-vibe'),
                    'next'=>_x('Next','','video-vibe'),
                    'previous'=>_x('Previous','','video-vibe'),
                ),
            );
       
        if(!empty($args)){

            if(!empty($args['translations'])){
                $args['translations'] = array_merge($data['translations'],$args['translations']);    
            }
            $data = array_merge($data,$args);
        }
        $data = apply_filters('vibe_wistia_after_data',$data);
        //wp_enqueue_style('vibe_wistia_main_uploader_css','https://fast.wistia.com/assets/external/uploader.css',array(),VIDEO_VIBE_VERSION,true);
        wp_enqueue_style( 'video-vibe-css', plugin_dir_url( __FILE__ ) . '../assets/css/wistia_library.css' );
        wp_enqueue_script('vibe_wistia_main_uploader_api_js','https://fast.wistia.com/assets/external/api.js',array('jquery'),VIDEO_VIBE_VERSION,true);


        //DEvelopment
        //wp_enqueue_script('vibe_wistia_main','http://localhost:3000/static/js/bundle.js',array('jquery','vibe_wistia_main_uploader_api_js'),VIDEO_VIBE_VERSION,true);

        //Live
        wp_enqueue_script('vibe_wistia_main',plugins_url('../assets/js/vv-wistia-admin.js',__FILE__),array('jquery','vibe_wistia_main_uploader_api_js'),VIDEO_VIBE_VERSION,true);

        ?>
	 	<script>
	 		document.addEventListener('insertWisitaVideo',(e)=>{
                let media = e.detail;
                console.log(media);
                var win = window.dialogArguments || opener || parent || top;
		        if(win){
		            if(media.id){
		                win.send_to_editor("[wplms_wistia file='"+media.id+"'][/wplms_wistia]");
		                if(parent && parent.window){
		                    parent.window.tb_remove();
		                }
		            }
		            
		        }
            });
            document.addEventListener('insertWisitaVideohash',(e)=>{
                let media = e.detail;
                console.log(media);
                var win = window.dialogArguments || opener || parent || top;
		        if(win){
		            if(media.hashed_id){
		                win.send_to_editor("[wplms_wistia file='"+media.hashed_id+"'][/wplms_wistia]");
		                if(parent && parent.window){
		                    parent.window.tb_remove();
		                }
		            }
		        }
            });
	 	</script>
	 	<?php

        wp_localize_script('vibe_wistia_main','vibe_wistia_main',$data);

        wp_nonce_field('vibe_wistia_security','vibe_wistia_security');
        ?>
        <?php
	}

	public function vv_vimeo_library_tab( $type = 'file', $errors = null, $id = null ) {
		if(is_user_logged_in() && current_user_can('manage_options')){
			$this->vimeo_privacy = apply_filters('vv_vimeo_privacy',0);
		}
		media_upload_header();
		wp_enqueue_style( 'media' );
		
		$user_id = get_current_user_id();
	
		wp_enqueue_style( 'video-vibe-css', plugin_dir_url( __FILE__ ) . '../assets/css/video-vibe.css' );
		wp_enqueue_script('video-vibe-js', plugin_dir_url( __FILE__ ) . '../assets/js/video-vibe.js'  );
		$translation_array = array( 

							'privacy_warning' => __( 'Please set a privacy for video','video-vibe' ),
							'name_warning' => __( 'Please set a name for video','video-vibe' ),
							'embed' => __( 'Embed','video-vibe' ),
							'load' => __( 'Load','video-vibe' ),
							'password_warning' => __( 'Please set a password for video','video-vibe' ),
							'next' => __( 'Next','video-vibe' ),
							'previous' => __( 'Previous','video-vibe' ),
							'no_album_found' => __( 'No albums Found .Reload the page ','video-vibe' ),
							'no_videos_found' => __( 'No videos Found .Please upload a video... ','video-vibe' ),
							'updating' => __('Updating...','video-vibe'),
							'video_uploaded' => _x('Video uploaded to vimeo.com','','video-vibe'),
							'name_file'=>_x('Please name your file and set its privacy','uploaded heading asking to set privacy and all ','video-vibe'),
							'privacy'=>_x('Privacy','privacy in form that shows after video upload','video-vibe'),
							'domain_only'=>_x('Allow this video for this site domain only.','','video-vibe'),
							'name'=>_x('Name','name in form that shows after video upload','video-vibe'),
							'choose_one'=>_x('Choose one','','video-vibe'),
							'update' =>_x('Update','','video-vibe'),
							'insert'=>__('Insert','video-vibe'),
							'settings' => array(
								'domain_privacy' => (!empty($this->settings['vimeo_domain_privacy_ed'])),
								'privacy_option_array'=>array(
									array('label'=>_x('Anybody','vimeo uploaded file dropdown','video-vibe'),'value'=>'anybody'),
									array('label'=>_x('Private','vimeo uploaded file dropdown','video-vibe'),'value'=>'nobody'),
									array('label'=>_x('Password','vimeo uploaded file dropdown','video-vibe'),'value'=>'password'),
									array('label'=>_x('Hide from vimeo.com','vimeo uploaded file dropdown','video-vibe'),'value'=>'disable'),
									
								),
							),
							
							);
       	wp_localize_script( 'video-vibe-js', 'video_vibe_strings', $translation_array );

		?>

		<div id="vv_vimeo_wrapper" class="<?php echo (empty($this->vimeo_privacy)?'show_albums':'') ?>">
		<div id="ajaxloader"></div>
		<?php 
		wp_nonce_field( 'get_albums'.$user_id, 'get_albums' ); 
		$user_album = get_user_meta($user_id,'vv_vimeo_user_album',true);
		?>

		<div id="vv_vimeo_tabs">
			<ul>
				<?php
				if(empty($this->vimeo_privacy)){
					echo '<li><a href="#vv_vimeo_albums_tab" class="active">'. _x('Albums','tabs label in vimeo library','video-vibe').'</a></li>';	
				}else{
					echo '<li><a href="#vv_vimeo_videos_tab" class="active">'. _x('Videos','tabs label in vimeo library','video-vibe').'</a></li>';	
				}
				if(!empty($this->vimeo_privacy)){
					echo '<li><a href="#vv_vimeo_upload_tab" class="'.(empty($user_album)?'disabled':'').'">'._x('Upload','tabs label in vimeo library','video-vibe').'</a></li>';
				}else{
					echo '<li><a href="#vv_vimeo_upload_tab">'._x('Upload','tabs label in vimeo library','video-vibe').'</a></li>';
				}

				?>
			</ul>
		</div>
		<?php
		if(empty($this->vimeo_privacy)){
		?>
			<div id="vv_vimeo_albums_tab">
			<?php
			
			
			echo '<table class="wp-list-table widefat fixed striped vimeo_albums" style="max-height: 500px;overflow-y:scroll;">
				<tr>
				<th>' . __( 'Album name', 'video-vibe' ) . '</th>
				<th>' . __( 'Actions', 'video-vibe' ) . '</th>
				</tr>
				</table>

				<span class="paging"></span>';

			echo '<div id="vimeo_videos"><p class="video_paging"></p><ul></ul></div>';
			?>
			</div>
		<?php
		}else{
			echo '<div  id="vv_vimeo_videos_tab">
					<span class="button button-primary videos_refresh">'._x('Refresh','refresh lable on refresh button','video-vibe').'</span>';
			$args = array(
			  	'headers' => array(
			    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
			  	),
			   	
				);
			$request = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/me/',$args);
			$me = wp_remote_retrieve_body( $request );
			$me = json_decode($me);
			preg_match("/users\/([0-9]*)/", $me->uri, $vimeo_user_id);
			
			
			/*if(!empty($user_album)){
				$reponse = wp_remote_get( 'https://api.vimeo.com/users/'.$vimeo_user_id[1].'/albums/'.$user_album,$args);
				$reponse =  wp_remote_retrieve_body( $reponse );
				$reponse = json_decode($reponse);
				print_r($reponse);	
			}*/
			
			if(empty($user_album)){
				//create new album 
			
				$args2 = array(
			  	'headers' => array(
			    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
			  	),
			   	'body' => array(
			   		'name' => $user_id,
			   		),

				);
				$create_album = wp_remote_post( VIMEO_ROOT_ENDPOINT.'/users/'.$vimeo_user_id[1].'/albums',$args2);
				$create_album =  wp_remote_retrieve_body( $create_album );
				$create_album = json_decode($create_album);
				if(empty($create_album->error)){
					preg_match("/albums\/([0-9]*)/", $create_album->uri, $vimeo_user_album);
					if(!empty($vimeo_user_album[1]))
						$meta = update_user_meta($user_id,'vv_vimeo_user_album',$vimeo_user_album[1]);
					if($meta){
						echo _x('Your album just has been created on vimeo.Please reload the page to begin uploading...','no videos message instructor','video-vibe');
						?>
						<script>
							alert('<?php echo _x('Your album just has been created on vimeo.Reloading page ...','no videos message instructor','video-vibe'); ?>');
							window.location.reload();
						</script>
						<?php
					}
					
				}else{
					echo _x('There was some error while creating your personal album. Error: ','video-vibe').$create_album->error;
				}
			}else{
				//make ajax call to fetch videos
				echo '<script>jQuery(window).load(function(){
					jQuery("#vv_vimeo_videos_tab").trigger("inst_album_loaded",'.$user_album.');
					jQuery("#vv_vimeo_videos_tab").append(\'<input type="hidden" class="instructor_vimeo_album" value="'.$user_album.'">\');
				}); </script>';
			}
			
			echo '<div id="vimeo_videos"><p class="video_paging"></p><ul></ul></div></div>';
		}
			
		
		?>
		<div class="vv_vimeo_tab_content" id="vv_vimeo_upload_tab" >
		<?php
			wp_enqueue_script('vibe_vimeo_tus_uploader_js',plugin_dir_url( __FILE__ ) . '../assets/js/tus.min.js',array('jquery'),VIDEO_VIBE_VERSION,true);
			$form_action_url = esc_url( add_query_arg( array( 'wplms_vimeo_action' => 'vimeo_upload' ), admin_url() ) );

		?>
			<div class="wrap">

				<form enctype="multipart/form-data" method="post" class="video-vibe-upload">
				

				<?php
				if(!empty($this->vimeo_privacy)){
					if(!empty($user_album))
					echo '<select style="display:none" class="albums_select"><option value="'.$user_album.'"></option></select>';
				}else{
					$args = array(
					  	'headers' => array(
					    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					    	),
					  	);
					$request =  wp_remote_get(VIMEO_ROOT_ENDPOINT.'/me/albums',$args);
					if(!empty($request)){
						$this->albums =  json_decode(wp_remote_retrieve_body( $request ),true);
					}
					if(!empty($this->albums['data'])){
						echo '<select class="albums_select">';
						foreach ($this->albums['data'] as $album) {
							preg_match("/albums\/([0-9]*)/", $album['uri'], $album_uri);
							echo '<option value="'.$album_uri[1].'">'.$album['name'].'</option>';
						}
						echo '</select>';
					}
				}
				
				
				

						
			?>
			<div class="uploader-area">
				<label for="upload_video">
					<?php echo _x('Select File','','video-vibe');?>
				</label>
			<input id="upload_video" type="file" accept="video/*" style="display:none;"/>
			</div>
			<div class="pl_vimeo_progress"></div>
			<div class="warning_plupload">
				<h3><?php echo __("Please do not close the window until process is completed","video-vibe") ?></h3>
			</div>
			<style>
				.uploader-area {
				    display: flex;
				    justify-content: center;
				    align-items: center;
				    border: 5px dashed rgba(0,0,0,0.2);
				    margin: 1rem 0;
				}.uploader-area label {padding:5rem;width:100%;text-align:center;}
			</style>
			<script>
			jQuery(document).ready(function($){
				document.querySelector('#upload_video').addEventListener("change", function(e) {
				    // Get the selected file from the input element
				    var file = e.target.files[0];
				    console.log(file);
					var video_file ={};
				    fetch(`https://api.vimeo.com/me/videos`,{
				    	headers: {
					      	'Authorization': 'bearer <?php echo $this->vimeo_access_token;?>',
			      			'Content-Type': 'application/json',
					      	'Accept':'application/vnd.vimeo.*+json;version=3.4'
					    },
			            method : 'POST',
			            body:JSON.stringify({
						  "upload": {
						    "approach": "tus",
						    "size": file.size
						  }
						})
			        }).then(function(res){return res.json()})
			        .then(function(res){
			            console.log(res);

			            if(res.hasOwnProperty('upload') && res.upload){
			            	video_file = res;
			            	jQuery('.pl_vimeo_progress').addClass('visible');
			          		var clone = jQuery('.pl_vimeo_progress').append('<div class="'+file.lastModified+'">'+file.name+'<i></i><strong><span></span></strong></div>');
			          		jQuery('.pl_vimeo_progress').append(clone);
			          		jQuery('.warning_plupload').show(300);
				            var upload = new tus.Upload(file, {
							    uploadUrl: video_file.upload.upload_link,
							    onError: function(error) {
							    	jQuery('.pl_vimeo_progress div.'+file.lastModified+' strong span').css('width', '0%');
	                      			jQuery('.pl_vimeo_progress div.'+file.lastModified+' strong').html("<i class='error'>"+error+"<i>");
	                      			setTimeout(function(){
								  		jQuery('.pl_vimeo_progress div.'+file.lastModified).fadeOut(600);
									}, 1200);
									jQuery('.warning_plupload').hide(300);
	                      			
							        console.log("Failed because: " + error)
							    },
							    onProgress: function(bytesUploaded, bytesTotal) {
							    	




						            var percentage = (bytesUploaded / bytesTotal * 100).toFixed(2);
						            
						            if(percentage <= 100 && percentage >= 1){
								        jQuery('.pl_vimeo_progress div.'+file.lastModified+' strong span').css('width', (percentage)+'%');
								        jQuery('.pl_vimeo_progress div.'+file.lastModified+' i').html( (percentage)+'%');
								    }
						        },
							    onSuccess: function() {
							        
									var _options_array = '';
									window.video_vibe_strings.settings.privacy_option_array.map(function(op,i){
										_options_array += '<option value="'+op.value+'">'+op.label+'</option>';
									});




									var user_id = video_file.user.uri.split('/')[2];
									var album = jQuery('.albums_select').val();
									var new_video_id = video_file.uri.split('/')[2];
									var html = '<div class="wplms_vimeo_errors"><p class="wplms_success"><h5>' +window.video_vibe_strings.video_uploaded+'<h5></p><p>'+window.video_vibe_strings.name_file+'</p><p>'+window.video_vibe_strings.name+'<input type="text" name="vimeo_video_file_name" class="vimeo_video_file_name"></p><p>'+window.video_vibe_strings.privacy+'<select class="vimeo_video_file_privacy" name="vimeo_video_file_privacy"><option value="">'+window.video_vibe_strings.choose_one+'</option>'+_options_array+'</select></p><p><input type="password" name="vimeo_video_password" class="vimeo_video_password" style="display:none"></p>'+((window.video_vibe_strings.settings.domain_privacy)?'<p>'+window.video_vibe_strings.domain_only+'<input type="checkbox" value="1" name="vimeo_domain_privacy_ed" class="vimeo_domain_privacy_ed"></p>':'')+'<p><a class="update_vimeo_video_file button button-primary" data-user-id="'+user_id+'" data-album-id="'+album+'" data-new-vimeo-file="'+new_video_id+'">'+window.video_vibe_strings.update+'</a><a class="video_vibe_insert_plupload button disabled" data-new-vimeo-file="'+new_video_id+'">'+window.video_vibe_strings.insert+'</a></p></div>';
	                      			//$('.stop_vimeo_plupload_upload').removeClass('disabled');
		                          	jQuery('.pl_vimeo_progress div.'+file.lastModified+' strong span').css('width', '100%');
		                          	jQuery('.pl_vimeo_progress div.'+file.lastModified+' i').html('100%');
		                          	
		                          		setTimeout(function(){
										  jQuery('.pl_vimeo_progress div.'+file.lastModified+' strong').fadeOut(500);
										}, 1200);
										jQuery('.pl_vimeo_progress div.'+file.lastModified).html(html);
										setTimeout(function(){
											if(jQuery('.pl_vimeo_progress strong').length < 1){
												jQuery('.warning_plupload').hide(300);
											}
										}, 1750);
												
												
								        
						   		 	
						   		 	jQuery('#vv_vimeo_upload_tab').trigger('vimeo_video_file_uploaded');
							    }
							});
				            upload.start();
			            }
					    // Start the upload
					    
			        });
				    
				});
			});

				
			</script>

						
			</form>
			</div>

		</div>
		</div>
		<?php
		
	}

	//@accepts : filepath 
	//@return values :  new_video_id and Vimeo_user_id
	public function upload_file_vimeo( $filePath ) {
		if(!is_user_logged_in() || !current_user_can('edit_posts'))
			return 0;
		$return = array();
		$args = array(
			  	'headers' => array(
			    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
			    	'Accept'=> 'application/vnd.vimeo.*+json;version=3.3.1'

			  	),
			   	'body' => array(
	                    'type'   => 'streaming',
	                ),
				);

		$request = wp_remote_post( VIMEO_ROOT_ENDPOINT.'/me/videos', $args );
		$ticket = json_decode( wp_remote_retrieve_body( $request ), true );

		if(!empty($ticket) && !empty($ticket['upload_link_secure'])){
			preg_match("/users\/([0-9]*)/", $ticket['user']['uri'], $vimeo_user_id);
			$return['vimeo_user_id'] = $vimeo_user_id[1]; 
			$file = @fopen( $filePath, 'r' );		
			$file_size = filesize( $filePath );	
		   

			$file_data = fread($file, $file_size);
			
			$args = array(
				    	'headers' => array(
					    	'Host' => '1.2.3.4:8080',//site_url(),
					    	'Content-Length'=>$file_size,
					    	'Content-Type'=> $_POST['type'],
				  		),
				  		'method' => 'PUT',
				  		'timeout' => 200,
			    		'body' => 	$file_data,
					);

			$response = wp_remote_request( $ticket['upload_link_secure'], $args);

			if(!empty($response) && !is_wp_error($response) && $response['response']['code'] == 200){
				$verify = wp_remote_request( $ticket['upload_link_secure'], array(
				    	'headers' => array(
					    	'Host' => '1.2.3.4:8080',//site_url(),
					    	'Content-Length'=>0,
					    	'Content-Range'=>'bytes */*',
				  		),
				  		'method' => 'PUT',
				  		'timeout' => 30,
					)
				);	

			   	if($verify['response']['code'] == 308){

			   		fclose($file); 
			   		unlink($filePath);
			   		$args4 = array(
					    	'headers' => array(
					    		'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					  		),
					  		'method' => 'DELETE',
					  		'timeout' => 30,
				    		
						);

		   			$response4 = wp_remote_request(VIMEO_ROOT_ENDPOINT.$ticket['complete_uri'],$args4);
		   			$new_video_uri = wp_remote_retrieve_header( $response4,'location');
		   			if(!empty($new_video_uri)){
				   			preg_match("/videos\/([0-9]*)/", $new_video_uri, $new_video_id);
				   			$return['new_video_id'] = $new_video_id[1];
				   	}
		   		}
	   		}

		}
   		return $return;
	}

	public function delete_file( $file = array() ) {
		// Create  Instructor folder
		
	}


	
	public function api_keys_entered() {

		$key = 1;//get_option( 'wplms_vimeo_license_key' );

		if( empty( $this->access_id ) || empty( $key ) ) {
			return false;
		}

		return true;
	}
	public function show_admin_notices() {

		if ( empty( $this->access_id   ) || empty( $this->secret_key   ) ) {
			$url = admin_url( 'admin.php?page=lms-settings&tab=video-vibe' );
			echo '<div class="update error"><p>' . sprintf( __( 'Please enter your <a href="%s">vimeo vimeo Access Key ID and Secret Key</a>', 'video-vibe' ), $url ) . '</p></div>';
		}
	}
}

global $vv;
$vv = Video_Vibe_Admin::init();

function wplms_vimeo_set_error($code,$error){
	global $vv;
	$vv->errors[$code] = $error;
}