<?php
/**
 * Settings in Admin
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	video-vibe/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Video_Vibe_Settings{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Video_Vibe_Settings();
        return self::$instance;
    }

	private function __construct(){

		if( !defined('VIDEO_VIBE_OPTION')){
		    define('VIDEO_VIBE_OPTION','video-vibe');
		}
		$this->settings = get_option(VIDEO_VIBE_OPTION);
		$this->vimeo_access_token = get_option('vimeo_access_token');
		add_action('template_redirect',array($this,'verify_vimeo'));

		add_filter('wplms_lms_settings_tabs',array($this,'setting_tab'));
		add_filter('lms_settings_tab',array($this,'tab'));
	}

	function setting_tab($tabs){
		$tabs['video-vibe'] = __('Video vibe','video-vibe');
		return $tabs;
	}

	function tab($name){
		if($name == 'video-vibe')
			return 'Video_Vibe_Settings';
		
		return $name;
	}

	function get_settings(){
		if ( function_exists( 'wp_max_upload_size') ) {
			$size =  wp_max_upload_size() ; 
 			$size = $size/1048576;
		}else{
			$size = 1048576;
		}
 			
		return apply_filters('Video_Vibe_Settings',array(
			array(
				'label' => __( 'Vimeo', 'video-vibe' ),
				'name' => 'vimeo',
				'type' => 'heading',
				'desc' => __( 'Vimeo Settings', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Instructor Privacy', 'video-vibe' ),
				'name' => 'vimeo_privacy',
				'type' => 'checkbox',
				'desc' => __( 'Instructors can access only their uploaded files. Creates different albums for every instructor on vimeo.', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Seek lock', 'video-vibe' ),
				'name' => 'vimeo_seek_lock',
				'type' => 'checkbox',
				'desc' => __( 'Does not allow user to seek video', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Auto mark complete unit when vimeo video finishes', 'video-vibe' ),
				'name' => 'vimeo_auto_mark_complete',
				'type' => 'checkbox',
				'desc' => __( 'Automatically marks the unit as complete as soon as video finishes.', 'video-vibe' ),
			),
			array(
				'label' => __( 'Vimeo Client Identifier', 'video-vibe' ),
				'name' => 'vimeo_client_id',
				'type' => 'text',
				'desc' => sprintf(__( 'Put here your vimeo app client id', 'video-vibe' ),'<a href="http://console.aws.amazon.com/" target="_blank">','</a>'),
			),
			array(
				'label' => __( 'Vimeo Client Secrets', 'video-vibe' ),
				'name' =>'vimeo_client_secret',
				'type' => 'text',
				'desc' => __( 'Put here your vimeo app client secret', 'video-vibe' ),
			),
		
			array(
				'label' => __( 'Max upload size', 'video-vibe' ),
				'name' =>'vimeo_plupload_max_size',
				'type' => 'text',
				'desc' => __( 'Set the maximum file size for plupload in vimeo ( In Mb(s) )', 'video-vibe' ),
				'std'=>(2*intval($size))
			),
			array(
				'label' => __( 'Enabled settings for video to allow on this domian.', 'video-vibe' ),
				'name' => 'vimeo_domain_privacy_ed',
				'type' => 'checkbox',
				'desc' => __( 'Enables setting to allow the video for this site domain only.', 'video-vibe' ),
			),
			array(
				'label' => __( 'Wistia', 'video-vibe' ),
				'name' => 'wistia',
				'type' => 'heading',
				'desc' => __( 'Wistia Settings', 'video-vibe' ),
			),
			array(
				'label' => __( 'Wistia access token', 'video-vibe' ),
				'name' =>'wistia_access_token',
				'type' => 'text',
				'desc' => __( 'Put here your wistia access token', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Instructor Privacy', 'video-vibe' ),
				'name' => 'wistia_privacy',
				'type' => 'checkbox',
				'desc' => __( 'Instructors can access only their uploaded files. Creates different albums for every instructor on wistia.', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Seek lock(wistia)', 'video-vibe' ),
				'name' => 'wistia_seek_lock',
				'type' => 'checkbox',
				'desc' => __( 'Does not allow user to seek video', 'video-vibe' ),
			),
			array(
				'label' => __( 'Enable Auto mark complete unit when vimeo video finishes(wistia)', 'video-vibe' ),
				'name' => 'wistia_auto_mark_complete',
				'type' => 'checkbox',
				'desc' => __( 'Automatically marks the unit as complete as soon as video finishes.', 'video-vibe' ),
			),
			
			
		));	
	}
	function settings(){

		echo '<form method="post">';
		wp_nonce_field('Video_Vibe_Settings');   
		echo '<table class="form-table">
				<tbody>';

		$settings = $this->get_settings();
		
		if(isset($this->settings['vimeo_client_id']) && isset($this->settings['vimeo_client_secret'])){

			$redirect_uri = admin_url( "admin.php?page=".$_GET["page"].'&tab='.$_GET["tab"]);
				if(isset($_GET['code'])){

					$args = array(
					  	'headers' => array(
					    	'Authorization' => 'Basic ' . base64_encode( $this->settings['vimeo_client_id'] . ':' . $this->settings['vimeo_client_secret'] ),
					  	),
					   	'body'      =>  array(	'grant_type' => 'authorization_code',
								             	'code' => $_GET['code'],
								             	'redirect_uri' => $redirect_uri),
					   	'method'    => 'POST'
					);
					
					$request = wp_remote_post( 'https://api.vimeo.com/oauth/access_token', $args );

					$response = json_decode( wp_remote_retrieve_body( $request ), true ); 
					if(isset($response['access_token'])){
						update_option('vimeo_access_token',$response['access_token']);	
					}
					
				}else{
					include_once dirname( __FILE__ ) . '/library/vimeo/vimeo.api.php';
					$lib = new Vimeo($this->settings['vimeo_client_id'], $this->settings['vimeo_client_secret']);
					$state = wp_create_nonce('wplms_security');
					$url = $lib->buildAuthorizationEndpoint($redirect_uri,array('public','private','edit','upload','create'), $state);
					if(empty($this->vimeo_access_token)){
						echo '<h3>'._x('Authorise Vimeo account to generate token','heading for vimeo authorize','video-vibe').'</h3><a href="'.$url.'" class="button">'._x('Generate Token','button to auth','video-vibe').'</a>';
						
					}else{
						echo '<h3>'._x('Re-generate token','heading for vimeo authorize','video-vibe').'</h3><a href="'.$url.'" target="_blank" class="button">'._x('Regenerate token','button to regenerate token auth','video-vibe').'</a>';
					}
				}
		}

		$this->generate_form($settings);
		echo '<tr valign="top"><th colspan="2"><input type="submit" name="save_Video_Vibe_Settings" class="button button-primary" value="'.__('Save Settings','video-vibe').'" /></th>';
		echo '</tbody></table></form>';				
	}

	function generate_form($settings){
		
		foreach($settings as $setting ){
			echo '<tr valign="top">';
			switch($setting['type']){
				case 'textarea':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><textarea name="'.$setting['name'].'" style="max-width: 560px; height: 240px;border:1px solid #DDD;">'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:'').'</textarea>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'heading':
					echo '<th scope="row" class="titledesc"><h2>'.$setting['label'].'</h2><span>'.$setting['desc'].'</span></th>';
				break;
				case 'select':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><select name="'.$setting['name'].'">';
					foreach($setting['options'] as $key=>$option){
						echo '<option value="'.$key.'" '.(isset($this->settings[$setting['name']])?selected($key,$this->settings[$setting['name']]):'').'>'.$option.'</option>';
					}
					echo '</select>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'checkbox':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="checkbox" name="'.$setting['name'].'" '.(isset($this->settings[$setting['name']])?'CHECKED':'').' />';
					echo '<span>'.$setting['desc'].'</span>';
				break;
				case 'number':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="number" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'text':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="text" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
			}
		}	
	}
	function save(){
		

		if(!isset($_POST['save_Video_Vibe_Settings']))
			return;

		if ( !isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'],'Video_Vibe_Settings') ){
		     echo '<div class="error notice is-dismissible"><p>'.__('Security check Failed. Contact Administrator.','video-vibe').'</p></div>';
		}

		$settings = $this->get_settings();
		foreach($settings as $setting){
			if(isset($_POST[$setting['name']])){
				$this->settings[$setting['name']] = $_POST[$setting['name']];
			}else if($setting['type'] == 'checkbox' && isset($this->settings[$setting['name']])){
				unset($this->settings[$setting['name']]);
			}
		}

		update_option(VIDEO_VIBE_OPTION,$this->settings);
		echo '<div class="updated notice is-dismissible"><p>'.__('Settings Saved.','video-vibe').'</p></div>';
	}
	function verify_vimeo(){
		if ( !isset($_GET['state']) || !wp_verify_nonce($_GET['state'],'wplms_security') || !is_user_logged_in() || !isset($_GET['code'])){
	        return;
      	}
      	$user_id = get_current_user_id();
      	update_user_meta($user_id,'video_vibe_vimeo_access_code',$_GET['code']);

      	$message = sprintf(_x('Vimeo successfully authorised on this web site. %s click to close %s','Vimeo success authorisation link','videovibe'),'<a onClick="window.close()">','</a>');
      	 wp_die($message);
		
	}
}

Video_Vibe_Settings::init();


function Video_Vibe_Settings(){
	$wplms_vimeo = Video_Vibe_Settings::init();
	$wplms_vimeo->save();
	$wplms_vimeo->settings();
}