<?php

/**
 * WPLMS Attendance API
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	Wplms-Attendance/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_VideoVibe_API{

	public static $instance;
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_VideoVibe_API();
        return self::$instance;
        
    }

	private function __construct(){
		if(!defined('WPLMS_API_NAMESPACE')){
			$this->namespace = BP_COURSE_API_NAMESPACE.'/videovibe';
		}else{
			$this->namespace = WPLMS_API_NAMESPACE.'/videovibe';
			
		}
		$this->type = 'videovibe';  // use for App
		$this->settings = get_option(VIDEO_VIBE_OPTION);
		$this->vimeo_access_token = get_option('vimeo_access_token');
		add_action( 'rest_api_init', array($this,'videovibe_endpoints'));
		
 
	}


	function videovibe_endpoints(){
		register_rest_route( $this->namespace, '/vimeo/gettabs', array(
			array(
				'methods'             =>  "POST",
				'callback'            =>  array( $this, 'gettabs' ),
				'permission_callback' => array( $this, 'videovibe_request_validate' ),
				
			),
		));
		register_rest_route( $this->namespace, '/vimeo/updatevideo', array(
			array(
				'methods'             =>  "POST",
				'callback'            =>  array( $this, 'updatevideo' ),
				'permission_callback' => array( $this, 'videovibe_request_validate' ),
				
			),
		));
		
	}


	function videovibe_request_validate($request){

		$body = json_decode($request->get_body(),true);
        
       

        if(!empty($body['token'])){
            
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
            	$this->user_id = $this->user->id;
                return true;
            }
        }

		
		return false;
	}

	function updatevideo($request){
		$post = json_decode($request->get_body(),true);
		if(!empty($post['video'])){
			$return = array('status'=>true);
			$args = array(
				'headers' => array(
			    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
			  	));
			$video_file = sanitize_text_field($post['video']);
			//fetch all video data to modify
			$request1 = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/videos/'.$video_file,$args);
			$video_json = json_decode(wp_remote_retrieve_body( $request1 ),true);
			if(!empty($video_json)){
				
				
				$video_json['name'] = $post['name'];
				$video_json['privacy']['view'] = $post['privacy'];
				if(!empty($post['vimeo_domain_privacy_ed'])){
					$video_json['privacy']['embed'] = 'whitelist';
				}
				if(!empty($post['password']) && $post['privacy']=='password'){
					$video_json['password'] = $post['password'];
				}
				/*if(!empty($this->settings['vimeo_seek_lock'])){
					$video_json['embed']['playbar'] = false;
				}*/

				if(!empty($video_json['type'])){
					unset($video_json['type']);
				}
				if(!empty($video_json['user'])){
					unset($video_json['user']);
				}
				if(!empty($video_json['embed']['logos'])){
					unset($video_json['embed']['logos']);
				}
				
				$args = array(
					'headers' => array(
				    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
				    	'Content-type' => 'application/json',
				  	),
				  	'method' => 'PATCH',
				  	'timeout' => 30,
				   	'body' => json_encode($video_json),
					);

				
				//edit video call
				$request2 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/videos/'.$post['video'],$args);
				$request2_body = json_decode(wp_remote_retrieve_body( $request2 ),true);
				
				if(!empty($request2_body['uri']) ){
					$args5 = array(
					    	'headers' => array(
					    		'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					  		),
					  		'method' => 'PUT',
					  		'timeout' => 200,
				    		
						);

			   		if(!empty($post['vimeo_domain_privacy_ed'])){
			   			$url = site_url();
						$parse = parse_url($url);
						if(!empty($parse)){
							$site_domain =  $parse['host'];
							if(!empty($site_domain)){
								preg_match("/[a-z0-9\-]{1,63}\.[a-z\.]{2,6}$/", $site_domain, $_domain_tld);
							}
						}
						
			   			if(!empty($_domain_tld[0])){
			   				$args6 = array(
						    	'headers' => array(
						    		'Authorization' => 'Bearer ' . $this->vimeo_access_token,
						  		),
						  		'method' => 'PUT',
						  		'timeout' => 200,
							);
			   				//add site domain to video call
				   			$response6 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/videos/'.$post['video'].'/privacy/domains/'.$_domain_tld[0],$args6);
				   			if(!empty($response6) && !empty($response6['response']['code']) && $response6['response']['code'] == '204'){
				   				echo '<p>'._x("Site domain added to allowed domain for this video",'','video-vibe').'</p>';
				   			}
				   		}
				   			
			   			
			   		}
			   		
			   		//adding video to album request
			   		$response5 = wp_remote_request( VIMEO_ROOT_ENDPOINT.'/users/'.$post['user'].'/albums/'.$post['album'].'/videos/'.$post['video'],$args5);
			   		if(!empty($response5) && !empty($response5['response']['code']) && $response5['response']['code'] == '204'){
			   			$return = array('status'=>true,'message'=>_x('Video Edited','','video-vibe'));
			   		}else{
			   			$return = array('status'=>false,'message'=>_x('Could not add to album','','video-vibe'));
			   		}
		   		}
			}else{
				$return = array('status'=>false,'message'=>_x('Could not edit video','','video-vibe'));
			}
		}else{
			$return = array('status'=>false,'message'=>_x('Video id missing','','video-vibe'));
		}
		return new WP_REST_Response( $return, 200 ); 
		

	}

	function gettabs($request){
		$body = json_decode($request->get_body(),true);
		$return  = array('status'=>false,'message'=>_x('Account not configured','','video-vibe'));
		if(!empty($this->vimeo_access_token)){
			$return  = array('status'=>true,'message'=>_x('tabs found','','video-vibe'));
			if(!empty($this->settings) && !empty($this->settings['vimeo_privacy']) && !user_can($this->user->id,'manage_options')){
				$user_album = get_user_meta($this->user->id,'vv_vimeo_user_album',true);
				if(empty($user_album)){

					$args = array(
					  	'headers' => array(
					    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					  	),
					   	
						);
					$request = wp_remote_get( VIMEO_ROOT_ENDPOINT.'/me/',$args);
					$me = wp_remote_retrieve_body( $request );
					$me = json_decode($me);
					preg_match("/users\/([0-9]*)/", $me->uri, $vimeo_user_id);
					if(!empty($vimeo_user_id)){
						$args2 = array(
					  	'headers' => array(
					    	'Authorization' => 'Bearer ' . $this->vimeo_access_token,
					  	),
					   	'body' => array(
					   		'name' => $this->user->id,
					   		),

						);
						$create_album = wp_remote_post( VIMEO_ROOT_ENDPOINT.'/users/'.$vimeo_user_id[1].'/albums',$args2);
						$create_album =  wp_remote_retrieve_body( $create_album );
						$create_album = json_decode($create_album);
						if(empty($create_album->error)){
							preg_match("/albums\/([0-9]*)/", $create_album->uri, $vimeo_user_album);
							if(!empty($vimeo_user_album[1])){
								$meta = update_user_meta($this->user->id,'vv_vimeo_user_album',$vimeo_user_album[1]);
							}
							if($meta){
								$user_album = $vimeo_user_album[1];
							}
						}
					}
				}

				$return['instructor_album'] = $user_album;
				if(empty($return['instructor_album'])){
					return new WP_REST_Response( array('status'=>false,'message'=>$create_album->error), 200 );
				}
	        	$tabs = array(
	        		array('value'=>'videos','label'=>_x('Videos','','video-vibe')),
	        		array('value'=>'upload','label'=>_x('Upload','','video-vibe')),
	        	);
	        }else{
	        	$tabs = array(
	        		
	        		array('value'=>'albums','label'=>_x('Albums','','video-vibe')),
	        		array('value'=>'upload','label'=>_x('Upload','','video-vibe')),
	        	);
	        }
	        $return['token'] = $this->vimeo_access_token;
	        $return['tabs'] = $tabs;
		}
        return new WP_REST_Response( $return, 200 );   
	}

}
add_action('init',function(){
Wplms_VideoVibe_API::init();
});
