<?php
/**
 * Initialise WPLMS Batches
 *
 * @class       Bp_Wplms_Videovibe_Init
 * @author      wplms-attendanceThemes
 * @category    Admin
 * @package     WPLMS-Batches/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
class Bp_Wplms_Videovibe_Init{


	public static $instance;
	
	public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Bp_Wplms_Videovibe_Init();
        return self::$instance;
    }

	private function __construct(){
        $this->settings = get_option(VIDEO_VIBE_OPTION);

		add_action('wp_enqueue_scripts',array($this,'enqueue_script'));
	}


	function enqueue_script(){
        if(!defined('WPLMS_PLUGIN_VERSION'))
            return;
		$blog_id = '';
        if(function_exists('get_current_blog_id')){
            $blog_id = get_current_blog_id();
        }

            
		$kb=apply_filters('wplms_videovibe_script_args',array(
			'api_url'=> get_rest_url($blog_id,WPLMS_API_NAMESPACE),
            'vimeo_api_url'=>VIMEO_ROOT_ENDPOINT,
            'settings'=>array(
                'domain_privacy' => (!empty($this->settings['vimeo_domain_privacy_ed'])),
                'privacy_option_array'=>array(
                    array('label'=>_x('Anybody','vimeo uploaded file dropdown','video-vibe'),'value'=>'anybody'),
                    array('label'=>_x('Private','vimeo uploaded file dropdown','video-vibe'),'value'=>'nobody'),
                    array('label'=>_x('Password','vimeo uploaded file dropdown','video-vibe'),'value'=>'password'),
                    array('label'=>_x('Hide from vimeo.com','vimeo uploaded file dropdown','video-vibe'),'value'=>'disable'),
                    
                ), 
                'albums_per_page' => 6,
                'videos_per_page'=>20,
            ),
            'client_id' => (function_exists('vibebp_get_setting')?vibebp_get_setting('client_id'):''),
            'per_page' => apply_filters('wplms_attendance_per_page',10),
            'timestamp'=>time(),

            'date_format'=>get_option('date_format'),
            'time_format'=>get_option('time_format'),
            'translations'=>array(
                'select_file'=>_x('Select file','','video-vibe'),
                'select'=>_x('Select','','video-vibe'),
                'close_window'=> _x("Please do not close the window until process is completed","","video-vibe"),
                'edit_video' => _x('Edit video','','video-vibe'),
                'video_name' => _x('Video Name','','video-vibe'),
                'privacy_warning' => __( 'Please set a privacy for video','video-vibe' ),
                'name_warning' => __( 'Please set a name for video','video-vibe' ),
                'embed' => __( 'Embed','video-vibe' ),
                'load' => __( 'Load','video-vibe' ),
                'password_warning' => __( 'Please set a password for video','video-vibe' ),
                'next' => __( 'Next','video-vibe' ),
                'previous' => __( 'Previous','video-vibe' ),
                'no_albums_found' => __( 'No albums Found .Reload the page ','video-vibe' ),
                'no_videos_found' => __( 'No videos Found .Please upload a video... ','video-vibe' ),
                'updating' => __('Updating...','video-vibe'),
                'video_uploaded' => _x('Video uploaded to vimeo.com','','video-vibe'),
                'name_file'=>_x('Please name your file and set its privacy','uploaded heading asking to set privacy and all ','video-vibe'),
                'privacy'=>_x('Privacy','privacy in form that shows after video upload','video-vibe'),
                'domain_only'=>_x('Allow this video for this site domain only.','','video-vibe'),
                'name'=>_x('Name','name in form that shows after video upload','video-vibe'),
                'choose_one'=>_x('Choose one','','video-vibe'),
                'update' =>_x('Update','','video-vibe'),
                'insert'=>__('Insert','video-vibe'),
                
                'account_not_configured'=>_x('Account not configured','','video-vibe'),
            ),
        ));

        global $post;
        if((function_exists('bp_is_user') && bp_is_user()) || has_shortcode($post->post_content,'vibebp_profile')){
            
            wp_enqueue_script('wplms-videovibe-js',plugins_url('../assets/js/videovibe.js',__FILE__),array('wp-element','wp-data'),VIDEO_VIBE_VERSION);
            wp_localize_script('wplms-videovibe-js','videovibe',$kb);
            wp_enqueue_style('wplms-videovibe-css',plugins_url('../assets/css/videovibe.css',__FILE__),array(),VIDEO_VIBE_VERSION);



        }
	}
	
}

add_action('plugins_loaded',function(){
	
	if(defined('WPLMS_PLUGIN_VERSION')){
		Bp_Wplms_Videovibe_Init::init();
	}
});
