<?php
/**
 * Actions WPLMS Appointments
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	wplms-appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}



class WPLMS_Appointments_Activity{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Activity();
        return self::$instance;
    }


    function __construct(){

        add_action('bp_activity_register_activity_actions',array($this,'register_actions'));
        add_action('wplms_appointments_booking_confirmed',array($this,'booking_confirmed'),10,2);
        add_action('wplms_appointments_booking_rejected',array($this,'rejected_appointment'),10,2);

        add_action('wplms_appointments_appointment_updated',array($this,'appointment_updated'),10,2);
        add_action('wplms_appointment_requsted_by_user',array($this,'wplms_appointment_requsted_by_user'),10,2);

        add_action('wplms_appointments_appointment_rescheduled',array($this,'wplms_appointments_appointment_rescheduled'),10,1);
        add_action('wplms_appointments_appointment_cancelled',array($this,'wplms_appointments_appointment_cancelled'),10,3);
        add_action('wplms_appointments_mark_appointment_attended',array($this,'wplms_appointments_mark_appointment_attended'),10,1);

        add_action('wplms_appointment_requested_cancellation',array($this,'requested_cancellation'),10,1);
        add_action('wplms_appointment_requested_reschedule',array($this,'requested_reschedule'),10,1);

        add_action('appointment_start',array($this,'appointment_started'),10,2);
        add_action('wplms_appointments_meeting_ended',array($this,'meeting_ended'));

        add_action('wplms_appointments_meeting_created',array($this,'meeting_created'),10,3);
        add_action('wplms_appointments_meeting_create_failed',array($this,'meeting_failed'),10,3);
        
    }

    function register_actions(){
        global $bp; 
        $bp_course_action_desc=apply_filters('bp_course_register_actions',array(
            'start_appointment' => __( 'Appointment Started', 'wplms-appointments' ),
            'meeting_finished' => __( 'Appointment meeting finished', 'wplms-appointments' ),
            'meeting_ended' => __( 'Appointment Meeting Ended', 'wplms-appointments' ),
            'appointment_attended' => __( 'Appointment is Attended by user', 'wplms-appointments' ),
            'appointment_cancelled' => __( 'Appointment is cancelled', 'wplms-appointments' ),
            'appointment_marked_attended' => __( 'Appointment is marked attended by instructor', 'wplms-appointments' ),
            'appointment_rescheduled' => __( 'Appointment is Rescheduled', 'wplms-appointments' ),
            'schedule_updated' => __( 'The Schedule of Appointment is Updated', 'wplms-appointments' ),
            'booking_confirmed' => __( 'Booking is Confirmed for an appointment', 'wplms-appointments' ),
            'booking_rejected' => __( 'Booking is Rejected by Instructor', 'wplms-appointments' ),
            'requested_cancellation' => __( 'Cancellation is Requested for a Booking', 'wplms-appointments' ),
            'requested_reschedule' => __( 'Reschedule is Requested for a Booking', 'wplms-appointments' ),
            'appointment_requsted' => __( 'Appointment is Requested', 'wplms-appointments' ),
            'appointment_meeting_created' => __( 'Meeting is Created for an Appointment', 'wplms-appointments' ),
            'appointment_meeting_create_failed' => __( 'Created Meeting for Appointment is failed', 'wplms-appointments' ),
            
            ));
        foreach($bp_course_action_desc as $key => $value){
            bp_activity_set_action($bp->activity->id,$key,$value);  
        }
    }

    function appointment_started($schedule_id,$appointment_id){

        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(array('appointment_id'=>$appointment_id));
        if(!empty($appointments)){
            $appointment = $appointments[0];
            $bookers = wplms_get_appointment_bookers($appointment_id);

            if(!empty($bookers)){
                foreach($bookers as $booker_id){
                    $this->record_activity(array(
                        'user_id'=>$booker_id,
                        'action'=> sprintf(_x('Appointment is starting','activity','wplms-appointments'),$this->get_activity_appointment($appointment)),
                        'content'=>sprintf(
                            _x('Appointment %s is about to start.','activity','wplms-appointments'),
                            $this->get_activity_appointment($appointment)),
                        'type'=>'start_appointment',
                        'item_id'=> $appointment['parent_appointment_id'],
                        'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
                    ));
                }
            }
        }
    }

    function meeting_ended($schedule_id){

        if(is_user_logged_in()){

            $user_id = get_current_user_id();

            $meta_db = new WPLMS_APPOINTMENTS_META_DB;

            $sq = $meta_db->get_appointments_meta(array('appointment_id' => $schedule_id,'meta_key'=>'schedule_queue'));

            $schedule_queue = unserialize($sq[0]['meta_value']);

            $meeting_appointment_id = $schedule_queue[0]['appointment_id'];

            if(!empty($meeting_appointment_id) && $meeting_appointment_id){
                
                $db = new WPLMS_APPOINTMENTS_DB;
                $appointments = $db->get_appointments(array('appointment_id'=>$meeting_appointment_id));
                if(!empty($appointments)){
                    $appointment = $appointments[0];


                    if($user_id == $appointment['author_id']){
                        $this->record_activity(array(
                            'user_id'=>$user_id,
                            'action'=> sprintf(_x('Appointment %s meeting finished','activity','wplms-appointments'),$this->get_activity_appointment($appointment)),
                            'content'=>sprintf(
                                _x('Appointment %s Meeting finished.','activity','wplms-appointments'),
                                $this->get_activity_appointment($appointment)),
                            'type'=>'meeting_finished',
                            'item_id'=> $appointment['parent_appointment_id'],
                            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
                        ));
                    }else{
                        $this->record_activity(array(
                            'user_id'=>$user_id,
                            'action'=> sprintf(_x('Appointment %s meeting attended','activity','wplms-appointments'),$this->get_activity_appointment($appointment)),
                            'content'=>sprintf(
                                _x('Appointment %s Meeting attended','activity','wplms-appointments'),
                                $this->get_activity_appointment($appointment)),
                            'type'=>'meeting_ended',
                            'item_id'=> $appointment['parent_appointment_id'],
                            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
                        ));

                        $args = array(
                            'appointment_id'=>$appointment['appointment_id'],
                            'schedule_id'=>$schedule_id,
                            'booker_id'=>$user_id,
                            'author_id'=>$appointment['author_id'],
                            'type'=>'appointment_attended',
                        );

                        do_action('wplms_appointments_mark_appointment_attended',$args);
                    }
                }
            }
        }
    }

    function wplms_appointments_appointment_cancelled($args,$metas,$order_details){
        /*Array
        (
            [appointment_id] => 53
            [schedule_id] => 41
            [booker_id] => 2
            [reason] => 
            [author_id] => 1
            [type] => instructor_cancel
        )*/
        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }
        $this->record_activity(array(
            'user_id'=>$args['author_id'],
            'action'=> sprintf(_x('Appointment %s cancelled by instructor','activity','wplms-appointments'),$this->get_activity_appointment($args)),
            'content'=>sprintf(
                _x('Appointment %s cancelled for %s','activity','wplms-appointments'),
                $this->get_activity_appointment($args),
                bp_core_get_userlink($args['booker_id'])
            ),
            'type'=>'appointment_cancelled',
            'item_id'=> $args['schedule_id'],
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        ));

    }


    function wplms_appointments_mark_appointment_attended($args){

        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }
        $this->args = array(
            'user_id'=>$args['author_id'],
            'action'=> sprintf(_x('Appointment %s Marked as Attended by instructor','activity','wplms-appointments'),$this->get_activity_appointment($args)),
            'content'=>sprintf(
                _x('Appointment %s Marked as Attended for %s','activity','wplms-appointments'),
                $this->get_activity_appointment($args),
                bp_core_get_userlink($args['booker_id'])
            ),
            'type'=>'appointment_marked_attended',
            'item_id'=> $args['schedule_id'],
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        );
        $activity_id = $this->record_activity($this->args);
        $this->args['id']=$activity_id;
    }

    function wplms_appointments_appointment_rescheduled($args){
        /*Array
        (
            [author_id] => 1
            [item_id] => 0
            [booker_id] => 2
            [appointment_id] => 54
            [new_appointment_id] => 53
            [security] => I!0iyhaxX03l
        )*/
        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }
        $this->record_activity(array(
            'user_id'=>$args['author_id'],
            'action'=> sprintf(_x('Appointment %s reschduled','activity','wplms-appointments'),$this->get_activity_appointment($args)),
            'content'=>sprintf(
                _x('Appointment %s reschduled to appointment %s for %s','activity','wplms-appointments'),
                $this->get_activity_appointment($args),
                $this->get_activity_appointment($args['new_appointment_id']),
                bp_core_get_userlink($args['booker_id'])
            ),
            'type'=>'appointment_rescheduled',
            'item_id'=> (empty($appointment['parent_appointment_id'])?$args['appointment_id']:$appointment['parent_appointment_id']),
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        ));

    }

    function bp_core_get_userlink( $user_id ){
        if(empty($this->userlink[$user_id])){
            $this->userlink[$user_id] = bp_core_get_userlink( $user_id );
        }
        return $this->userlink[$user_id];
    }

    function bp_core_get_user_displayname($user_id){

        if(empty($this->displayname[$user_id])){
            $this->displayname[$user_id]=bp_core_get_user_displayname($user_id);
        }


        return $this->displayname[$user_id];

    }

    function get_the_title($item_id){

        if(empty($this->title[$item_id])){
            $this->title[$item_id]=get_the_title($item_id);
        }

        return $this->title[$item_id];
    }

    function get_courselink($item_id){
        if(empty($this->link[$item_id])){
            $this->link[$item_id] = '<a href="'.get_permalink($item_id).'">'.$this->get_the_title($item_id).'</a>';
        }

        return $this->link[$item_id];
    }

    function get_activity_appointment($appointment){
        //return '<span>'.json_encode(array('start'=>array('date'=>$appointment['start_date'],'time'=>$appointment['start_time']),'end'=>array('date'=>$appointment['end_date'],'time'=>$appointment['end_time']))).'</span>';
        if(!empty($appointment)){
            if(is_array($appointment) && isset($appointment['appointment_id'])){

                return apply_filters('wplms_appointments_appointment_id_html_activity','<a class="activity_appointment_id" data-id="'.$appointment['appointment_id'].'">'.$appointment['appointment_id'].'</a>',$appointment);
            }else{
                return apply_filters('wplms_appointments_appointment_id_html_activity','<a class="activity_appointment_id" data-id="'.$appointment.'">'.$appointment.'</a>',$appointment);
            }
        }
        
    }

    function record_activity($args){

        global $bp;
        $defaults = array(
            'id' => false,
            'user_id' => $bp->loggedin_user->id,
            'action' => '',
            'content' => '',
            'primary_link' => '',
            'component' => 'appointments',
            'type' => false,
            'item_id' => false,
            'secondary_item_id' => false,
            'recorded_time' => gmdate( "Y-m-d H:i:s" ),
            'hide_sitewide' => false
        );

        $r = wp_parse_args( $args, $defaults );
        extract( $r );
        return bp_activity_add( 
            apply_filters('wplms_appointments_record_activity',
                array( 
                    'id' => $id, 
                    'user_id' => $user_id, 
                    'action' => $action, 
                    'content' => $content, 
                    'primary_link' => $primary_link, 
                    'component' => $component, 
                    'type' => $type, 
                    'item_id' => $item_id, 
                    'secondary_item_id' => $secondary_item_id, 
                    'recorded_time' => $recorded_time, 
                    'hide_sitewide' => $hide_sitewide 
                ) 
            ));

    }

    function appointment_updated($schedule,$details){
        

        $this->record_activity(array(
            'user_id'=>$schedule['author_id'],
            'action'=>_x('Appointment  schedule updated by Instructor','activity','wplms-appointments'),
            'content'=>sprintf(_x('Appointment %s schedule updated by Instructor %s','activity','wplms-appointments'),$schedule['appointment_id'],$schedule['author_id']),
            'type'=>'schedule_updated',
            'item_id'=> $schedule['appointment_id'],
            'secondary_item_id'=>$schedule['item_id']
        ));
    }

    function booking_confirmed($booker_id,$args){

        $this->record_activity(array(
            'user_id'=>$args['author_id'],
            'action'=> sprintf(_x('Appointment %s booking confirmed','activity','wplms-appointments'),$this->get_activity_appointment($args)),
            'content'=>sprintf(
                _x('Appointment %s booking confirmed by %s for %s','activity','wplms-appointments'),
                $this->get_activity_appointment($args),
                bp_core_get_userlink($args['author_id']),
                bp_core_get_userlink($booker_id)
            ),
            'type'=>'booking_confirmed',
            'item_id'=> (empty($args['parent_appointment_id'])?$args['appointment_id']:$args['parent_appointment_id']),
            'secondary_item_id'=>!empty($args['item_id'])?$args['item_id']:$args['appointment_id']
        ));
        
    }

    function rejected_appointment($booker_id,$args){
        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(Array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }
        $this->record_activity(array(
            'user_id'=>$args['instructor_id'],
            'action'=> sprintf(_x('Appointment %s booking rejected','activity','wplms-appointments'),$this->get_activity_appointment($args)),
            'content'=>sprintf(
                _x('Appointment %s booking rejected by %s for %s' ,'activity','wplms-appointments'),
                $this->get_activity_appointment($args),
                bp_core_get_userlink($args['instructor_id']),
                bp_core_get_userlink($booker_id)
            ),
            'type'=>'booking_rejected',
            'item_id'=> (empty($args['parent_appointment_id'])?$args['appointment_id']:$args['parent_appointment_id']),
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        ));
    }

    function requested_cancellation($args){
        

        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(Array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }

        $this->record_activity(array(
            'user_id'=>$args['booker_id'],
            'action'=>_x('Cancellation requested for Appointment','activity','wplms-appointments'),
            'content'=>sprintf(_x('Cancellation request for Appointment %s sent to instructor %s','activity','wplms-appointments'),(empty($appointment['parent_appointment_id'])?$appointment['appointment_id']:$appointment['parent_appointment_id']),bp_core_get_userlink($args['author_id'])),
            'type'=>'requested_cancellation',
            'item_id'=> (empty($appointment['parent_appointment_id'])?$appointment['appointment_id']:$appointment['parent_appointment_id']),
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        ));
    }

    function requested_reschedule($args){
 
        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(Array('appointment_id'=>$args['appointment_id']));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }
        $appointment_id = (empty($appointment['parent_appointment_id'])?$appointment['appointment_id']:$appointment['parent_appointment_id']);
        $this->record_activity(array(
            'user_id'=>$args['booker_id'],
            'action'=>_x('Reschedule requested for Appointment','activity','wplms-appointments'),
            'content'=>sprintf(_x('Reschedule request for Appointment %s sent to instructor %s','activity','wplms-appointments'),$appointment_id,bp_core_get_userlink($args['author_id'])),
            'type'=>'requested_reschedule',
            'item_id'=> $appointment_id,
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$args['appointment_id']
        ));
    }


    function wplms_appointment_requsted_by_user($booker_id,$schedule){
        
        if(!empty($schedule['parent_appointment_id'])){
            $appointment_id = $schedule['parent_appointment_id'];
            $content = sprintf(
                _x('Appointment %s slot %s requested by %s' ,'activity','wplms-appointments'),
                $this->get_activity_appointment($schedule['parent_appointment_id']),
                $this->get_activity_appointment($schedule),
                bp_core_get_userlink($booker_id)
            );
        }else{
            $appointment_id = $schedule['appointment_id'];
            $content = sprintf(
                _x('Appointment slot %s requested by %s' ,'activity','wplms-appointments'),
                $this->get_activity_appointment($schedule),
                bp_core_get_userlink($booker_id)
            );
        }
        $appointment = array();
        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(Array('appointment_id'=>$appointment_id));
        if(!empty($appointments)){
            $appointment = $appointments[0];
        }

        
        $this->record_activity(array(
            'user_id'=>$booker_id,
            'action'=> sprintf(_x('Appointment %s requested','activity','wplms-appointments'),$this->get_activity_appointment($appointment_id)),
            'content'=>$content,
            'type'=>'appointment_requsted',
            'item_id'=> (empty($schedule['parent_appointment_id'])?$schedule['appointment_id']:$schedule['parent_appointment_id']),
            'secondary_item_id'=>!empty($appointment['item_id'])?$appointment['item_id']:$appointment_id
        ));
    }


    function meeting_created($schedule_id,$appointment_id,$author_id){

        $this->record_activity(array(
            'user_id'=>$author_id,
            'action'=> sprintf(_x('Appointment %s meeting created','activity','wplms-appointments'),$this->get_activity_appointment($appointment_id)),
            'content'=>sprintf(_x('Appointment %s meeting is created for Schedule %s','activity','wplms-appointments'),$this->get_activity_appointment($appointment_id),$this->get_activity_appointment($schedule_id)),
            'type'=>'appointment_meeting_created',
            'item_id'=> $schedule_id,
            'secondary_item_id'=>$appointment_id
        ));
    }
    function meeting_failed($schedule_id,$appointment_id,$author_id){
        $this->record_activity(array(
            'user_id'=>$author_id,
            'action'=> sprintf(_x('Unable to create meeting for appointment %s','activity','wplms-appointments'),$this->get_activity_appointment($appointment_id)),
            'content'=>sprintf(_x('Appointment %s meeting could not be created for Schedule %s','activity','wplms-appointments'),$this->get_activity_appointment($appointment_id),$this->get_activity_appointment($schedule_id)),
            'type'=>'appointment_meeting_create_failed',
            'item_id'=> $schedule_id,
            'secondary_item_id'=>$appointment_id
        ));
    }
}


WPLMS_Appointments_Activity::init();