<?php
/**
 *  WPLMS appointments Admin
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	wplms-appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}



class WPLMS_Appointments_Admin{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Admin();
        return self::$instance;
    }


    function __construct(){

    	add_action('admin_notices',array($this,'table_exists'));
    	add_action( 'current_screen', array($this,'install_tables'));

        add_filter('manage_users_columns', array($this,'vibe_add_user_labels_column'));
        add_action('manage_users_custom_column',  array($this,'vibe_show_user_labels_column_content'), 10, 3);
        add_Action('admin_enqueue_scripts',array($this,'users_script'));
        add_action('wp_ajax_wplms_appointment_user_label',array($this,'wplms_appointment_user_label'));
    }


    function table_exists(){
    	$appointment = new WPLMS_APPOINTMENTS_DB;
        $meta = new WPLMS_APPOINTMENTS_META_DB;
    	if(!$appointment->table_exists($appointment->table_name) || !$meta->table_exists($meta->table_name)){
    		echo '<div class="message error"><p>'.sprintf(__('Install WPLMS Appointments tables. %s Click to install %s','wplms-appointments'),'<a href="admin.php?page=lms-settings&tab=appointments&install_tables" class="button-primary">','</a>').'</p></div>';
    	}
    }

    function install_tables(){

    	$current_screen = get_current_screen();
    	if($current_screen->id == 'lms_page_lms-settings' && $_GET['tab'] == 'appointments' && isset($_GET['install_tables'])){
           
    		$appointment = new WPLMS_APPOINTMENTS_DB;
            $meta = new WPLMS_APPOINTMENTS_META_DB;
    		if(!$appointment->table_exists($appointment->table_name) || !$meta->table_exists($meta->table_name)){
    			
                if(!$appointment->table_exists($appointment->table_name)){
                    $appointment->create_table();    
                }
    			if(!$meta->table_exists($meta->table_name)){
                    $meta->create_table();
                }

    			add_action('admin_notices',function(){echo '<div class="message updated"><p>'.__('WPLMS Appointments tables added.','wplms-appointments').'</p></div>';});
    			
    		}
    	}
    	
    }
    
    function vibe_add_user_labels_column($columns){

        $columns['user_labels'] = __('Labels','vibe');
        return $columns;
    }

    function vibe_show_user_labels_column_content($value, $column_name, $user_id) {

        if ( 'user_labels' == $column_name ){
            $init = WPLMS_Appointments_Init::init();
            $init->get_global_settings();

            $labels = $init->settings['appointments_instructor_labels'];
            
            if(!empty($labels)){
                $label = get_user_meta($user_id,'wplms_appointment_user_label',true);
                $return ='<select class="wa_set_user_label" data-user_id="'.$user_id.'"><option value="0">'.__('None','wplms-appointments').'</option>';
                foreach($labels['key'] as $i=>$v){
                   
                    $return .='<option value="'.$v.'" '.selected($label,$v).'>'.$labels['value'][$i].'</option>';
                }
                $return .='</select>';
                
                return $return;
            }

            
        }

        return $value;
    }

    function users_script($hook){
  
        if($hook != 'users.php'){
            return;
        }

        add_action('admin_footer',function(){
           ?>
           <script>

                jQuery(document).ready(function($){
                    var $ = jQuery;
                    $('.wa_set_user_label').on('change',function(){
                        $.ajax({
                          type: "POST",
                          url: ajaxurl,
                          async: true,
                          data: { action: 'wplms_appointment_user_label',
                                  security:'<?php echo wp_create_nonce('wplms_appointment_user_label'); ?>',
                                  user_id: $(this).attr('data-user_id'),
                                  label: $(this).val(),
                                },
                          cache: false,
                        });
                    });
                });
                        
                        
           </script>
           <?php
        });
        
        ?>
        <style>.wa_set_user_label{width:60px}</style>
        <?php
    }

    function wplms_appointment_user_label(){
        if(!current_user_can('manage_options') || !is_numeric($_POST['user_id']) || !wp_verify_nonce($_POST['security'],'wplms_appointment_user_label')){
            die();
        }

        update_user_meta($_POST['user_id'],'wplms_appointment_user_label',$_POST['label']);



        die();
    }
}


WPLMS_Appointments_Admin::init();