<?php
/**
 * API WPLMS appointments
 *
 * @author      VibeThemes
 * @category    Init
 * @package     wplms-appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}


class WPLMS_Appointments_Api{


    public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Api();
        return self::$instance;
    }


    function __construct(){

        add_action('rest_api_init',array($this,'register_routes'));
    }
 

    function register_routes(){

        

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/availability', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointments_slots' ),
                'permission_callback'       => array( $this, 'get_appointments_creds_permissions_check' ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointment/(?P<appointment_id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_details' ),
                'permission_callback'       => array($this,'get_appointments_details_permissions_check'),
                'args'                      =>  array(
                        'appointment_id'    =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        //search_users
        register_rest_route( BP_COURSE_API_NAMESPACE,'/appointments/appointment/(?P<appointment_id>\d+)?',array(
                array(
                    'methods'             =>  'POST',
                    'callback'            =>  array( $this, 'save_appointment_details' ),
                    'permission_callback'       => array($this,'update_appointments_details_permissions_check'),
                    'args'                      =>  array(
                            'appointment_id'    =>  array(
                                'validate_callback'     =>  function( $param, $request, $key ) {
                                                            return is_numeric( $param );
                                                        }
                            ),
                        ),
                ),
            )
        );

        
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/courses', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointments_courses' ),
                'permission_callback'       => array( $this, 'get_appointments_creds_permissions_check' ),
            ),
        ));


        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/instructors', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_instructors' ),
                'permission_callback'       => array( $this, 'get_appointments_creds_permissions_check' ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/bookers', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_bookers' ),
                'permission_callback'       => array( $this, 'get_appointments_creds_permissions_check' ),
            ),
        ));
        
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/instructor/(?P<id>\d+)/(?P<type>(bookings|courses))', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_instructor' ),
                'permission_callback' => array( $this, 'get_appointments_creds_permissions_check' ),
                 'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));
        

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/directory/searchItems', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_search_items' ),
                'permission_callback'       => array($this,'get_appointments_directory_permissions_check'),
            ),
        ));
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/directory/getinstructors', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_directory_instructors' ),
                'permission_callback'       => array($this,'get_appointments_directory_permissions_check'),
            ),
        ));
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/directory/get_instructor_profile/(?P<id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_directory_instructor_profile' ),
                'permission_callback'       => array($this,'get_appointments_directory_permissions_check'),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));


        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/user/(?P<id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_user' ),
                'permission_callback'       => array( $this, 'get_appointments_user_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/item/(?P<id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_item' ),
                'permission_callback'       => array( $this, 'get_appointments_item_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_schedule_details' ),
                'permission_callback'       => array( $this, 'get_appointments_schedule_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/save_schedule/(?P<id>\d+)?', 
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'update_appointment_schedule_details' ),
                'permission_callback'       => array( $this, 'update_appointments_schedule_permissions_check' ),
                
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
        ));


        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/save_availability/(?P<id>\d+)?', array(
            array(
                'methods'             =>  WP_REST_Server::EDITABLE,
                'callback'            =>  array( $this, 'update_availability' ),
                'permission_callback'       => array( $this, 'update_appointments_availability_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)/messages', array(
            array(
                'methods'             =>  'GET',
                'callback'            =>  array( $this, 'get_appointment_schedule_messages' ),
                'permission_callback'       => array( $this, 'get_appointments_schedule_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));


        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)/message', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'send_appointment_schedule_message' ),
                'permission_callback'       => array( $this, 'post_appointments_schedule_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)/activity', array(
            array(
                'methods'             =>  'GET',
                'callback'            =>  array( $this, 'get_appointment_schedule_activity' ),
                'permission_callback'       => array( $this, 'get_appointments_schedule_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/cancelled_rescheduled/', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_cancelled_rescheduled_events' ),
                'permission_callback'       => array( $this, 'get_appointments_schedule_permissions_check' ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/search_users/', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_users' ),
                'permission_callback'       => array( $this, 'get_appointments_search_users_permissions_check' ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/reschedule_booker/', array(
            array(
                'methods'             =>'POST',
                'callback'            =>  array( $this, 'reschedule_booker' ),
                'permission_callback'       => array( $this, 'update_appointments_schedule_permissions_check' ),
            ),
        ));
        


        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)/booking/(?P<type>(cancel|reschedule|instructor_cancel|instructor_reschedule))', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'update_appointment_schedule_booking' ),
                'permission_callback'       => array( $this, 'update_appointments_schedule_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        //Activity Recording for Mark Attended button

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<id>\d+)/booking/(?P<type>(appointment_attended))', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'mark_appointment_attended' ),
                'permission_callback'       => array( $this, 'mark_appointment_attended_permissions_check' ),
                'args'                      =>  array(
                        'id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        // BigBlueButton Meetings

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<schedule_id>\d+)/meetings/set/', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'update_schedule_meeting' ),
                'permission_callback'       => array( $this, 'appointments_schedule_enable_meetings_permissions_check' ),
                'args'                      =>  array(
                        'schedule_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<schedule_id>\d+)/meeting/(?P<appointment_id>\d+)/join', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'join_schedule_meeting' ),
                'permission_callback'       => array( $this, 'appointments_schedule_metting_permissions_check' ),
                'args'                      =>  array(
                        'schedule_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                        'appointment_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<schedule_id>\d+)/meeting/(?P<appointment_id>\d+)/cancel', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'cancel_schedule_meeting' ),
                'permission_callback'       => array( $this, 'appointments_schedule_metting_permissions_check' ),
                'args'                      =>  array(
                        'schedule_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                        'appointment_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));
        
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/schedule/(?P<schedule_id>\d+)/meeting/upload_presentation', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'upload_schedule_presentation' ),
                'permission_callback'       => array( $this, 'appointments_schedule_metting_permissions_check' ),
                'args'                      =>  array(
                        'schedule_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                        'appointment_id'                        =>  array(
                            'validate_callback'     =>  function( $param, $request, $key ) {
                                                        return is_numeric( $param );
                                                    }
                        ),
                    ),
            ),
        ));

/*
        register_rest_route( BP_COURSE_API_NAMESPACE, '/appointments/bookers_statistics', array(
            array(
                'methods'             =>  WP_REST_Server::READABLE,
                'callback'            =>  array( $this, 'get_appointment_bookers_statistics' ),
                'permission_callback'       => array( $this, 'get_appointments_creds_permissions_check' ),
            ),
        ));
*/
    }


    //BBB Schedule_meetings
    function appointments_schedule_enable_meetings_permissions_check($request){

        $post = json_decode($request->get_body(),true);  

        if($post['security'] !== vibe_get_option('security_key') || empty($post['user_id'])){
            return false;
        }

        return true;

    }

    function appointments_schedule_metting_permissions_check($request){

        $post = json_decode($request->get_body(),true);  

        if($post['security'] !== vibe_get_option('security_key') || empty($post['user_id'])){
            return false;
        }

        return true;
    }

    function upload_schedule_presentation($request){
        

        $files = $request->get_body();
        //$files = file_get_contents('php://input');
        $pdf_mime_Types = array(
                                'application/pdf',
                                'application/x-pdf',
                                'application/acrobat',
                                'applications/vnd.pdf',
                                'text/pdf',
                                'text/x-pdf');


        if(in_array($_FILES['file']['type'],$pdf_mime_Types)){

            if($_FILES['file']['size'] > 2097152){
                return new WP_REST_Response(array('status'=>0,'message'=>__('Presentation size more than 2 mb','wplms-appointments')),200);
            }
            

            if ( ! function_exists( 'wp_handle_upload' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/file.php' );
            }
            $uploadedfile = $_FILES['file'];
            $upload_overrides = array( 'test_form' => false);
            $movefile = wp_handle_upload( $_FILES['file'],$upload_overrides );



            $filePath=$movefile['file'];
            $attachment = array(
                            'guid'           => $filePath, 
                            'post_mime_type' => $movefile['type'],
                            'post_title'     => preg_replace( '/\.[^.]+$/', '', basename( $filePath ) ),
                            'post_content'   => '',
                            'post_status'    => 'inherit'
                            );   
                    

            $attachment_id = wp_insert_attachment($attachment,$filePath);  
            $attachment_url= wp_get_attachment_url($attachment_id);

            return new WP_REST_Response(array('status'=>1,'message'=>__('File uploaded','wplms-appointments'),'file_url'=>$attachment_url),200);
        }else{

            return new WP_REST_Response(array('status'=>0,'message'=>__('Only PDF files allowed!','wplms-appointments')),200);
        }
        
        return new WP_REST_Response(array('status'=>0,'message'=>__('Unknown error !','wplms-appointments')),200);
    }

    function cancel_schedule_meeting($request){
        
        $appointment_id = $request->get_param('appointment_id');
        $metadb = new WPLMS_APPOINTMENTS_META_DB;

        $metainfo = $metadb->get_appointments_meta(array('appointment_id'=>$appointment_id,'meta_key'=>'meeting_args'));

        // print_r($appointment_id.'##');
        // print_r($metainfo);

        if(!empty($metainfo) && !empty($metainfo[0]['meta_value'])){


            do_action('wplms_appointments_cancel_schedule_meeting',$appointment_id,$metainfo[0]['meta_value']);

            $meeting_arr = unserialize($metainfo[0]['meta_value']);
            // print_r($meeting_arr);
            
            $bbb_api = WPLMS_Appointments_BBB_API::init();

            $bbb_api->endMeeting( array('meetingID'=>$meeting_arr['meetingID'],'password'=>$meeting_arr['moderatorPW']));

            $metadb->delete_appointment_meta($metainfo[0]['meta_id']);

            $message = __('Meeting cancelled','wplms-appointments');
            $status = 1;

        }else{
            $status = 0;
            $message = __('Meeting can not be cancelled !','wplms-appointments');
        }
        return new WP_REST_Response( array('status'=>$status,'message'=>$message), 200 );
    }

    function join_schedule_meeting($request){

        

        $post = json_decode($request->get_body(),true);  

        $schedule_id = $request->get_param('schedule_id');
        $appointment_id = $request->get_param('appointment_id');
        $this->debug=array();

        // $schedule_id = (int)$post['schedule_id'];
        // $appointment_id = (int)$post['appointment_id'];

        $appointments_meta_db = new WPLMS_APPOINTMENTS_META_DB;

        $metas = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$appointment_id,'meta_key'=>'meeting_args'));

        $this->debug['metas']=$metas;
        if(empty($metas)){

            $appointments_db = new WPLMS_APPOINTMENTS_DB;
            $appointment =  $appointments_db->get_appointments(array('appointment_id'=>$schedule_id));
            if(!empty($appointment)){
                $appointment=$appointment[0];    
            }


            if(!empty($appointment) && $appointment['author_id'] == $post['user_id']){
                
                $meeting_ended = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$appointment_id,'meta_key'=>'meeting_ended'));

                if(empty($meeting_ended)){
                //print_r('---finalised ----');
                    $meeting = WPLMS_Appointments_Meetings::init();
                    $set_meeting = $meeting->set_meeting($schedule_id,$appointment_id);
                    $this->debug['set_meeting']=$set_meeting;
                    //print_r($set_meeting);
                }
            }
            $metas = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$appointment_id,'meta_key'=>'meeting_args'));
        }
        $this->debug['new_metas']=$metas;
        if(!empty($metas)){
            if($metas[0]['meta_value']){

                $userName = bp_core_get_user_displayname($post['user_id']);//

                $bbb_api = WPLMS_Appointments_BBB_API::init();

                $appointments_db = new WPLMS_APPOINTMENTS_DB;

                $appointment =  $appointments_db->get_appointments(array('appointment_id'=>$schedule_id));
                if(!empty($appointment)){
                    $appointment=$appointment[0];    
                }
                
                
                $meeting_arr = unserialize($metas[0]['meta_value']);

               
                if(!empty($appointment) && $appointment['author_id'] == $post['user_id']){
                    //print_r($bbb_api->getJoinURL());
                    
                    $url = $bbb_api->getJoinURL( $meeting_arr['meetingID'], $userName, $meeting_arr['moderatorPW']);
                }else{
                    $url = $bbb_api->getJoinURL( $meeting_arr['meetingID'], $userName, $meeting_arr['attendeePW']);
                }


                return new WP_REST_Response( apply_filters('wplms_appointments_join_meeting_schedule',
                array('status'=>1,
                    'join_url'=>$url,
                    'debug'=>$this->debug,
                    'message'=>__('Meeting schedule is set.','wplms-appointments')
                )),
                        200 );
            }
        }


        return new WP_REST_Response( array('status'=>0,'debug'=>$this->debug,'message'=>__('Unable to set join url','wplms-appointments')), 200 );
    }
    


    function update_schedule_meeting($request){
        
        $schedule_id = $request->get_param('schedule_id');
        $post = json_decode($request->get_body(),true);  


        $db = new WPLMS_APPOINTMENTS_DB;
        $appointment = $db->get_appointments(Array('appointment_id'=>$schedule_id));

        if(!user_can($post->user_id,'manage_options') && $appointment[0]['author_id'] != $post['user_id']){

            return new WP_REST_Response( array('status'=>0,'message'=>__('User does not have sufficient permissions to edit this schedule.','wplms-appointments')), 200 );
        }
        
        //Fetch all schedule events
        $events = $db->get_appointments(Array('parent_appointment_id'=>$schedule_id));
        if(!empty($events)){

            $init = WPLMS_Appointments_Init::init();
            $init->get_global_settings();
            $meeting_buffer_time = $init->settings['meeting_buffer_time'];

            $meta_value = array();
            foreach($events as $event){
                $chron_timestamp = round($event['start_date']/1000,0) + ($event['start_time']+$meeting_buffer_time)*60;
                if(empty($meta_value['current_meeting']) || $meta_value['current_meeting'] > (time() + $meeting_buffer_time*60) ){
                $meta_value['current_meeting'] = $chron_timestamp;
            }

            $meta_value['meeting_schedule'][] = array('timestamp'=>$chron_timestamp,'event_id'=>$event['appointment_id']);

        }

       //echo time() + $meeting_buffer_time*60;


        $appointments_meta_db = new WPLMS_APPOINTMENTS_META_DB;
        $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$schedule_id,'meta_key'=>'meetings'));

        $appointments_meta_db->update_appointment_meta(array(
            'appointment_id'=>$schedule_id,
            'meta_key'=>'enable_meetings',
            'meta_value'=> 1
        ));
       

       //print_r($events);
       //echo tofriendlytime($meta_value['current_meeting'];

            do_action('wplms_set_schedule_meeting',$schedule_id,$meta_value);
            
            return new WP_REST_Response( 
                array('status'=>1,
                    'message'=>__('Meetings Enabled. ','wplms-appointments')
                ), 
                        200 );

        }
        

    
        
    }

    // ENd BBB schedule meetings

    function update_appointments_details_permissions_check($request){
        
        $post = json_decode($request->get_body(),true);  

        if($post['security'] !== vibe_get_option('security_key') || empty($post['appointment_id'])){
            return false;
        }

        return true;
    }



    function save_appointment_details($request){
        $post = json_decode($request->get_body(),true);

        if(!empty($post['type'])){

            $instructor_pending_slots = wplms_appointments_get_pending_slots(array('instructor_id'=>$post['instructor_id']));
            $booker_pending_slots = wplms_appointments_get_pending_slots(array('booker_id'=>$post['booker_id']));


            if($post['type'] === 'accept'){

                wplms_appointments_confirm_booking($post['booker_id'],$post['appointment_id'],$post['extras']);

            }
            if($post['type'] === 'reject'){


                do_action('wplms_appointments_booking_rejected',$post['booker_id'],$post);
            }


            if(count($instructor_pending_slots[$post['appointment_id']])){
                foreach($instructor_pending_slots[$post['appointment_id']] as $key=>$a){
                    if($a['booker_id'] == $post['booker_id']){

                        unset($instructor_pending_slots[$post['appointment_id']][$key]);
                    }
                }
            }
            
                
            
            unset($booker_pending_slots[$post['appointment_id']]);
            
            update_user_meta($post['instructor_id'],'pending_appointments',$instructor_pending_slots);
            update_user_meta($post['booker_id'],'pending_bookings',$booker_pending_slots);   
        }

        return new WP_REST_Response( array('status'=>1,'message'=>__('Details saved','wplms-appointments')), 200 );
    }

    function get_cancelled_rescheduled_events($request){

        $return = array();
        $instructor_ids = json_decode($request->get_param('instructors'),true);
        
        global $wpdb;

        $str = implode(',',$instructor_ids);
        $datas = $wpdb->get_results("SELECT meta_key,meta_value FROM {$wpdb->usermeta} WHERE meta_key in ('requested_cancellation','requested_reschedule') AND user_id IN ($str)");

        
        if(!empty($datas)){
            foreach($datas as $data){

                if(empty($return[$data->meta_key])){
                    $return[$data->meta_key]=array();
                }
                $data->meta_value=(unserialize($data->meta_value));
                if(is_Array($data->meta_value)){
                    $appointment_ids = array_keys($data->meta_value);
                }
                foreach($appointment_ids as $appointment_id){

                    if(!array_search($appointment_id, array_column($return[$data->meta_key], 'appointment_id'))){

                        if(empty($reason)){
                            $reason = array(
                                $data->meta_value[$appointment_id]['user_id']=>$data->meta_value[$appointment_id]['reason']
                            );
                        }else{
                            $reason[$data->meta_value[$appointment_id]['user_id']]=$data->meta_value[$appointment_id]['reason'];
                        }
                        

                        //print_R($data->meta_value);
                        $return[$data->meta_key][]=
                        array(
                            'appointment_id'=>$appointment_id,
                            'users'=>(is_Array($data->meta_value[$appointment_id]['user_id'])?$data->meta_value[$appointment_id]['user_id']:array($data->meta_value[$appointment_id]['user_id'])),
                            'reason'=>$reason
                        );
                        
                   }    
                }
                
            }
        }
        return new WP_REST_Response($return, 200 );
    }

    function update_appointments_schedule_permissions_check($request){

        $post = json_decode($request->get_body(),true);
        if(!wp_verify_nonce($post['security'],'security')){
            //return false;
        }

        return true;
    } 


    function mark_appointment_attended_permissions_check($request){

        $appointment_id = $request->get_param('id');
        $type = $request->get_param('type');
        if(in_Array($type,array('appointment_attended','appointment_unattended')) && is_numeric($appointment_id)){
            return true;
        }
        return false;
    }


    function mark_appointment_attended($request){

        $post = json_decode($request->get_body(),true);

        $args = array(
            'appointment_id'=>$post['appointment_id'],
            'schedule_id'=>$request->get_param('id'),
            'booker_id'=>$post['booker_id'],
            'author_id'=>$post['author_id'],
            'type'=>$request->get_param('type'),
        );

        


        if($args['type'] === 'appointment_attended'){
            
            do_action('wplms_appointments_mark_appointment_attended',$args);
            
            $activity = WPLMS_Appointments_Activity::init();

            $activity->args['date_recorded']=tofriendlytime(time() - strtotime($activity->args['date_recorded']));
            return new WP_REST_Response( array('status'=>1,'message'=>__('Appointment is marked as attended by student','wplms-appointments'),'activity'=>$activity->args), 200 ); 
        }       


    }


    function update_appointment_schedule_booking($request){

        

        $post = json_decode($request->get_body(),true);
        /*if(strlen($post['reason']) < 2){
            return new WP_REST_Response( array('status'=>0,'message'=>__('Missing reason.','wplms-appointments')), 200 );
        }*/

        $args = array(
            'appointment_id'=>$post['appointment_id'],
            'schedule_id'=>$request->get_param('id'),
            'booker_id'=>$post['booker_id'],
            'reason'=>$post['reason'],
            'author_id'=>$post['author_id'],
            'type'=>$request->get_param('type'),
        );

        $init = WPLMS_Appointments_Init::init();
        $init->get_global_settings();


        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments_meta_db = new WPLMS_APPOINTMENTS_META_DB;
        $appointment = $db->get_appointments(Array('appointment_id'=>$post['appointment_id']));
        

        $appointments_buffer_time = $init->settings['appointments_buffer_time'];

        if($args['type'] == 'cancel'){

            $meta_key = 'requested_cancellation';

            if(isset($init->settings['appointments_buffer_time']) && $appointments_buffer_time < 9999){
                if(empty($appointments_buffer_time)){
                    return new WP_REST_Response( array('status'=>0,'message'=>__('Cancellation is disabled.','wplms-appointments')), 200 );
                }
                
               
                $time = ($appointment[0]['start_date']+60000*$appointment[0]['start_time'] - $appointments_buffer_time*60000) - 2*intval($post['timezoneoffset']);

               
                if($post['currentdate'] > $time ){
                    return new WP_REST_Response( array('status'=>0,'message'=>__('Cancellation is disabled as appointment as either started or calcellation buffer time is over.','wplms-appointments')), 200 );
                }

                $message = __('Cancellation Requested !','wplms-appointments');
            }
        }

        
        if($args['type'] == 'reschedule'){
            $meta_key = 'requested_reschedule';
            if(isset($init->settings['appointments_buffer_time']) && $appointments_buffer_time < 9999){
                if(empty($appointments_buffer_time)){
                    return new WP_REST_Response( array('status'=>0,'message'=>__('Reschedule is disabled.','wplms-appointments')), 200 );
                }
                $time = $appointment[0]['start_date']+60000*$appointment[0]['start_time'] - $appointments_buffer_time*60000  - 2*intval($post['timezoneoffset']); // subtract because we comare with date in current timezone

               
                if($post['currentdate'] > $time ){
                    return new WP_REST_Response( array('status'=>0,'message'=>__('Reschedule is disabled as appointment as either started or reschedule buffer time is over.','wplms-appointments')), 200 );
                }
            }
            $message = __('Reschedule Requested !','wplms-appointments');
        }


        if($args['type'] == 'cancel' || $args['type'] == 'reschedule'){

            $appointments = get_user_meta($args['author_id'], $meta_key,true);
            if(empty($appointmnets)){
                $appointments=array();
            }
            $appointments[$post['appointment_id']]=array('user_id'=>$args['booker_id'],'reason'=>$post['reason']);

            update_user_meta(intval($args['author_id']), $meta_key,$appointments);
            
            do_action('wplms_appointment_'.$meta_key,$args);
        }

        if($args['type'] === 'instructor_cancel'){


            $metas = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$appointment['appointment_id'],'meta_key'=>'order_details'));


            $order_details = array();
            if(!empty($metas)){
                foreach($metas as $meta){
                    if($meta['meta_key'] == 'order_details'){
                        $order_details=$meta['meta_value'];
                        if(is_string($order_details)){
                            $order_details=unserialize($order_details);
                        }
                        break;
                    }
                }
            }

            /*$order_details[$user_id]=array(
                'type'=>'woocommerce',
                'order_id'=>$order_id,
                'item_id'=>$item_id
            );*/

            //Find the purchase and cancel the order
            if(!empty($metas) && !empty($order_details[$args['booker_id']])){

                if(empty($init->settings['appointments_order_instructor_approve'])){
                    return new WP_REST_Response( array('status'=>0,'message'=>__('Administrator has disabled cancellation of paid orders. Please contact administrator.','wplms-appointments')), 200 );    
                } 
                
                $message = apply_filters('wplms_appointment_cancel_booking_'.$order_details[$args['booker_id']]['type'],0,$order_details[$args['booker_id']],$args);

                if(!empty($message)){
                    return new WP_REST_Response( array('status'=>0,'message'=>$message), 200 ); 
                }
                
            }else{
                if($appointment[0]['booker_id'] == $args['booker_id']){
                    $db->update_appointment(array('appointment_id'=>$args['appointment_id'],'booker_id'=>0));
                }else{
                    $booker_ids = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$args['appointment_id'],'meta_key'=>'booker_id'));
                    $key = array_search($args['booker_id'], array_column($booker_ids, 'meta_value'));
                    if( $key == false){
                        return new WP_REST_Response( array('status'=>0,'message'=>__('Booker missing in Appointment bookers','wplms-appointments')), 200 ); 
                    }else{
                        $appointments_meta_db->remove_appointments_meta(array('meta_id'=>$booker_ids[$key]['meta_id']));
                    }
                }
            }

            do_action('wplms_appointments_appointment_cancelled',$args,$metas,$order_details);

            return new WP_REST_Response( array('status'=>1,'message'=>__('Booking cancelled ! Slot is available for booking.','wplms-appointments'),'data'=>$order_details,'meta'=>$metas,'args'=>$args), 200 ); 
        }

        if($args['type'] === 'instructor_reschedule'){
            $schedule=array();
            if(!empty($appointment[0]['parent_appointment_id'])){
                $schedule = $db->get_appointments(Array('appointment_id'=>$appointment[0]['parent_appointment_id']));
            }
        }
        
        return new WP_REST_Response( array('status'=>1,'message'=>$message), 200 );
    }


    function update_appointment_schedule_details($request){

        $post = json_decode($request->get_body(),true);
        
       
        $details = json_decode($post['details'],true);
        $schedule = json_decode($post['schedule'],true);

        $db = new WPLMS_APPOINTMENTS_DB;


        $success = $db->update_appointment($schedule);

        //Set booker ID for all child
        //Set type,status for all child
        //Set item_id

        if($success){
            //GEt all child appointmnets
            $children = $db->get_appointments(array('parent_appointment_id'=>$schedule['appointment_id']));
            $child_metas=array();

            if(!empty($children)){
                
                foreach($children as $child){
                    $args = array(
                        'appointment_id'=>$child['appointment_id'],
                    );
                    if(!empty($schedule['item_id'])){
                        $args['item_id']=$schedule['item_id'];
                    }
                    if(!empty($schedule['booker_id'])){
                        if(empty($child['booker_id'])){
                            $args['booker_id']=$schedule['booker_id'];    
                        }else{
                            $child_metas[]=array('appointment_id'=>$child['appointment_id'],'meta_key'=>'booker_id','meta_value'=>$schedule['booker_id']);
                        }
                        
                    }
                    if(!empty($schedule['type'])){
                        $args['type']=$schedule['type'];
                    }
                    if(!empty($schedule['status'])){
                        $args['status']=$schedule['status'];
                    }
                    $db->update_appointment($args);
                }
            }
            if(!empty($details)){
                $metadb = new WPLMS_APPOINTMENTS_META_DB;
                
                foreach($details as $detail){

                    $detail['appointment_id'] = $schedule['appointment_id']; 

                    unset($detail['meta_id']);
                    $metadb->update_appointment_meta($detail);

                    if(!empty($children)){
                        $continue = false;
                        if(in_array($detail['meta_key'],apply_filters('wplms_appointments_in_save_schedule_save_add_to_slots',array('booker_id','max_seats') ))){
                            $continue = true;
                        }
                        if($continue){
                            foreach($children as $child){
                                $child_metas[]=array('appointment_id'=>$child['appointment_id'],'meta_key'=>$detail['meta_key'],'meta_value'=>$detail['meta_value']);
                            }    
                        }
                        
                    }
                }
                if(!empty($child_metas)){
                    //remove all booker ids
                    foreach($child_metas as $meta){
                        if($meta['meta_key'] == 'booker_id'){
                            $metadb->add_appointment_meta($meta);       
                        }else{
                            $metadb->update_appointment_meta($meta);           
                        }
                        
                    }
                }
            }

            do_action('wplms_appointments_appointment_updated',$schedule,$details);
            $return = array(
                'status'=>1,
                'message'=>__('Successfully saved','wplms-appointments')
            );


        }else{
                $return = array(
                'status'=>0,
                'message'=>__('Error in saving','wplms-appointments')
            );
        }


        

        return new WP_REST_Response( $return, 200 );
        
    }

    function reschedule_booker($request){
        /*
        author_id:this.state.event.author_id,
        item_id:this.state.event.item_id,
        booker_id:this.state.event.booker_id,
        appointment_id:this.state.event.appointment_id,
        new_appointment_id:this.state.rescheduledEventID,
        security:window.wplms_appointments.security
        */

        $post = json_decode($request->get_body(),true);
        

       
        $db = new WPLMS_APPOINTMENTS_DB;
        $meta_db = new WPLMS_APPOINTMENTS_META_DB;

        $appointments = $db->get_appointments(array('appointment_id'=>$post['appointment_id']));
        if(empty($appointments)){
            return new WP_REST_Response( array('status'=>0,'message'=>__('Appointment ID not found','wplms-appointments')), 200 );
        }

        if($appointments[0]['author_id'] != $post['author_id']){
            return new WP_REST_Response( array('status'=>0,'message'=>__('Appointment author mismatch','wplms-appointments')), 200 );
        }

        if($appointments[0]['item_id'] != $post['item_id']){
            return new WP_REST_Response( array('status'=>0,'message'=>__('Appointment Item mismatch','wplms-appointments')), 200 );
        }

        $bookers = wplms_get_appointment_bookers($post['new_appointment_id']);

        if(in_array($post['booker_id'],$bookers)){
            return new WP_REST_Response( array('status'=>0,'message'=>__('User already in new appointment','wplms-appointments'),'bookers'=>$bookers), 200 );
        }

        if($appointments[0]['booker_id'] === $post['booker_id'] ){
            //remove
            $db->update_appointment(Array('appointment_id'=>$post['appointment_id'],'booker_id'=>0));
            //add
            if(count($bookers)){
                
                $meta_db->add_appointment_meta(array('appointment_id'=>$post['new_appointment_id'],'meta_key'=>'booker_id','meta_value'=>$post['booker_id']));

            }else{
                $db->update_appointment(Array('appointment_id'=>$post['new_appointment_id'],'booker_id'=>$post['booker_id']));
            }

            do_action('wplms_appointments_appointment_rescheduled',$post);

            return new WP_REST_Response( array('status'=>1,'message'=>__('Booker rescheduled','wplms-appointments')), 200 );
        }else{
            
            $bookers = $meta_db->get_appointments_meta(Array('appointment_id'=>$post['appointment_id'],'meta_key'=>'booker_id'));
            $found = array_search($post['booker_id'],array_column('meta_value',$bookers));
            if( $found !== false ){

                //remove
                $meta_db->delete_appointment_meta($found['meta_id']);

                //add
                if(count($bookers)){
                    $meta_db->add_appointment_meta(array('appointment_id'=>$post['new_appointment_id'],'meta_key'=>'booker_id','meta_value'=>$post['booker_id']));
                }else{
                    $db->update_appointment(Array('appointment_id'=>$post['new_appointment_id'],'booker_id'=>$post['booker_id']));
                }

                do_action('wplms_appointments_appointment_rescheduled',$post);
                
                return new WP_REST_Response( array('status'=>1,'message'=>__('Booker rescheduled for new appointment.','wplms-appointments')), 200 );
            }else{
                return new WP_REST_Response( array('status'=>0,'message'=>__('User not found as booker for appointment','wplms-appointments')), 200 );
            }
        }
        

    }

    function get_appointments_directory_permissions_check($request){
        return true;
    } 

    function get_appointment_search_items($request){

        $settings = get_option(WPLMS_APPOINTMENTS_OPTION);      
        if(empty($settings) || empty($settings['appointments_directory_search'])){
            $return =array(
                    array(
                        'type'=>'course',
                        'label'=> __('Search Course','wplms-appointments'),
                    ),
                    array(
                        'type'=>'availability',
                        'label'=> __('Select Times','wplms-appointments'),
                    ),
                );
        }else{
            $return = json_decode(stripslashes($settings['appointments_directory_search']));
        }
        

        return new WP_REST_Response( $return, 200 );
    }


    function get_appointments_creds_permissions_check(){
       return true;
    }

    function get_appointments_details_permissions_check(){
         if($request->get_param('security') == vibe_get_option('security_key')){
            return true;
        }

        return false;
    }

    function get_appointments_courses($request){


        $course_ids = json_decode($request->get_param('course_ids'));
        $style = $request->get_param('style');

        if(!empty($course_ids) && is_numeric($course_ids)){
            $course_ids = array($course_ids);
        }
        $hide_courses = vibe_get_option('hide_courses');
        $args = array(
            'post_type'=>'course',
            'posts_per_page'=>-1,
            'post__not_in' => $hide_courses,
            'author'=>get_current_user_id()
        );

        if(!empty($course_ids)){
            $args['post__in']=$course_ids;
        }
        
        $query = new WP_Query(apply_filters('wplms_appointments_api_course_args',$args));

        $return = array();
        if($query->have_posts()){
            while($query->have_posts()){
                $query->the_post();

                switch($style){
                    case 'basic':

                        $color = get_post_meta(get_the_ID(),'vibe_wplms_appointment_booking_color',true);
                        $add = array(
                            'id'=>get_the_id(),
                            'title'=>get_the_title(),
                            'post_author'=>get_the_author_id(),
                            'image'=>get_the_post_thumbnail_url(),
                        );
                        if(!empty($color)){
                            $add['color']=$color;
                        }
                        $return[]=$add;
                    break;
                    default:
                        $return[]=array('label'=>get_the_title(),'value'=>get_the_ID());
                    break;
                }

                
            }
        }

        return new WP_REST_Response( $return, 200 );
    }

    function get_appointment_bookers($request){


        $booker_ids = $request->get_param('booker_ids');
        $booker_ids = json_decode($booker_ids);
        if(is_array($booker_ids)){
            $booker_ids = implode(',',$booker_ids);
        }
        global $wpdb;
        $appointments_table_name = $wpdb->prefix.'wplms_appointments';
        
        if(!empty($booker_ids)){
            $users = $wpdb->get_results("SELECT u.ID,u.display_name FROM {$wpdb->users} as u LEFT JOIN ${appointments_table_name} as at on u.ID = at.booker_id WHERE at.booker_id in ({$booker_ids}) GROUP BY at.booker_id");
            
        }else{
            $users = $wpdb->get_results("SELECT u.ID,u.display_name FROM {$wpdb->users} as u LEFT JOIN ${appointments_table_name} as at on u.ID = at.booker_id GROUP BY at.booker_id");
            
        }
        if(!empty($users)){
            foreach ($users as $key => $user) {
                $return[]=array('label'=>$user->display_name,'value'=>$user->ID);
            }
        }

         return new WP_REST_Response( $return, 200 );
    }

    function get_appointment_instructors($request){


        $instructor_ids = $request->get_param('instructor_ids');

        $args = array(
            'role__in' => array('Administrator','Instructor'),
            'meta_key' => 'enable_appointments', 
            'meta_value' => 1
        );

        if(!empty($instructor_ids)){
            $args['include'] = json_decode($instructor_ids);
        }

        $user_query = new WP_User_Query( apply_filters('wplms_appointments_api_instructor_args',$args));

         $return = array();
        if ( ! empty( $user_query->get_results() ) ) {
            foreach ( $user_query->get_results() as $user ) {
               $return[]=array('label'=>$user->display_name,'value'=>$user->ID);
            }
        }

         return new WP_REST_Response( $return, 200 );
    }

    function get_appointment_instructor($request){

        $instructor_id = $request->get_param('id');
        $type  = $request->get_param('type');
        $return = array();
        switch($type){


            case 'bookings':
                $db = new WPLMS_APPOINTMENTS_DB;
                $args = apply_filters('wplms_appointments_get_slots',array(
                    'number'=>999,
                    'start_date'=>$args->start_date,
                    'end_date'=>$args->end_date,
                    'start_time'=>$args->start_time,
                    'end_time'=>$args->end_time,
                    'author_id'=>(($args->instructor)?$args->instructor:(($args->author_id)?$args->author_id:'')),
                    'booker_id'=>(empty($args->booker_id)?'':$args->booker_id),
                    'item_id'=>(($args->item_id)?$args->item_id:''),
                    'parent_appointment_id'=>'EXISTS',
                    'status'=>(($args->status)?$args->status:'open'),
                    'sort'=>'upcoming'
                ),$request);

            break;
            case 'courses':

                $query = new WP_Query(apply_filters('wplms_appointments_instructor_courses',array(
                    'post_type'=>'course',
                    'author'=>$instructor_id,
                    'posts_per_page' => 999
                )));
                
                if($query->have_posts()){
                    while($query->have_posts()){
                        $query->the_post();
                        $return[]=array('label'=>get_the_title(),'value'=>get_the_ID());
                    }
                }
            break;
        }
        
        return new WP_REST_Response( $return, 200 );
    }


    function get_appointments_slots($request){

        
        $db = new WPLMS_APPOINTMENTS_DB;
        $meta = new WPLMS_APPOINTMENTS_META_DB;

        $args = json_decode($request->get_param('args'));

        if(isset($args->schedule_ids)){
            $args = apply_filters('wplms_appointments_get_slots',array(
                'appointment_id'=>$args->schedule_ids
            ));
        }else{
            $args = apply_filters('wplms_appointments_get_slots',array(
                'number'=>999,
                'start_date'=>$args->start_date,
                'end_date'=>$args->end_date,
                'start_time'=>$args->start_time,
                'end_time'=>$args->end_time,
                'author_id'=>(($args->instructor)?$args->instructor:(($args->author_id)?$args->author_id:'')),
                'booker_id'=>(($args->booker_id)?$args->booker_id:''),
                'item_id'=>(($args->item_id)?$args->item_id:''),
                'parent_appointment_id'=>'EXISTS',
                'status'=>(($args->status)?$args->status:'open'),
                'sort'=>(($args->sort)?$args->sort:'upcoming')
            ),$request);
        }
        


        if(!empty($args['booker_id'])){
            $booker_id = $args['booker_id'];
            unset($args['parent_appointment_id']);
        }
         if(!empty($args['author_id'])){
            $instructor_id = $args['author_id'];
        }
        
        

        $events = $db->get_appointments($args);
         //print_r($args);
        // print_r($events);


        $schdule_array = array();

        $full_schedules = array();
        if(!empty($events)){
            $event_ids = array();
            foreach($events as $i=>$event){

                $event_ids[]=$event['appointment_id'];
                if(!empty($event['parent_appointment_id']) && !in_Array($event['parent_appointment_id'],$schdule_array)){
                    $schdule_array[]=$event['parent_appointment_id'];
                }

                if(empty($event['parent_appointment_id'])){
                    $full_schedules[]=$event['appointment_id'];
                    $schdule_array[]=$event['appointment_id'];
                }
            }

            if(!empty($full_schedules)){
                $args = array(
                        'number'=>999,
                        'parent_appointment_id'=>$full_schedules
                    );

                $schedule_events = $db->get_appointments($args);
                foreach($schedule_events as $event){
                    if(!in_array($event['appointment_id'],$event_ids)){
                        $events[]=$event;
                    }
                }
            }

            $metas = $meta->get_appointments_meta(array('appointment_id'=>$event_ids));
            if(!empty($metas)){
                foreach($metas as $info){

                    $info['meta_value'] = maybe_unserialize($info['meta_value']);
                    foreach($events as $i=>$event){
                        if($info['appointment_id'] === $event['appointment_id']){
                            if(empty($events[$i][$info['meta_key']])){
                                $events[$i][$info['meta_key']]=$info['meta_value'];
                            }else{
                                if(!is_array($events[$i][$info['meta_key']])){
                                    $events[$i][$info['meta_key']]=array($events[$i][$info['meta_key']]);
                                }
                                
                                $events[$i][$info['meta_key']][]=$info['meta_value'];
                            }
                            
                            break;
                        }
                    }
                }
            }

        }


        if(!empty($schdule_array)){

            $args = apply_filters('wplms_appointments_get_schedules',array(
                'appointment_id'=>$schdule_array
            ),$request);

            $schedules = $db->get_appointments($args);
            if(!empty($schedules)){
                
                $schedule_ids = array();
                foreach($schedules as $key=>$schedule){
                    $schedule_ids[]=$schedule['appointment_id'];
                }

                $metas = $meta->get_appointments_meta(array('appointment_id'=>$schedule_ids));
                
                if(!empty($metas)){
                    foreach($metas as $info){
                        foreach($schedules as $i=>$schedule){
                            if($schedule['appointment_id'] == $info['appointment_id']){
                                
                                if(empty($schedules[$i][$info['meta_key']])){
                                    $schedules[$i][$info['meta_key']] = $info['meta_value'];
                                }else{

                                    if(!is_array($schedules[$i][$info['meta_key']])){
                                        $schedules[$i][$info['meta_key']]=array($schedules[$i][$info['meta_key']]);
                                    }
                                    
                                    $schedules[$i][$info['meta_key']][]=$info['meta_value'];
                                }

                                break;
                            }
                        }
                    }
                }
            }
        }

        $pending_slots = array();
        
        if(!empty($booker_id) && is_numeric($booker_id)){
            $p_slots = wplms_appointments_get_pending_slots(array('booker_id'=>$booker_id));
        }
        if(!empty($instructor_id)){

            $p_slots = wplms_appointments_get_pending_slots(array('instructor_id'=>$instructor_id));
        }

        if(!empty($p_slots)){
            foreach($p_slots as $slot_id => $value){
                $value['appointment_id']=$slot_id;
                $pending_slots[]=$value;
            }
        }


        
        return new WP_REST_Response( array('schedules'=>$schedules,'events'=>$events,'active_slots'=>wplms_appointments_get_active_slots(),'pending_slots'=>$pending_slots ), 200 );
    }



    function get_appointments_user_permissions_check($request){
        return true;
        // if($request->get_param('security') == vibe_get_option('security_key')){
        //     return true;
        // }

        // return false;
    }

    function get_appointments_item_permissions_check($request){
       if($request->get_param('security') == vibe_get_option('security_key')){
            return true;
        }

        return false;
    }

    function get_appointment_user($request){
        $user_id = $request['id'];        

        if(class_exists('Vibe_CustomTypes_Permalinks')){
            $x = Vibe_CustomTypes_Permalinks::init();
            $permalinks = $x->permalinks;    
        }else{
            $permalinks = get_option( 'vibe_course_permalinks' );
        }
    
        $booking_permalink = $permalinks['appointments_slug'];

        if(empty($permalinks['appointments_slug'])){
            $booking_permalink = 'appointments';
        }

        $return = apply_filters('wplms_appointments_get_appointment_user',array(
            'id'    => $user_id,
            'name'  => bp_core_get_user_displayname($user_id),
            'image' => bp_core_fetch_avatar ( array( 'item_id' => $user_id, 'type' => 'full','html'=>false )),
            'url' => bp_core_get_user_domain($user_id).$booking_permalink.'/',
        
        ));

        return new WP_REST_Response( $return, 200 );
    }

    function get_appointment_item($request){
        $item_id = $request['id'];  

        $return = apply_filters('wplms_appointments_get_appointment_item',array(
            'title'=> get_the_title($item_id),
            'id'=>$item_id,
            'image'=> get_the_post_thumbnail_url($item_id,'medium')
        ));

        return new WP_REST_Response( $return, 200 );
    }

    function get_appointment_directory_instructors($request){

        $db = new WPLMS_APPOINTMENTS_DB;
        $args = json_decode($request->get_param('args'),true);
       
        $args['booker_id'] = 0; 
        $number = $args['number'];
        $args['number'] = -1;
        
        
        //$args['group_by']='author_id';
        global $wpdb;


        if(!empty($args['meta'])){

            
            $from ='';
            $where='';
            if(count($args['meta']) == 1){
                $meta = $args['meta'][0];

                if(is_array($meta['value'])){
                    $meta['value'] = '"'.implode('","',$meta['value']).'"';
                }
                if(!empty($meta['value'])){
                    $from = $wpdb->usermeta.' as um %s';

                    $where = ' um.meta_key="instructor_searchable_field_'.$meta['key'].'" AND um.meta_value IN ('.$meta['value'].')';    
                }
                
            }else{
                foreach($args['meta'] as $i=>$meta){

                    if(!empty($meta['value'])){
                        if(is_array($meta['value'])){
                            $meta['value'] = '"'.implode('","',$meta['value']).'"';
                        }

                        if($i > 0){
                            if(!empty($meta['key'])){
                                $from .= ' INNER JOIN '.$wpdb->usermeta.' as um'.$i.' ON um.user_id = um'.$i.'.user_id ';
                                $where .=' AND  um'.$i.'.meta_key="instructor_searchable_field_'.$meta['key'].'" AND  um'.$i.'.meta_value IN ('.$meta['value'].')';
                            }
                        }else{
                            if(!empty($meta['key'])){
                                $from .= ' '.$wpdb->usermeta.' as um %s';
                                $where .= ' um.meta_key="instructor_searchable_field_'.$meta['key'].'" AND um.meta_value IN ('.$meta['value'].')';
                            }
                        }
                    }
                }
            }
           
            $args['join'] = array(
                'on_p' => 'author_id',
                'on_c' => 'um.user_id',
                'from' => $from,
                'where'=>$where
            );

        }

        if(!empty($args['s'])){
            if(empty($args['join'])){
                $args['join'] = array(
                    'on_p' => 'author_id',
                    'on_c' => 'u.ID',
                    'from' => $wpdb->users.' as u %s',
                    'where'=> ' (u.user_login LIKE "%%'.$args['s'].'%%" OR u.user_email LIKE "%%'.$args['s'].'%%" OR u.user_nicename LIKE "%%'.$args['s'].'%%" OR u.display_name LIKE "%%'.$args['s'].'%%")'
                );
            }else{
                $args['from']= ' INNER JOIN '.$wpdb->users.' as u ON um.user_id = u.ID';
                $args['where']=' AND (u.user_login LIKE "%%'.$args['s'].'%%" OR u.user_email LIKE "%%'.$args['s'].'%%" OR u.user_nicename LIKE "%%'.$args['s'].'%%" OR u.display_name LIKE "%%'.$args['s'].'%%")';
            }
        }

        //print_r($args);
        
        $appointments = $db->get_appointments(apply_filters('wplms_appointments_directory_instructors',$args));

        

        $author_ids = array();
        $return = array();

        if(!empty($appointments)){
            
            foreach($appointments as $appointment){
                if(empty($return[$appointment['author_id']])){
                    $return[$appointment['author_id']] = array();
                }

                $return[$appointment['author_id']]['slots'][]=$appointment;
            }        
        }

        
        return new WP_REST_Response( $return, 200 );
    }
    
    function get_appointment_directory_instructor_profile($request){
        $instructor_id = $request->get_param('id');
        
        if(is_numeric($request->get_param('course_id'))){
            $course_id = $request->get_param('course_id');
        }


        if(!empty($course_id)){
            $return = wp_cache_get('instructor_'.$instructor_id.'_'.$course_id,'wplms_appointments');
        }else{
            $return = wp_cache_get('instructor_'.$instructor_id,'wplms_appointments');
        }

        if(!$return){

            $init = WPLMS_Appointments_Init::init();
            $init->get_global_settings();
            

            $labels = array();

            $appointments_instructor_labels = $init->settings['appointments_instructor_labels'];
            if(!empty($appointments_instructor_labels)){


                $stored_key = get_user_meta($instructor_id,'wplms_appointment_user_label',true);
                
                $instructor_label = '';

                

                if(!empty($appointments_instructor_labels['key'])){
                    foreach($appointments_instructor_labels['key'] as $k=>$v){
                        if($v == $stored_key){
                            $instructor_label = $appointments_instructor_labels['value'][$k];
                            $color  = $appointments_instructor_labels['color'][$k];
                        }
                    }
                    $labels = apply_filters('wplms_appointment_instructor_label',array(
                        array(
                            'color'=> $color,
                            'label'=>$instructor_label
                        )
                    ),$instructor_id);
                }
            }


            $profile = WPLMS_Appointments_Profile::init();
            $profile->get_user_appointment_settings($instructor_id);

            $instructor_fields = array();

            if(!empty($init->settings['appointments_instructor_fields'])){
                
                $fields = json_decode(stripslashes($init->settings['appointments_instructor_fields']),true);


                foreach($fields as $setting){

                     $saved = $profile->settings['instructor_searchable_field_'.$setting['key']];

                    $index = array_search($saved, array_column($setting['options'], 'value'));
                    $instructor_fields[]= array('label'=>$setting['label'],'key'=>$setting['key'],'value'=>$setting['options'][$index]['label']);
                    
                }
            }

            if(class_exists('Vibe_CustomTypes_Permalinks')){
                $x = Vibe_CustomTypes_Permalinks::init();
                $permalinks = $x->permalinks;    
            }else{
                $permalinks = get_option( 'vibe_course_permalinks' );
            }
        
            $booking_permalink = $permalinks['appointments_slug'];

            if(empty($permalinks['appointments_slug'])){
                $booking_permalink = 'appointments';
            }
            //Course specific
            if(!empty($course_id)){

                

                global $wpdb;
                $slot_product_id = $wpdb->get_var("SELECT m.post_id FROM {$wpdb->postmeta} as m LEFT JOIN {$wpdb->posts} as p ON p.id = m.post_id WHERE p.post_type='product' AND p.post_author = $instructor_id AND m.meta_key = 'vibe_wplms_appointment_booking_slot_price' AND m.meta_value = $course_id");

                if(!empty($slot_product_id)){
                    $product = wc_get_product( $slot_product_id );
                    $slot_price = $product->get_price_html();    
                }
                

                $schedule_product_id = $wpdb->get_var("SELECT m.post_id FROM {$wpdb->postmeta} as m LEFT JOIN {$wpdb->posts} as p ON p.id = m.post_id WHERE p.post_type='product' AND p.post_author = $instructor_id AND m.meta_key = 'vibe_wplms_appointment_booking_schedule_price' AND m.meta_value = $course_id");

                if(!empty($schedule_product_id)){
                    $product = wc_get_product( $schedule_product_id );
                    $schedule_price = $product->get_price_html();    
                }
                
                
                

                $return = array(
                    'author_id'=> $instructor_id,
                    'name'=> bp_core_get_user_displayname($instructor_id),
                    'link'=>bp_core_get_user_domain($instructor_id).'/'.$booking_permalink,
                    'profile_link'=>bp_core_get_user_domain($instructor_id),
                    'image'=> bp_core_fetch_avatar(array('item_id'=>$instructor_id,'type'=>'full','html'=>'false')),
                    'rating'=> vibe_get_instructor_average_rating($instructor_id),
                    'price'=>array(
                        'slot'=>array('id'=>$slot_product_id,'price'=>$slot_price),
                        'schedule'=>array('id'=>$schedule_product_id,'price'=>$schedule_price),
                    ),
                    'studentcount'=>vibe_get_instructor_student_count($instructor_id),
                    'labels'=>$labels,
                    'fields'=>$instructor_fields,
                    'description'=>wp_trim_words(bp_course_get_instructor_description(array('instructor_id'=>$instructor_id)), 40, '...')
                );

                //if enabled
                $return['featuredVideo']=$profile->settings['appointments_video'];
                wp_cache_set('instructor_'.$instructor_id.'_'.$course_id,$return,'wplms_appointments',86400);
            }else{
            //Directory specific
                $args = array(
                    'author_id'=>$instructor_id,
                    'post_type'=>'course',
                    'post_status'=>'publish',
                    'posts_per_page'=>-1
                );
                if(function_exists('get_coauthors')){
                    //unset($args['author_id']);
                    $user_info = get_userdata($instructor_id);
                    $args['author_name']=$user_info->user_nicename;
                }

                
                $courses = new WP_Query($args);
               
                $course_ids = array();
                if(!empty($courses)){
                    while($courses->have_posts()){ 
                        $courses->the_post();
                        
                        $course_details = array(
                            'id'=>get_the_ID(),
                            'name'=>get_the_title(),

                        );
                        $course_id = get_The_ID();
                        $enabled = get_post_meta($course_id,'vibe_wplms_appointments',true);

                        if($enabled == 'S'){
                           
                            $course_details['message']=do_shortcode(get_post_meta($course_id,'vibe_wplms_appointment_booking_form',true));
                            global $wpdb;
                            $slot_product_id = $wpdb->get_var("SELECT m.post_id FROM {$wpdb->postmeta} as m LEFT JOIN {$wpdb->posts} as p ON p.id = m.post_id WHERE p.post_type='product' AND p.post_author = $instructor_id AND m.meta_key = 'vibe_wplms_appointment_booking_slot_price' AND m.meta_value = $course_id");

                            if(!empty($slot_product_id)){
                                $product = wc_get_product( $slot_product_id );
                                $slot_price = $product->get_price_html();    
                                $course_details['price']=array(
                                    'slot'=>array(
                                        'id'=>$slot_product_id,
                                        'price'=>$slot_price
                                    ),
                                );
                            }
                            

                            $schedule_product_id = $wpdb->get_var("SELECT m.post_id FROM {$wpdb->postmeta} as m LEFT JOIN {$wpdb->posts} as p ON p.id = m.post_id WHERE p.post_type='product' AND p.post_author = $instructor_id AND m.meta_key = 'vibe_wplms_appointment_booking_schedule_price' AND m.meta_value = $course_id");

                            if(!empty($schedule_product_id)){
                                $product = wc_get_product( $schedule_product_id );
                                $schedule_price = $product->get_price_html();    

                                $course_details['price']['schedule']=array(
                                        'id'=>$schedule_product_id,
                                        'price'=>$schedule_price
                                );
                            }
                        }
                        

                        $course_ids[]=$course_details;
                    }
                }
                //


                $slot_product_id = $profile->settings['instructor_appointments_price'];

                $return = array(
                    'author_id'=> $instructor_id,
                    'name'=> bp_core_get_user_displayname($instructor_id),
                    'link'=>bp_core_get_user_domain($instructor_id).'/'.$booking_permalink,
                    'profile_link'=>bp_core_get_user_domain($instructor_id),
                    'image'=> bp_core_fetch_avatar(array('item_id'=>$instructor_id,'type'=>'full','html'=>'false')),
                    'rating'=> vibe_get_instructor_average_rating($instructor_id),
                    'studentcount'=>vibe_get_instructor_student_count($instructor_id),
                    'labels'=>$labels,
                    'courses'=>$course_ids,
                    'fields'=>$instructor_fields,
                    'message'=>do_shortcode($profile->settings['appointments_message']),
                    'description'=>wp_trim_words(bp_course_get_instructor_description(array('instructor_id'=>$instructor_id)), 40, '...') 
                );

                $return['featuredVideo']=$profile->settings['appointments_video'];

                if(!empty($slot_product_id)){
                    $product = wc_get_product( $slot_product_id );
                    if(!empty($product)){
                        $slot_price = $product->get_price_html(); 

                        $return['price']=array(
                            'slot'=>array('id'=>$slot_product_id,'price'=>$slot_price),
                        );

                        $return['price_val']=$product->get_price();
                    }
                    
                }

                wp_cache_set('instructor_'.$instructor_id,$return,'wplms_appointments',86400);
            }
        }

        return new WP_REST_Response( apply_filters('wplms_appointments_directory_instructor_profile',$return,$request), 200 );

    }


    function  get_appointment_schedule_details($request){
        $schedule_id = $request->get_param('id');
        
        $db = new WPLMS_APPOINTMENTS_META_DB;
        $appointments_meta = $db->get_appointments_meta(apply_filters('wplms_appointments_schedule_details',array(
            'appointment_id'=>$schedule_id,
            'number'=>999,
        )));
        $booker_count=1;
        $meta_key_index=-1;
        if(!empty($appointments_meta)){
            foreach($appointments_meta as $k=> $meta){
                if(is_serialized($meta['meta_value'])){
                    $appointments_meta[$k]['meta_value'] = unserialize($meta['meta_value']);
                }
                if($meta['meta_key'] == 'max_seats'){
                    $meta_key_index=$k;
                }
            }
        }
        $appointment_id= $request->get_param('appointment_id');
        if($meta_key_index >-1 && is_numeric($appointment_id)){
            $bookers = wplms_get_appointment_bookers($appointment_id);
            $appointments_meta[$meta_key_index]['meta_value']= count($bookers).'/'.$appointments_meta[$meta_key_index]['meta_value'];

            $appointments_meta[]=array('bookers'=>$bookers);
        }
        
        return new WP_REST_Response( $appointments_meta, 200 );
        
    }
    function get_appointments_schedule_permissions_check($request){
        return true;

        // if($request->get_param('security') == vibe_get_option('security_key')){
        //     return true;
        // }

        // return false;
    }

    function get_appointments_search_users_permissions_check($request){

      return true;

    }


    function get_appointment_details($request){

         $appointment_id = $request->get_param('appointment_id');

        $db = new WPLMS_APPOINTMENTS_DB;
        $appointments = $db->get_appointments(array('appointment_id'=>$appointment_id));
        $appointment=array();
        if(!empty($appointments)){
            $appointment = $appointments[0];
            $meta_db = new WPLMS_APPOINTMENTS_META_DB;
            $appointments_meta = $meta_db->get_appointments_meta(array(
                'appointment_id'=>$appointment_id,
                'number'=>999,
            ));
            if(!empty($appointments_meta)){

                foreach($appointments_meta as $meta){
                        $appointment[$meta['meta_key']] = maybe_unserialize($meta['meta_value']);  
                } 
            }  
        }

        return new WP_REST_Response($appointment , 200 );
    }


    function get_users($request){
        $s = $request->get_param('s');
        $fields = $request->get_param('fields');
        $return = array();
        if(Empty($fields)){
            $fields = array('ID');
        }else{
            $fields = explode(',',$fields);
        } 
        $args = apply_filters('wplms_appointments_user_search',array(
            'search'=>$s,
            'search_columns' => array( 'user_login', 'user_email' ,'user_nicename','display_name'),
            'fields'=>$fields
        ));
        $user_query = new WP_User_Query( $args );


        return new WP_REST_Response( $user_query->get_results(), 200 );
    }

    function get_appointment_schedule_messages($request){

        $schedule_id = $request->get_param('id');
        $user_id = $request->get_param('instructor_id');

        $results = array();


        global $wpdb;
        $bp = buddypress();
        

        if(empty($user_id)){
            $message_id_array = $wpdb->get_results($wpdb->prepare("SELECT meta.message_id as message_id,recipients.unread_count as unread FROM {$bp->messages->table_name_meta} as meta INNER JOIN {$bp->messages->table_name_recipients} as recipients ON meta.message_id = recipients.thread_id WHERE meta.meta_key = 'schedule_id' AND meta.meta_value = %d ORDER BY meta.id DESC",$schedule_id));
        }else{
            $message_id_array = $wpdb->get_results($wpdb->prepare("SELECT meta.message_id as message_id,recipients.unread_count as unread FROM {$bp->messages->table_name_meta} as meta INNER JOIN {$bp->messages->table_name_recipients} as recipients ON meta.message_id = recipients.thread_id WHERE meta.meta_key = 'schedule_id' AND meta.meta_value = %d AND recipients.user_id = %d ORDER BY meta.id DESC",$schedule_id,$user_id));
        }

        $message_id_array = $wpdb->get_results($wpdb->prepare("SELECT meta.message_id as message_id,recipients.unread_count as unread FROM {$bp->messages->table_name_meta} as meta INNER JOIN {$bp->messages->table_name_recipients} as recipients ON meta.message_id = recipients.thread_id WHERE meta.meta_key = 'schedule_id' AND meta.meta_value = %d AND recipients.user_id = %d ORDER BY meta.id DESC",$schedule_id,$user_id));

        if(!empty($message_id_array)){
            $message_ids = array();
            $return['total_unread']=0;
            $unread_count =array();
            foreach($message_id_array as $message){
                $message_ids[]=$message->message_id;
                $unread_count[$message->message_id] = intval($message->unread);
                $return['total_unread'] = $return['total_unread']+intval($message->unread);
            }

            $message_ids = implode(',',$message_ids);
            $messages = $wpdb->get_results(  "SELECT * FROM {$bp->messages->table_name_messages} WHERE thread_id IN ($message_ids) GROUP BY thread_id ORDER BY date_sent DESC" );
            
            if(!empty($messages)){

                foreach($messages as $message){

                    $r= array('message_id'=>intval($message->thread_id),'subject'=>$message->subject,'message'=>$message->message,'time'=>tofriendlytime(time()-strtotime($message->date_sent)),'unread'=>$unread_count[$message->thread_id]);

                    if($message->sender_id != $user_id){
                        if(empty($this->users[$message->sender_id])){
                            $this->users[$message->sender_id] = array('name'=>bp_core_get_user_displayname(),'image' => bp_core_fetch_avatar ( array( 'item_id' => $message->sender_id, 'type' => 'full','html'=>false )));    
                        }

                        $r['user'] = $this->users[$message->sender_id];
                    }

                    $results[]=$r;
                }
            }
        }


        return new WP_REST_Response( $results, 200 );
    } 

    function post_appointments_schedule_permissions_check($request){
        $post = json_decode($request->get_body(),true);  
        if($post['security'] == vibe_get_option('security_key')){
            return true;
        }

        return false;
    }
    function send_appointment_schedule_message($request){
        
        $post = json_decode($request->get_body(),true);  

        if(function_exists('messages_new_message')){
            $message_args = array('sender_id' =>  $post['sender_id'],
                  'subject' => sprintf(_x('Message from %s for appointment %s','appointment admin panel','wplms-appointments'),bp_core_get_user_displayname($post['sender_id']),$post['appointment_id']),
                  'content' => $post['message'],
                  'thread_id'  => empty($post['message_id'])?false:$post['message_id'],
                  'recipients' => $post['receiving_id']
                );
            $message_id = messages_new_message( 
                $message_args
            );
            bp_messages_add_meta( $message_id, 'schedule_id', intval($request->get_param('id')) );

            $message_args['message_id'] = $message_id;
            $message_args['time']= __('Just now','wplms-appointments');
            
            return new WP_REST_Response(array('status'=>1,'message'=>__('Messages sent.','wplms-appointments'),'sent_message'=>$message_args));
        }else{
            return new WP_REST_Response(array('status'=>0,'message'=>__('Messages disabled in site.','wplms-appointments')));
        }
        
    }

    function get_appointment_schedule_activity($request){
        
        $schedule_id = $request->get_param('id');
        $instructor_id= $request->get_param('instructor_id');
        $user_id = $request->get_param('user_id');
        $paged = $request->get_param('paged');
        if(empty($paged)){$paged=0;}

        $return = array();
        
        if(empty($user_id)){
            $args = array(
            'display_comments' => 'false',
            'filter_query'=>array(
                array(
                    'relation'=>'AND',
                    'column'=>'component',
                    'value'=>'appointments',
                ),
                array(
                    'relation'=>'AND',
                    'column'=>'item_id',
                    'value'=>$schedule_id
                ),
            ),
            'show_hidden'      => false
        );
        }else{
            $args = array(
            'display_comments' => 'false',
            'filter_query'=>array(
                array(
                    'relation'=>'AND',
                    'column'=>'component',
                    'value'=>'appointments',
                ),
                array(
                    'relation'=>'AND',
                    'column'=>'item_id',
                    'value'=>$schedule_id
                ),
                array(
                    'relation'=>'AND',
                    'column'=>'user_id',
                    'value'=>array($user_id,$instructor_id),
                ),
            ),
            'show_hidden'      => false
        );
        }
        $activities = bp_activity_get($args );

        
        if(!empty($activities)){
            
            $return = (array)$activities['activities'];
            foreach($return as $k=>$v){
                $return[$k]=(Array)$return[$k];
                $return[$k]['date_recorded']=tofriendlytime(time() - strtotime($v->date_recorded));
            }
            //$activity_obj = new BP_Activity_Activity;
           ;//$activity_obj->get_activity_data($activity_ids);
        }
        
        return new WP_REST_Response( $return, 200 );
    }


    //Save appointments schedule
    //function(){
    // $init->settings['ppointment_reminder_time'] //do_action('jasdgjhasdg','schedules','events0');

    // Save Appointments Availabilty

     function update_appointments_availability_permissions_check($request){
        //To debug whether it is working or not
        //return true;

        $post = json_decode($request->get_body(),true);  

        if($post['security'] !== vibe_get_option('security_key') || empty($request->get_param('id')) || !get_userdata($request->get_param('id') )){
            return false;
        }

        return true;

    }
/*

    function get_appointment_bookers_statistics($request){

        $booker_ids = $request->get_param('booker_ids');
        $booker_ids = json_decode($booker_ids);
        if(is_array($booker_ids)){
            $booker_ids = implode(',',$booker_ids);
        }
        global $wpdb;
        $appointments_table_name = $wpdb->prefix.'wplms_appointments';
        
        if(!empty($booker_ids)){
            $users = $wpdb->get_results("SELECT u.ID,u.display_name FROM {$wpdb->users} as u LEFT JOIN ${appointments_table_name} as at on u.ID = at.booker_id WHERE at.booker_id in ({$booker_ids}) GROUP BY at.booker_id");

        }else{
            $users = $wpdb->get_results("SELECT u.ID,u.display_name FROM {$wpdb->users} as u LEFT JOIN ${appointments_table_name} as at on u.ID = at.booker_id GROUP BY at.booker_id");
            
        }

        $total = count_users();
        $total = $total['total_users'];
        $users = count($users);


        $return = array(
            'bookers'=> $users,
            'total'  => $total,
            'non_bookers'=>($total - $users),
        );

        return new WP_REST_Response( $return, 200 );
    }
    */

    function update_availability($request){

        $user_id = $request->get_param('id');
        $availability = json_decode($request->get_body(),true);  


        //debugging
        $message = __('Saved','wplms-appointments');
       

        if(!empty($availability)){

            $appointments_db = new WPLMS_APPOINTMENTS_DB();
            $metadb = new WPLMS_APPOINTMENTS_META_DB;

            
            if(!empty($availability['removed'])) {
                $availability['removed'] = json_decode($availability['removed'],true);

                foreach($availability['removed'] as $appointment){
                    $message .= 'Removing '.$appointment['appointment_id'].' # ';
                    $rm = apply_filters('wplms_appointment_can_remove_appointment_slot',true,$appointment); 
                    if($rm){
                        $metainfo = $metadb->get_appointments_meta(array('appointment_id'=>$appointment['appointment_id']));

                        $appointments_db->delete_appointment($appointment['appointment_id']);
                        $metadb->delete_appointment_meta($metainfo[0]['meta_id']);
                    }
                }
            }

            $schedule_index_appointment_id =array();
            if(!empty($availability['schedule'])){

                $availability['schedule'] = json_decode($availability['schedule'],true);
                foreach($availability['schedule'] as $k=>$a){
                    //'appointment_'.$a['type'].'_'.$a['id']]
                    
                    if(empty($a['appointment_id'])){


                        $args = array(
                            'start_date'        => $a['date_start'],
                            'end_date'          => $a['date_end'],
                            'start_time'        => $a['time_start'],
                            'end_time'          => $a['time_end'],
                            'type'              => 'available',
                            'author_id'         => $a['author_id'],
                            'item_id'           => 0,
                            'status'            => 'open',
                        );

                        if(current_user_can('manage_options') && get_current_user_id() != bp_displayed_user_id()){
                            $args['author_id'] = bp_displayed_user_id();
                        }
                        
                        $appointment_id = $appointments_db->insert_appointment($args);

                        $availability['schedule'][$k]['appointment_id'] = $appointment_id;
                        $schedule_index_appointment_id[$k] = $appointment_id;
                        

                        
                    }else{

                        if(!empty($a['start_time']) && !empty($a['time_start'])){
                            $a['start_time']=$a['time_start'];
                        }
                        if(!empty($a['end_time']) && !empty($a['time_end'])){
                            $a['end_time']=$a['time_end'];
                        }
                        $appointments_db->update_appointment($a);
                        //
                        $schedule_index_appointment_id[$k] = $a['appointment_id'];


                    }


                }
            }
            
            if(!empty($availability['events'])){

                $availability['events'] = json_decode($availability['events'],true);
              

                foreach($availability['events'] as $k=>$a){ 
                    //'appointment_'.$a['type'].'_'.$a['id']]
                    $parent_appointment_id = empty($schedule_index_appointment_id[$a['parent_appointment_id']])?$a['parent_appointment_id']:$schedule_index_appointment_id[$a['parent_appointment_id']];

         
                    if(empty($a['appointment_id'])){

                        $args = array(
                            'parent_appointment_id'=> $parent_appointment_id,
                            'start_date'        => $a['date_start'],
                            'end_date'          => $a['date_end'],
                            'start_time'        => $a['time_start'],
                            'end_time'          => $a['time_end'],
                            'author_id'         => $a['author_id'],
                            'type'              => 'available',
                            'item_id'           => 0,
                            'status'            => 'open',
                        );
                        
                        if(current_user_can('manage_options') && get_current_user_id() != bp_displayed_user_id()){
                            $args['author_id'] = bp_displayed_user_id();
                        }
                        
                        $appointment_id = $appointments_db->insert_appointment($args);
                        $availability['events'][$k]['appointment_id'] = $appointment_id;
                        do_action('wplms_insert_appointment', $parent_appointment_id,$appointment_id);
                        
                    }else{
                        //print_r($args);
                        //print_r($a);
                        $appointment = $a;

                        if(!empty($appointment['start_time']) && !empty($appointment['time_start'])){
                            $appointment['start_time']=$appointment['time_start'];
                        }
                        if(!empty($appointment['end_time']) && !empty($appointment['time_end'])){
                            $appointment['end_time']=$appointment['time_end'];
                        }


                        $appointments_db->update_appointment($appointment);
                        $appointments_db->update_appointment(array('appointment_id'=>$appointment['parent_appointment_id'],'start_time'=>$appointment['time_start']));

                        do_action('wplms_update_appointment', $parent_appointment_id,$appointment['appointment_id']);
                    }

                }
            }
            //$this->settings[$setting['name']] = json_encode($saved_appointments);
        }

        // Array Schedule ID >-< eventID <-> Timestamp

        return new WP_REST_Response( 
                apply_filters('wplms_update_availability',array('status'=>1,
                    'message'=>$message,
                    'availability'=> json_encode($availability)
                ),$availability), 
            200 );
    }

    
//}


}

WPLMS_Appointments_Api::init();
