<?php
/**
 * Initialise WPLMS appointments
 * 
 * @author 		VibeThemes
 * @category 	Init
 * @package 	wplms-appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class WPLMS_Appointments_Init{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Init();
        return self::$instance;
    }


    function __construct(){
        
        add_action('wp_enqueue_scripts',array($this,'load_scripts'));
        add_shortcode('wplms_appointments',array($this,'appointments'));
        add_shortcode('wplms_appointments_directory',array($this,'wplms_appointments_directory_shortcode'));
    }

    

    function get_global_settings(){
        if(empty($this->settings)){
            $this->settings=get_option(WPLMS_APPOINTMENTS_OPTION);    
        }
        
        return $this->settings;
    }

    function appointments( $atts, $content = null ) {
        extract(shortcode_atts(array(
                'type'   => '',
                'item_id'=>0,
                'instructor_id'=>0,
                'booker_id'=>0,
                'schedule_ids'=>array(),
                'start'=>0,//Date
                'end'=>0,
            ), $atts));
        $return = '';
        
        $return .= '<div class="wplms_instructor_course_appointments">';
        $return .= '<div id="wplms_instructor_course_appointments"></div>
            </div>';

       
        if(!empty($start)){
            $atts['start'] = strtotime($start)*1000;
        }

        if(!empty($end)){
            $atts['end'] = strtotime($end)*1000;
        }

        $profile = WPLMS_Appointments_Profile::init();
        
        if(strpos($instructor_id, ',') !== false){
            $instructor_id = explode(',',$instructor_id);
        }else{
            $instructor_id = array($instructor_id);
        }
        $settings = array();
        foreach($instructor_id as $instructor){
            $settings[$instructor]=apply_filters('wplms_appointment_instructor_settings',$profile->get_user_appointment_settings($instructor),$instructor);
        }

        $args = array(
            'user_id'=>bp_displayed_user_id(),
            'instructors'=>(empty($instructors)?array():$instructors),
            'type'=>'instructor_booking',
            'settings'=>$settings,
            'approval'=> ($approval === 'H')?0:1
        );
        
        $args = apply_filters('wplms_appointments_course_book_args',$args);

        $this->get_global_settings();

        if(empty($this->settings))
            return;
        
       
        ?>
        <script>
            setTimeout(function(){ //allow react script to load
                var event = new CustomEvent("wplms_appointment_set_args", { "detail": JSON.stringify({ 'shortcode': <?php echo json_encode($atts); ?> })});
                document.dispatchEvent(event);
            },500);
            
        </script>
        <?php
       
        

        $appointment_types = apply_filters('wplms_appointments_schedule_types',array(
            'available'=>__('Available','wplms-appointments'),
            'free_slots'=>__('Free slots','wplms-appointments'),
            'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
            'reserved'=>__('Reserved','wplms-appointments'),
            'booked'=>__('Booked','wplms-appointments'),
        ));

        $this->get_global_settings();
        $booking_time = 15;
        if(!empty($this->settings['appointments_booking_timer'])){
            $booking_time = $this->settings['appointments_booking_timer'];
        }

        $data = apply_filters('wplms_appointments_booking_calendar_data',array(
            'settings'=> $this->settings,
            'args'=>$args,
            'user'=>array('id'=>get_current_user_id()),
            'types'=>$appointment_types,
            'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
            'translations'=>array(
                'select_course'=>__('Select Course','wplms-appointments'),
                'select_instructor'=>__('Select Instructor','wplms-appointments'),
                'select_availability'=>__('Select Availability','wplms-appointments'),
                'show_schedule'=>__('Show schedule','wplms_appointments'),
                'hide_schedule'=>__('Hide schedule','wplms_appointments'),
                'available'=>__('Available','wplms-appointments'),
                'confirm_booking'=>__('Do you want to confirm this booking ?','wplms-appointments'),
                'confirm_pay_booking'=>sprintf(__('This booking purchase will remain valid for another %s minutes, please make payment for this booking. Do you want to confirm this booking ?','wplms-appointments'),$booking_time),
                'search_instructors'=>__('Search Available Instructors','wplms-appointments'),
                'no_instructors'=>__('No Instructors available in this time slot','wplms-appointments'),
                'view_profile'=>__('View Profile','wplms-appointments'),
                'open'=>__('Open','wplms-appointments'),
                'mine'=>__('My Booking','wplms-appointments'),
                'past_event_notice'=>__('Past slots can not be booked','wplms-appointments'),
                'minutes_remaining'=>__('minutes remaining','wplms-appointments'),
                'schedule_price'=>__('Schedule Price','wplms-appointments'),
                'slot_price'=>__('Slot Price','wplms-appointments'),
                'all'=>__('All','wplms-appointments'),
                'none'=>__('None','wplms-appointments'),
                'show_calendar'=>__('Show Calendar','wplms-appointments'),
            ),
        ));
        
        //$return .='<script> var wplms_appointments = '.json_encode($data).'</script>';



        //wp_enqueue_script('wplms_appointments_booking','http://localhost:3000/static/js/bundle.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
        wp_enqueue_script('wplms_appointments_booking',plugins_url('../assets/js/wplms_appointments_booking.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);
        wp_localize_script('wplms_appointments_booking','wplms_appointments',$data);
         wp_enqueue_style('wplms_appointments_booking_style',plugins_url('../assets/css/appointments.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);
          wp_enqueue_style('wplms_appointments_calendar',plugins_url('../assets/css/react-big-calendar.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);
        


        return $return;
    }

    function wplms_appointments_directory_shortcode( $atts, $content = null ) {
        extract(shortcode_atts(array(
                'type'   => 'instructor_booking',
                'status'=>'open',
                'primary_id'=>'',
                'notify_users'=>'',
                'appointments_title'=>'',
                'appointments_subtitle'=>'',
                'welcome_message'=>'',
                'user_id'=>'',
                'class'=>'',
            ), $atts));
        $return = '';
        
        $return .='<div class="wplms_appointments_directory">';

        $return .='<div id="wplms_appointments_directory"></div>';

        $return .='</div>';

        ob_start();
        ?>
        <script>
            document.addEventListener("wplms_appointment_instructor_mouseover",(evnt)=>{
                
                let scrollto = jQuery('[data-instructorid="'+parseInt(evnt.detail)+'"]').offset().top - jQuery('.wplms_appointments_directory_instructor_list').offset().top;
                
                jQuery('.wplms_appointments_calendar').css('transform','translateY('+scrollto+'px)');
            });
        </script>
        <?php
        $return .= ob_get_clean();
        $this->load_scripts(array('type'=>'instructor_booking'));
        return $return;
    }

    function load_scripts($args=null){
        
        global $post;
        $this->get_global_settings();
        if(empty($this->settings) || empty($this->settings['appointments_slot_time']) || empty($this->settings['appointments_directory']))
            return;

        if(empty($args) && !is_page($this->settings['appointments_directory']))
            return;

        $appointment_types = apply_filters('wplms_appointments_schedule_types',array(
            'available'=>__('Available','wplms-appointments'),
            'free_slots'=>__('Free slots','wplms-appointments'),
            'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
            'reserved'=>__('Reserved','wplms-appointments'),
            'booked'=>__('Booked','wplms-appointments'),
        ));
        $this->get_global_settings();
        $booking_time = 15;
        if(!empty($this->settings['appointments_booking_timer'])){
            $booking_time = $this->settings['appointments_booking_timer'];
        }
        $data = apply_filters('wplms_appointments_booking_calendar_data',array(
            'settings'=> $this->settings,
            'args'=>$args,
            'user'=>array('id'=>get_current_user_id()),
            'types'=>$appointment_types,
            'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
            'translations'=>array(
                'select_course'=>__('Select Course','wplms-appointments'),
                'select_instructor'=>__('Select Instructor','wplms-appointments'),
                'select_availability'=>__('Select Availability','wplms-appointments'),
                'show_schedule'=>__('Show schedule','wplms_appointments'),
                'hide_schedule'=>__('Hide schedule','wplms_appointments'),
                'available'=>__('Available','wplms-appointments'),
                'confirm_booking'=>__('Do you want to confirm this booking ?','wplms-appointments'),
                'confirm_pay_booking'=>sprintf(__('This booking purchase will remain valid for another %s minutes, please make payment for this booking. Do you want to confirm this booking ?','wplms-appointments'),$booking_time),
                'search_instructors'=>__('Search Available Instructors','wplms-appointments'),
                'no_instructors'=>__('No Instructors available in this time slot','wplms-appointments'),
                'view_profile'=>__('View Profile','wplms-appointments'),
                'open'=>__('Open','wplms-appointments'),
                'mine'=>__('My Booking','wplms-appointments'),
                'past_event_notice'=>__('Past slots can not be booked','wplms-appointments'),
                'minutes_remaining'=>__('minutes remaining','wplms-appointments'),
                'schedule_price'=>__('Schedule Price','wplms-appointments'),
                'slot_price'=>__('Slot Price','wplms-appointments'),
                'per_slot'=>__('Per Slot','wplms-appointments'),
                'all'=>__('All','wplms-appointments'),
                'none'=>__('None','wplms-appointments'),
                'search'=>__('Search ...','wplms-appointments'),
                'book_lesson'=>__('Book lesson','wplms-appointments'),
                'show_calendar'=>__('Show Calendar','wplms-appointments'),
                'agenda'=>__('Agenda','wplms-appointments'),
                'day'=>__('Day','wplms-appointments'),
                'week'=>__('Week','wplms-appointments'),
                'month'=>__('Month','wplms-appointments'),
                'today'=>__('Today','wplms-appointments'),
                'status'=>__('Status','wplms-appointments'),
                'type'=>__('Type','wplms-appointments'),
                'course'=>__('Course','wplms-appointments'),
                'instructor'=>__('Instructor','wplms-appointments'),
                'read_more'=>__('Read More','wplms-appointments'),
            ),
        ));


        $data = apply_filters('wplms_appointments_after_data',$data);
        
        //Live
        if(is_page($this->settings['appointments_directory'])){
            //DEvelopment
            
            //wp_enqueue_script('wplms_appointments','http://localhost:3001/static/js/bundle.js',array(),21,true);
            //wp_enqueue_script('wplms_appointments_chunk0','http://localhost:3001/static/js/0.chunk.js',array(),2,true);
            //wp_enqueue_script('wplms_appointments_chunk1','http://localhost:3001/static/js/1.chunk.js',array(),2,true);
            //wp_enqueue_script('wplms_appointments_main','http://localhost:3001/static/js/main.chunk.js',array(),21,true);

             
             
            
            //Live
            
            wp_enqueue_script('wplms_appointments',plugins_url('../assets/js/wplms_appointments_directory.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true); 

        }else if(!empty($args['course_id'])){
            //DEvelopment
            //wp_enqueue_script('wplms_appointments','http://localhost:3000/static/js/bundle.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
            //live
            wp_enqueue_script('wplms_appointments',plugins_url('../assets/js/wplms_appointments_booking.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);  
        }else if(!empty($args['user_id'])){

            //wp_enqueue_script('wplms_appointments','http://localhost:3000/static/js/bundle.js',array(),22,true);
            //echo '<script>var gmt_offset = 180;</script>';
            wp_enqueue_script('wplms_appointments',plugins_url('../assets/js/wplms_appointments_booking.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);  
        }else{
            //Inside Shortcode
            //wp_enqueue_script('wplms_appointments','http://localhost:3001/static/js/bundle.js',array(),21,true);
            wp_enqueue_script('wplms_appointments',plugins_url('../assets/js/wplms_appointments_booking.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true); 
        }
        //

        wp_localize_script('wplms_appointments','wplms_appointments',$data);
        wp_enqueue_style('wplms_appointments',plugins_url('../assets/css/appointments.css',__FILE__),array(),rand(1,9999));
        wp_enqueue_style('wplms_appointments_settings_calendar',plugins_url('../assets/css/react-big-calendar.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);
        
    }
}

WPLMS_appointments_Init::init();