<?php
/**
 * Initialise WPLMS appointments
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	wplms-appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WPLMS_Appointments_Profile{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Profile();
        return self::$instance;
    }

    function __construct(){
    	add_action( 'bp_setup_nav', array($this,'add_appointments'), 100 );
		add_action('wplms_customizer_custom_css',array($this,'appointment_profile_colors'));
    }

    function add_appointments(){


    	if(class_exists('Vibe_CustomTypes_Permalinks')){
            $x = Vibe_CustomTypes_Permalinks::init();
            $permalinks = $x->permalinks;    
        }else{
            $permalinks = get_option( 'vibe_course_permalinks' );
        }

        if(!empty($permalinks['appointments_slug'])){
        	$slug = str_replace('/','',$permalinks['appointments_slug']);
        }else{
        	$slug = BP_APPOINTMENTS_SLUG;
        }

        $check = get_user_meta(bp_displayed_user_id(),'enable_appointments',true);
        
		if(empty($check) && !current_user_can('manage_options') && !bp_is_my_profile()) {
			return;
		}
		global $bp;
		//Add Appointments tab in profile menu
	    bp_core_new_nav_item( array( 
	        'name' => __('Appointments','wplms-appointments'),
	        'slug' => $slug, 
	        'item_css_id' => 'appointments',
	        'screen_function' => array($this,'show_screen'),
	        'default_subnav_slug' => 'home', 
	        'position' => 50
	    ) );

	    $init = WPLMS_Appointments_Init::init();
	    $init->get_global_settings();
	    if(!empty($init->settings['appointments_recordings'])){
	    	bp_core_new_subnav_item( array(
					'name' 		  => __( 'Recordings', 'wplms-appointments' ),
					'slug' 		  => 'recordings',
					'parent_slug' => $slug,
		        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
					'screen_function' => array($this,'recordings'),
				) );
	    }
        

	    if(current_user_can('manage_options') || bp_is_my_profile()){

		    	bp_core_new_subnav_item( array(
					'name' 		  => __( 'Bookings', 'wplms-appointments' ),
					'slug' 		  => _x('home','default appointments slug in profile ','wplms-appointments'),
					'parent_slug' => $slug,
		        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
					'screen_function' => array($this,'show_upcoming'),
				) );

		    	
		    	


		    	if(empty($init->settings['enable_appointments']) || (!empty($init->settings['enable_appointments']) && current_user_can($init->settings['enable_appointments']))){
				    
				    $count_pending_appointments = 0;
				    $pending_appointments = wplms_appointments_get_pending_slots(array('instructor_id'=>bp_displayed_user_id()));

				    if(!empty($pending_appointments)){
				    	foreach($pending_appointments as $appointment){
				    		unset($appointment['appointment_id']);
				    		$count_pending_appointments += count($appointment);
				    	}
				    }

				    bp_core_new_subnav_item( array(
						'name' 		  => sprintf(__( 'Instructing Appointments', 'wplms-appointments' ).'<span>%s</span>',$count_pending_appointments),
						'slug' 		  => _x('instructing','default appointments slug in profile ','wplms-appointments'),
						'parent_slug' => $slug,
			        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
						'screen_function' => array($this,'show_instructing'),
					) );

				    bp_core_new_subnav_item( array(
							'name' 		  => __( 'Settings', 'wplms-appointments' ),
							'slug' 		  => BP_APPOINTMENTS_SETTINGS_SLUG,
							'parent_slug' => $slug,
				        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
							'screen_function' => array($this,'show_settings'),
						) );

				    bp_core_new_subnav_item( array(
						'name' 		  => __( 'Student View', 'wplms-appointments' ),
						'slug' 		  => 'student',
						'parent_slug' => $slug,
			        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
						'screen_function' => array($this,'booking_form'),
					) );
				}
		    

		}else{
			
			 	//Add gifts sent submenu in gift tab
				bp_core_new_subnav_item( array(
					'name' 		  => __( 'Appointments', 'wplms-appointments' ),
					'slug' 		  => 'home',
					'parent_slug' => $slug,
		        	'parent_url' => $bp->displayed_user->domain.$slug.'/',
					'screen_function' => array($this,'booking_form'),
				) );
			
		}
	}


	function show_screen(){
		
	}

	function booking_form(){
		add_action('bp_template_title',function(){
			_ex('Book Instructor','appointments profile screen','wplms-appointments'); 
		});
		add_action( 'bp_template_content', array($this,'book_appointments'));
    	bp_core_load_template( 'members/single/plugins');
	}

	function book_appointments(){
		

		echo '<div class="wplms_instructor_course_appointments">';
        echo '<div id="wplms_instructor_course_appointments"></div>
            </div>';

        $init = WPLMS_Appointments_Init::init();

        $profile = WPLMS_Appointments_Profile::init();
        
        $instructors = array(bp_displayed_user_id());
        $settings = array();
        foreach($instructors as $instructor){

            $settings[$instructor]=apply_filters('wplms_appointment_instructor_settings',$profile->get_user_appointment_settings($instructor),$instructor);
        }

        $args = array(
        	'user_id'=>bp_displayed_user_id(),
            'instructors'=>$instructors,
            'type'=>'instructor_booking',
            'settings'=>$settings,
            'approval'=> ($approval === 'H')?0:1
        );
        
        $args = apply_filters('wplms_appointments_course_book_args',$args);
        
        ?>
        <script>
            setTimeout(function(){ //allow react script to load
                var event = new CustomEvent("wplms_appointment_set_args", { "detail": JSON.stringify({author_id:<?php echo bp_displayed_user_id(); ?>,'item_id':'any'})});
                document.dispatchEvent(event);
            },500);
            
        </script>
        <?php
        do_action('wplms_appointments_booking_form','profile',bp_displayed_user_id());

        $init->load_scripts($args);

		
		?>
		<style>
			/*
			#buddypress .item-list-tabs#subnav{display:none;}
			*/
		</style>
		<?php
	}


	function show_upcoming(){
		add_action('bp_template_title',function(){_ex('My Appointments','appointments profile screen','wplms-appointments'); });
		add_action( 'bp_template_content', array($this,'upcoming_appointments'));
    	bp_core_load_template( 'members/single/plugins');
	}


	function show_instructing(){
		add_action('bp_template_title',function(){_ex('Instructing Appointments','appointments profile screen','wplms-appointments'); });
		add_action( 'bp_template_content', array($this,'instructing_appointments'));
    	bp_core_load_template( 'members/single/plugins');
	}

	function show_settings(){

		add_action('wp_enqueue_scripts',array($this,'enqueue_settings_script'));
		add_action('bp_template_title',function(){_ex('Appointment Settings','appointments profile screen','wplms-appointments'); });
		add_action( 'bp_template_content', array($this,'appointment_settings'));
    	bp_core_load_template( 'members/single/plugins');
	}

	function upcoming_appointments(){

		$init = WPLMS_Appointments_Init::init();
		$init->get_global_settings();  

		do_action('wplms_appointments_profile_myappointments',bp_displayed_user_id());
		echo '<div id="wplms_instructor_course_appointments"></div>';

		wp_enqueue_style('wplms_appointments_settings',plugins_url('../assets/css/appointment_settings.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);

		wp_enqueue_style('wplms_appointments_settings_calendar',plugins_url('../assets/css/react-big-calendar.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);

		$appointment_types = apply_filters('wplms_appointments_schedule_types',array(
            'available'=>__('Available','wplms-appointments'),
            'free_slots'=>__('Free slots','wplms-appointments'),
            'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
            'reserved'=>__('Reserved','wplms-appointments'),
            'booked'=>__('Booked','wplms-appointments'),
        ));

		$translation = apply_filters('wplms_appointments_profile_args',array(
			'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
			'type'=>'mybookings',
			'exclude_meta_in_schedule_popup'=>array('appointment_product_id','appointment_price','appointment_price','enable_meetings','schedule_queue','meeting_args','break_slots'),
			'types'=>$appointment_types,
			'settings'=>$init->settings,
			'default_member_image'=>vibe_get_option('default_avatar'),
			'dateformat'=>apply_filters('wplms_appotintments_date_format','MMM,DD YYYY'),//get_option('date_format'),
			'timeformat'=>apply_filters('wplms_appotintments_time_format','hh:mm a'),//
			'security'=>vibe_get_option('security_key'),
			//

			'user'=>array(
				'id'=>bp_displayed_user_id(),
				'name'  => bp_core_get_user_displayname(bp_displayed_user_id()),
        		'image' => bp_core_fetch_avatar ( array( 'item_id' => bp_displayed_user_id(), 'type' => 'full','html'=>false )),
				'messages'=>array(
					'unread'=>wplms_appointments_get_unread_count(bp_displayed_user_id()),
					'url'=>((function_exists('bp_is_active') && bp_is_active('messages'))?trailingslashit( bp_loggedin_user_domain() . bp_get_messages_slug() ):0)
				),
			),
			'translations'=>array(
				'all'=>__('All','wplms-appointments'),
				'add_availability_schedule'=>__('Add Availability Schedule','wplms-appointments'),
				'save_schedules'=>__('Save schedules','wplms-appointments'),
				'set_availability_period'=>__('Set Availability period','wplms-appointments'),
				'set_availability_time'=>__('Set Avaialibity Time','wplms-appointments'),
				'available'=>__('Available','wplms-appointments'),
				'start_date'=>__('Start Date','wplms-appointments'),
				'end_date'=>__('End Date','wplms-appointments'),
				'start_time'=>__('Start Time','wplms-appointments'),
				'end_time'=>__('end Time','wplms-appointments'),
				'show_active_month_events'=>__('Show active month','wplms-appointments'),
				'hide_past_events'=>__('Hide past events','wplms-appointments'),
				'today'=>__('Today','wplms-appointments'),
				'by'=>__('by','wplms-appointments'),
				'pending'=>__('Pending','wplms-appointments'),
				'confirmed'=>__('Confirmed','wplms-appointments'),
				'agenda'=>__('Agenda','wplms-appointments'),
				'day'=>__('Day','wplms-appointments'),
				'week'=>__('Week','wplms-appointments'),
				'month'=>__('Month','wplms-appointments'),
				'more_details'=>__('More Details','wplms-appointments'),
				'open'=>__('Open','wplms-appointments'),
				'closed'=>__('Closed','wplms-appointments'),
				'confirmed'=>__('Confirmed','wplms-appointments'),
				
				'status'=>__('Status','wplms-appointments'),
				'type'=>__('Type','wplms-appointments'),
				'course'=>__('Course','wplms-appointments'),
				'booker'=>__('Booker','wplms-appointments'),
				'instructor'=>__('Instructor','wplms-appointments'),
				'more_details'=>__('More Details','wplms-appointments'),
				'details'=>__('Details','wplms-appointments'),
				'messages'=>__('Messages','wplms-appointments'),
				'activity'=>__('Activity Log','wplms-appointments'),
				'connected_bookings'=>__('Connected Bookings','wplms-appointments'),
				'reschedule_appointment'=>__('Reschedule Appointment','wplms-appointments'),
				'cancel_appointment'=>__('Cancel Appointment','wplms-appointments'),
				'cancel_meeting'=>__('Cancel Meeting','wplms-appointments'),
				'cancel_all_meeting'=>__('Cancel All Meeting','wplms-appointments'),
				'message_instructor'=>__('Message instructor','wplms-appointments'),
				'message_user'=>__('Message User','wplms-appointments'),
				'book_another_slot'=>__('Book another slot','wplms-appointments'),
				'nomessage'=>__('No messages found.','wplms-appointments'),
				'noactivity'=>__('No activities found.','wplms-appointments'),
				'reply'=>__('Reply','wplms-appointments'),
				'cancel_appointment_reason'=>__('Reason for Cancellation','wplms-appointments'),
				'reschedule_appointment_reason'=>__('Reason for Reschedule','wplms-appointments'),
				'load_more'=>__('Load more','wplms-appointments'),
				'send'=>__('Send Message','wplms-appointments'),
				'sending'=>__('Sending','wplms-appointments'),
				'cancelled'=>__('Cancelled','wplms-appointments'),
				'reschdule'=>__('Rescheduled','wplms-appointments'),
				'close'=>__('Close','wplms-appointments'),
				'requested_cancellation'=>__('Cancellation Requested','wplms-appointments'),
				'requested_reschedule'=>__('Reschedule Requested','wplms-appointments'),
				'more'=>__('More','wplms-appointments'),
				'message_all_members'=>__('Message all members','wplms-appointments'),
				'admin'=>__('Admin','wplms-appointments'),
				'select_member'=>__('Select Member','wplms-appointments'),
				'cancel_confirm'=>__('Confirm appointment cancellation.','wplms-appointments'),
				'same_schedule'=>__('Same schedule.','wplms-appointments'),
				'mark_appointment_attended'=>__('Mark Appointment Attended','wplms-appointments'),
				'mark_appointment_attended_confirm'=>__('Do you want to manually mark this appointment as attended by this user ?','wplms-appointments'),

				'days'=>__('days','wplms-appointments'),
				'hours'=>__('hours','wplms-appointments'),
				'minutes'=>__('minutes','wplms-appointments'),
				'seconds'=>__('seconds','wplms-appointments'),

				'accept'=>__('Accept','wplms-appointments'),
				'reject'=>__('Reject','wplms-appointments'),
				'details'=>__('Details','wplms-appointments'),
				'timesup'=>__('Time Out','wplms-appointments'),

				'form_data'=>__('Form data','wplms-appointments'),
				'hide_form_data'=>__('Hide data','wplms-appointments'),
				'form_attachment'=>__('Form attachment','wplms-appointments'),
				'awaiting_instructor'=>__('Pending approval','wplms-appointments'),
				'join_meeting'=> __('Join Meeting','wplms_appointments'),
				'meeting_ended'=> __('Meeting Ended','wplms_appointments'),
				
				'message_member'=>__('Message member','wplms-appointments'),
				'type_message'=>__('type your message','wplms-appointments'),
				'send_message'=>__('Send message','wplms-appointments'),
				'sending'=>__('Sending...','wplms-appointments'),
				'cancel'=>__('cancel','wplms-appointments'),
				'show_pending'=>__('Show pending appointments','wplms-appointments'),
				'hide_pending'=>__('Hide pending appointments','wplms-appointments'),
				'show_filters'=>__('Show Filters','wplms-appointments'),
				'hide_filters'=>__('Hide Filters','wplms-appointments'),
				
			)
			
		));

		
		if($translation['type'] == 'mybookings'){
			//Bookings Tab
			
			// wp_enqueue_script('wplms_appointments_mybookings','http://localhost:3000/static/js/bundle.js',array(),10,true);
			// wp_enqueue_script('wplms_appointments_mybookings_3','http://localhost:3000/static/js/0.chunk.js',array(),10,true);
			// wp_enqueue_script('wplms_appointments_mybookings_2','http://localhost:3000/static/js/1.chunk.js',array(),10,true);
			// wp_enqueue_script('wplms_appointments_mybookings_1','http://localhost:3000/static/js/main.chunk.js',array(),10,true);
			wp_enqueue_script('wplms_appointments_mybookings',plugins_url('../assets/js/mybookings.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);	
			wp_localize_script('wplms_appointments_mybookings','wplms_appointments',$translation);
		}else{

			//MyAppointments
			//Instructing Appointments Tab
			
			//change 3001
			// wp_enqueue_script('wplms_appointments_myappointments','http://localhost:3000/static/js/bundle.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
		 // 	wp_enqueue_script('wplms_appointments_myappointments_2','http://localhost:3000/static/js/0.chunk.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
			// wp_enqueue_script('wplms_appointments_myappointments_3','http://localhost:3000/static/js/1.chunk.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
			// wp_enqueue_script('wplms_appointments_myappointments_1','http://localhost:3000/static/js/main.chunk.js',array(),WPLMS_APPOINTMENTS_VERSION,true);
			
			wp_enqueue_script('wplms_appointments_myappointments',plugins_url('../assets/js/myappointments.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);
			
			wp_localize_script('wplms_appointments_myappointments','wplms_appointments',$translation);
		}
		
		

		?>
		<script>

		 document.addEventListener("wplms_appointment_calendar_height", function(event) {
                       

            var calendar_type =event.detail.view;

			if(calendar_type === 'agenda'){
				setTimeout(function(){
					let height = jQuery('.agenda_event').height()+15;
                	
                	jQuery('.rbc-calendar').css('height',(event.detail.length*height +jQuery('.toolbar_container').height() )+'px');        	
				},300);
                
                
            }
            
            if(calendar_type === 'month'){
                jQuery('.rbc-calendar').css('height','600px');    
            }
        });

		</script>
		<?php

		add_action('wp_footer',function(){
			?><style>.bp-user.p3 #item-nav+.container, .bp-user.p4 #item-nav+.container{z-index:2;}</style>
			<?php
		});
	}

	function instructing_appointments(){
		add_filter('wplms_appointments_profile_args',function($args){
			$args['type']='instructing';
			return $args;
		});
		$this->upcoming_appointments();
	}

	function enqueue_settings_script(){

		$init = WPLMS_Appointments_Init::init();
		$init->get_global_settings();  

		$user_id = bp_displayed_user_id();
		/*
		

		wp_enqueue_script('wplms_appointments_settings',plugins_url('../assets/js/appointment_settings.js',__FILE__),array('jQuery','jquery-ui-core','jquery-ui-datepicker'),1,true);*/
		
		wp_enqueue_style('wplms_appointments_settings',plugins_url('../assets/css/appointment_settings.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);
		wp_enqueue_style('wplms_appointments_settings_calendar',plugins_url('../assets/css/react-big-calendar.css',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION);
		
		$appointment_types = apply_filters('wplms_appointments_schedule_types',array(
            'available'=>__('Available','wplms-appointments'),
            'free_slots'=>__('Free slots','wplms-appointments'),
            'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
            'reserved'=>__('Reserved','wplms-appointments'),
            'booked'=>__('Booked','wplms-appointments'),
        ));

		$args = apply_filters('wplms_appointments_appointment_settings',array(
			
			'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
			'type'=>'instructor_settings',
			'security'=>vibe_get_option('security_key'),
			'timestamp'=>time(),
			'settings'=>$init->settings,
			'types'=>$appointment_types,
			'exclude_meta_in_schedule_popup'=>array('appointment_product_id','appointment_price','enable_meetings','schedule_queue','meeting_args','break_slots'),
			'user'=>array(
				'id'=>bp_displayed_user_id(),
				'name'  => bp_core_get_user_displayname(bp_displayed_user_id()),
        		'image' => bp_core_fetch_avatar ( array( 'item_id' => bp_displayed_user_id(), 'type' => 'full','html'=>false )), 
				'settings'=>$this->get_user_appointment_settings($user_id),
			),
			'translations'=>array(
				'add_availability_schedule'=>__('Add Availability Schedule','wplms-appointments'),
				'set_availability_period'=>__('Set Dates','wplms-appointments'),
				'set_availability_time'=>__('Set Times','wplms-appointments'),
				'available'=>__('Available','wplms-appointments'),
				'start_date'=>__('Start Date','wplms-appointments'),
				'end_date'=>__('End Date','wplms-appointments'),
				'start_time'=>__('Start Time','wplms-appointments'),
				'end_time'=>__('end Time','wplms-appointments'),
				'show_active_month_events'=>__('Show active month','wplms-appointments'),
				'hide_past_events'=>__('Hide past events','wplms-appointments'),
				'edit'=>__('Edit','wplms-appointments'),
				'type'=>__('Type','wplms-appointments'),
				'open'=>__('Open','wplms-appointments'),
				'closed'=>__('Closed','wplms-appointments'),
				'close'=>__('Close','wplms-appointments'),
				'view'=>__('View','wplms-appointments'),
				'cancel'=>__('Cancel','wplms-appointments'),
				'remove'=>__('Remove','wplms-appointments'),
				'status'=>__('Status','wplms-appointments'),
				'add_new_file'=>__('Add new file','wplms-appointments'),
				'save'=>__('Save Availability Schedules','wplms-appointments'),
				'save_editpopup'=>__('Save Schedule Fields','wplms-appointments'),
				'saving'=>__('Saving','wplms-appointments'),
				'type_description'=>__('Appointment Type','wplms-appointments'),
				'all'=>__('All','wplms-appointments'),
				'add_meeting_presentation'=>__('Meeting Presentation','wplms-appointments'),
				'show_details'=>__('Show Details','wplms-appointments'),
				'show_calendar'=>__('Show Calendar','wplms-appointments'),
				'slots'=>__('Slots','wplms-appointments'),
				'seats'=>__('Seats','wplms-appointments'),
				
				'add_booker'=>__('Add booker','wplms-appointments'),
				'search_booker'=>__('Search booker','wplms-appointments'),
				
				'bookers'=>__('Bookers','wplms-appointments'),
				'bookers_count_more_max_seats'=>__('Booker count can not be more than max seats','wplms-appointments'),
				'status_set_to_closed'=>__('Status set to closed','wplms-appointments'),
				'assign_course'=>__('Assign to Course','wplms-appointments'),
				'select_course'=>__('Select Course','wplms-appointments'),
				'add_note'=>__('Add a note','wplms-appointments'),
				'add_meeting_note'=>__('Add meeting note','wplms-appointments'),
				
				'select_date_time_notice'=>__('Click on select date to set dates for this schedule. Click on set times to set timings for this schedule. Each schedule is composed of multiple events, where each event per day.','wplms-appointments'),
				'meeting'=>__('Online Meeting / Conference','wplms-appointments'),
				'set_schedule_meeting'=>__('Enable meeting schedule','wplms-appointments'),
				'file_upload'=>__('Upload File','wplms-appointments'),
				'uploading'=>__('...uploading','wplms-appointments'),

				'setting_schedule'=>__('... setting meeting reminder','wplms-appointments'),
				'meeting_enabled'=>__('Meeting  schedule enabled','wplms-appointments'),
				'disable_meeting'=>__('Disable meeting schedule','wplms-appointments'),
				'meeting_status'=>__('Check meeting status','wplms-appointments'),
				
				'join_meeting'=>__('Join Meeting','wplms-appointments'),
				'break_slots'=>__('Break into Slots of Minutes Per Day','wplms-appointments'),
				'hours'=>__('hours','wplms-appointments'),
				'minutes'=>__('minutes','wplms-appointments'),
				'full_slot'=>__('Full Slot','wplms-appointments'),
				'save_schedule_required'=>__('Requires save schedules','wplms-appointments'),

				/*translations for walkthrough*/
				'walkthrough_intro'=>__('Welcome to Appointments Walkthrough. Lets go through all the steps required to create new schedules and slots.','wplms-appointments'),
				'walkthrough_checkbox'=>__('Would you like the walkthrough to complete steps as we proceed ?','wplms-appointments'),
				'click_to_add_schedule'=>__('Click on this button to Add a New Schedule','wplms-appointments'),
				'show_all_available_schedules'=>__('Click on this icon to show all schedules of this instructor.','wplms-appointments'),
				'show_current_month_schedule'=>__('Check this checkbox to show only current month schedules.','wplms-appointments'),
				'hide_passed_events'=>__('Check this Checkbox to hide schedules and events that have already happened.','wplms-appointments'),
				'set_the_date'=>__('Click this link to set the date of schedule.','wplms-appointments'),
				'select_dates'=>__('Hold & drag over dates to select the date range for this schedule.','wplms-appointments'),
				'set_the_time'=>__('Click this link to set the time of schedule.','wplms-appointments'),
				'select_time'=>__('Hold & drag over time slots to select the time range for 
					this schedule.','wplms-appointments'),
				'save_schedules'=>__('Click to save all schedules.','wplms-appointments'),
				'show_schedule_details'=>__('Click this link to show the details of the 
					schedule.','wplms-appointments'),
				'edit_schedule_details'=>__('Click this link to edit the schedule.','wplms-appointments'),
				'show_available_events_schedule'=>__('Click this link to check the available events of that schedule','wplms-appointments'),
				'remove_schedule'=>__('Click this link to remove the schedule and then click on button save availability schedule','wplms-appointments'),
				'edit_calendar_details'=>__('Click this link to edit the details of the current selected schedule','wplms-appointments'),
				'eventpopup_fields'=>__('Show Eventpopup of that schedule','wplms-appointments'),
				'edit_event'=>__('Click to edit this schedule','wplms-appointments'),
				'break_slots_select'=>__('Select to break selected time period into time slots of selected interval in the day.','wplms-appointments'),
				'type_select'=>__('Set the type of the schedule.','wplms-appointments'),
				'status_select'=>__('Set if schedule is open or closed for bookings.','wplms-appointments'),
				'course_select'=>__('Assign a course.','wplms-appointments'),
				'seats_number'=>__('Set seats per slot in the schedule.','wplms-appointments'),
				'bookers_number'=>__('Bookers for the Schedule and all the connected slots.','wplms-appointments'),
				'search_bookers_option'=>__('Users can Search the Bookers here','wplms-appointments'),
				'notes_text'=>__('Set notes for the schedule.','wplms-appointments'),
				'bbb_button'=>__('Enable meeting reminders.','wplms-appointments'),
				'meeting_notes_area'=>__('Notes for the meeting.','wplms-appointments'),
				'presentation_upload'=>__('Add a presentation for the upcoming meeting.','wplms-appointments'),
				'editpopupbutton'=>__('After changing the details, hit on save schedule fields button','wplms-appointments'),
				'save_availability_breakslots'=>__('Save availability schedules to confirm change in slots !','wplms-appointments'),
			),
			
		));

		//Appointment settings

		// echo '<script>var gmt_offset = 180;</script>';
		 // wp_enqueue_script('wplms_appointments_settings','http://localhost:3000/static/js/bundle.js',array(),10,true);
		 // wp_enqueue_script('wplms_appointments_settings_chunk','http://localhost:3000/static/js/0.chunk.js',array(),10,true);
		 // wp_enqueue_script('wplms_appointments_settings_chunk1','http://localhost:3000/static/js/1.chunk.js',array(),10,true);
		 // wp_enqueue_script('wplms_appointments_settings_main','http://localhost:3000/static/js/main.chunk.js',array(),10,true);

		wp_enqueue_script('wplms_appointments_settings',plugins_url('../assets/js/profile_appointment_settings.js',__FILE__),array(),WPLMS_APPOINTMENTS_VERSION,true);
		
		wp_localize_script('wplms_appointments_settings','wplms_appointments_settings',$args);

		add_action('wp_footer',function(){
			?><style>.bp-user.p3 #item-nav+.container, .bp-user.p4 #item-nav+.container{z-index:2;}</style>
			<?php
		});
	}

	function appointment_settings(){

		?>
		<div class="appointment_setting_tabs">
			<a class="general_tab active" data-tab="general_settings_content"><?php _e('General','wplms-appointments'); ?></a>
			<a class="schedule_tab" data-tab="schedule_settings_content"><?php _e('Schedule Calendar','wplms-appointments'); ?></a>
		</div>
		<div class="appointment_settings_content">
			<div class="general_settings_content active">
				<?php $this->settings(); ?>
			</div>
			<div class="schedule_settings_content">
				<?php $this->show_schedule(); ?>
			</div>
		</div>
		<style>
			.appointment_setting_tabs{margin-top:30px;display:grid;grid-template-columns: repeat(auto-fill,minmax(120px,240px));align-items:center;text-align:center;}@media(max-width:600px){.appointment_setting_tabs{grid-template-columns: repeat(auto-fit,minmax(120px,1fr));}}.appointment_setting_tabs a{color:#222;padding:15px 0;cursor:pointer;    border-bottom: 2px solid rgba(0,0,0,0.08);}.appointment_setting_tabs a.active{text-shadow:0 5px 20px rgba(0,0,0,0.08);}.appointment_settings_content{padding:30px 0;}.appointment_settings_content > div{display: none;transform:translateY(30px);transition:all 0.5s;}.appointment_settings_content > div.active{transform:translateY(0px);display:block;}.availability_controls>strong{font-size:11px;margin-right:12px;}

			.appointment_setting_tabs a.active {border-bottom: 2px solid #79BD9A;}
		</style>
		<script>
			jQuery(document).ready(function(){
			jQuery('.appointment_setting_tabs > a').on('click',function(event){
				event.preventDefault();
				jQuery('.appointment_setting_tabs .active').removeClass('active');
				jQuery(this).addClass('active');
				jQuery('.appointment_settings_content .active').removeClass('active');
				jQuery('.'+jQuery(this).data('tab')).addClass('active');
			});});
		</script>
		<?php
		
	}


	function fetch_settings(){

		$init = WPLMS_Appointments_Init::init();
		$init->get_global_settings();

		$settings = array(
			array(
				'label' => __('Enable Appointments.','wplms-appointments'),
				'name' => 'enable_appointments',
				'type' => 'checkbox',
				'default'=>'0',
				'value'=>'1',
				'desc' => __('Enable booking','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Minimum Slot Time','wplms-appointments'),
				'name' => 'appointments_slot_time',
				'type' => 'number',
				'default'=> (empty($init->settings['appointments_slot_time'])?'15':$init->settings['appointments_slot_time']) ,
				'desc' => __('Set minimum appointment slot time in minutes','wplms-appointments'),
			),
			array(
				'label' => __('Appointments booking buffer Time','wplms-appointments'),
				'name' => 'appointments_buffer_time',
				'type' => 'number',
				'default'=>(empty($init->settings['appointments_buffer_time'])?'999':$init->settings['appointments_buffer_time']),
				'desc' => __('Set a default time in minutes before which the appointments are freezed and appointments are cancelled','wplms-appointments'),
			),
			array(
				'label' => __('Maximum time (in minutes) for manually approving appointments.','wplms-appointments'),
				'name' => 'appointments_approval_wait_time',
				'type' => 'text',
				'default'=>'30',
				'value'=>'0',
				'desc' => __('Only applicable for manuallly approving appointments','wplms-appointments'),
			),
			array(
				'label' => __('Message for Bookers.','wplms-appointments'),
				'name' => 'appointments_message',
				'type' => 'editor',
				'default'=>'',
				'value'=>'0',
				'desc' => __('Default message for bookers','wplms-appointments'),
			),
		);
 		
 		if(function_exists('WC')){
 			$settings[]=array(
				'label' => __('Default Slot price.','wplms-appointments'),
				'name' => 'instructor_appointments_price',
				'type' => 'price',
				'default'=>'0',
				'value'=>'0',
				'desc' => __('Set a price for slot.','wplms-appointments'),
			);
 		}
		

		if(!empty($init->settings['appointments_instructor_video'])){
			
			$settings[]=array(
				'label' => __('Promotional Video','wplms-appointments'),
				'name' => 'appointments_video',
				'type' => 'text',
				'level'=>'video',
				'value_type'=>'single',
				'upload_button'=>__('Set as Instructor Video','wplms-appointments' ),
				'upload_title'=>__('Upload Featured Video','wplms-appointments' ),
				'upload_multiple'=>0,
				'style'=>'',
				'from'=>'post',
				'id' => 'post_video',
				'desc' => __('Instructors can add a promotional video link here describing their teaching','wplms-appointments'),
			);
			
		}

		
		if(!empty($init->settings['appointments_instructor_fields'])){

			
			$fields = json_decode(stripslashes($init->settings['appointments_instructor_fields']),true);
			foreach($fields as $setting){
				if($setting['type'] == 'select'){
					$settings[]=array(
						'label' => $setting['label'],
						'name' => 'instructor_searchable_field_'.$setting['key'],
						'type' => $setting['type'],
						'options'=>$setting['options'],
						'desc' => __('searchable instructor field','wplms-appointments'),
					);
				}
			}
		}

		if(!empty($init->settings['appointments_schedule_seats'])){
			$settings[]=array(
						'label' => __('Default seats per schedule','wplms-appointments'),
						'name' => 'appointments_schedule_seats',
						'type' => 'text',
						'default'=>1,
						'desc' => __('Set default number of seats per schedule','wplms-appointments'),
					);
		}


		 return apply_filters('wplms_appointments_settings',$settings);
	}

	function settings(){

		$settings = $this->fetch_settings();
		

		$this->generate_settings_form($settings);
	}

	function show_schedule(){
		$settings=array();
		$settings[]=array(
				'label' => __('Appointments Availability','wplms-appointments'),
				'name' => 'appointments_available',
				'type' => 'availability',
				'default'=>'',
				'desc' => __('Set availability of instructor for appointments','wplms-appointments'),
			);
		$this->generate_settings_form($settings);
	}


	function generate_settings_form($settings){

		if(empty($this->settings)){
      		$user_id = bp_displayed_user_id();
      		$this->get_user_appointment_settings($user_id);
      		
      	}
		if(isset($_POST['save_wplms_settings_form'])){

			$this->save_settings();
		}

		if(empty($this->settings)){
			$this->settings=array();
		}

		echo '<form method="post"><ul class="wplms_appointment_settings">';
		foreach($settings as $setting ){

			if(!isset($this->settings[$setting['name']]) && !empty($setting['default']) ){
				$this->settings[$setting['name']]=$setting['default'];
			}
			
			echo '<li>';
			switch($setting['type']){
				case 'textarea':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp"><textarea name="'.$setting['name'].'" style="width: 50%; height: 240px;border:1px solid #DDD;">'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:'').'</textarea>';
					echo '<span>'.$setting['desc'].'</span></div>';
				break;
				case 'select':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp"><select name="'.$setting['name'].'">';
					foreach($setting['options'] as $option){
						echo '<option value="'.$option['value'].'" '.(isset($this->settings[$setting['name']])?selected($option['value'],$this->settings[$setting['name']]):'').'>'.$option['label'].'</option>';
					}
					echo '</select>';
					echo '<span>'.$setting['desc'].'</span></div>';
				break;
				case 'checkbox':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp"><div class="checkbox"><input type="checkbox" id="'.$setting['name'].'" name="'.$setting['name'].'" value="'.$setting['value'].'" '.(empty($this->settings[$setting['name']])?'':'CHECKED').' /><label for="'.$setting['name'].'">'.$setting['desc'].'</label></div>';
					echo '</div></li>';
				break;
				case 'number':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp"><input type="number" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></div>';
				break;
				case 'text':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp"><input type="text" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></div>';
				break;
				case 'editor':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp">';
						wp_editor($this->settings[$setting['name']],$setting['name'],array('editor_class'=>'post_field','editor_height'=>200));
					
					echo '<script>jQuery(document).ready(function($){
						tinyMCE.execCommand("mceRemoveEditor", true, "'.$field['id'].'");
                        tinyMCE.execCommand("mceAddEditor", false, "'.$field['id'].'"); 
                        quicktags({id : "element_content"});
                        tinyMCE.triggerSave();});
                    </script>';
					echo '<span>'.$setting['desc'].'</span></div>';
				break;

				case 'price':

					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp">';
					if(!empty($this->settings[$setting['name']]) && $this->settings[$setting['name']] != $setting['default'] && function_exists('wc_get_product')){
						
						$product = wc_get_product($this->settings[$setting['name']]);
						$price = get_post_meta($product->id, '_price', true);

						echo '<strong style="background:#fafafa;padding:5px;margin-right:20px;"><span class="woocommerce-Price-amount amount"><span class="woocommerce-Price-currencySymbol">'.get_woocommerce_currency().'</span>'.(is_object($product)?$price:'').'</span></strong>';


						//echo '<strong style="background:#fafafa;padding:5px;margin-right:20px;">'.(is_object($product)?$price:'').'</strong>';
					}

					$price = get_post_meta($product->id, '_price', true);

					echo '<input type="hidden" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />

						'.get_woocommerce_currency().' <input type="number" name="'.$setting['name'].'_price" value="'.(is_object($product)?$price:'').'" /> ';

						
					echo '<span>'.$setting['desc'].'</span></div>';
					//print_r($price);

					
				break;
				case 'availability':
					if(count($settings) > 1){
						echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
						echo '<div class="forminp"><span>'.$setting['desc'].'</span>';
					}else{
						echo '<style>.availability{grid-column:1/3;}</style>';
					}
					
					echo '<div class="availability">';

					echo '<div id="wplms_appointments_calendar_settings"></div>';

					/*echo "<input type='hidden' class='wplms_appointments_calendar_settings_schedule' id='wplms_appointments_calendar_settings_schedule' name='".$setting['name']."' 
					value='' />";*/

					
					if(isset($this->settings[$setting['name']])){

					}else{
						if(isset($_POST['save_wplms_settings_form'])){

						?>
						<script>
							//localStorage.removeItem('wplms_appointment_availability_schedule');

						</script>
						<?php
						}
					}
					
					echo '</div></div>';
				break;

				case 'media':
					echo '<div class="settings_label"><label>'.$setting['label'].'</label></div>';
					echo '<div class="forminp">';
					echo '<div class="add_url">';
					echo '<div class="input-group add_url_text_wrapper">';
					echo '<input type="text" id="'.$setting['id'].'" class="post_field add_url_text form-control" placeholder="" value="'.$setting['value'].'">
					  <span class="input-group-addon"><i class="upload_video_button fa fa-upload" data-id="'. $setting['id'].'" data-title="'.$setting['upload_title'].'"  data-upload-button="'.$setting['upload_button'].'" data-uploader-allow-multiple="'.$setting['upload_multiple'].'"></i></span>
					</div>';
					echo '</div>';
					?>
					<script>
							var media_uploader_<?php echo $setting['id']; ?>;
							jQuery(".upload_video_button[data-id='<?php echo $setting['id']; ?>']").on("click", function( event ){
								var button = jQuery( this );
								if ( media_uploader_<?php echo $setting['id']; ?> ) {
									media_uploader_<?php echo $setting['id']; ?>.open();
									return;
								}


								// Create the media uploader.
								media_uploader_<?php echo $setting['id']; ?> = wp.media.frames.media_uploader = wp.media({
								title: button.data( "title" ),
								// Tell the modal to show only videos.
								library: {
								type: "video",
								query: false
								},
								button: {
									text: button.data( "upload_button" ),
								},
									multiple: button.data( "upload_multiple" )

								});


								media_uploader_<?php echo $setting['id']; ?>.on( "select", function() {
									var selection = media_uploader_<?php echo $setting['id']; ?>.state().get('selection');
						            selection.map( function( attachment ) {
							            attachment = attachment.toJSON();
							            //console.log(attachment);
							            //console.log('---------->');
							            var url_video='';
							            if(attachment &&  attachment.url !== undefined ){
							               url_video=attachment.url;
							                //console.log(url_video);
							            }else{
							            	alert('<?php echo _x("Unable to find url of selected video","","wplms-appointments");?>');
							            }
							            jQuery('#<?php echo $setting['id']; ?>').val(url_video);
							            //console.log(url_video);
							            //console.log('<------->');

									});
								 });

							
					    });

						</script>
						<?php

					echo '</div>';
					break;

			}
			echo '</li>';
		}	

		echo '</ul>';
			?>
			
			<?php
			if(count($settings) > 1){
				wp_nonce_field('wplms_appointments_settings','security');
			echo '<ul class="wplms_appointment_settings"><li><span></span><span><input type="submit" value="'.__('Save settings','wplms-appointments').'" name="save_wplms_settings_form"></span></li></ul>';
			}
			echo '</form>';
	}

	function save_settings(){

		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'wplms_appointments_settings') || !is_user_logged_in()){
           echo '<div class="message">'.__('Security check Failed. Contact Administrator.','wplms-appointments').'</div>';
         	return;
      	}
      	$user_id = bp_displayed_user_id();

      	

      	if(empty($this->settings)){
      		$this->get_user_appointment_settings($user_id);
      	}
      	if(empty($this->settings) || !is_array($this->settings)){$this->settings = array();}

		$settings = $this->fetch_settings();
      	foreach($settings as $setting){
      		if(isset($_POST[$setting['name']]) && !in_array($setting['type'],array('availability')) && $setting['name'] != 'enable_appointments' && strpos($setting['name'], 'instructor_searchable_field_') === false){

      			$this->settings[ $setting['name'] ] = stripcslashes($_POST[$setting['name']]);

      		}else if($setting['name'] == 'enable_appointments'){
      			
      			if(isset($_POST[$setting['name']]) && $_POST[$setting['name']] == 1){
      				
      				$this->settings[ $setting['name'] ] = $_POST[$setting['name']];
      			}else{
      				$this->settings[ $setting['name'] ] = 0;
      			}
      			update_user_meta($user_id,'enable_appointments',$this->settings[$setting['name']]);

      		}else if(strpos($setting['name'], 'instructor_searchable_field_') !== false){
      			
      			$this->settings[ $setting['name'] ] = $_POST[$setting['name']];
      			update_user_meta($user_id,$setting['name'],$this->settings[$setting['name']]);

      		}else if($setting['type'] != 'availability'){
      			unset($this->settings[ $setting['name'] ]);
      		}


			if($setting['type'] == 'media'){
				//print_r($setting['id']);
				if(isset($_POST[$setting['name']])){
      				$this->settings[ $setting['name'] ] = $_POST[$setting['name']];
      			}else{
      				$this->settings[ $setting['name'] ] = 0;
      			}
      			update_user_meta($user_id,'appointments_video',$this->settings[$setting['name']]);
			}

      		if($setting['type'] == 'price'){
      			

      			if($_POST[$setting['name'].'_price'] > 0){

	      			$product_id = $_POST[$setting['name']];
	      			$product_price = $_POST[$setting['name'].'_price'];


	      			if(empty($product_id) && !empty($product_price)){

	      				$product_id = wp_insert_post(apply_filters('wplms_appointments_instructor_product',array(
	      					'post_title'=> sprintf(_x('Appointment slot price for %s','instructor product args','wplms-appointments'),bp_core_get_user_displayname( bp_loggedin_user_id() )),
	      					'post_type'=>'product',
	      					'post_status'=>'publish',
	      					'post_content'=>_x('Appointment slot price for Instructor','instructor product args','wplms-appointments'),
	      					'author_id'=>$user_id
	      				)));
	      				if(is_numeric($product_id)){
	      					update_post_meta($product_id,'_regular_price',$product_price);
	      					update_post_meta($product_id,'_price',$product_price);
		      				wp_set_object_terms($product_id, 'simple', 'product_type');
				            update_post_meta($product_id,'_visibility','hidden');
				            update_post_meta($product_id,'_virtual','yes');
				            update_post_meta($product_id,'_downloadable','yes');
				            update_post_meta($product_id,'_sold_individually','no');	
				            update_post_meta($product_id,'vibe_wplms_appointment_instructor_booking_slot_price',$user_id);
				            $this->settings[ $setting['name'] ] = $product_id;
	      				}
	      			}else{
	      				update_post_meta($product_id,'_regular_price',$product_price);
	      				update_post_meta($product_id,'_price',$product_price);
	      			}
	      		}else{
	      			if(!empty($_POST[$setting['name']]) && $_POST[$setting['name'].'_price'] == 0){
	      				$author_id = get_post_field('post_author',$_POST[$setting['name']]);
	      				if($author_id == $user_id || current_user_can('manage_options')){
	      					wp_delete_post($_POST[$setting['name']],true);
	      					unset($this->settings[ $setting['name'] ]);
	      				}
	      				
	      			}
	      		}

      		}

  		}

  		update_user_meta($user_id,'appointment_settings',$this->settings);
  		
      	echo '<div class="message">'.__('Settings Saved.','wplms-appointments').'</div>';
      	return;
      	
	}

	function filter($element){

	}
	function get_user_appointment_settings($user_id){

		if(!empty($this->settings))
			return $this->settings;

		$settings = get_user_meta($user_id,'appointment_settings',true);	
		if(empty($settings)){
			$settings = array();
		}
		$settings_fields = $this->fetch_settings();
      	foreach($settings_fields as $setting){
      		if(strpos($setting['name'], 'instructor_searchable_field_') !== false){
      			$settings[$setting['name']] = get_user_meta($user_id,$setting['name'],true);

      		}
      	}
   

		$this->settings = $settings;

		return $settings;
	}

	function appointment_profile_colors($customizer){

		
		if(bp_current_component() === 'appointments'){
			?>.appointment_setting_tabs .active{color:<?php echo $customizer['primary_bg'];?>;border-color:<?php echo $customizer['primary_bg'];?> ;}
			<?php
		}
		
	}


	function recordings(){
		add_action('bp_template_title',function(){_ex('My Recordings','appointments profile screen','wplms-appointments'); });
		add_action( 'bp_template_content', array($this,'show_recordings'));
    	bp_core_load_template( 'members/single/plugins');
	}

	function show_recordings(){
		$user_id = bp_displayed_user_id();
		$meetings = get_user_meta($user_id,'bbb_meetings',true);
		if(empty($meetings)){
			echo '<div class="message">'.__('No recordings','wplms-appointments').'</div>';
		}else{

			foreach($meetings as $meetingID=>$meeting){
				echo '<div class="meeting">'.$meetingObject['meetingID'].' - '.date_i18n( get_option( 'date_format' ), strtotime( $meeting['createTime'])).' <a class="view_meeting link" data-meeting="'.$meetingID.'">'.__('View','wplms-appointments').'</a>';
			}
		}
	}
}

WPLMS_Appointments_Profile::init();
