<?php
/**
 * Settings in Admin
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	wplms_appointments/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WPLMS_Appointments_Settings{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Settings();
        return self::$instance;
    }

	private function __construct(){
		add_filter('wplms_lms_settings_tabs',array($this,'setting_tab'));
		add_filter('lms_settings_tab',array($this,'tab'));

		add_action('admin_enqueue_scripts',array($this,'admin_scripts'));
		add_action('wp_ajax_create_test_meeting',array($this,'create_test_meeting'));
	}

	function setting_tab($tabs){
		
		$tabs['appointments'] = __('Appointments','video-vibe');
		return $tabs;
	}

	function tab($name){
		if($name == 'appointments')
			return 'WPLMS_Appointments_Settings';
		
		return $name;
	}

	function get_settings(){
 			
			$locale = get_locale();
        $locale=str_replace('_','-',$locale);

		return apply_filters('wplms_appointments_settings',array(

			array(
				'label' => __('Enable Appointment Bookings for Users','wplms-appointments'),
				'name' => 'enable_appointments',
				'type' => 'select',
				'options'=>Array(
					''=>__('No','wplms-appointments'),
					'read'=>__('All Site Users','wplms-appointments'),
					'edit_posts'=>__('Instructors and Administrators only','wplms-appointments'),
				),
				'desc' => __('Enable appointment bookings for following members','wplms-appointments'),
			),
			
			array(
				'label' => __('Date Format','wplms-appointments'),
				'name' => 'appointments_date_format',
				'type' => 'select',
				'options'=>array(
					'MM-DD-YYYY'=>'MM-DD-YYYY',
					'DD-MM-YYYY'=>'DD-MM-YYYY',
					'MM-DD-YY'=>'MM-DD-YY',
					'DD-MM-YY'=>'DD-MM-YY',
					'MM-DD-YYYY'=>'MM-DD-YYYY',
					'DD-MMM-YY'=>'DD-MMM-YY',
					'DD-MMMM-YYYY'=>'DD-MMMM-YYYY',
					'YYYY-MM-DD'=>'YYYY-MM-DD',
				),
				'default'=>'MM-DD-YYYY',
				'desc' => __('Date format','wplms-appointments'),
			),
			array(
				'label' => __('Language','wplms-appointments'),
				'name' => 'appointments_locale',
				'type' => 'text',
				'default'=>$locale,
				'desc' => __('Set moment locale','wplms-appointments').'<a href="https://cdnjs.com/libraries/moment.js/" target="_blank">?</a>',
			),
			array(
				'label' => __('Time Format','wplms-appointments'),
				'name' => 'appointments_time_format',
				'type' => 'select',
				'options'=>array(
					'HH:mm'=>'HH:mm',
					'hh:mm a'=>'hh:mm a',
					'HH:mm Z'=>'HH:mm Z ( Timezone )',
				),
				'default'=>'HH:mm',
				'desc' => __('Time format','wplms-appointments'),
			),

			array(
				'label' => __('Appointments Minimum Slot Time','wplms-appointments'),
				'name' => 'appointments_slot_time',
				'type' => 'text',
				'default'=>'30',
				'desc' => __('Set minimum appoint slot time in minutes','wplms-appointments'),
			),
			array(
				'label' => __('Maximum appointment booking time to complete purchase of slot/schedules','wplms-appointments'),
				'name' => 'appointments_booking_timer',
				'type' => 'number',
				'default'=>'15',
				'desc' => __('Once the slot has been added to cart, it should be completed in these many minutes','wplms-appointments'),
			),
			array(
				'label' => __('Payment processing Time for Slots','wplms-appointments'),
				'name' => 'appointments_booking_payment_processing_timer',
				'type' => 'number',
				'default'=>'1440',
				'desc' => __('When payments do not finish in realtime. Once the slot has been purchased and the payment did not go through in realtime, block the slot for these many minutes. After which the order should be marked as cancelled or refunded.','wplms-appointments'),
			),
			array(
				'label' => __('Cancellation / Reschedule Buffer time for Request and approval','wplms-appointments'),
				'name' => 'appointments_buffer_time',
				'type' => 'number',
				'default'=>0,
				'desc' => __('Set time (in minutes) before appointment start the booker can request cancellation ( 0 to disable ).','wplms-appointments'),
			),
			array(
				'label' => __('Send out reminders [emails/notifications] for Appointments X minutes before the start','wplms-appointments'),
				'name' => 'appointment_reminder_time',
				'type' => 'number',
				'default'=> 30,
				'desc' => __('Send out reminders X minutes to students and instructors before the appointment starts. 0 to Disable.','wplms-appointments'),
			),
			array(
				'label' => __('Show book button in Course featured blocks','wplms-appointments'),
				'name' => 'appointments_featured_blocks',
				'type' => 'checkbox',
				'default'=>0,
				'desc' => __('Book Instructor Button will be visible on single course page, in course directory, in featured blocks of the courses','wplms-appointments'),
			),
			array(
				'label' => __('Show book tab in Course','wplms-appointments'),
				'name' => 'appointments_in_course',
				'type' => 'checkbox',
				'default'=>0,
				'desc' => __('Book tab will be visible in course menu on single course page','wplms-appointments'),
			),
			array(
				'label' => __('Show book button in Instructor names','wplms-appointments'),
				'name' => 'appointments_in_profile',
				'type' => 'checkbox',
				'default'=>0,
				'desc' => __('A book button appear below instructor name in instructor profile','wplms-appointments'),
			),
			array(
				'label' => __('Allow instructors to refund/cancel orders','wplms-appointments'),
				'name' => 'appointments_order_instructor_approve',
				'type' => 'checkbox',
				'default'=>0,
				'desc' => __('Instructors can cancel or refund orders from their appointments manage option.','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Slot available color','wplms-appointments'),
				'name' => 'appointments_slot_available_color',
				'type' => 'color',
				'default'=>'#28a745',
				'desc' => __('Available slot type color','wplms-appointments'),
			),
			
			array(
				'label' => __('Appointments Slots for Schedule Booking only color','wplms-appointments'),
				'name' => 'appointments_slot_schedule_booking_color',
				'type' => 'color',
				'default'=>'#0a9e8a',
				'desc' => __('Complete Schedule booking only color','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Slots Free slots color','wplms-appointments'),
				'name' => 'appointments_slot_free_slots_color',
				'type' => 'color',
				'default'=>'#3aba58',
				'desc' => __('Free Slots color','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Slots Reserved Slots color','wplms-appointments'),
				'name' => 'appointments_slot_reserved_color',
				'type' => 'color',
				'default'=>'#265daa',
				'desc' => __('Reserved Slots color','wplms-appointments'),
			),

			array(
				'label' => __('Appointments Slot booked color','wplms-appointments'),
				'name' => 'appointments_slot_booked_color',
				'type' => 'color',
				'default'=>'#007bff',
				'desc' => __('Booked Slots color','wplms-appointments'),
			),
			array(
				'label' => __('Meeting Creation/Reminder/Notifications buffer time','wplms-appointments'),
				'name' => 'meeting_buffer_time',
				'type' => 'text',
				'default'=> '30',
				'desc' => __('Meeting will be created x minutes before the start of actual event. The reminders and emails are sent to users x minutes prior to the event.','wplms-appointments'),
			),
			array(
				'label' => __('Enable Booking Seats per appointmment schedule','wplms-appointments'),
				'name' => 'appointments_schedule_seats',
				'type' => 'checkbox',
				'default'=>'0',
				'desc' => __('Instructors can set number of Seats per schedule','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Instructor fields (for directory search)','wplms-appointments'),
				'name' => 'appointments_instructor_fields',
				'type' => 'instructor_field',
				'default'=>array(
					array(
						'type'=>'select',
						'key'=>'gender',
						'label'=> __('Gender','wplms-appointments'),
						'options'=>array(
							array('value'=>'male','label'=>__('Male','wplms-appointments')),
							array('value'=>'female','label'=>__('Female','wplms-appointments'))
						),
					),
				),
				'desc' => __('Set Directory search','wplms-appointments'),
			),
			array(
				'label' => __('Enable Instructor promotional video','wplms-appointments'),
				'name' => 'appointments_instructor_video',
				'type' => 'checkbox',
				'default'=>'0',
				'desc' => __('Instructors can create and upload a promotional video describing their teaching','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Directory Page','wplms-appointments'),
				'name' => 'appointments_directory',
				'type' => 'page',
				'desc' => __('Set a appointments directory page.','wplms-appointments'),
			),
			array(
				'label' => __('Instructor Labels','wplms-appointments'),
				'name' => 'appointments_instructor_labels',
				'type' => 'repeatable',
				'multiple'=>array('key','value','color'),
				'desc' => __('Set instructor labels like Featured instructor etc.','wplms-appointments'),
			),
			array(
				'label' => __('Appointments Directory Search','wplms-appointments'),
				'name' => 'appointments_directory_search',
				'type' => 'directory_search',
				'default'=>array(
					array(
						'type'=>'course',
						'label'=> __('Search Course','wplms-appointments'),
					),
					array(
						'type'=>'availability',
						'label'=> __('Select Times','wplms-appointments'),
					),
				),
				'desc' => __('Set Directory search','wplms-appointments'),
			),
/*
			array(
				'label' => __('Directory Instructor Block Style','wplms-appointments'),
				'name' => 'directory_instructor_block',
				'type' => 'selectimg',
				'options'=>array(
					''=>array(
						'label'=>__('Default','wplms-appointments'),
						'value'=>1,
						'image'=>plugins_url('../assets/images/default.png',__FILE__)
					),
					'style1'=>array(
						'label'=>__('Style 1','wplms-appointments'),
						'value'=>2,
						'image'=>plugins_url('../assets/images/style1.png',__FILE__)
					),
				),
				'desc' => __('Enable appointment bookings for following members','wplms-appointments'),
			),
*/
			array(
				'label' => __('Live Webinars [ BigBlueButton Server ]','wplms-appointments'),
				'name' => 'bigbluebutton_server',
				'type' => 'text',
				'default'=> 'https://bbb.videoencrypt.com/b/',
				'desc' => __('Set BigBlueButton server','wplms-appointments').'<a href="https://videoencrypt.com/pricing" target="_blank">VideoEcnrypt</a>',
			),
			array(
				'label' => __('Server Secret [BigBlueButton sercret or VideoEncrypt email]','wplms-appointments'),
				'name' => 'bigbluebutton_secret',
				'type' => 'text',
				'default'=> 'https://bbb.videoencrypt.com/b/',
				'desc' => __('Set BigBlueButton server secret or videoencrypt email id','wplms-appointments').'    <a href="#" class="button" id="test_connection">Test Connection</a>',
			),
			array(
				'label' => __('Default Presentation','wplms-appointments'),
				'name' => 'bigbluebutton_default_presentation',
				'type' => 'file',
				'default'=> '',
				'desc' => __('Set a default BigBlueButton PDF presentation, less than 2 mb in size','wplms-appointments'),
			),
			/*	
			array(
				'label' => __('Enable Recordings','wplms-appointments'),
				'name' => 'appointments_recordings',
				'type' => 'checkbox',
				'default'=>0,
				'desc' => __('Recordings for appointments','wplms-appointments'),
			),*/
		));	
	}


	function settings(){
		
		$settings = $this->get_settings();
		

		$template_array = apply_filters('wplms_lms_appointments_tabs',array(
			'general'=> __('General Settings','wplms-appointments'),
			'calendar'=> __('Admin Calendar','wplms-appointments'),
			//'stats'=> __('Statistics','wplms-appointments'),
			'help'=> __('Help','wplms-appointments'),
			));
		echo '<ul class="subsubsub">';
		foreach($template_array as $k=>$value){
			if(empty($_GET['sub']) && empty($current)){
				$current = $k;
			}else if(!empty($_GET['sub']) && empty($current)){
				$current = $_GET['sub'];
			}
			echo '<li><a href="?page=lms-settings&tab=appointments&sub='.$k.'" '.(($k == $current)?'class="current"':'').'>'.$value.'</a>  &#124; </li>';
		}
		echo '</ul><hr class="clear"/>';
		if(!isset($_GET['sub'])){$_GET['sub']='';}

		switch($_GET['sub']){
			case 'calendar':
				$this->get_calendar();
			break;
			// case 'stats':
			// 	$this->statistics();
			// break;
			case 'help':
				$this->help();
			break;
			default:

				echo '<form method="post">';
				wp_nonce_field('wplms_appointments_Settings');   
				echo '<table class="form-table">
						<tbody>';

						 $this->settings=get_option(WPLMS_APPOINTMENTS_OPTION);
				

				$this->generate_form($settings);

				?>
				<script>
					var $ = jQuery;
					jQuery(document).ready(function($){
						$('#test_connection').on('click',function(event){
							event.preventDefault();
							if($(this).hasClass('disabled'))
								return;
							$(this).text('.......');
							$(this).addClass('disabled');
							var $this = $(this);
							$.ajax({
								type: "POST",
						        url: ajaxurl,
						        data: { 
						        	action: 'create_test_meeting',
				                  	security:'<?php echo wp_create_nonce('security'); ?>',
				                },
					         	cache: false,
					          	success: function (html) {
						            $this.removeClass('disabled');
						            $this.after(html);
						            $this.remove();
					          	}
							})
						});
					});
				</script>
				<?php
				echo '<tr valign="top"><th colspan="2"><input type="submit" name="save_wplms_appointments_settings" class="button button-primary" value="'.__('Save Settings','video-vibe').'" /> </th>';
				echo '</tbody></table></form>';			
			break;
		}	
	}

	function admin_scripts($hook){

		if($hook != 'lms_page_lms-settings' || empty($_GET['tab']) || (!empty($_GET['tab']) && $_GET['tab'] != 'appointments' ))
			return;


		if(!empty($_GET['sub']) && $_GET['sub'] == 'calendar'){
			
			wp_enqueue_style('wplms_appointments_admin_settings',plugins_url('../assets/css/appointments_admin_settings.css',__FILE__),array(),rand(0,1234));
			wp_enqueue_style('wplms_appointments_big_calendar',plugins_url('../assets/css/react-big-calendar.css',__FILE__));
			//wp_enqueue_script('wplms_appointments_admin_calendar','http://localhost:3001/static/js/bundle.js',array(),1,true);
			wp_enqueue_script('wplms_appointments_admin_calendar',plugins_url('../assets/js/admin-calendar.js',__FILE__),array(),1,true);


			$init = WPLMS_Appointments_Init::init();
			$init->get_global_settings();  
			
			$appointment_types = apply_filters('wplms_appointments_schedule_types',array(
	            'available'=>__('Available','wplms-appointments'),
	            'free_slots'=>__('Free slots','wplms-appointments'),
	            'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
	            'reserved'=>__('Reserved','wplms-appointments'),
	            'booked'=>__('Booked','wplms-appointments'),
	        ));

			$translation = apply_filters('wplms_appointments_admin_calendar',array(
				'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
				'type'=>'mybookings',
				'types'=>$appointment_types,
				'settings'=>$init->settings,
				'default_member_image'=>vibe_get_option('default_avatar'),
				'dateformat'=>apply_filters('wplms_appotintments_date_format','MMM,DD YYYY'),//get_option('date_format'),
				'timeformat'=>apply_filters('wplms_appotintments_time_format','hh:mm a'),//
				'security'=>vibe_get_option('security_key'),
				//

				'user'=>array(
					'id'=>bp_displayed_user_id(),
					'name'  => bp_core_get_user_displayname(bp_displayed_user_id()),
	        		'image' => bp_core_fetch_avatar ( array( 'item_id' => bp_displayed_user_id(), 'type' => 'full','html'=>false )),
					'messages'=>array(
						'unread'=>wplms_appointments_get_unread_count(bp_displayed_user_id()),
						'url'=>((function_exists('bp_is_active') && bp_is_active('messages'))?trailingslashit( bp_loggedin_user_domain() . bp_get_messages_slug() ):0)
					),
				),
				'translations'=>array(
					'all'=>__('All','wplms-appointments'),
					'add_availability_schedule'=>__('Add Availability Schedule','wplms-appointments'),
					'set_availability_period'=>__('Set Availability period','wplms-appointments'),
					'set_availability_time'=>__('Set Avaialibity Time','wplms-appointments'),
					'available'=>__('Available','wplms-appointments'),
					'start_date'=>__('Start Date','wplms-appointments'),
					'end_date'=>__('End Date','wplms-appointments'),
					'start_time'=>__('Start Time','wplms-appointments'),
					'end_time'=>__('end Time','wplms-appointments'),
					'show_active_month_events'=>__('Show active month','wplms-appointments'),
					'hide_past_events'=>__('Hide past events','wplms-appointments'),
					'today'=>__('Today','wplms-appointments'),
					'by'=>__('by','wplms-appointments'),
					'pending'=>__('Pending','wplms-appointments'),
					'confirmed'=>__('Confirmed','wplms-appointments'),
					'agenda'=>__('Agenda','wplms-appointments'),
					'day'=>__('Day','wplms-appointments'),
					'week'=>__('Week','wplms-appointments'),
					'month'=>__('Month','wplms-appointments'),
					'more_details'=>__('More Details','wplms-appointments'),
					'open'=>__('Open','wplms-appointments'),
					'closed'=>__('Closed','wplms-appointments'),
					'confirmed'=>__('Confirmed','wplms-appointments'),
					'showBookings'=>__('Show Bookings','wplms-appointments'),
					'status'=>__('Status','wplms-appointments'),
					'type'=>__('Type','wplms-appointments'),
					'course'=>__('Course','wplms-appointments'),
					'booker'=>__('Booker','wplms-appointments'),
					'instructor'=>__('Instructor','wplms-appointments'),
					'more_details'=>__('More Details','wplms-appointments'),
					'details'=>__('Details','wplms-appointments'),
					'messages'=>__('Messages','wplms-appointments'),
					'activity'=>__('Activity Log','wplms-appointments'),
					'connected_bookings'=>__('Connected Bookings','wplms-appointments'),
					'reschedule_appointment'=>__('Reschedule Appointment','wplms-appointments'),
					'cancel_appointment'=>__('Cancel Appointment','wplms-appointments'),
					'cancel_meeting'=>__('Cancel Meeting','wplms-appointments'),
					'cancel_all_meeting'=>__('Cancel All Meeting','wplms-appointments'),
					'message_instructor'=>__('Message instructor','wplms-appointments'),
					'message_user'=>__('Message User','wplms-appointments'),
					'book_another_slot'=>__('Book another slot','wplms-appointments'),
					'nomessage'=>__('No messages found.','wplms-appointments'),
					'noactivity'=>__('No activities found.','wplms-appointments'),
					'reply'=>__('Reply','wplms-appointments'),
					'cancel_appointment_reason'=>__('Reason for Cancellation','wplms-appointments'),
					'reschedule_appointment_reason'=>__('Reason for Reschedule','wplms-appointments'),
					'load_more'=>__('Load more','wplms-appointments'),
					'send'=>__('Send Message','wplms-appointments'),
					'sending'=>__('Sending','wplms-appointments'),
					'cancelled'=>__('Cancelled','wplms-appointments'),
					'reschdule'=>__('Rescheduled','wplms-appointments'),
					'close'=>__('Close','wplms-appointments'),
					'requested_cancellation'=>__('Cancellation Requested','wplms-appointments'),
					'requested_reschedule'=>__('Reschedule Requested','wplms-appointments'),
					'more'=>__('More','wplms-appointments'),
					'message_all_members'=>__('Message all members','wplms-appointments'),
					'admin'=>__('Admin','wplms-appointments'),
					'select_member'=>__('Select Member','wplms-appointments'),
					'cancel_confirm'=>__('Confirm appointment cancellation.','wplms-appointments'),
					'same_schedule'=>__('Same schedule.','wplms-appointments'),
					'mark_appointment_attended'=>__('Mark Appointment Attended','wplms-appointments'),
					'mark_appointment_attended_confirm'=>__('Appointment Attended','wplms-appointments'),
				

					'days'=>__('days','wplms-appointments'),
					'hours'=>__('hours','wplms-appointments'),
					'minutes'=>__('minutes','wplms-appointments'),
					'seconds'=>__('seconds','wplms-appointments'),

					'accept'=>__('Accept','wplms-appointments'),
					'reject'=>__('Reject','wplms-appointments'),
					'details'=>__('Details','wplms-appointments'),
					'timesup'=>__('Time Out','wplms-appointments'),

					'form_data'=>__('Form data','wplms-appointments'),
					'hide_form_data'=>__('Hide data','wplms-appointments'),
					'form_attachment'=>__('Form attachment','wplms-appointments'),
					'remove'=>__('Remove','wplms-appointments'),
					'showSchedules'=>__('Show Schedules','wplms-appointments'),
					'search_instructor'=>__('Search Instructors','wplms-appointments'),
					'search_course'=>__('Search Courses','wplms-appointments'),
					'search_booker'=>__('Search Bookers','wplms-appointments'),
					'showAvailability'=>__('Show Availability','wplms-appointments'),
				)
				
			));
			wp_localize_script('wplms_appointments_admin_calendar','wplms_appointments_settings',$translation);
		}
/*
		if(!empty($_GET['sub']) && $_GET['sub'] == 'stats'){

			wp_enqueue_style('wplms_appointments_admin_statistics_setting',plugins_url('../assets/css/appointment_admin_statistics.css',__FILE__),array(),rand(0,1234));

			wp_enqueue_script('wplms_appointments_admin_statistics','http://localhost:3000/static/js/bundle.js',array(),1,true);
			 wp_enqueue_script('wplms_appointments_admin_statistics_2','http://localhost:3000/static/js/0.chunk.js',array(),10,true);
			 wp_enqueue_script('wplms_appointments_admin_statistics_3','http://localhost:3000/static/js/main.chunk.js',array(),10,true);

			//wp_enqueue_script('wplms_appointments_admin_statistics',plugins_url('../assets/js/admin-statistics.js',__FILE__),array(),1,true);

			$init = WPLMS_Appointments_Init::init();
			$init->get_global_settings();  
			

			$translation = apply_filters('wplms_appointments_admin_statistics',array(
				'api_url'=>home_url().'/wp-json/wplms/v1/appointments',
				'default_member_image'=>vibe_get_option('default_avatar'),
				'dateformat'=>apply_filters('wplms_appotintments_date_format','MMM,DD YYYY'),//get_option('date_format'),
				'timeformat'=>apply_filters('wplms_appotintments_time_format','hh:mm a'),//
				'security'=>vibe_get_option('security_key'),
				//

				'user'=>array(
					'id'=>get_current_user_id(),
					'name'  => bp_core_get_user_displayname(get_current_user_id()),
	        		'image' => bp_core_fetch_avatar ( array( 'item_id' => get_current_user_id(), 'type' => 'full','html'=>false )),
					'messages'=>array(
						'unread'=>wplms_appointments_get_unread_count(get_current_user_id()),
						'url'=>((function_exists('bp_is_active') && bp_is_active('messages'))?trailingslashit( bp_loggedin_user_domain() . bp_get_messages_slug() ):0)
					),
				),
				'translations'=>array(
					'bookers_slot_title'=>__('Bookers Slot Distribution','wplms-appointments'),
				)
				
			));

			wp_localize_script('wplms_appointments_admin_statistics','wplms_appointments_statistics',$translation);


		}
		*/

		
	}
	function get_calendar(){
		echo '<div class="wplms_appointments_admin_calendar">
				<div id="wplms_appointments_admin_calendar"></div>
			</div>';
	}

	function statistics(){
		echo '<div class="wplms_appointments_admin_statistics">
				<div id="wplms_appointments_admin_statistics"></div>
			</div>';

	}

	function generate_form($settings){
		
		foreach($settings as $setting ){

			if(!isset($this->settings[$setting['name']]) && !empty($setting['default']) ){
				if(is_array($setting['default'])){$setting['default']=json_encode($setting['default'],true);}
				$this->settings[$setting['name']]=$setting['default'];
			}
			
			echo '<tr valign="top">';
			switch($setting['type']){
				case 'textarea':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><textarea name="'.$setting['name'].'" style="width: 50%; height: 240px;border:1px solid #DDD;">'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:'').'</textarea>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'select':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><select name="'.$setting['name'].'">';
					foreach($setting['options'] as $key=>$option){
						echo '<option value="'.$key.'" '.(isset($this->settings[$setting['name']])?selected($key,$this->settings[$setting['name']]):'').'>'.$option.'</option>';
					}
					echo '</select>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'checkbox':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="checkbox" name="'.$setting['name'].'" '.(isset($this->settings[$setting['name']])?'CHECKED':'').' />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'number':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="number" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'text':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="text" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'color':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="text" class="color_picker" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" style="background:'.$this->settings[$setting['name']].'"/>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'file':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>
					';
					?>
					<script>
						var $ = jQuery;
	                	var media_uploader2;
	                	jQuery(document).ready(function(){
						jQuery('.upload_pdf_button').on('click', function( event ){
						  	console.log('#1');
						    var button = jQuery( this );
						    if ( media_uploader2 ) {
						      media_uploader2.open();
						      return;
						    }
						    // Create the media uploader.
						    media_uploader2 = wp.media.frames.media_uploader = wp.media({
						        title: button.data( 'uploader-title' ),
						        // Tell the modal to show only images.
						        library: {
						            type: 'application',
						            query: false
						        },
						        button: {
						            text: button.data( 'uploader-button-text' ),
						        },
						        multiple: false
						    });

						    // Create a callback when the uploader is called
						    media_uploader2.on( 'select', function() {
						        var selection = media_uploader2.state().get('selection');
						            
						            selection.map( function( attachment ) {
						            attachment = attachment.toJSON();
						            var pdf_url='';
						            if(attachment &&  attachment.url !== undefined ){
						               pdf_url=attachment.url;
						            }else{
						            	alert('<?php echo _x("Unable to find url of selected PDF","","wplms-appointments");?>');
						            }
						            button.parent().find('.presentation_url').val(pdf_url);
						            button.parent().find('.presentation_name').html('<a href="'+pdf_url+'" class="dashicons dashicons-visibility" target="_blank"></a>');
						         });

						    });
						    // Open the uploader
						    media_uploader2.open();
						  });
					});
	                </script>
					<?php
					echo '<td class="forminp">
							<a class="upload_pdf_button button" data-uploader-button-text="'.__('Upload PDF','wplms-appointments').'" data-input-name="'.$setting['name'].'" >'.__('Upload Default Presentation','wplms-appointments').'</a>
							<input type="hidden" class="presentation_url" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" /><span class="presentation_name">'.(isset($this->settings[$setting['name']])?'<a href="'.$this->settings[$setting['name']].'" class="dashicons dashicons-visibility" target="_blank"></a>':'').'</span>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'selectimg':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><div class="selectimg_options">';
					foreach($setting['options'] as $option){

						if(empty($this->settings[$setting['name']])){
							$this->settings[$setting['name']] = $option['value'];
						}
						echo '<div class="selectimg_option"><input type="radio" id="select_img'.$option['value'].'" name="'.$setting['name'].'" value="'.$option['value'].'" '.(($this->settings[$setting['name']] == $option['value'])?'checked':'').' /><label for="select_img'.$option['value'].'"><img src="'.$option['image'].'" /></label></div>';
					}
					
					echo '</div><span>'.$setting['desc'].'</span></td><style>.selectimg_options { display: grid; grid-template-columns: repeat(auto-fit,minmax(120px,1fr)); grid-gap: 15px; width: 100%; } .selectimg_option img { width: 100%; } .selectimg_option input { display: none; }.selectimg_option input[type="radio"]:checked+label img { box-shadow: 0 5px 20px rgba(0,0,0,0.4); border-radius: 5px;} </style>';
				break;
				/*
				case 'media':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					//echo '<td class="forminp"><input type="image" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<td class="forminp">';
					echo '<div class="add_url">';
					echo '<div class="input-group add_url_text_wrapper">';
					echo '<input type="text" id="'.$setting['id'].'" class="post_field add_url_text form-control" placeholder="" value="'.(isset($this->settings[$setting['name']])?stripslashes($this->settings[$setting['name']]):json_encode($setting['std'])).'"/>
					  <span class="input-group-addon"><i class="upload_image_button fa fa-upload" data-id="'. $setting['id'].'" data-title="'.$setting['upload_image_title'].'"  data-upload-button="'.$setting['upload_image_button'].'" data-uploader-allow-multiple="'.$setting['upload_image_multiple'].'"></i></span>';
					echo '</div>';
					?>
					<script>
							var media_uploader_<?php echo $setting['id']; ?>;
							jQuery(".upload_image_button[data-id='<?php echo $setting['id']; ?>']").on("click", function( event ){
								var button = jQuery( this );
								if ( media_uploader_<?php echo $setting['id']; ?> ) {
									media_uploader_<?php echo $setting['id']; ?>.open();
									return;
								}


								// Create the media uploader.
								media_uploader_<?php echo $setting['id']; ?> = wp.media.frames.media_uploader = wp.media({
								title: button.data( "title" ),
								// Tell the modal to show only videos.
								library: {
								type: "image",
								query: false
								},
								button: {
									text: button.data( "upload_image_button" ),
								},
									multiple: button.data( "upload_image_multiple" )

								});


								media_uploader_<?php echo $setting['id']; ?>.on( "select", function() {
									var selection = media_uploader_<?php echo $setting['id']; ?>.state().get('selection');
						            selection.map( function( attachment ) {
							            attachment = attachment.toJSON();
							            console.log(attachment);
							           
							            if(attachment &&  attachment.url !== undefined ){
							            	$('#<?php echo $setting['id']; ?>').val(attachment.url);
							            	//$('#<?php echo $setting['id']; ?>').val(JSON.stringify(attachment.url));
							               //console.log(JSON.stringify(attachment.url));
							            }else{
							            	alert('<?php echo _x("Unable to find url of selected image","","wplms-appointments");?>');
							            }
							            //$('#<?php echo $setting['id']; ?>').val(url_image);
							             //console.log('########2');
									});
								 });

							
					    });

						</script>
						<style>
							.input-group-addon {
							padding: 6px 12px;
							font-size: 14px;
							font-weight: 400;
							line-height: 1;
							color: #555;
							text-align: center;
							background-color: #eee;
							border: 1px solid #ccc;
							border-radius: 4px;
							}

							.fa-upload:before {
							    content: "\f093";
							    font-family: fontawesome;
							}
						</style>
						<?php

					echo '</div>';
					
					//echo "<input type='hidden'  name='".$setting['name']."' id='".$setting['name']."' value='".(isset($this->settings[$setting['name']])?stripslashes($this->settings[$setting['name']]):json_encode($setting['std']))."' />";

					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				*/
				case 'repeatable':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp">';

					echo '<input type="hidden" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';


					echo '<a class="wa_meta_box_repeatable_add button button-primary button-large" href="#">'.__('Add More','wplms-appointments').'</a>
									<ul id="' . $field['id'] . '-repeatable" class="wa_meta_box_repeatable">';
	
							if ( !empty($this->settings[$setting['name']]) && is_Array($this->settings[$setting['name']])) {
								foreach( $this->settings[$setting['name']][$setting['multiple'][0]] as $k=>$key ) {
									echo '<li><span class="sort handle dashicons dashicons-sort"></span>';
									foreach($setting['multiple'] as $v){
										echo '<input type="text" name="' . $setting['name'] . '['.$v.'][]" id="' . $setting['name'] . '_'.$v.'" value="' . esc_attr( $this->settings[$setting['name']][$v][$k] ) . '" size="30" />';
									}
									echo '<a class="wa_meta_box_repeatable_remove" href="#"><span class="dashicons dashicons-no"></span></a></li>';
								}
							} 
								echo '<li class="hide"><span class="sort handle dashicons dashicons-sort"></span>';

								foreach($setting['multiple'] as $v){
									echo '<input type="text" rel-name="' . $setting['name'] . '['.$v.'][]" id="' . $setting['name'] . '_'.$v.'" placeholder="'.$v.'" value="" size="30" />';
								}
								echo '<a class="wa_meta_box_repeatable_remove" href="#"><span class="dashicons dashicons-no"></span></a></li>';
							
							echo '</ul>
								<span class="description">' . $field['desc'] . '</span>';

					?>


					<script>
						var $ = jQuery;
						jQuery(document).ready(function($){
							jQuery('.wa_meta_box_repeatable_add').on('click', function(event) {
						        event.preventDefault();
								// clone
						        var repeatable = jQuery(this).siblings('.wa_meta_box_repeatable');
								var row = repeatable.find('li.hide');
						        var lastrow = repeatable.find('li:last-child');
								var clone = row.clone();
						        clone.removeClass('hide');
						        var inputname='';
								clone.find('input').each(function(){
									$(this).val('');
									inputname=$(this).attr('rel-name');
        							$(this).attr('name',inputname);
        						});

        						lastrow.after(clone);
        					});

        					jQuery('.wa_meta_box_repeatable_remove').live('click', function(){
		
						        var repeatable=jQuery(this).closest('.wa_meta_box_repeatable');
						        jQuery(this).closest('li').remove();
						        
								return false;
							});
						        
							jQuery('.wa_meta_box_repeatable').sortable({
								opacity: 0.6,
								revert: true,
								cursor: 'move',
								handle: '.handle',
						        update: function( event, ui ) {
						            
						        }
							});

						});
					</script><style>.hide{display:none;}.wa_meta_box_repeatable_remove{color:red;}</style>
					<?php
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'page':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><select name="'.$setting['name'].'"><option>'.__('Select a page','wplms-appointments').'</option>';

					$posts = new WP_Query(array('post_type'=>'page','posts_per_page'=>-1));
					if($posts->have_posts()){
						while($posts->have_posts()){
							$posts->the_post();
							echo '<option value="'.get_the_ID().'" '.(isset($this->settings[$setting['name']])?selected(get_the_ID(),$this->settings[$setting['name']]):'').'>'.get_the_title().'</option>';
						}
					}
					echo '</select>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'instructor_field':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp">';
					?>
					<a id="add_instructor_block" class="button"><?php _e('Add Searchable Instructor Field','wplms-appointments'); ?></a>
					<ul id="instructor_blocks">
					</ul>
					<script>
						var $ = jQuery;
						jQuery(document).ready(function($){
							var instructor_blocks = [];

							var isValidJSON = true;
							try { JSON.parse($('#<?php echo $setting['name']; ?>').val()) } catch { isValidJSON = false }


							if(isValidJSON){
								instructor_blocks = JSON.parse($('#<?php echo $setting['name']; ?>').val());	
							}
							//console.log(instructor_blocks);

							var set_instructor_field = function(element){
								//console.log('covered');
								
								element.find('.set_field').on('change',function(){
									let i = $(this).val();
									//console.log('new field');
									if(i === 'new' ){
										$(this).after('<div class="newoption"><input type="text" placeholder="<?php _e('Value','wplms-appointments'); ?>" class="field_value"><input type="text" class="field_label " placeholder="<?php _e('Label','wplms-appointments'); ?>"><a class="add_newoption button"><?php _e('Add','wplms-appointments'); ?></a></div>');

										element.find('.add_newoption').on('click',function(){
											let label = $(this).parent().find('.field_label').val();
											let value = $(this).parent().find('.field_value').val();
											let i = $(this).parent().parent().attr('data-i');

											//console.log(instructor_blocks);
											if(!instructor_blocks[i]){
												instructor_blocks[i]=[];
											}
											if(!instructor_blocks[i].hasOwnProperty('options')){
												instructor_blocks[i]['options']=[];
											}else{
												if(!instructor_blocks[i]['options'] || instructor_blocks[i]['options'].length < 0){
													instructor_blocks[i]['options']=[];	
												}
												
											}
											
											instructor_blocks[i]['options'].push({'label':label,'value':value});
											$('#<?php echo $setting['name']; ?>').val(JSON.stringify(instructor_blocks));

											$(this).parent().parent().find('.set_field').append('<option>'+label+'</option>');
											$(this).parent().parent().find('.set_field').val('none');
											$(this).parent().remove();
										});
									}
									
								});


								
							}

							if(instructor_blocks.length){
								instructor_blocks.map(function(item,i){
									if(item && item.hasOwnProperty('label')){


										let block = '<li data-i="'+i+'"><span class="remove"></span><select class="set_field"><option value="none">'+item.label+'</option><option value="new"><?php _e('Add new option','wplms-appointments'); ?></option>';

										 item.options.map(function(it){
										 	if(it.hasOwnProperty('label')){
										 		block +='<option value="'+it.value+'">'+it.label+'</option>';
										 	}
										 	
										 });

										block +='</li>';
										jQuery('#instructor_blocks').append(block);

										set_instructor_field(jQuery('#instructor_blocks>li[data-i="'+i+'"]'));

										if(i === (instructor_blocks.length -1)){
											//console.log('all covered');
	    									$( "#instructor_blocks" ).sortable({
												stop : function(event,ui){ 
													let new_instructor_blocks = [];
													$('#instructor_blocks > li').each(function(index,item){
														let i = parseInt($(this).attr('data-i'));
														new_instructor_blocks.push(instructor_blocks[i]);
														$(this).attr('data-i',index);
													});
													instructor_blocks = new_instructor_blocks;
													$('#<?php echo $setting['name']; ?>').val(JSON.stringify(instructor_blocks));
										        }
											});
											$( "#instructor_blocks" ).disableSelection();
										}
									}
								});

								
							}

							jQuery('#instructor_blocks').on('click','.remove',function(){
								$(this).parent().remove();
								let new_instructor_blocks = [];
								$('#instructor_blocks > li').each(function(index,item){
									let i = parseInt($(this).attr('data-i'));
									new_instructor_blocks.push(instructor_blocks[i]);

									$(this).attr('data-i',index);
								});
								instructor_blocks = new_instructor_blocks;
								$('#<?php echo $setting['name']; ?>').val(JSON.stringify(instructor_blocks));
							});

							jQuery('#add_instructor_block').on('click',function(){


								let appender = '<li data-i="'+instructor_blocks.length+'"><div style="display: grid; padding: 10px; text-align: center; grid-gap: 5px;"><input type="text" class="field_key" placeholder="<?php _e('Set Field key','wplms-appointments'); ?>"><input type="text" class="field_name" placeholder="<?php _e('Set Field label','wplms-appointments'); ?>"><a class="add_field_label button-primary"><?php _e('Add Field','wplms-appointments'); ?></a></div><span class="remove"></span></li>';

								jQuery('#instructor_blocks').append(appender);

								jQuery('.add_field_label').on('click',function(){

									if($(this).parent().find('input.field_name').val().length){
										let new_instructor_field_block = {'type':'select','label':$(this).parent().find('input.field_name').val(),'key':$(this).parent().find('input.field_key').val(),'options':[]};
										instructor_blocks.push(new_instructor_field_block);

										//console.log('blocks');
										$('#<?php echo $setting['name']; ?>').val(JSON.stringify(instructor_blocks));

										$(this).parent().after('<select class="set_field"><option value="none">'+$(this).parent().find('input.field_name').val()+'</option><option value="new"><?php _e('Add new option','wplms-appointments'); ?></option></select>');
										

										set_instructor_field($(this).parent().parent());
										$(this).parent().remove();
									}
									
							});
							
							$( "#instructor_blocks" ).sortable({
								stop : function(event,ui){ 
									let new_instructor_blocks = [];
									$('#instructor_blocks > li').each(function(index,item){
										let i = parseInt($(this).attr('data-i'));
										new_instructor_blocks.push(instructor_blocks[i]);
										$(this).attr('data-i',index);
									});
									instructor_blocks = new_instructor_blocks;
									$('#<?php echo $setting['name']; ?>').val(JSON.stringify(instructor_blocks));
						        }
							});
							$( "#instructor_blocks" ).disableSelection();
						});


						});
					</script>
					<style>
					#instructor_blocks{max-width:320px;}
					#instructor_blocks span.remove{    position: absolute;right: 0;top: 0;}#instructor_blocks span.remove:after {
					    content: "\f158";
					    color: red;
					    font-family: dashicons;
					    font-size: 16px;
					    float: right;
					    padding: 5px;
					}#instructor_blocks .newoption{display:grid;grid-template-columns:1fr 1fr 48px;padding:5px;grid-gap:5px;}#instructor_blocks .newoption input{width:100%;}
					ul#instructor_blocks li:before{
					    content:"\f156";font-family: dashicons;line-height:2;
					    padding:0px 5px;font-size:12px;opacity:0.6;
					}
					#instructor_blocks li{border: 1px solid rgba(0,0,0,0.2);background:#fff;position:relative;}
					</style>
					<?php
					echo	"<input type='hidden'  name='".$setting['name']."' id='".$setting['name']."' value='".(isset($this->settings[$setting['name']])?stripslashes($this->settings[$setting['name']]):json_encode($setting['std']))."' />";
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'directory_search':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp">';
					?>
					<a id="add_search_block" class="button"><?php _e('Add Search block','wplms-appointments'); ?></a>
					<ul id="search_blocks">
					</ul>
					<script>
						var $ = jQuery;

						jQuery(document).ready(function($){
							var search_blocks = [];

							var isValidJSON = true;
							try { JSON.parse($('#<?php echo $setting['name']; ?>').val()) } catch { isValidJSON = false }

							

							if(isValidJSON){
								//console.log($('#<?php echo $setting['name']; ?>').val());
								search_blocks = JSON.parse($('#<?php echo $setting['name']; ?>').val());	
							}
							
							
							var all_possible_search_blocks = <?php 
							
							$filters = array(
								array(
									'type'=>'course',
									'label'=> __('Search Course','wplms-appointments'),
								),
								array(
									'type'=>'availability',
									'label'=> __('Select Times','wplms-appointments'),
								),
								array(
									'type'=>'course-cat',
									'label'=> __('Select Category','wplms-appointments'),
								),
								array(
									'type'=>'location',
									'label'=> __('Select Location','wplms-appointments'),
								),
								array(
									'type'=>'level',
									'label'=> __('Select Levels','wplms-appointments'),
								),
								array(
									'type'=>'text',
									'label'=> __('Select Text','wplms-appointments'),
								),
							);

							
							echo json_encode(apply_filters('wplms_appointments_search_filters',$filters));

							?>;
							<?php
							if(!empty($this->settings['appointments_instructor_fields'])) {
								echo 'var more_search_blocks = '.stripslashes($this->settings['appointments_instructor_fields']).';';
							}
							?>
							more_search_blocks.map(function(item){
							if(item){
								item['type'] = item.key;
								all_possible_search_blocks.push(item);}
							});
							if(search_blocks.length){
								search_blocks.map(function(item,i){
									jQuery('#search_blocks').append('<li data-i="'+i+'">'+item.label+'<span class="remove"></span></li>');
									if(i === (search_blocks.length -1)){
										$( "#search_blocks" ).sortable({
											stop : function(event,ui){ 
												let new_search_blocks = [];
												$('#search_blocks > li').each(function(index,item){
													let i = parseInt($(this).attr('data-i'));
													new_search_blocks.push(search_blocks[i]);
													$(this).attr('data-i',index);
												});
												search_blocks = new_search_blocks;
												$('#<?php echo $setting['name']; ?>').val(JSON.stringify(search_blocks));
									        }
										});
    									$( "#search_blocks" ).disableSelection();
									}
								});
							}
							jQuery('#search_blocks').on('click','.remove',function(){
								$(this).parent().remove();
								let new_search_blocks = [];
								$('#search_blocks > li').each(function(index,item){
									let i = parseInt($(this).attr('data-i'));
									new_search_blocks.push(search_blocks[i]);

									$(this).attr('data-i',index);
								});
								search_blocks = new_search_blocks;
								$('#<?php echo $setting['name']; ?>').val(JSON.stringify(search_blocks));
							});
							jQuery('#add_search_block').on('click',function(){

								let options ='<option><?php _e('Select search option','wplms-appointments'); ?></option>';
								all_possible_search_blocks.map(function(item,i){
									if(search_blocks.findIndex(function(element, index, array){return element.type === item.type; }) < 0){
										options+='<option value="'+i+'">'+item.label+'</option>';
									}
								});

								jQuery('#search_blocks').append('<li data-i="'+search_blocks.length+'"><select class="set_block">'+options+'</select><span class="remove"></span></li>');
								jQuery('.set_block').on('change',function(){
									let i = $(this).val();
									search_blocks.push(all_possible_search_blocks[i]);
									$('#<?php echo $setting['name']; ?>').val(JSON.stringify(search_blocks));
								});
								$( "#search_blocks" ).sortable({
									stop : function(event,ui){ 
										let new_search_blocks = [];
										$('#search_blocks > li').each(function(index,item){
											let i = parseInt($(this).attr('data-i'));
											new_search_blocks.push(search_blocks[i]);
											$(this).attr('data-i',index);
										});
										search_blocks = new_search_blocks;
										$('#<?php echo $setting['name']; ?>').val(JSON.stringify(search_blocks));
							        }
								});
								$( "#search_blocks" ).disableSelection();
							});

						});
					</script>
					<style>
					#search_blocks{max-width:240px;}#search_blocks span.remove:after {
					    content: "\f158";
					    color: red;
					    font-family: dashicons;
					    font-size: 16px;
					    float: right;
					    padding: 5px;
					}
					ul#search_blocks li:before{
					    content:"\f156";font-family: dashicons;line-height:2;
					    padding:0px 5px;font-size:12px;opacity:0.6;
					}
					#search_blocks li{border: 1px solid rgba(0,0,0,0.2);background:#fff;}
					</style>
					<?php
					echo	"<input type='hidden'  name='".$setting['name']."' id='".$setting['name']."' value='".(isset($this->settings[$setting['name']])?stripslashes($this->settings[$setting['name']]):$setting['std'])."' />";
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
			}
		}	
		?>
		<script>
		var $ = jQuery;
		jQuery(document).ready(function(){
			$(".color_picker").each(function(){
			    var $this = $(this);
			    $this.css("background",$this.val());

			    $this.iris({
			        width: 200,
			        hide: true,
			        change: function(event, ui) {
			            $this.css( "background", ui.color.toString());
			        }
			    });
			});

			$(".color_picker").on("click",function(){
			    $(this).parent().find(".iris-picker").toggle(100);
			});
		});
		</script><style>.iris-picker{position:absolute;}</style>
		<?php
	}

	function save(){
		
		if(!isset($_POST['save_wplms_appointments_settings']))
			return;
		
		if ( !isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'],'wplms_appointments_Settings') ){
		     echo '<div class="error notice is-dismissible"><p>'.__('Security check Failed. Contact Administrator !','video-vibe').'</p></div>';
		}

		$settings = $this->get_settings();
		//print_r($settings);
		foreach($settings as $setting){
			if(isset($_POST[$setting['name']])){
				$this->settings[$setting['name']] = $_POST[$setting['name']];
			}else if($setting['type'] == 'checkbox' && isset($this->settings[$setting['name']])){
				unset($this->settings[$setting['name']]);
			}
		}
		
		update_option(WPLMS_APPOINTMENTS_OPTION,$this->settings);
		echo '<div class="updated notice is-dismissible"><p>'.__('Settings Saved.','video-vibe').'</p></div>';
	}
	
	function help(){

		$myFile = plugin_dir_path(__FILE__).'/help.html';
        if (file_exists($myFile)) {
          $fh = fopen($myFile, 'r');
        }
        $contents = fread($fh,filesize($myFile));
        print_r($contents);
        fclose($fh); 
	}


	function create_test_meeting(){


		if(!current_user_can('manage_options') || wp_verify_nonce('security',$_POST['security'])){
			echo '<strong>You do not have permissions</strong>';
			die();
		}

		$bbb_api = WPLMS_Appointments_BBB_API::init();

		$meetingID = wp_generate_password(12,false,false);
		$object = $bbb_api->create_meeting(array(
			'name' => 'Sample',
    		'meetingID'=>$meetingID,//AppointmentID
    		'attendeePW'=> '1234',
    		'moderatorPW'=>'2345',
    		'welcome'=>'Test Meeting',
    		'maxParticipants'=>'2',
    		'record'=>false,
    		'duration'=>'', //(minutes)
    		'meta'=>'',//appointmentID
    		'allowStartStopRecording'=>false,
    		'webcamsOnlyForModerator'=>'',//boolean
    		'logo'=>'',//url
    		'bannerText'=>'',
    		'bannerColor'=>'',
    		'copyright'=>''
    		)
		);

		
		
		$meeting_arr = $bbb_api->object2array($object);



		$userName = 'Sample';//wordpress username 

					
		echo '&nbsp;&nbsp;&nbsp;<a target="_blank" class="button button-primary"data-id="'.$meeting_arr['meetingID'].'" href="'.$bbb_api->getJoinURL( $meetingID, $userName, '2345').'">'._x('Join Sample meeting','','wplms-appointments').'</a>';
		
		die();
		
	}
}

WPLMS_Appointments_Settings::init();

function WPLMS_Appointments_Settings(){
	$settings = WPLMS_Appointments_Settings::init();
	$settings->save(); 
	$settings->settings();	
}