<?php
/**
 * Filters WPLMS CHAT
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	wplms-chat/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}



class WPLMS_Appointments_Filters{


	public static $instance;

    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WPLMS_Appointments_Filters();
        return self::$instance;
    }


    function __construct(){

        $appointments=WPLMS_Appointments_Init::init();
        $appointments->get_global_settings();
        if(!empty($appointments->settings['appointments_in_course'])){

            add_filter('wplms_course_nav_menu',array($this,'course_appointments'));
    		add_filter('wplms_save_vibe_course_permalinks',array($this,'save_permalinks'));
    		add_action('init', array($this,'add_endpoints'));
        	add_filter( 'request', array($this,'filter_request' ));
    		add_action( 'template_redirect', array($this,'catch_vars' ),9);
    	}
        if(!empty($appointments->settings['appointments_featured_blocks'])){	
		  add_action('wplms_instructor_meta',array($this,'show_bookable_instructor'),10,2);
        }

        add_filter('wplms_appointments_cart_timer',array($this,'apply_cart_timer'));
        

		add_filter('wplms_course_creation_tabs',array($this,'add_appointments'));
		add_filter('wplms_front_end_generate_fields_default',array($this,'appointment_slot'),10,2);
        add_filter('wplms_front_end_process_compoenents',array($this,'skip_post_meta_save'),10,2);
        add_action('wplms_front_end_save_course_components',array($this,'save_course_appointments'),10,2);

        add_filter('wplms_course_creation_tabs',array($this,'add_appointment_pricing'));

        //WooCommerce
        add_filter('woocommerce_add_cart_item',array($this,'add_appointment_data'),10,2);
        add_filter('woocommerce_get_cart_item_from_session',array($this,'get_cart_items_from_session'), 10, 3 );
        add_filter('woocommerce_cart_item_name',array($this,'add_cart_timer'),10,2);
        
        //add_action( 'woocommerce_product_query_meta_query', array($this,'hide_appointments_products'), 10, 1 );

        add_filter('pre_get_posts',array($this,'hide_appointments_products'));
        //this will remove the add to cart button for appointments product from single product page
        add_action( 'woocommerce_single_product_summary', array($this,'remove_appointments_product_add_to_cart_button'), 1 );

        //add_filter('wplms_appointment_slot_add_to_cart',array($this,'check_cart_expiry'),10,3);

        add_filter( 'woocommerce_cart_item_quantity', array($this,'disable_quatitiy_change_in_cart'), 10, 3);
       

        add_filter('wplms_appointment_cancel_booking_woocommerce',array($this,'cancel_booking'),10,3);

        //Instructor booking main filter
        add_filter('wplms_appointments_after_data',array($this,'check_instructor_booking'));

        add_filter('woocommerce_cart_item_thumbnail',array($this,'show_instructor_thumbnail'),10,3);

        //function to show courses which is created by that instructor only in edit details popup
        add_filter('wplms_appointments_instructor_courses',array($this,'wplms_appointments_instructor_courses_visibility'));
        
        
    }


    function check_instructor_booking($args){
        if(!empty($args['args']['type']) && $args['args']['type'] == 'instructor_booking'){
            //print_r($args['args']);
        }
        return $args;
    }
    function add_cart_timer($product_name,$cart_item){


        if(!empty($cart_item['expires'])){

            $product_name .= '<strong class="appointment_info">
            ';
            if(!empty($cart_item['slot_details'])){

                foreach($cart_item['slot_details'] as $appointment_id=>$detail){

                    $product_name .= '<span class="convert_appointment_datetime">'.strtotime($detail['start']).'-'.strtotime($detail['end']).'</span><span>['.$appointment_id.']</span>';
                }
            }
            $product_name .='
            <span class="wplms_appointment_cart_timer">'.$cart_item['expires'].'</span>
            </strong>
            ';     
        }
       
        return $product_name;
    }

    function hide_appointments_products( $query ){

        if(is_admin()){
            return;
        }

        if($query->query_vars['post_type'] != 'product')
            return $query;

        if(is_admin())
            return $query;

        $meta_query = $query->get('meta_query');
        if(!empty($meta_query)){
            $meta_query[] = array(
                    'key'     => 'vibe_courses',
                    'compare' => 'LIKE'
                    );
            $args['meta_query']['relation']= 'AND';
        }else{
            $meta_query = array(
                'relation'=>'AND',
                array(
                    'key'     => 'vibe_courses',
                    'compare' => 'LIKE'
                    )
            );
        }
         $query->set('meta_query',$meta_query);

        
         return $query;
    }

    function remove_appointments_product_add_to_cart_button() {
    global $product;
    //print_r($product);
    //echo $product->id;

    $check = get_post_meta($product->id,'vibe_wplms_appointment_instructor_booking_slot_price',true);

    // For simple products type
        if( $product->is_type( 'simple' ) && $product->is_purchasable() && $check) {
            remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30 );
            add_action( 'woocommerce_single_product_summary',  array($this,'show_stock_info'), 30 );
        }
    }

    function show_stock_info() {
        global $product;
        echo wc_get_stock_html( $product );
    }


    function apply_cart_timer($timer){
        $init =WPLMS_Appointments_Init::init();
        $settings = $init->get_global_settings();

        if(!empty($settings) && !empty($settings['appointments_booking_timer'])){
            $timer = $settings['appointments_booking_timer']*60;
        }

        return $timer;
    }


    function show_bookable_instructor($meta,$instructor_id){
       
        $course_id = get_the_ID();
        if(empty($this->permalinks))
            $this->permalinks = get_option( 'vibe_course_permalinks' );

        $appointments_slug = isset($this->permalinks['appointments_slug'])?$this->permalinks['appointments_slug']:'book';

        $link = apply_filters('wplms_course_instructor_booking_link',bp_get_course_permalink().$appointments_slug,$instructor_id,$course_id);
        return $meta.'<a href="'.$link.'" class="button small" style="margin:0;grid-column: 1/3;width: 100%;text-align: center;">'._x('Book instructor','button below instructor in course','wplms-appointments').'</a>';
    
    	
        return $meta;
    }

    function course_appointments($menu){
    	$nav = bp_course_get_nav_permalinks();

        $appointments=WPLMS_Appointments_Init::init();


        if(!empty($appointments->settings['appointments_in_course'])){
        	$menu['appointments']= array(
                'id'    => 'appointments',
                'label' =>__('Book','vibe'),
                'can_view' => 1,
                'action'=> (empty($nav['appointments_slug'])?__('book','vibe'):$nav['appointments_slug']),
                'link'  =>bp_get_course_permalink(),
            );
        }

    	return $menu;
    }


    function save_permalinks($permalinks){
    	if(isset($_POST['appointments_slug'])){
    		$appointments_slug = trim( sanitize_text_field( $_POST['appointments_slug'] ), '/' );
			$appointments_slug = '/' . $appointments_slug;
			$permalinks['appointments_slug'] = untrailingslashit( $appointments_slug );
		}
		return $permalinks;
	}

	function add_endpoints(){
    	if(empty($this->permalinks))
    		$this->permalinks = get_option( 'vibe_course_permalinks' );
        
        $appointments_slug = isset($this->permalinks['appointments_slug'])?$this->permalinks['appointments_slug']:'book';
        $appointments_slug = str_replace('/','',$appointments_slug);
        add_rewrite_endpoint($appointments_slug, EP_ALL);    
    }

    function filter_request( $vars ){

    	if(empty($this->permalinks))
			$this->permalinks = get_option( 'vibe_course_permalinks' );

		$appointments_slug = isset($this->permalinks['appointments_slug'])?$this->permalinks['appointments_slug']:'book';
		$appointments_slug = str_replace('/','',$appointments_slug);
		if(isset( $vars[$appointments_slug])){
			$vars[$appointments_slug] = true;	
		}
	    return $vars;
	}


	function catch_vars(){ 
		global $bp,$wp_query;	
		
		if(empty($this->permalinks))
			$this->permalinks = get_option( 'vibe_course_permalinks' );


		if($bp->unfiltered_uri[0] == trim($this->permalinks['course_base'],'/') || $bp->unfiltered_uri[0] == BP_COURSE_SLUG){
				
        		$appointments_slug = (!empty($this->permalinks['appointments_slug'])?$this->permalinks['appointments_slug']:'book');
				$appointments_slug = str_replace('/','',$appointments_slug);
				
			    if( get_query_var( $appointments_slug )){ 
			    	$bp->current_component = BP_COURSE_SLUG;
			    	$bp->current_item = get_The_ID();
			        $bp->current_action = 'appointments';

			        add_action('bp_course_plugin_template_content',array($this,'course_appointments_page'));
					bp_get_template_part('course/single/plugins');
					exit;
			    }
		}
	}


	function course_appointments_page(){

        echo '<div class="wplms_instructor_course_appointments">';
        echo apply_filters('the_content',get_post_meta(get_the_ID(),'vibe_wplms_appointment_booking_message',true));
        echo '<div id="wplms_instructor_course_appointments"></div>
            </div>';
        do_action('wplms_appointments_booking_form','course',get_the_ID());

        $init = WPLMS_Appointments_Init::init();

        $color = get_post_meta($post->ID,'vibe_wplms_appointment_booking_color',true);
        $approval = get_post_meta($post->ID,'vibe_wplms_appointment_booking_approval',true);

        $course_settings = $this->get_course_appointment_settings();
        $instructor = apply_filters('wplms_course_instructors',get_the_author_id(),get_the_ID());
        if(is_Array($instructor)){
            $instructors = $instructor;
        }else{
            $instructors = array($instructor);
        }

        $profile = WPLMS_Appointments_Profile::init();
        

        $settings = array();
        foreach($instructors as $instructor){

            $settings[$instructor]=apply_filters('wplms_appointment_instructor_settings',$profile->get_user_appointment_settings($instructor),$instructor);
        }

        $args = array(
            'course_id'=>get_the_ID(),
            'instructors'=>$instructors,
            'type'=>'course_booking',
            'settings'=>$settings,
            'approval'=> ($approval === 'H')?0:1
        );
        if(!empty($color)){
            $args['color']= $color;
        }
        $args = apply_filters('wplms_appointments_course_book_args',$args);
        

        $init->load_scripts($args);
	}

	
    function get_course_appointment_settings(){

        $settings = get_user_meta(get_current_user_id(),'appointment_settings',true);
        if(empty($settings) || empty($settings['appointments_approval_wait_time'])){
            $wait_time = 30;
        }else{
            $wait_time = $settings['appointments_approval_wait_time'];
        }
        
        return apply_filters('wplms_appointments_course_options',
            array(
                        array(
                            'label'=> __('Appointments / Bookings','wplms-appointments' ),
                            'text'=>__('Enable Course Bookings by Students','wplms-appointments' ),
                            'type'=> 'conditionalswitch',
                            'hide_nodes'=> array('vibe_wplms_appointment_booking_criteria','vibe_wplms_appointment_booking_approval','vibe_wplms_appointment_booking_message','vibe_wplms_appointment_booking_form','vibe_wplms_appointment_booking_slots','vibe_wplms_appointment_booking_color'),
                            'options'  => array('H'=>__('DISABLE','wplms-appointments' ),'S'=>__('ENABLE','wplms-appointments' )),
                            'style'=>'',
                            'id' => 'vibe_wplms_appointments',
                            'from'=> 'meta',
                            'default'=>'H',
                            'desc'=> __('Enable Appointments/Bookings for Course.','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Who can book course','wplms-appointments' ),
                            'text'=>__('Pre-requisites for Appointment or Booking','wplms-appointments' ),
                            'type'=> 'select',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_criteria',
                            'from'=> 'meta',
                            'default'=>'all',
                            'options'  => 
                                array(array('label'=>__('All Loggedin Users','wplms-appointments' ),'value'=>'all'),array('label'=>__('All course Members','wplms-appointments' ),'value'=>'course_members')),
                            'desc'=> __('Who can book course','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Booking approval','wplms-appointments' ),
                            'text'=>__('Appointment booking approval','wplms-appointments' ),
                            'type'=> 'select',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_approval',
                            'from'=> 'meta',
                            'default'=>'all',
                            'options'  => 
                                array(array('label'=>__('Automatic','wplms-appointments' ),'value'=>''),array('label'=>sprintf(__('Manually Approve ( %s minutes )','wplms-appointments' ),$wait_time),'value'=>'manual')),
                            'desc'=> __('Who can book course','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Appointment Color','wplms-appointments' ),
                            'text'=>__('Appointment color for easy identification','wplms-appointments' ),
                            'type'=> 'color',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_color',
                            'from'=> 'meta',
                            'default'=>'#418b3d',
                            'desc'=> __('Who can book course','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Appointment/Booking Message & Form','wplms-appointments' ),
                            'text'=>__('Add a Booking message and Form','wplms-appointments' ),
                            'type'=> 'editor',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_form',
                            'from'=> 'meta',
                            'default'=>'',
                            'desc'=> __('Booking Message and Form','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Appointment Slots','wplms-appointments' ),
                            'text'=>__('Select appointment slots for this course','wplms-appointments' ),
                            'type'=> 'appointment_slots',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_slots',
                            'from'=> 'meta',
                            'default'=>'',
                            'desc'=> __('Booking Message and Form','wplms-appointments' ),
                            ),
                    ));
    }
	function add_appointments($fields){

        $appointments = $this->get_course_appointment_settings();
        foreach($fields['course_components']['fields'] as $key => $setting){
            $settings[] = $setting;
            if($key == 1){
                foreach($appointments as $appointment){
                    $settings[] = $appointment;
                }
            }
        }
        $fields['course_components']['fields'] = $settings;
        return $fields;
    
	}

	function appointment_slot($field,$course_id=null){

		if($field['type'] == 'appointment_slots'){
            $appointments_db = new WPLMS_APPOINTMENTS_DB;


            if(empty($this->permalinks)){
                $this->permalinks = get_option( 'vibe_course_permalinks' );
            }


            $appointments_slug = (!empty($this->permalinks['appointments_slug'])?$this->permalinks['appointments_slug']:'book');

            $course_slots = array();
            $not_in = array();
            if($course_id){
                $course_slots = $appointments_db->get_appointments(apply_filters('wplms_appointments_course_slots',array(
                    'author_id'=> (current_user_can('manage_options')?0:get_current_user_id()),
                    'status'=>'open',
                    'start_date'=>time()*1000,
                    'parent_appointment_id'=> 0,
                    'item_id'=>$course_id,
                    'sort'=>'on_and_upcoming'
                )));
            }

            $slots = $appointments_db->get_appointments(apply_filters('wplms_appointments_open_slots',array(
                'author_id'=>get_current_user_id(),
                'type'=> array('available','free_slots','schedule_booking'),
                'status'=>'open',
                'item_id'=>'none',
                'parent_appointment_id'=> 0,
                'sort'=>'on_and_upcoming',
                'start_date'=>(time()-86400)*1000,
                'orderby'=>'start_date',
                'order'=>'ASC'
            )));

            
            $appointment_types = apply_filters('wplms_appointments_schedule_types',array(
                'available'=>__('Available','wplms-appointments'),
                'free_slots'=>__('Free slots','wplms-appointments'),
                'schedule_booking'=>__('Schedule Booking','wplms-appointments'),
                'reserved'=>__('Reserved','wplms-appointments'),
                'booked'=>__('Booked','wplms-appointments'),
            ));
            
            $available=array();
            
			echo '<div class="field_wrapper '.(isset($field['style'])?$field['style']:'').'">';
			echo '<label>'. $field['label'].(empty($field['desc'])?'':'<a class="tip" title="'.$field['desc'].'"><i class="icon-question"></i></a>').'</label>
            <script>let timezone = new Date().getTimezoneOffset()*60000;</script>
            <div class="set_course_appointment_slots">';

            $locale = get_locale();
            $locale=str_replace('_','-',$locale);
            $course_seats_array = array();
            ?>
            <script>
                let appointment_types = JSON.parse('<?php echo json_encode($appointment_types); ?>');
                var options = { weekday: 'long', year: 'numeric', month: 'long', day: 'numeric' };
            </script>
            <?php
            if(!empty($course_slots)){
                foreach($course_slots as $slot){
                    echo '<div class="course_slot slot_id_'.$slot['appointment_id'].'">';
                    $course_seats_array[$slot['appointment_id']]=wplms_appointments_get_max_seats(array('schedule_id'=>$slot['appointment_id'],'instructor_id'=>get_current_user_id()));


                    echo '<input type="hidden" class="post_field" data-id="appointment_slot_id" value="'.$slot['appointment_id'].'" /></div>';
                }

                ?>
                <script>
                        let schedule = JSON.parse('<?php echo stripslashes((string)json_encode($course_slots)); ?>');

                        let course_slots_seats = JSON.parse('<?php echo json_encode($course_seats_array); ?>');

                        schedule.map(function(item,index){
                           
                            
                            let start_date = new Date(parseInt(item.start_date) - timezone);
                            //capture time
                            let time_mapper = new Date();
                            let y = new Date(time_mapper.setHours(Math.floor(parseInt(item['start_time'])/60),Math.floor(parseInt(item['start_time'])%60)) - timezone);
                            start_date.setHours(y.getHours(),y.getMinutes());
                            //end capture
                            let start = start_date;



                            let end_date = new Date(parseInt(item['end_date']) - timezone);
                            //capture time
                            time_mapper = new Date();
                            y = new Date(time_mapper.setHours(Math.floor(parseInt(item['end_time'])/60),Math.floor(parseInt(item['end_time'])%60)) - timezone);
                            end_date.setHours(y.getHours(),y.getMinutes());
                            //end capture
                            let end =end_date;


                             jQuery('.slot_id_'+item.appointment_id).append('<div class="slot_info">\
                                <div class="slot_details" style="grid-column:1/3">\
                                    <strong><?php _e('Type','wplms-appointments'); ?> : '+appointment_types[item.type]+'</strong>\
                                    <strong><?php _e('Seats','wplms-appointments'); ?> : '+course_slots_seats[item.appointment_id]+'</strong>\
                                    \
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-calendar tip" data-tip="<?php _e('Start Date','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(start).toLocaleDateString('<?php echo $locale; ?>',options)+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-calendar tip" data-tip="<?php _e('End Date','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(end).toLocaleDateString('<?php echo $locale; ?>',options)+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-clock-o tip" data-tip="<?php _e('Start Time','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(start).toLocaleTimeString('<?php echo $locale; ?>')+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-clock-o tip" data-tip="<?php _e('End Time','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(end).toLocaleTimeString('<?php echo $locale; ?>')+'">\
                                </div>\
                                <span class="remove_course_slot"></span>\
                            </div>');

                        });

                        jQuery('body').on('click','.remove_course_slot',function(){
                            let stop=false;
                            schedule.map(function(item,i){
                                let end = new Date(parseInt(item.end_date)).setHours(Math.floor(item.end_time/60),Math.floor(item.end_time%60),0);
                                // console.log(end);
                                // console.log('#########1');
                                // console.log(new Date().getTime());
                                //console.log(stop);
                                if(end < new Date().getTime()){
                                    stop=true;
                                }

                            });
                            if(stop){
                                return;
                            }
                            var parent = jQuery(this).parent().parent();
                            parent.find('.slot_info').attr('data-slot_id',parent.find('.post_field').val());
                            var cloned = parent.find('.slot_info').clone();
                            parent.remove();
                            jQuery('.new_appointment_slots').append(cloned);
                        });
                    </script>
                <?php
            }

            echo '<span class="button add_appointment_slots">'.__('Add Appointment Slots','wplms-appointments').'</span>
            <div class="new_appointment_slots">
            </div>';

            $seats_array = array();
            if(!empty($slots)){
                foreach($slots as $slot){
                    $seats_array[$slot['appointment_id']]=wplms_appointments_get_max_seats(array('schedule_id'=>$slot['appointment_id'],'instructor_id'=>get_current_user_id()));
                }    
            }
            
			echo (empty($field['extras'])?'':$field['extras']);
            ?><script>
                let openschedule = JSON.parse('<?php echo stripslashes((string)json_encode($slots)); ?>');
                
                let slots_seats = JSON.parse('<?php echo json_encode($seats_array); ?>');

                var options = { weekday: 'long', year: 'numeric', month: 'long', day: 'numeric' };
                jQuery('.add_appointment_slots').on('click',function(){
                    var $ = jQuery;
                    $(this).after('<span><?php echo sprintf(__('Click to select Aavailable Appointment slots. %s Add more slots %s ','wplms-appointments'),'<a href="'.bp_core_get_user_domain(get_current_user_id()).$appointments_slug.'" class="button" target="_blank">','</a>'); ?></span>');
                    $(this).remove();

                    if(!openschedule){
                        jQuery('.new_appointment_slots').append('<div class="message"><?php _e('No open slot available ! Click to add new slots','wplms-appointments'); ?></div>');

                        return;
                    }


                    openschedule.map(function(item,index){

                            let start = new Date(parseInt(item.start_date) -timezone).setHours(Math.floor(parseInt(item.start_time)/60),Math.floor(parseInt(item.start_time)%60))-timezone;
                            let end = new Date(parseInt(item.end_date)-timezone).setHours(Math.floor(item.end_time/60),Math.floor(item.end_time%60),0)-timezone;


                             jQuery('.new_appointment_slots').append('<div class="slot_info" data-slot_id="'+item.appointment_id+'">\
                                <div class="slot_details" style="grid-column:1/3">\
                                    <strong><?php _e('Type','wplms-appointments'); ?> : '+appointment_types[item.type]+'</strong>\
                                    <strong><?php _e('Seats','wplms-appointments'); ?> : '+slots_seats[item.appointment_id]+'</strong>\
                                    \
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-calendar tip" data-tip="<?php _e('Start Date','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(start).toLocaleDateString('<?php echo $locale; ?>',options)+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-calendar tip" data-tip="<?php _e('End Date','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(end).toLocaleDateString('<?php echo $locale; ?>',options)+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-clock-o tip" data-tip="<?php _e('Start Time','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(start).toLocaleTimeString('<?php echo $locale; ?>')+'">\
                                </div>\
                                <div class="input-group">\
                                    <div class="input-group-addon"><i class="fa fa-clock-o tip" data-tip="<?php _e('End Time','wplms-appointments'); ?>"></i></div>\
                                    <input type="text" readOnly class="form-control" id="exampleInputAmount" value="'+new Date(end).toLocaleTimeString('<?php echo $locale; ?>')+'">\
                                </div>\
                                \
                            </div>');

                        });

                    jQuery('.new_appointment_slots .slot_info').on('click',function(){
                        jQuery(this).toggleClass('active');
                        jQuery('.new_appointment_slot_field').remove();
                        jQuery('.new_appointment_slots .slot_info.active').each(function(){
                            jQuery('.set_course_appointment_slots').append('<input type="hidden" class="post_field new_appointment_slot_field" data-id="appointment_slot_id" value="'+$(this).attr('data-slot_id')+'" />');
                        });
                    });
                }); 
            </script>
            <style>
                .slot_details{
                    grid-column: 1/3;
                    display: grid;
                    grid-template-columns: repeat(auto-fill,minmax(120px,1fr));
                    grid-gap: 1em;
                }
                .slot_info {
                    background: #fff;
                    position:relative;
                    padding: 15px;
                    border-radius: 15px;
                    margin-bottom: 15px;
                    display: grid;
                    grid-template-columns: 1fr 1fr;
                    grid-gap: 15px;
                    box-shadow: 0 2px 5px rgba(0,0,0,0.2);
                }
                .slot_info.active{
                    box-shadow: 0 2px 5px #70c989;
                    border: 1px solid #70c989;
                }
                .remove_course_slot{
                    position: absolute;
                    top: 0;
                    right: 0;
                }
                .remove_course_slot:before {
                    content: "\f00d";
                    font-family: fontawesome;
                    position: absolute;
                    top: -10px;
                    right: -10px;
                    background: red;
                    padding: 1px 5px;
                    border-radius: 50%;
                    color: #fff;
                }
                </style>
                <?php
            
			echo   '</div></div>';	
		}
	}

    function skip_post_meta_save($return,$setting){
        if($setting->id == 'appointment_slot_id'){
            return 0;
        }

        return $return;
    }

    function save_course_appointments($course_id,$settings){
        if(!empty($settings)){
            $appointments_db = new WPLMS_APPOINTMENTS_DB;
            
            $enabled= false;
            $existing_slots=[];
            foreach($settings as $setting){
                if($setting->id == 'vibe_wplms_appointments' && $setting->value == 'S'){
                    $enabled = true;
                }
                if($setting->id == 'appointment_slot_id'){
                    $existing_slots[]=$setting->value;
                }
            }
            foreach($settings as $setting){
                if($setting->id == 'appointment_slot_id'){
                    $appointments_db->update_appointment(array(
                        'appointment_id'=>(int)$setting->value,
                        'item_id'=>$course_id
                    ));
                    $appointments = $appointments_db->get_appointments(array('parent_appointment_id'=>(int)$setting->value));
                    if(!empty($appointments)){
                        foreach($appointments as $appointment){
                            $appointments_db->update_appointment(array(
                                'appointment_id'=>$appointment['appointment_id'],
                                'item_id'=>$course_id
                            ));
                        }
                    }

                }
            }

            if($enabled){
                $course_slots = $appointments_db->get_appointments(array(
                    'author_id'=> (current_user_can('manage_options')?0:get_current_user_id()),
                    'status'=>'open',
                    'start_date'=>time()*1000,
                    'parent_appointment_id'=> 0,
                    'item_id'=>$course_id,
                    'sort'=>'on_and_upcoming'
                ));

                if(!empty($course_slots)){
                    foreach($course_slots as $id){
                        if(!in_array($id['appointment_id'],$existing_slots)){
                            $appointments_db->update_appointment(array(
                                'appointment_id'=>$id['appointment_id'],
                                'item_id'=>0
                            ));
                            $appointments = $appointments_db->get_appointments(array('parent_appointment_id'=>$id['appointment_id']));
                            if(!empty($appointments)){
                                
                                foreach($appointments as $appointment){
                                    $appointments_db->update_appointment(array(
                                        'appointment_id'=>$appointment['appointment_id'],
                                        'item_id'=>0
                                    ));
                                }
                            }
                        }

                    }

                }
            }
        }
    }

    function add_appointment_pricing($fields){

         $appointments =    apply_filters('wplms_appointments_course_pricing_options',
            array(
                        array(
                            'label'=> __('Appointments / Bookings type','wplms-appointments' ),
                            'text'=>__('Enable pricing Appointments/Bookings','wplms-appointments' ),
                            'type'=> 'conditionalswitch',
                            'options'  => array('H'=>__('FREE','wplms-appointments' ),'S'=>__('PAID','wplms-appointments' )),
                            'hide_nodes'=> array('vibe_wplms_appointment_booking_slot_price','vibe_wplms_appointment_booking_schedule_price'),
                            'style'=>'',
                            'id' => 'vibe_wplms_appointments_pricing',
                            'from'=> 'meta',
                            'default'=>(empty($settings['chat_course_status'])?'H':'S' ),
                            'desc'=> __('Enable Appointments/Bookings for Course.','wplms-appointments' ),
                            ),

                        array(
                            'label'=> __('Price per Slot','wplms-appointments' ),
                            'text'=>__('Set booking price per slot','wplms-appointments' ),
                            'type'=> 'appointment_hidden_product',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_slot_price',
                            'from'=> 'meta',
                            'default'=>'0',
                            'desc'=> __('Price per Booking Slot','wplms-appointments' ),
                            ),
                        array(
                            'label'=> __('Price per Schedule','wplms-appointments' ),
                            'text'=>__('Set booking price per schedule, 0 to disable','wplms-appointments' ),
                            'type'=> 'appointment_hidden_product',
                            'style'=>'" style="padding:10px 0;',
                            'id' => 'vibe_wplms_appointment_booking_schedule_price',
                            'from'=> 'meta',
                            'default'=>'0',
                            'desc'=> __('Price per Booking Slot','wplms-appointments' ),
                            ),
                    ));

        array_splice($fields['course_pricing']['fields'],(count($fields['course_pricing']['fields'])-1),0,$appointments);
        
        return $fields;
    }



    function add_appointment_data($cart_item_data,$cart_item_key){

        // Add Gift Course Data in cart
        if ( isset($_REQUEST['appointment_id'])){

            $appointment_id = $_REQUEST['appointment_id'];
            $course_id = $_REQUEST['course_id'];

            //Check if someone already added the appointment in cart
            $non_bookable = apply_filters('wplms_appointment_course_non_bookable',0);

            if(empty($appointment_id)){
                wc_add_notice(__('The appointment slot is already added in cart by some user. Please wait','wplms-appointments'),'error');

                //Remove product from cart item data
                $this->key = $cart_item_key;
                add_action( 'woocommerce_after_calculate_totals',array($this,'remove_product_from_cart'));
            } 

            if($non_bookable){

                wc_add_notice(__('The appointment slot is already added in cart by some user. Please wait','wplms-appointments'),'error');
                echo '<style>.woocommerce .woocommerce-message {display:none;}</style>';

                //Remove product from cart item data
                $this->key = $cart_item_key;
                add_action( 'woocommerce_after_calculate_totals',array($this,'remove_product_from_cart'));
                
                return $cart_item_data;
            }

            //optional get appointment details and check if appointment is of forward time 
            // Check if 30 minutes remain in appotinment booking


            // Check if approval is required for appointment

           

            //Add gift data in item data
            $new_value = array(
                'appointment_id' => $appointment_id
            );

            if(!empty($course_id)){
                $new_value['course_id'] = $course_id;
                $new_value['course_title'] = get_the_title($course_id);
            }
            return array_merge($cart_item_data,$new_value);
        }

        return $cart_item_data;
    }

    function remove_product_from_cart(){
        WC()->cart->remove_cart_item( $this->key );
    }

    function get_cart_items_from_session($item,$values,$key){

        if (array_key_exists( 'appointment_id', $values ) ){
            $item['appointment_id'] = $values['appointment_id'];
            $item['instructor'] = $values['instructor'];
            $item['slot_info'] = $values['slot_info'];
        }
        if (array_key_exists( 'course_id', $values ) ){
            $item['course_id'] = $values['course_id'];
        }

        return $item;
    }


    function disable_quatitiy_change_in_cart( $product_quantity, $cart_item_key, $cart_item ) {
        $product_id = $cart_item['product_id'];

        $check = get_post_meta($product_id,'vibe_wplms_appointment_booking_slot_price',true);
        if(empty($check)){
            $check = get_post_meta($product_id,'vibe_wplms_appointment_booking_schedule_price',true);
        }
        if(empty($check)){
            $check = get_post_meta($product_id,'vibe_wplms_appointment_instructor_booking_slot_price',true);
        }
        
        if ( !empty($check) ) {
            return '<strong>' . $item['quantity'] . '</strong>';
        }

        return $product_quantity;
    }
    

    function cancel_booking($message,$order_details,$args){
        
        $order = new WC_Order( $order_details['order_id'] );
        $items = $order->get_items();
        $user_id=$order->get_user_id();
        
        if($args['booker_id'] != $user_id && !current_user_can('manage_options')){
            return __('User mismatch ! Order user not same as being refunded.','wplms-appointments');
        }

        $found = false;
        $refund_amount = 0;
        foreach($items as $item_id=>$item){

            //$appointment_ids = $item['_appointment_ids'];

            $appointment_ids = $order->get_item_meta($item_id, '_appointment_ids', true);
            
            if(!empty($appointment_ids)){

                //if(in_array($args['appointment_id'],$appointment_ids)){
                    
                    $found = true;
                    //
                    $item_total = $order->get_item_total( $item );
                    $refund_amount = round($item_total/count($appointment_ids),2);


                    $item_meta    = $order->get_item_meta( $item_id );
                    $product_data = wc_get_product( $item_meta["_product_id"][0] );
                    $item_ids[] = $item_id;
                    $tax_data = $item_meta['_line_tax_data'];
                    $refund_tax = 0;
                    if( is_array( $tax_data[0] ) ) {
                        $refund_tax = array_map( 'wc_format_decimal', $tax_data[0] );
                    }

                    $refund_amount = wc_format_decimal( $refund_amount ) + wc_format_decimal( $item_meta['_line_total'][0] );

                    $refund_amount = round($item_total/count($appointment_ids),2);

                    $line_items[ $item_id ] = array( 'qty' => $item_meta['_qty'][0], 'refund_total' => wc_format_decimal( $item_meta['_line_total'][0] ), 'refund_tax' =>  $refund_tax );

                    $refund = wc_create_refund( array(
                        'amount'         => $refund_amount,
                        'reason'         => $args['reason'],
                        'order_id'       => $order_details['order_id'],
                        'line_items'     => $line_items,
                        'refund_payment' => true
                    ) );


                    //update appointment set booker ID
                    $appointments_db = new WPLMS_APPOINTMENTS_DB;
                    $appointments_meta_db = new WPLMS_APPOINTMENTS_META_DB;

                    $appointment = $appointments_db->get_appointments(array('appointment_id'=>$args['appointment_id']));

                    if(empty($appointment[0]['parent_appointment_id'])){
                        //Schedule
                        $slots=$appointments_db->get_appointments(array('parent_appointment_id'=>$args['appointment_id']));
                    }else{
                        if($appointment[0]['booker_id'] == $args['booker_id']){
                            $appointments_db->update_appointment(array('appointment_id'=>$args['appointment_id'],'booker_id'=>0));
                        }else{
                            $booker_ids = $appointments_meta_db->get_appointments_meta(array('appointment_id'=>$args['appointment_id'],'meta_key'=>'booker_id'));
                            $key = array_search($args['booker_id'], array_column($booker_ids, 'meta_value'));
                            if( $key == false){
                                return __('Booker missing in Appointment bookers','wplms-appointments');
                            }else{
                                $appointments_meta_db->remove_appointments_meta(array('meta_id'=>$booker_ids[$key]['meta_id']));
                            }
                        }
                    }


                //}

            }
        }
        
        if(!$found){
            return __('Appointment ID missing in order','wplms-appointments');
        }
        return $message;
    }


    function show_instructor_thumbnail($url, $cart_item, $cart_item_key ){

        if(!empty($cart_item['appointment_ids']) && !empty($cart_item['instructor'])){
            return '<div class="instructor_in_cart"><img src="'.$cart_item['instructor']['image'].'" />'.'<span>'.$cart_item['instructor']['name'].'</span></div>';
        }

        return $url;
    }


    function wplms_appointments_instructor_courses_visibility($args){
        if(function_exists('get_coauthors')){
            $user = get_userdata($args['post_author']);
            unset($args['post_author']);
            $args['author_name'] = $user->user_nicename;
        }
        return $args;
    }
}

WPLMS_Appointments_Filters::init();