import React, {Component} from 'react';
import DateTimestamp from './date';
import DatePicker from "react-datepicker";
import UserSelector from "./select";

import "react-datepicker/dist/react-datepicker.css";

class AttendanceWidget extends Component {


    constructor(props){
        super(props);

        this.state={
            records:[],
            courses:[],
            users:[],
            search:'',
            context:'',
            selectedDate:0,
            selectedCourse:0,
            selectedStudent:0,
            currentPage:1,
            maxRecords:1,
            loading:true,
            perPage:parseInt(window.wplms_attendance.per_page,10)
        }

        this.paginationPages=this.paginationPages.bind(this);
        this.fetchAttendance=this.fetchAttendance.bind(this);
        this.handleDateChange=this.handleDateChange.bind(this);
        this.setCourseChange=this.setCourseChange.bind(this);
        this.setStudentChange=this.setStudentChange.bind(this);
        this.fetchAtt = this.fetchAtt.bind(this);
        this.displayRecords=this.displayRecords.bind(this);
    }


    componentWillMount(){

        let url = `${window.wplms_attendance.api_url}user/${window.wplms_attendance.user_id}/?per_page=${this.state.perPage}&security=${window.wplms_attendance.security}`;
        if(window.wplms_attendance.type === 'instructor'){
            url = `${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/?per_page=${this.state.perPage}&security=${window.wplms_attendance.security}`;
        }

        fetch(url)
        .then(res => res.json())
        .then((records)=>{

            this.setState({loading:false});
            this.setState({records});

            let countUrl = `${window.wplms_attendance.api_url}user/${window.wplms_attendance.user_id}/?count=1&per_page=${this.state.perPage}&security=${window.wplms_attendance.security}`;
            if(window.wplms_attendance.type === 'instructor'){
                countUrl=`${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/?count=1&per_page=${this.state.perPage}&security=${window.wplms_attendance.security}`;
            }

            let users = [];

            if(records.length && window.wplms_attendance.type === 'instructor'){
                records.map((record)=>{
                    if(users.findIndex((user)=>{ return parseInt(user.id) === parseInt(record.user_id)}) < 0){
                        users.push({id:parseInt(record.user_id),'name':'...'});
                    }
                });

                if(users.length){
                    let unique_user_ids =[];
                    users.map((user)=>{
                        if(unique_user_ids.indexOf(user.id) < 0){
                            unique_user_ids.push(user.id);
                        }
                    });

                    fetch(`${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/students/?students=${JSON.stringify(unique_user_ids)}&security=${window.wplms_attendance.security}`)
                    .then(res => res.json())
                    .then((users)=>{
                        this.setState({users});
                    });

                }
            }


            fetch(countUrl)
            .then(res => res.json())
            .then((maxRecords)=>{
                this.setState({maxRecords});
            });


            let courseUrl = `${window.wplms_attendance.api_url}user/${window.wplms_attendance.user_id}/courses?security=${window.wplms_attendance.security}`;
            if(window.wplms_attendance.type === 'instructor'){
                courseUrl=`${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/courses?security=${window.wplms_attendance.security}`;
            }

            fetch(courseUrl)
            .then(res => res.json())
            .then((allcourses)=>{

                let courses =[];

                allcourses.map((course)=>{
                    if(courses.findIndex((item)=>{return item.id === course.id}) < 0){
                        courses.push(course);
                    }
                });
                this.setState({courses});
            });
        });
    }

    fetchAttendance(obj){
        
        let records = [...this.state.records];
        this.setState({loading:true});
        if( (!obj.paginate)  || (obj.paginate && obj.page > 1 && obj.page*this.state.perPage > records.length)){


            let url = `${window.wplms_attendance.api_url}user/${window.wplms_attendance.user_id}/?per_page=${this.state.perPage}&page=${obj.page}&date_start=${obj.date_start}&date_end=${obj.date_end}&course_id=${obj.course_id}&security=${window.wplms_attendance.security}`;
            if(window.wplms_attendance.type === 'instructor'){
                url = `${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/?per_page=${this.state.perPage}&page=${obj.page}&date_start=${obj.date_start}&date_end=${obj.date_end}&course_id=${obj.course_id}&student_id=${obj.student_id}&security=${window.wplms_attendance.security}`;
            }

            let users = [...this.state.users];

            fetch(url)
            .then(res => res.json())
            .then((rrecords)=>{
                this.setState({loading:false});
                
                

                if(!obj.paginate){

                    let pagUrl = `${window.wplms_attendance.api_url}user/${window.wplms_attendance.user_id}/?count=1&per_page=${this.state.perPage}&page=${obj.page}&date_start=${obj.date_start}&date_end=${obj.date_end}&course_id=${obj.course_id}&security=${window.wplms_attendance.security}`;
                    if(window.wplms_attendance.type === 'instructor'){
                        pagUrl = `${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/?count=1&per_page=${this.state.perPage}&page=${obj.page}&date_start=${obj.date_start}&date_end=${obj.date_end}&course_id=${obj.course_id}&student_id=${obj.student_id}&security=${window.wplms_attendance.security}`;
                    }
                    fetch(pagUrl)
                    .then(res => res.json())
                    .then((maxRecords)=>{
                        this.setState({maxRecords});
                    });
                    
                    records = rrecords;
                    

                }else{

                    rrecords.map((record)=>{
                        if(records.findIndex((r)=>{return r.activity_id === record.activity_id}) < 0){
                            records.push(record);
                        }
                    });
                }

                if(rrecords.length && window.wplms_attendance.type === 'instructor'){
                    let unique_user_ids =[];
                    console.log(users);

                    rrecords.map((record)=>{
                        if(users.findIndex((user)=>{ return parseInt(user.id) === parseInt(record.user_id)}) < 0){
                            if(unique_user_ids.indexOf(record.user_id) < 0){
                                unique_user_ids.push(record.user_id);
                            }
                        }
                    });

                    
                    if(unique_user_ids.length){

                        fetch(`${window.wplms_attendance.api_url}instructor/${window.wplms_attendance.user_id}/students/?students=${JSON.stringify(unique_user_ids)}&security=${window.wplms_attendance.security}`)
                        .then(res => res.json())
                        .then((newusers)=>{
                            if(newusers.length){
                                newusers.map((user)=>{
                                    users.push(user);
                                });
                                this.setState({users});
                            }
                        });

                    }
                }

                this.setState({records});
                
            });

        }else{
            this.setState({loading:false});
        }
    }
    
    fetchAtt(){

        let obj = {page:1,course_id:0,date_start:0 ,date_end:0,paginate:false };
        if(this.state.selectedCourse){
            obj.course_id = this.state.selectedCourse;
        }

        if(this.state.selectedStudent){
            obj.student_id = this.state.selectedStudent;
        }

        if(this.state.selectedDate){
            obj.date_start = Math.round(this.state.selectedDate.setHours(0,0)/1000);
            obj.date_end = Math.round(this.state.selectedDate.setHours(23,59)/1000);
        }

       
        
        this.fetchAttendance(obj);
    }

    setPaged(currentPage){
        //console.log('clicked => '+currentPage);
        this.setState({currentPage});
        let obj = {page:currentPage,course_id:0,date_start:0,date_end:0,paginate:true};
        if(this.state.selectedCourse){
            obj.course_id = this.state.selectedCourse;
        }

        if(this.state.selectedDate){
            obj.date_start = Math.round(this.state.selectedDate.setHours(0,0)/1000) ;
            obj.date_end = Math.round(this.state.selectedDate.setHours(23,59)/1000) ;
        }
        
        
        this.fetchAttendance(obj);
    }

    handleDateChange(date){
        this.setState({currentPage:1});
        this.setState({selectedDate:date});
    }
    setCourseChange(event){
        
        if(event.target.value === "0"){
            event.target.value=0;
        }
        this.setState({currentPage:1});
        this.setState({selectedCourse:event.target.value});
    }
    setStudentChange(event){
        
        if(event.target.value === "0"){
            event.target.value=0;
        }

        if(event.hasOwnProperty('user')){
            let users = [...this.state.users];
            users.push(event.user);
            this.setState({users});

        }
        
        this.setState({currentPage:1});
        this.setState({selectedStudent:event.target.value});
    }
    paginationPages(records){

        let recordsLength = records.length;


        if(records.length === this.state.records.length){
            recordsLength= this.state.maxRecords;
        }

        let $return =[];
        let addfrontdots = true;let addenddots=true;
        let maxPage = Math.floor(recordsLength/this.state.perPage);
        if(Math.floor(recordsLength%this.state.perPage) != 0 ){
            maxPage = maxPage+1;
        }

        for(let i=1;i<=maxPage;i++){


            if(this.state.currentPage === i){
                $return.push(<span className="current" key={i}>{i}</span>);
            }else{

                if(maxPage => 5){
                    if(i === 1  || i === maxPage || i === (this.state.currentPage-1) || i=== (this.state.currentPage+1)){

                        if( i === maxPage && this.state.currentPage !== maxPage -1){
                            $return.push(<span key={i}>{i}</span>);
                        }else{
                            $return.push(<a onClick={()=>this.setPaged(i)} className="inactive" key={i}>{i}</a>);
                        }
                        
                    }else{
                        if(addfrontdots && i > 1 && i < (this.state.currentPage-1)){
                            addfrontdots = false;
                            $return.push(<span key={i}>...</span>);
                        }
                        if(addenddots && i < maxPage && i > (this.state.currentPage+1)){
                            addenddots = false;
                            $return.push(<span key={i}>...</span>);
                        }
                    }

                }else{
                    if(i < this.state.currentPage || i === (this.state.currentPage+1)){
                        $return.push(<a onClick={()=>this.setPaged(i)} className="inactive" key={i}>{i}</a>);
                    }else{
                        if( i === maxPage && i > 1){
                            $return.push(<span key={i}>{i}</span>);
                        }
                    }
                }

                
                
            }
        }

        return $return;
    }

    displayRecords(records){


        if(records.length){

            let rreturn=[];
            records.map((record,index)=>{

                let user = 0;
                if(this.state.users.length){
                    user = this.state.users[this.state.users.findIndex((u)=>{return parseInt(u.id) === parseInt(record.user_id)})];    
                }
                
                let course = this.state.courses[this.state.courses.findIndex((c)=>{return c.id === record.course_id})];
                rreturn.push(
                    <div className="record" key={index}>
                        <div>
                            <DateTimestamp date={record.attendance_timestamp} />
                        </div>
                        <div>
                            <div className="course">
                            {
                                (typeof course === 'undefined' || !course.hasOwnProperty('name') || !course.name)?'':
                                <span>{course.name}</span>
                            }
                            </div>
                        </div>
                        {
                                (window.wplms_attendance.type === 'instructor')?
                            <div>
                                {
                                    (user)?
                                        <div className="attendance_student_profile">
                                            <span>{user.name}</span>
                                        </div>
                                        :''
                                }
                                
                            </div>:''
                        }
                        <div>
                            {
                                (parseInt(record.present,10))?
                                <span>{window.wplms_attendance.translations.present}</span>
                                :
                                <span>{window.wplms_attendance.translations.absent}</span>
                            }
                        </div>
                    </div>
                );
            });

            
            
            return rreturn;            
                     
        }else{
            return(
                <div className="noattendancerecords"><div className="message">
                {window.wplms_attendance.translations.no_records}</div></div>
            )
        }      
    }

    render(){

        //console.log(this.state);

        let course ={};
        let records = [...this.state.records];

        let maxRecords = this.state.maxRecords;

        if(this.state.selectedCourse > 0){
            records = records.filter((record,index)=>{
                    return record.course_id === this.state.selectedCourse;
            });
        }

        if(this.state.selectedStudent > 0){
            records = records.filter((record,index)=>{
                    return parseInt(record.user_id) === parseInt(this.state.selectedStudent);
            });
        }

        if(this.state.selectedDate){

            let selectedDate = new Date(this.state.selectedDate.getTime());
            records = records.filter((record,index)=>{
                
                let record_attendance_timestamp = new Date().getTime() - (parseInt(window.wplms_attendance.timestamp,10) - parseInt(record.attendance_timestamp))*1000;

                //console.log(record_attendance_timestamp +' > '+selectedDate.setUTCHours(0,0)+' && '+record_attendance_timestamp +'< '+ selectedDate.setUTCHours(23,59));
                return record_attendance_timestamp > selectedDate.setUTCHours(0,0) && record_attendance_timestamp <  selectedDate.setUTCHours(23,59);
                
            });
        }

        let rrecords = [...records];
        records = records.filter((record,index)=>{

            if(this.state.currentPage){
                //console.log(index +' >= '+(this.state.currentPage - 1)*this.state.perPage +'||'+ index +'<'+ ((this.state.currentPage)*this.state.perPage));
                if(index >= (this.state.currentPage - 1)*this.state.perPage && index < (this.state.currentPage)*this.state.perPage){
                    return record;
                }
            }
        });

        if(rrecords.length !== this.state.records.length){
            maxRecords = rrecords.length;
        }

        return (
                <div>
                {
                    (this.state.loading)?
                    <div className="attendanceloader"><svg width="120" height="30" viewBox="0 0 120 30" xmlns="http://www.w3.org/2000/svg" fill="#222"> <circle cx="15" cy="15" r="15"> <animate attributeName="r" from="15" to="15" begin="0s" dur="0.8s" values="15;9;15" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="1" to="1" begin="0s" dur="0.8s" values="1;.5;1" calcMode="linear" repeatCount="indefinite" /> </circle> <circle cx="60" cy="15" r="9" fill-opacity="0.3"> <animate attributeName="r" from="9" to="9" begin="0s" dur="0.8s" values="9;15;9" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="0.5" to="0.5" begin="0s" dur="0.8s" values=".5;1;.5" calcMode="linear" repeatCount="indefinite" /> </circle> <circle cx="105" cy="15" r="15"> <animate attributeName="r" from="15" to="15" begin="0s" dur="0.8s" values="15;9;15" calcMode="linear" repeatCount="indefinite" /> <animate attributeName="fill-opacity" from="1" to="1" begin="0s" dur="0.8s" values="1;.5;1" calcMode="linear" repeatCount="indefinite" /> </circle> </svg></div>
                    :
                    <div className="attendanceRecords">
                        <div className="attendanceSorter">
                            <div className="selectdate">
                                <DatePicker
                                    placeholderText="Select a date" 
                                    isClearable={true}
                                    selected={this.state.selectedDate}
                                    onChange={this.handleDateChange}
                                />
                            </div>
                            <div className="selectCourses">
                                <select onChange={this.setCourseChange} value={this.state.selectedCourse}>
                                    <option value="0">{window.wplms_attendance.translations.select_course}</option>
                                    {
                                        this.state.courses.map((course)=>{
                                            if(course.hasOwnProperty('name') && course.name){
                                                return (<option value={course.id} key={course.id}>{course.name}</option>)
                                            }
                                        })
                                    }
                                </select>
                            </div>
                            {
                                (window.wplms_attendance.type === 'instructor')?
                                <div>
                                    <UserSelector users={this.state.users} change={(event)=>this.setStudentChange(event)} value={this.state.selectedStudent} />
                                </div>
                                :''
                            }
                            <button className="button" onClick={this.fetchAtt}>{window.wplms_attendance.translations.go}</button>
                        </div>
                        <div className="attendanceData">
                            <div className="record recordHeader"><strong>{window.wplms_attendance.translations.date}</strong><strong>{window.wplms_attendance.translations.course}</strong><strong>{window.wplms_attendance.translations.student}</strong><strong>{window.wplms_attendance.translations.attendance}</strong></div>
                        {this.displayRecords(records)}
                        {
                            (this.state.maxRecords > 1)?
                            <div className="pagination" key={records.length}>
                                <span>{window.wplms_attendance.translations.showing} {((this.state.currentPage-1)*this.state.perPage)+'-'+((this.state.currentPage == 1 && records.length < this.state.perPage)?records.length:this.state.currentPage*this.state.perPage)} {window.wplms_attendance.translations.of} {maxRecords}</span>
                                {this.paginationPages(rrecords)}
                            </div>:''
                        }
                        </div> 
                         
                    </div> 

                }
                </div>

        )
    }

}

export default AttendanceWidget;
