<?php
/**
 * WPLMS S3 Init
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	Wplms-S3/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_Import_Attendance_Init{

	public static $instance;
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Import_Attendance_Init();
        return self::$instance;
        
    }

	private function __construct(){
		$this->date_format_map = array(
        	'F j, Y' => 'MM d, yy',
        	'Y-m-d' => 'yy-mm-dd',
        	'm/d/Y' => 'mm/dd/yy',
        	'd/m/Y' => 'dd/mm/yy',
        );
        $this->date_format = get_option('date_format');

		add_action('wp_ajax_import_attendance_through_csv',array($this,'import_attendance_admin'));
		add_action('wplms_course_stats_panel',array($this,'wplms_course_attendance_csv_import'), 10,2);
		add_action('wplms_course_stats_panel',array($this,'adding_import_attendance_action'), 10,2);
	}
	function wplms_course_attendance_csv_import($list) {
		$course_id =  get_the_ID(); 
		$enabled=get_post_meta($course_id,'vibe_course_attendance', true);
		if(!empty($enabled) && vibe_validate($enabled)){
			$instructor_mark_attendance = vibe_get_option('instrustor_mark_attendance');

			if(isset($instrustor_mark_attendance) && $instrustor_mark_attendance || current_user_can('manage_options')) {
				echo '<h3 class="heading import_attendance" id="import_attendance">'.__("Import Attendance","wplms-attendance").'<span class="attendance-plus-minus right"></h3>';
			}?>
			<script>
				jQuery(document).ready(function($){ 
					$('.import_attendance').on('click',function(event){
						event.preventDefault();
						$('.import__attendance_field').toggle('500');
						$('.attendance-plus-minus').toggleClass('active');
						
					});
				});
			</script>
			<style>
				h3#import_attendance span.attendance-plus-minus.right{font-size:18px}
				.attendance-plus-minus:before{
					content:"\f068";
					font-family: 'fontawesome';
				}
				.attendance-plus-minus.active:before{
					content:"\f067";
					font-family: 'fontawesome';
				}
			</style>
			<?php
			return $list;
		}
	}

	function adding_import_attendance_action() {
		$this->import_data();
		$post_max_size = ini_get('post_max_size');
		$post_max_size_kb = $this->wplms_let_to_num( $post_max_size );
	  	echo '
			<div class="import__attendance_field" id = "" >
				<form id= "import_csv" method="post" enctype="multipart/form-data" action= "#">
			    <h6>
			    	'. sprintf(_x( 'Select Csv File To Import (Upload limit : %s, Date Format : mm/dd/YYYY)', 'wplms-attendance' ), $post_max_size) .'
			    	<span class = "sample_download" style="float:right;"><a href="'.plugins_url( '../sample_csv.csv' , __FILE__ ).'" target="_blank">Download Sample</a></span>
			    </h6>
			    <input type="file" name="upfile" id="fileToUpload">
			    <input id = "upload_csv_button" type="submit" value="'.__('Upload File','wplms-attendance').'" name="import_attendance" class="button-primary">';
			echo '</form>
			</div>'; 
				
	}

	function import_data(){
		if(isset($_POST['import_attendance'])){
			try {
		    
			    // Undefined | Multiple Files | $_FILES Corruption Attack
			    // If this request falls under any of them, treat it invalid.
			    if (
			        !isset($_FILES['upfile']['error']) ||
			        is_array($_FILES['upfile']['error'])
			    ) {
			        throw new RuntimeException(__('Invalid parameters.','wplms-attendance'));
			    }
			    $post_max_size = ini_get('post_max_size');
				$post_max_size_kb = $this->wplms_let_to_num( $post_max_size );
			    // Check $_FILES['upfile']['error'] value.
			    switch ($_FILES['upfile']['error']) {
			        case UPLOAD_ERR_OK:
			            break;
			        case UPLOAD_ERR_NO_FILE:
			            throw new RuntimeException(__('No file selected.','wplms-attendance'));
			        case UPLOAD_ERR_INI_SIZE:
			        case UPLOAD_ERR_FORM_SIZE:
			            throw new RuntimeException(__('Exceeded filesize limit, Max Upload size limit: '.$post_max_size.'.','wplms-attendance'));
			        default:
			            throw new RuntimeException(__('Unknown errors.','wplms-attendance'));
			    }

			    // You should also check filesize here. 

			    if ($_FILES['upfile']['size'] > $post_max_size_kb) {
			        throw new RuntimeException(__('Exceeded filesize limit, Max Upload size limit: '.$post_max_size.'.','wplms-attendance'));
			    }

			    // DO NOT TRUST $_FILES['upfile']['mime'] VALUE !!
			    // Check MIME Type by yourself.
			    $finfo = new finfo(FILEINFO_MIME_TYPE);
			    if (false === $ext = array_search(
			        $finfo->file($_FILES['upfile']['tmp_name']),
			        array(
			            'text/plain','text/csv','text/tsv','text/html'
			        ),
			        true
			    )) {
			        throw new RuntimeException(__('Invalid file format.','wplms-attendance'));
			    }
			    $import	= $this->process_csv($_FILES['upfile']['tmp_name']);
			    if($import['status']){
			    	echo '<div class="message success full"><span class="file-success">'.sprintf(_x('Import Complete for %s users and %s dates','wplms-attendance'), $import['users'], $import['dates'] ).'</span></div>';

			    }else{
			    	echo '<div class="message error full"><span class="file-success">'.__('Import Failed','wplms-attendance').'</span></div>';
			    };

			} catch (RuntimeException $e) {
			    echo '<div class="message success full"><span class="file-success"><span class="file-error">'.$e->getMessage().'</span></div>';
			}
		}
	}
	function wplms_let_to_num( $size ) {
		$l   = substr( $size, -1 );
		$ret = substr( $size, 0, -1 );
		switch ( strtoupper( $l ) ) {
			case 'P':
				$ret *= 1024;
			case 'T':
				$ret *= 1024;
			case 'G':
				$ret *= 1024;
			case 'M':
				$ret *= 1024;
			case 'K':
				$ret *= 1024;
		}
		return $ret;
	}

	function process_csv($file){
		global $wpdb;
		$course_id =  get_the_ID(); 
		if (($handle = fopen($file, "r")) !== FALSE) {
			$temp = 0;
			$user_count = 0;
			$date_count = 0;
			$datesArray= array();
			$import= array();
		    while ( ($data = fgetcsv($handle,1000,",") ) !== FALSE ) {	
				$new_dates= array();
				$meta_value= array();
				if($temp) {
					$user_id = $data[0];
					$exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
					$user_data = array();
					if(!empty($exisiting_dates)){
						foreach ($exisiting_dates as $exisiting_date) {
						    $user_data[] = trim(date('m/d/Y', $exisiting_date));
						}
					}
					foreach ($data as $key => $value) {
						if($key) {
							if($value ==1) {
								$present_date = strtotime($datesArray[$key]);
								$new_date = trim(date('m/d/Y', $present_date));
								$new_dates[] = $new_date;
								if (($index = array_search($new_date, $user_data)) !== false) {
						            unset($user_data[$index]);
						        }
						    } else {
						    	$present_date = strtotime($datesArray[$key]);
						    	$new_date = trim(date('m/d/Y', $present_date));
								if (($index = array_search($new_date, $user_data)) !== false) {
						            unset($user_data[$index]);
						        }
						    }
						}
						
					}
					if(!empty($user_data)) {
						foreach($user_data as $user_date) {
							$meta_value[] = strtotime($user_date);
						}
					}

					if(!empty($new_dates)) {
						foreach($new_dates as $date) {
					        $meta_value[] = strtotime($date);
					        do_action('wplms_mark_user_attendance',$user_id,$course_id,strtotime($date));
					    }
					}
					update_user_meta($user_id,'wplms_course_attendance_'.$course_id, $meta_value);
					$user_count++;
				} else {
					foreach ($data as $key => $value) {
						if($key) {
							$datesArray[$key] = $value;
						}
					}
					$date_count = $key;
				}
				$temp++;
		    }
		    fclose($handle);
		}
		$import ['status'] = 1;
		$import ['dates'] = $date_count;
		$import ['users'] = $user_count;
		return $import;
	}
}
Wplms_Import_Attendance_Init::init();