<?php
/**
 * WPLMS Attendance Init
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	Wplms-Attendance/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_Attendance_Init{

	public static $instance;
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Attendance_Init();
        return self::$instance;
        
    }

	private function __construct(){

		add_action('init',array($this,'db_upgrade'));


		$this->date_format_map = array(
        	'F j, Y' => 'MM d, yy',
        	'Y-m-d' => 'yy-mm-dd',
        	'm/d/Y' => 'mm/dd/yy',
        	'd/m/Y' => 'dd/mm/yy',
        );
        $this->date_format = get_option('date_format');
        
		add_action('wp_enqueue_scripts',array($this,'enqueue_the_scripts'),999);
		add_filter('wplms_course_creation_tabs',array($this,'add_in_front_end'));
		add_action('wplms_front_end_generate_fields_default',array($this,'wplms_front_end_generate_field_default_customdiv'),10,2);
		add_action('wplms_before_start_course',array($this,'wplms_before_start_course'));

		add_action('wplms_before_course_status_hit',array($this,'mark_auto_attendance'),10,2);

		add_action('wplms_course_admin_bulk_actions_list',array($this,'wplms_course_admin_bulk_actions_mark_attendance'), 10,2);
		
		add_action('wplms_user_course_admin_member',array($this,'profile_attendance_count'), 10,2);

		add_action('wplms_course_admin_bulk_actions',array($this,'add_attendance_admin_bulk_actions'), 999,2);

		add_action('wp_ajax_add_attendance_admin',array($this,'save_attendance_admin'));

		add_filter('wplms_course_stats_list',array($this,'add_attendance_stat'));

		add_action('wplms_course_stats_process',array($this,'process_attendance_stat'),10,6);
		add_filter('vibe_option_custom_sections',array($this,'add_attendance_in_course_manager'));
		add_filter('wplms_touch_points',array($this,'add_attendance_touch_point'));

		add_filter('bp_course_all_mails',array($this,'add_wplms_attendance_email'));
		add_action('wplms_mark_user_attendance',array($this,'record_activity_for_attendance'),10,3);
		add_action('wplms_mark_user_attendance_remove',array($this,'record_activity_for_attendance_absent'),10,3);


		add_action('init',array($this,'check_touchpoints'));
		
		add_filter( 'bp_course_format_notifications',array($this,'set_format_for_notifications') ,11,5);

	}

	/* Enqueue Scripts */
	
	/*
		DB  UPGRADE Make necessary changes in the DB when required
	*/

	function db_upgrade(){

		$db_version = get_option('WPLMS_ATTENDANCE_DB_VERSION');
		if(empty($db_version) || version_compare( $db_version, WPLMS_ATTENDANCE_DB_VERSION, '<' ) ){

			global $wpdb,$bp;

			$activity_ids = $wpdb->get_results("SELECT activity_id FROM {$bp->activity->table_name_meta} WHERE meta_key LIKE '%wplms_course_attendance%'", ARRAY_A);

			if(!empty($activity_ids)){
				foreach($activity_ids as $data){
					$activity_id = $data['activity_id'];
					bp_course_record_activity_meta(array('id'=>$activity_id,'meta_key'=>'present','meta_value'=>1));
				}
			}
			update_option('WPLMS_ATTENDANCE_DB_VERSION',WPLMS_ATTENDANCE_DB_VERSION);
		}

	}

	function set_format_for_notifications($notification, $action, $item_id, $secondary_item_id, $total_items){
		
		switch ($action) {
			case 'instructor_marked_attendance':
				$notification = sprintf(_x('Attendance recorded for student %s in course %s','Attendance record for noitification action','wplms-attendance'),bp_core_get_userlink($secondary_item_id),'<a href="'.get_permalink($item_id).'">'.get_the_title($item_id).'</a>');
				break;
			
			case 'record_attendance_course':
				$notification = sprintf(_x('Your attendance recorded for course %s','Attendance record for activity action','wplms-attendance'),'<a href="'.get_permalink($item_id).'">'.get_the_title($item_id).'</a>');
				break;
			
		}
		return $notification;
	}

	function check_touchpoints(){
		if(class_exists('WPLMS_tips')){
	        $wplms_settings = WPLMS_tips::init();
	        if(!empty($wplms_settings->lms_settings) && !empty($wplms_settings->lms_settings['touch']))
	        $this->lms_settings = $wplms_settings->lms_settings['touch'];
      	}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['student']['message'])){
			add_action('wplms_mark_user_attendance',array($this,'student_message_wplms_attendance'),10,3);
		}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['student']['email'])){
			add_action('wplms_mark_user_attendance',array($this,'student_email_wplms_attendance'),10,3);
		}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['student']['notification'])){
			add_action('wplms_mark_user_attendance',array($this,'student_notification_wplms_attendance'),10,3);
		}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['instructor']['message'])){
			add_action('wplms_mark_user_attendance',array($this,'instructor_message_wplms_attendance'),10,3);
		}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['instructor']['notification'])){
			add_action('wplms_mark_user_attendance',array($this,'instructor_notification_wplms_attendance'),10,3);
		}
		if(!empty($this->lms_settings) && !empty($this->lms_settings['wplms_attendance']['instructor']['email'])){
			add_action('wplms_mark_user_attendance',array($this,'instructor_email_wplms_attendance'),10,3);
		}
		
	}

	function add_wplms_attendance_email($bp_emails){
		//for student
		$bp_emails['record_attendance_course']=array(
        'description'=> __('Attendance recorded for Course','wplms-attendance'),
        'subject' =>  sprintf(__('Your attendance recorded for course : %s on %s','wplms-attendance'),'{{course.name}}','{{student.time}}'),
        'message' =>  sprintf(__('Your attendance recorded for course : %s on %s','wplms-attendance'),'{{course.titlelink}}','{{student.time}}'),
        );

		//for instructor
        $bp_emails['instructor_marked_attendance']=array(
        'description'=> __('Instructor Attendance recorded for Course','wplms-attendance'),
        'subject' =>  sprintf(__('Student %s marked as present for course : %s on %s','wplms-attendance'),'{{student.name}}','{{course.name}}', '{{student.time}}'),
        'message' => sprintf(__('Student %s marked as present for course : %s on %s','wplms-attendance'),'{{student.userlink}}','{{course.titlelink}}', '{{student.time}}'),
        );
        return $bp_emails;
	}

	function record_activity_for_attendance($user_id,$course_id,$time){
		if(empty($user_id))
			return;
		if(!function_exists('bp_course_record_activity') || !function_exists('bp_course_record_activity_meta'))
			return;
		$date_format = get_option('date_format');
		$activity_id = bp_course_record_activity(array(
                    'user_id' => $user_id,
                    'action' => _x('Attendance recorded','Attendance record for activity action','wplms-attendance'),
                    'content' => sprintf(_x('Attendance recorded for course %s on %s','Attendance record for activity action','wplms-attendance'),get_the_title($course_id),date($date_format,$time)),
                    'component' => 'course',
                    'type' => 'wplms_attendance',
                    'item_id' => $course_id,
                    'secondary_item_id' => false,
                    'hide_sitewide' => apply_filters('wplms_attendance_hide_sitewide',false,$user_id),
                ));


		if(!empty($activity_id) ){

			bp_course_record_activity_meta(array('id'=>$activity_id,'meta_key'=>'present','meta_value'=>1));
			bp_course_record_activity_meta(array('id'=>$activity_id,'meta_key'=>'wplms_course_attendance_'.$course_id,'meta_value'=>$time));
			//do_actions('wplms_record_attendance_activity',$activity_id);
		}
	}

	function record_activity_for_attendance_absent($user_id,$course_id,$time){
		
		if(!function_exists('bp_course_record_activity') || !function_exists('bp_course_record_activity_meta'))
			return;
		$date_format = get_option('date_format');
		$activity_id = bp_course_record_activity(array(
                    'user_id' => $user_id,
                    'action' => _x('Attendance marked absent','Attendance record for activity action','wplms-attendance'),
                    'content' => sprintf(_x('Attendance marked absent for course %s on %s','Attendance record for activity action','wplms-attendance'),get_the_title($course_id),date($date_format,$time)),
                    'component' => 'course',
                    'type' => 'wplms_attendance',
                    'item_id' => $course_id,
                    'secondary_item_id' => false,
                    'hide_sitewide' => apply_filters('wplms_attendance_hide_sitewide',false,$user_id),
                ));


		if(!empty($activity_id) ){
			bp_course_record_activity_meta(array('id'=>$activity_id,'meta_key'=>'present','meta_value'=> 0));
			bp_course_record_activity_meta(array('id'=>$activity_id,'meta_key'=>'wplms_course_attendance_'.$course_id,'meta_value'=>$time));
			//do_actions('wplms_record_attendance_activity',$activity_id);
		}
	}


	//touch pints functions
	function student_message_wplms_attendance($user_id,$course_id,$time){


		if(!is_user_logged_in())
			return;

		$sender_id=get_post_field('post_author', $course_id);
		$message = sprintf(__('Your attendance recorded for course : %s on %s','wplms-attendance'),'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>',date($this->date_format,$time));
	    if(bp_is_active('messages') && function_exists('bp_course_messages_new_message'))
	      bp_course_messages_new_message( array('sender_id' => $sender_id, 'subject' => __('Attendance recorded for Course','wplms-attendance'), 'content' => $message,   'recipients' => $user_id ) );
	}

	function student_notification_wplms_attendance($user_id,$course_id,$time){
		if(!function_exists('bp_course_add_notification'))
			return;
		
		bp_course_add_notification(array(
				'user_id' => $user_id,
				'item_id' => $course_id,
				'secondary_item_id' => false,
				'component_action'  => 'record_attendance_course'
			));
	}

	function student_email_wplms_attendance($user_id,$course_id,$time){
		if(!function_exists('bp_course_wp_mail'))
			return;
		$enable = get_user_meta($user_id,'record_attendance_course',true);
		if($enable !== 'no'){
			$user = get_user_by( 'id', $user_id);
			$to = $user->user_email;
			$subject = sprintf(__('Your attendance recorded for course : %s on %s','wplms-attendance'),get_the_title($course_id),date($this->date_format,$time));
			$message = sprintf(__('Your attendance recorded for course : %s on %s','wplms-attendance'),'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>',date($this->date_format,$time));

			bp_course_wp_mail($to,$subject,$message,array('action'=>'record_attendance_course','item_id'=>$course_id,'secondary_item_id'=>$user_id,'tokens'=>array('student.time'=>date($this->date_format,$time),'course.name'=>get_the_title($course_id),'course.titlelink'=>'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>')));
		}

	}

	//Instructor functions
	function instructor_message_wplms_attendance($user_id,$course_id,$time){
		$instructor_ids=apply_filters('wplms_course_instructors',get_post_field('post_author', $course_id),$course_id);
		if(!is_array($instructor_ids))
			$instructor_ids = array($instructor_ids);

		$message = sprintf(__('Student %s marked as present for course : %s on %s','wplms-attendance'),bp_core_get_userlink($user_id),'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>', date($this->date_format,$time));
	    if(bp_is_active('messages') && function_exists('bp_course_messages_new_message'))
	      bp_course_messages_new_message( array('sender_id' => $user_id, 'subject' => __('Attendance marked for Course','wplms-attendance'), 'content' => $message,   'recipients' => $instructor_ids ) );
	}

	function instructor_notification_wplms_attendance($user_id,$course_id,$time){
		if(!function_exists('bp_course_add_notification'))
			return;
		$instructor_ids=apply_filters('wplms_course_instructors',get_post_field('post_author', $course_id),$course_id);
		if(!is_array($instructor_ids))
			$instructor_ids = array($instructor_ids);

		foreach($instructor_ids as $instructor_id){
			bp_course_add_notification(array(
				'user_id' => $instructor_id,
				'item_id' => $course_id,
				'secondary_item_id' => $user_id,
				'component_action'  => 'instructor_marked_attendance'
			));
		}
	}

	function instructor_emails($course_id,$key){
		$instructor_ids =apply_filters('wplms_course_instructors',get_post_field('post_author', $course_id),$course_id);
		if(!is_array($instructor_ids)){
			$instructor_ids = array($instructor_ids);
		}
		$to = array();
		foreach($instructor_ids as $instructor_id){
			$enable = get_user_meta($instructor_id,'instructor_'.$key,true);
			if($enable !== 'no'){
				$user = get_user_by( 'id', $instructor_id);
				$to[] = $user->user_email;
			}
		}
		return $to;
	}

	function instructor_email_wplms_attendance($user_id,$course_id,$time){
		$to = $this->instructor_emails($course_id,'wplms_attendance');
		$subject = sprintf(__('Student %s marked as present for course : %s on %s','wplms-attendance'),bp_core_get_user_displayname($user_id),get_the_title($course_id),date($this->date_format,$time)); 
		$message = sprintf(__('Student %s marked as present for course : %s on %s','wplms-attendance'),bp_core_get_userlink($user_id),'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>', $time);
		if(count($to)){
			bp_course_wp_mail($to,$subject,$message,array('action'=>'instructor_marked_attendance','item_id'=>$course_id,'secondary_item_id'=>false,'tokens'=>array('student.time'=>date($this->date_format,$time),'course.name'=>get_the_title($course_id),'course.titlelink'=>'<a href="'.get_permalink($course_id).'">'.get_the_title($course_id).'</a>','student.userlink'=>bp_core_get_userlink($user_id),'student.name'=>bp_core_get_user_displayname($user_id))));
		}
	}

	function add_attendance_in_course_manager($sections) {
	  foreach ($sections as $key => $value) {
	    if(!empty($value['title']) && $value['title'] == 'Course Manager') {
	      $sections[$key]['fields'][] =  array(
	                'id' => 'instrustor_mark_attendance',
	                'type' => 'pages_select',
	                'title' => __('Mark Attendance', 'wplms-attendance'),
	                'sub_desc' => __('enable attendance', 'wplms-attendance'),
	                'type' => 'button_set',
	                'options' => array('' => __('No','wplms-attendance'),'1'=>__('Yes','wplms-attendance')),
	                'std' => ''
	                );
	       return $sections;
	    }
	  }  
	  return $sections;
	}

	function add_attendance_stat($list){
        $list['attendance_data']= __('Attendance','wplms-attendance');
        $date_format = get_option('date_format');
        $javascript_format = $this->date_format_map[$date_format];
        $end_date_value = date($date_format);
        $start_date_value = date($date_format,strtotime("-30 day"));
        wp_enqueue_script('jquery-ui-datepicker');
        ?>
		<div id = 'range' style= 'display:none'>
			<label for='start_date'>Start date</label>
			<input class ='att_start_date mid_box' type='text' data-id='start_date_input' value= '<?php echo $start_date_value;?>'/>

			<label for='end_date'>End date</label>
			<input class ='att_end_date mid_box' type='text' data-id='end_date_input' value= '<?php echo $end_date_value;?>'/>
			
		</div>
		<script>
			function empty_value( val ) {
				if (val === undefined || val=== '')
				return true; 
				return false;
			}
			jQuery(document).ready(function($) {
				var start_date = $('.att_start_date').val();
				var end_date = $('.att_end_date').val();
				var date_changer_id = start_date+'|'+end_date;
				$('.select_download_options ul').append('<input type="checkbox" style="display:none!important;" id ="'+date_changer_id+'" class="field date_changer" value="1">');
				$('.att_start_date').datepicker({ dateFormat: "<?php echo $javascript_format;?>" });
				$('.att_end_date').datepicker({ dateFormat: "<?php echo $javascript_format;?>" });
				var checked = $('#attendance_data').is(':checked');
				if(checked) {
					$('#range').show();
					$('.date_changer').prop('checked', true);
				}

				$('#attendance_data').click(function() {
					if($('#attendance_data').is(':checked')) {
						$('#range').show(); 
						$('.date_changer').prop('checked', true);
					} else {
						$('.date_changer').prop('checked', false);
						$('#range').hide();
					}
				});

				$('.att_start_date,.att_end_date').on('change',function (){
					var start_date = $('.att_start_date').val();
					var end_date = $('.att_end_date').val();
					if(!empty_value(start_date) && !empty_value(end_date)){
						$('.date_changer').attr('id',start_date+'|'+end_date);
					}
				});

				$('#download_stats').on('click',function (event){
					var start_date = $('.att_start_date').val();
					var end_date = $('.att_end_date').val();
					if($('#attendance_data').is(':checked')){
						if(empty_value(start_date) || empty_value(end_date)){
							event.preventDefault();
							event.stopPropagation();
							alert('<?php echo _x('Please select date for Attendance!','','wplms-attendance')?>');
						}
					}
				});
			});
		</script>
		<?php
		return $list;
    }

    function process_attendance_stat(&$csv_title, &$csv,&$i,&$course_id,&$user_id,&$field){
         if( strpos($field, '|') === false)
            return;
         // Custom Stat Calculation //
        $dates = explode('|', $field);
        $start_date = $dates[0];
        $end_date = $dates[1];
        $start_date_timestamp =strtotime($start_date);
        $end_date_timestamp =strtotime($end_date);
        $user_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
        $incrementer = $start_date_timestamp;
        $gmt_offset = get_option('gmt_offset');
        $user_dates_array = array();

        if(!empty($user_dates)){
          foreach ($user_dates as $key => $user_date) {
            $user_dates_array[$user_date] = trim(date('m/d/Y', $user_date+($gmt_offset*60*60)));
          }
        }
	  	
        while($incrementer <= $end_date_timestamp){
          	$title=date('m/d/Y', $incrementer);
          	if(!in_array($title,$csv_title))
            	$csv_title[$i]=$title; // ----> This is Title of the CSV Column
	      	$searched_date_index = array_search($title, $user_dates_array);
	      	if(!empty($user_dates_array) && !empty($searched_date_index)){
	           $csv[$i][] = date('m/d/Y H:i:s',($searched_date_index+($gmt_offset*60*60)) );
	        }else{
	           $csv[$i][]= _x('NA','attendance','wplms-attendance');
	        }
          	$incrementer = $incrementer + 24*60*60;
          	$i++;
        }
    }

	function save_attendance_admin(){
	  $members=$_POST['members'];
	  $members_count = count($members);
	  $course_id=$_POST['course'];
	  if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'security'.$course_id) ){
	      echo 'Security check failed ! **';
	      die();
	  }
	  foreach($members as $user_id) {
	    $dates = $_POST['dates'];
	    $type = $_POST['type']; 
	    $remove_date = [];

	    $exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
	    $user_data = array();
	    if(!empty($exisiting_dates)){
	      foreach ($exisiting_dates as $exisiting_date) {
	        $user_data[] = trim(date('m/d/Y', $exisiting_date));
	      }
	    }
	    foreach($dates as $date) {
	      if (($key = array_search($date, $user_data)) !== false) {
	            unset($user_data[$key]);
	        }
	    }
	    $meta_value= [];
	    if(!empty($user_data)) {
	    	foreach($user_data as $user_date) {
		      $meta_value[] = strtotime($user_date);
		    }
	    }
	    
	    if($type == 'present') {
	      foreach($dates as $date) {
	        $meta_value[] = strtotime($date);
	        do_action('wplms_mark_user_attendance',$user_id,$course_id,strtotime($date));
	      }
	    } else {
	      foreach($dates as $date) {
	        do_action('wplms_mark_user_attendance_remove',$user_id,$course_id,strtotime($date));
	      }
	    }

	    update_user_meta($user_id,'wplms_course_attendance_'.$course_id, $meta_value);	  
	  }
	  echo sprintf(_x('Attendance updated for %s users','wplms-attendance'),
			$members_count);die;
	} 

	function add_attendance_admin_bulk_actions(){
	  $user_id = get_current_user_id(); 
	  $course_id =  get_the_ID(); 
	  $exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
	  wp_enqueue_script( 'jq-multidatespicker-js', plugins_url( '../assets/js/multidatespicker.js' , __FILE__ ), array( 'jquery','jquery-ui-core','jquery-ui-datepicker' ) ,'1.6.6',true);
	  echo'
	      <div class="bulk_attendance_calender" id = "bulk_attendance_calender" style="display: none;">
	          <div id = "attendance_calender">
	            <input class="mid_box post_field  " type="hidden" data-id="attendance_calender" class="form_field" value="" >
	           </div>
	           <div class = "type" id = "type">
	              <select id ="type">
	                <option value="present">MARK AS PRESENT</option>
	                <option value="absent">MARK AS ABSENT</option>
	              </select>
	            </div>
	            <div>
	               <a href="#" id="attendance_save" class="button full">Save</a>
	               <input type="hidden" id="course_id" value="'.$course_id.'">
	            </div>
	      </div>'; 
	  }

	function wplms_course_admin_bulk_actions_mark_attendance($list){
		$course_id =  get_the_ID(); 
		$enabled=get_post_meta($course_id,'vibe_course_attendance', true);
		if(!empty($enabled) && vibe_validate($enabled)){

		// GET ALL AVAILABLE DATES IN FORMAT: ["02/13/2012","02/19/2012","04/13/2012"]; // mm/dd/yyyy
	    	$data=get_post_meta($course_id,'vibe_course_attendance_data', true);
			$data = explode(',',$data);
			if(!empty($data[0]) && is_array($data)){
				foreach ($data as $key => $date) {
					$new_data[] = trim($date); 
				}
			}
			if(!empty($new_data[0])){
				foreach($new_data as $k=>$date){
					$dates[$k]="'".trim($date)."'";
				}
				$schedule_date= implode(',',$dates);
			} 
			$enabled_calender = 0;
			if(!empty($schedule_date)) {
				$schedule_date= "[".$schedule_date."]";
			} else {
				$enabled_calender = 1;
				$schedule_date = "''";
			}

			$instructor_mark_attendance = vibe_get_option('instrustor_mark_attendance');
			if(isset($instrustor_mark_attendance) && $instrustor_mark_attendance || current_user_can('manage_options'))
				$list .= '<li><a href="#" class="instrustor_mark_attendance tip" title="'.__('Attendance','wplms-attendance').'"><i class="icon-book"></i></a></li>';
			?>
			<script>
				jQuery(document).ready(function($){
					var allDates= '';
					var availableDates = <?php echo $schedule_date; ?>; 
					var enabled_calender = <?php echo $enabled_calender; ?>; 
					$('.instrustor_mark_attendance').on('click',function(event){
						event.preventDefault();
						$('.bulk_attendance_calender').toggle('slow');
					});
					function unavailable(date) {
						ymd = ("0"+(date.getMonth()+1)).slice(-2) + "/" + ("0"+date.getDate()).slice(-2) + "/" + date.getFullYear();
						day = new Date(ymd).getDay();
						if (($.inArray(ymd, availableDates) >= 0) || enabled_calender) {
							return [true, "enabled"];
						} else {
							return [false,"disabled"];
						}
					}
					$("#attendance_calender").multiDatesPicker({
						beforeShowDay: unavailable,
						dateFormat: "mm/dd/yy",
						onSelect:function(){
							allDates = $('#attendance_calender').multiDatesPicker('getDates');
							$('[data-id="attendance_calender"]').val(allDates);
						} 
					});
					$('#attendance_save').click(function(e) {
						var $this = $(this);
						var org_text = $(this).text();
						$this.text('Processing......');
						e.preventDefault();
						var i=0;
						var members=[];
						var type = $('#type option:selected').val();
						$('.member').each(function(){
							if($(this).is(':checked')){
								members[i]=$(this).val();
								i++;
							}
						});
						if(members && members.length != 0) { 
							var total_members = members.length;
							if(allDates != '') {
								var course_id= $('#course_id').val(); 
								$.ajax({
									type: "POST",
									url: ajaxurl,
									data: { action: 'add_attendance_admin', 
											dates: allDates,
											course: course_id,
											type: type,
											members:members,
											security: $('#bulk_action').val(),
									},
									cache: false,
									success: function(json){ 
										$this.text(json);	
										setTimeout(function(){ $this.text(org_text); }, 3000);
										
											
									}
								});
							}else {
								setTimeout(function(){ $this.text(org_text); }, 3000);
								$this.text('<?php echo _x('Please select a date','','wplms-attendance');?>');
							}
						} else {
							setTimeout(function(){ $this.text(org_text); }, 3000);
							$this.text('<?php echo _x('Please select a student','','wplms-attendance');?>');
						}
					})
				});
			</script>
			<?php
			return $list;
	    }


	}

	function mark_auto_attendance($course_id,$user_id){
		$enabled=get_post_meta($course_id,'vibe_course_attendance', true);
	    if(!empty($enabled) && vibe_validate($enabled)){
	      $exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
	      $new_data = array();
	      $user_data = array();
	      if(empty($exisiting_dates)){
	        $exisiting_dates = array();
	      }else{
	        foreach ($exisiting_dates as $k => $exisiting_date) {
	          $user_data[] = trim(date('m/d/Y', $exisiting_date));	          
	        }
	      }
	      $mark = 0;
	      $data=get_post_meta($course_id,'vibe_course_attendance_data', true);
	      $data = explode(',',$data);
	      
	      $current_date = trim(date('m/d/Y', time()));
	      if(!empty($data[0]) && is_array($data)){
	        foreach ($data as $key => $date) {
	          $date = trim($date);
	          $new_data[] = $date; 
	          
	        }
	        if(in_array($current_date,$new_data) && !in_array($current_date,$user_data)){
	          $mark = 1;
	        }
	        if($mark){
	           $this->mark_user_attendance($user_id,$course_id,time());
	        }
	      }else{
	        $enabled_auto=get_post_meta($course_id,'vibe_course_attendance_auto', true);
	        if(!empty($enabled_auto) && vibe_validate($enabled_auto)) {
	          $mark= 1;
	          foreach ($exisiting_dates as $k => $exisiting_date) {
	            $user_date = date('m/d/Y', $exisiting_date);
	            $current_date = date('m/d/Y', time());
	            if($user_date == $current_date){
	              $mark = 0;
	              break;
	            }
	          }
	          if($mark){
	             $this->mark_user_attendance($user_id,$course_id,time());
	          }
	        }
	      }
	    }
	}

	function wplms_before_start_course(){
		$user_id = get_current_user_id(); 
	    $course_id = '';
	    if ( isset($_POST['course_id']) ){
	      $course_id=$_POST['course_id'];
	    }
	    if(empty($course_id) && $_COOKIE['course']){
	      $course_id = $_COOKIE['course'];
	    }
	    $enabled=get_post_meta($course_id,'vibe_course_attendance', true);
	    if(!empty($enabled) && vibe_validate($enabled)){
	      $exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
	      $new_data = array();
	      $user_data = array();
	      if(empty($exisiting_dates)){
	        $exisiting_dates = array();
	      }else{
	        foreach ($exisiting_dates as $k => $exisiting_date) {
	          $user_data[] = trim(date('m/d/Y', $exisiting_date));	          
	        }
	      }
	      $mark = 0;
	      $data=get_post_meta($course_id,'vibe_course_attendance_data', true);
	      $data = explode(',',$data);
	      
	      $current_date = trim(date('m/d/Y', time()));
	      if(!empty($data[0]) && is_array($data)){
	        foreach ($data as $key => $date) {
	          $date = trim($date);
	          $new_data[] = $date; 
	          
	        }
	        if(in_array($current_date,$new_data) && !in_array($current_date,$user_data)){
	          $mark = 1;
	        }
	        if($mark){
	           $this->mark_user_attendance($user_id,$course_id,time());
	        }
	      }else{
	        $enabled_auto=get_post_meta($course_id,'vibe_course_attendance_auto', true);
	        if(!empty($enabled_auto) && vibe_validate($enabled_auto)) {
	          $mark= 1;
	          foreach ($exisiting_dates as $k => $exisiting_date) {
	            $user_date = date('m/d/Y', $exisiting_date);
	            $current_date = date('m/d/Y', time());
	            if($user_date == $current_date){
	              $mark = 0;
	              break;
	            }
	          }
	          if($mark){
	             $this->mark_user_attendance($user_id,$course_id,time());
	          }
	        }
	      }
	    }
	}

	function mark_user_attendance($user_id,$course_id,$time){
		if(empty($user_id))
			return;
	  $gmt_offset = get_option('gmt_offset');
	  $exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
	  if(!empty($exisiting_dates)){
	    array_push($exisiting_dates, $time);
	  }else{
	    $exisiting_dates = array($time);
	  }

	  do_action('wplms_mark_user_attendance',$user_id,$course_id,$time+($gmt_offset*60*60));
	  return update_user_meta($user_id,'wplms_course_attendance_'.$course_id,$exisiting_dates);
	}

	function wplms_front_end_generate_field_default_customdiv($field,$action){
	  if($field['type'] != 'customdiv')
	  return $field;
	  echo '<div id="'.$field['id'].'"></div>
	  <input type="hidden" placeholder="" value="'.$field['value'].'" data-id="'.$field['id'].'" class="mid_box post_field  "/>';

	  $dates = explode(',',$field['value']);
	  $schedule_date= '';
	  if(!empty($dates[0])){
	    foreach($dates as $k=>$date){
	      $dates[$k]="'".trim($date)."'";
	    }
	    $schedule_date= implode(',',$dates);
	  } 
	  if(!empty($schedule_date)) {
	    $schedule_date= "[".$schedule_date."]";
	  } else {
	    $schedule_date = "''";
	  }
	  ?>
	  <script>
	  	jQuery(document).ready(function($){
	        $('#<?php echo $field['id']; ?>').multiDatesPicker({
	          dateFormat: "mm/dd/yy",
	          addDates: <?php echo $schedule_date;?>,
	          firstDay: 1,
	          dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],
	          numberOfMonths: 1,
		      showButtonPanel: true,
	          onSelect:function(){
	            var allDates = $('#<?php echo $field['id']; ?>').multiDatesPicker('getDates');
	            $('[data-id="<?php echo $field['id']; ?>"]').val(allDates);
	          } 
	        });	       
	      });</script>
	  <?php
	}

	function add_in_front_end($settings){
        $fields = $settings['course_components']['fields'];
	    $arr=array(array(
	                'label'=> __('Attendance','wplms-attendance' ),
	                'text'=>__('Course Attendance ','wplms-attendance' ),
	                'type'=> 'conditionalswitch',
	                'hide_nodes'=> array('vibe_course_attendance_data', 'vibe_course_attendance_auto'),
	                'options'  => array('H'=>__('DISABLE','wplms-attendance' ),'S'=>__('ENABLE','wplms-attendance' )),
	                'style'=>'',
	                'id' => 'vibe_course_attendance',
	                'from'=> 'meta',
	                'default'=>'H',
	                'desc'=> __('Enable attendance and also set schedule below, If you want to mark attendance automatically for everyday, dont set schedule and enable Auto Attendance.','wplms-attendance' )
	                ),
	               array(
	                'label'=> __('Set Attendance Schedule','wplms-attendance' ),
	                'text'=>__('Choose dates for which you want to check presence','wplms-attendance' ),
	                'type'=> 'customdiv',
	                'style'=>'',
	                'id' => 'vibe_course_attendance_data',
	                'from'=> 'meta',
	                'desc'=> __('Choose date/ Date range','wplms-attendance' ),

	                'date_format' => 'Y-m-d'
	                ),
	             array(
	                'label'=> __('Auto Attendance','wplms-attendance' ),
	                'text'=>__('Enable Auto Attendance ','wplms-attendance' ),
	                'type'=> 'switch',
	                'options'  => array('H'=>__('DISABLE','wplms-attendance' ),'S'=>__('ENABLE','wplms-attendance' )),
	                'style'=>'',
	                'id' => 'vibe_course_attendance_auto',
	                'from'=> 'meta',
	                'default'=>'H',
	                'desc'=> __('Mark attendance automatically on course open. If no schedule is set then attendance will be automatically marked everyday on course open.','wplms-attendance' )
	                ),);


	    array_splice($fields, (count($fields)-2), 0,$arr );
	    $settings['course_components']['fields'] = $fields;  
	  return $settings;
	}

	function enqueue_the_scripts(){
		if(function_exists('bp_current_action')){
			$action = bp_current_action();
		}
		if(empty($action)){
			$action = $_GET['action'];
		}
		if(!is_page(vibe_get_option('create_course')) && (!is_singular('course') && $action != 'admin')){
			return;
		}

		if(function_exists('vibe_get_option')){
			$edit_course = vibe_get_option('create_course');
			if(function_exists('icl_object_id'))
				$edit_course = icl_object_id($edit_course, 'page', true);

			if(is_numeric($edit_course) && (is_page($edit_course))){

				wp_enqueue_script( 'jq-multidatespicker-js', plugins_url( '../assets/js/multidatespicker.js' , __FILE__ ), array( 'jquery','jquery-ui-core','jquery-ui-datepicker'  ) ,'1.6.6',true);
			}
		}

		add_action('wp_head',function(){
		    ?>
		    <style>
		    /* jQuery UI Datepicker moving pixels fix */
		    #vibe_course_attendance_data table.ui-datepicker-calendar, #attendance_calender table.ui-datepicker-calendar {border-collapse: separate;}
		    #attendance_calender.ui-datepicker-calendar td, #vibe_course_attendance_data.ui-datepicker-calendar td  {border: 1px solid transparent;}
		    /* jQuery UI Datepicker hide datepicker helper */
		    #vibe_course_attendance_data #ui-datepicker-div, #vibe_course_attendance_data #ui-datepicker-div {display:none;}
		    /* jQuery UI Datepicker emphasis on selected dates */
		    #vibe_course_attendance_data .ui-datepicker .ui-datepicker-calendar .ui-state-highlight a, #attendance_calender .ui-datepicker .ui-datepicker-calendar .ui-state-highlight a  {
		        background: <?php if(function_exists('vibe_get_customizer')) echo vibe_get_customizer('primary_bg'); ?> none;
		        color: white;
		    }
		    #vibe_course_attendance_data .ui-datepicker-today .ui-state-default {background: rgba(0,0,0,0.8);}
		    #vibe_course_attendance_data .ui-datepicker-inline{border:1px solid rgba(0,0,0,0.1);margin: 15px 0 30px;
			}#vibe_course_attendance_data .ui-datepicker-title{font-size:12px;text-transform:uppercase;letter-spacing:1px;background: rgba(0,0,0,0.8);}
			#vibe_course_attendance_data  .ui-state-default{background: #fff;height:auto;border: none;border-left: 1px solid rgba(0,0,0,0.05);border-top: 1px solid rgba(0,0,0,0.05);}
			#vibe_course_attendance_data  .ui-datepicker-calendar tr:nth-last-child(2) .ui-state-default{border-bottom:1px solid rgba(0,0,0,0.05);}
			#vibe_course_attendance_data  .ui-datepicker-calendar tr:last-child .ui-state-default{border-top:none;}

		    #vibe_course_attendance_data .ui-state-default, .ui-widget-content .ui-state-default, #attendance_calender .ui-state-default, .ui-widget-content .ui-state-default{width:100%;}

		    #attendance_calender .ui-datepicker-prev {
		      float: left !important;
		    }
		    .bulk_attendance_calender #attendance_save, .import__attendance_field #attendance_import_csv {margin: 10px 0 0 0 !important;}
		    .bulk_message, .bulk_add_students, .bulk_change_status, .bulk_assign_students, .bulk_extend_subscription_students, .bulk_attendance_calender {margin-bottom: 70px!important;}
		    .import__attendance_field, .bulk_attendance_calender{
				margin:24px 0 0;
				border-top:5px solid rgba(0,0,0,0.08);
				padding-top:10px;
			}
			.bulk_attendance_calender .type {margin: 10px 0 0 0; }
			.bulk_attendance_calender .type select {width: 43%; }
			#buddypress #import_attendance {margin-bottom: 0px;}
			#buddypress .import__attendance_field {margin-top: 0px;}

		    </style>
		    <?php
		});
	}

	function profile_attendance_count($student,$course_id){
		$user_dates = get_user_meta($student,'wplms_course_attendance_'.$course_id,true);
		
		if(!empty($user_dates)){
			$count = sizeof($user_dates);
			echo sprintf(_x('( Attendance: %s days present)','days prenset text in course admin','wplms-attendance'),$count);
		}
    }
	function add_attendance_touch_point ($settings){

		$settings['wplms_attendance'] = array(
           'label' => __('Attendance Record','vibe-customtypes'),
           'name' =>'wplms_attendance',
           'value' => array(
               'instructor' => admin_url('edit.php?taxonomy=bp-email-type&term=instructor_marked_attendance&post_type=bp-email'),
               'student' => admin_url('edit.php?taxonomy=bp-email-type&term=record_attendance_course&post_type=bp-email'),
           ),
           'type' => 'touchpoint',
           'hook' => '',
           'params'=>2,
       	);
		return $settings;
	}
}
Wplms_Attendance_Init::init();



