<?php

/**
 * WPLMS Attendance API
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	Wplms-Attendance/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if ( ! defined( 'BP_COURSE_ATTENDANCE_API_TYPE' ) )
   define( 'BP_COURSE_ATTENDANCE_API_TYPE', 'app' );
class Wplms_Attendance_API{

	public static $instance;
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Attendance_API();
        return self::$instance;
        
    }

	private function __construct(){
		if(!defined('WPLMS_API_NAMESPACE')){
			$this->namespace = BP_COURSE_API_NAMESPACE.'/attendance/';
		}else{
			$this->namespace = WPLMS_API_NAMESPACE.'/attendance/';
			
		}
		$this->type = BP_COURSE_ATTENDANCE_API_TYPE;  // use for App

		add_action( 'rest_api_init', array($this,'attendance_endpoints'));
		add_filter('bp_course_api_get_user_profile_tabs',array($this,'add_attendance_in_profile_tab'));
		add_filter('bp_course_api_get_user_profile_tab_attendance',array($this,'profile_attendance_courses'),10,4);
 
	}


	function attendance_endpoints(){
		register_rest_route( $this->namespace, '/user/(?P<id>\d+)?', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_user_attendance' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));

		register_rest_route( $this->namespace, '/instructor/(?P<id>\d+)?', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_instructor_attendance' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));

		register_rest_route( $this->namespace, '/user/(?P<id>\d+)?/courses', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_user_courses' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));

		register_rest_route( $this->namespace, '/instructor/(?P<id>\d+)?/courses', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_instructor_courses' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));


		register_rest_route( $this->namespace, '/instructor/(?P<id>\d+)/students/', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_instructor_students' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));
		

		register_rest_route( $this->namespace, '/course/(?P<id>\d+)?', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'get_course_details' ),
				'permission_callback' => array( $this, 'course_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));


		register_rest_route( $this->namespace, '/instructor/(?P<id>\d+)/search/', array(
			array(
				'methods'             =>  WP_REST_Server::READABLE,
				'callback'            =>  array( $this, 'search_instructor_students' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				'args'                     	=>  array(
					'id'                       	=>  array(
						'validate_callback'     =>  function( $param, $request, $key ) {
													return is_numeric( $param );
												}
					),
				),
			),
		));

		register_rest_route( $this->namespace, 'instructor/geteableddates/', array(
			array(
				'methods'             =>  'POST',
				'callback'            =>  array( $this, 'geteableddates' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				
			),
		));

		register_rest_route( $this->namespace, 'instructor/marattendance/', array(
			array(
				'methods'             =>  'POST',
				'callback'            =>  array( $this, 'marattendance' ),
				'permission_callback' => array( $this, 'attendance_request_validate' ),
				
			),
		));

		register_rest_route( $this->namespace, '/'.$this->type.'/course/(?P<id>\d+)', array(
			'methods'                   =>   WP_REST_Server::READABLE,
			'callback'                  =>  array( $this, 'get_user_course_attendance' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );
		register_rest_route( $this->namespace, '/'.$this->type.'/course/mark', array(
			'methods'                   =>   WP_REST_Server::CREATABLE,
			'callback'                  =>  array( $this, 'set_user_course_attendance' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );
	}



	function add_attendance_in_profile_tab($tabs){
		$newtabs = array();;
		foreach($tabs as $i=>$tab){
			if($tab['key'] == 'results'){
				$newtabs[]= array(
					'key'=>'attendance',
					'type'=> 'tab',
					'label'=>_x('Attendance','api','wplms-batches'),
					'value'=>'md-calendar',
				);
			}
			$newtabs[]=$tab;
		}
		return $newtabs;
	}
	
	function profile_attendance_courses($data,$user_id,$per_view,$paged){
		$user_id=(int)$user_id;
		$course_ids = bp_course_get_user_courses($user_id,'subscribe_course');
		$course_ids = array_unique($course_ids);
		$courses = array();
		if(!empty($course_ids) && count($course_ids) >= 1){
			foreach($course_ids as $course_id){
				// check here if attendance enable for course
				$check_att_enable = get_post_meta($course_id,'vibe_course_attendance',true);
				if($check_att_enable == 'S'){
					$courses[]=array('id'=>$course_id,'name'=>get_the_title($course_id));
				}
			}
		}
		$return = array();
		if(is_array($courses) && !empty($courses)){
			$return = array(
				'status' => 1,
				'message' =>_x('My-Courses found for Attendance','vibe'),
				'data' => $courses
			);
		}else{
			$return = array(
				'status' => 0,
				'message' =>_x('My-Courses not found for Attendance','vibe'),
				'data' => array()
			);
		}
		return $return;
	}

	function geteableddates($request){
		$post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);

        $course_id = $body['course'];
        if(!empty($course_id)){
        	$dates = get_post_meta($course_id,'vibe_course_attendance_data',true);
        	if(!empty($dates)){
        		$dates = explode(',', $dates);
        		if(!empty($dates)){
        			$return['status'] = true;
        			$return['data'] = $dates;
        		}else{
        			$return['status'] = false;
        		}
        	}else{
    			$return['status'] = false;
    		}
        }else{
        	$return['status'] = false;
        }
        return new WP_REST_Response( $return, 200 );
	}

	function marattendance($request){
		$post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);

        $course_id = $body['course_id'];
        $members = $body['members'];
        $dates = $body['dates'];
        $return  = array('status'=>false);
        $type =  $body['type'];
         if(empty($type)){
        	$return['message'] = _x('Type not set','','wplms-attendance');
        	return new WP_REST_Response( $return, 200 );
        }
        if(empty($course_id)){
        	$return['message'] = _x('Course not set','','wplms-attendance');
        	return new WP_REST_Response( $return, 200 );
        }
        if(empty($members)){
        	$return['message'] = _x('Please select students','','wplms-attendance');
        	return new WP_REST_Response( $return, 200 );
        }
        if(empty($dates)){
        	$return['message'] = _x('Please select a date to mark','','wplms-attendance');
        	return new WP_REST_Response( $return, 200 );
        }

        foreach ($members as  $user_id) {
        	$exisiting_dates = get_user_meta($user_id,'wplms_course_attendance_'.$course_id,true);
		    $user_data = array();
		    if(!empty($exisiting_dates)){
		      foreach ($exisiting_dates as $exisiting_date) {
		        $user_data[] = trim(date('m/d/Y', $exisiting_date));
		      }
		    }
		    foreach($dates as $date) {
		      if (($key = array_search($date, $user_data)) !== false) {
		            unset($user_data[$key]);
		        }
		    }
		    $meta_value= [];
		    if(!empty($user_data)) {
		    	foreach($user_data as $user_date) {
			      $meta_value[] = strtotime($user_date);
			    }
		    }
		    
		    if($type == 'present') {
		      foreach($dates as $date) {
		        $meta_value[] = strtotime($date);
		        do_action('wplms_mark_user_attendance',$user_id,$course_id,strtotime($date));
		      }
		    } else {
		      foreach($dates as $date) {
		        do_action('wplms_mark_user_attendance_remove',$user_id,$course_id,strtotime($date));
		      }
		    }

		    update_user_meta($user_id,'wplms_course_attendance_'.$course_id, $meta_value);	
        }
        $return['status'] = true;
        $return['message'] = sprintf(_x('Attendance updated for %s users','wplms-attendance'),
			count($members));
        return new WP_REST_Response( $return, 200 );
	}

	function get_instructor_courses($request){
		$user_id = $request->get_param('id');

		global $wpdb;
		$query = apply_filters('wplms_dashboard_courses_instructors',$wpdb->prepare("
          SELECT posts.ID as course_id
            FROM {$wpdb->posts} AS posts
            WHERE   posts.post_type   = 'course'
            AND   posts.post_author  = %d
          ",$user_id),$user_id);

        $course_ids=$wpdb->get_results($query,ARRAY_A);


		$courses = array();
		if(!empty($course_ids)){
			foreach($course_ids as $course_id){
				$courses[]=array('id'=>$course_id['course_id'],'name'=>get_the_title($course_id['course_id']));
			}
		}
		return new WP_REST_Response( $courses, 200 );

	}

	function get_user_courses($request){
		$user_id = $request->get_param('id');
		$course_ids = bp_course_get_user_courses($user_id,'subscribe_course');
		$courses = array();
		if(!empty($course_ids)){
			foreach($course_ids as $course_id){
				$courses[]=array('id'=>$course_id,'name'=>get_the_title($course_id));
			}
		}
		return new WP_REST_Response( $courses, 200 );
	}

	function attendance_request_validate($request){

		$body = json_decode($request->get_body(),true);
        
        if (empty($body['token']) && function_exists('vibebp_get_setting')){
            $client_id = $request->get_param('client_id');
            if($client_id == vibebp_get_setting('client_id')){
                return true;
            }
        }else{
            $token = $body['token'];
        }

        if(!empty($body['token'])){
            
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
            	$this->user_id = $this->user->id;
                return true;
            }
        }

		$user_id = $request->get_param('id');

		$security = $request->get_param('security');

		$user = get_userdata( $user_id );
		if ( $user === false ) {
		    return false;
		} else {
		   return true;
		}

		if($security == urlencode(vibe_get_option('security_key'))){
			return true;
		}

		return false;
	}


	function get_instructor_attendance($request){

		
		$user_id = $request->get_param('id');
		$count = $request->get_param('count');
		$paged = $request->get_param('page');
		$per_page = $request->get_param('per_page');	
		$course_id =$request->get_param('course_id');	
		$date_start = $request->get_param('date_start');	
		$date_end = $request->get_param('date_end');
		$student_id = $request->get_param('student_id');

		$activity_id_not_in = json_decode($request->get_param('activity_ids'),true);

		if(empty($paged)){
			$paged = 1;
		}


		if(empty($activity_id_not_in)){
			$activity_id_not_in = array();
		}


		$return = array();	

		add_filter('wplms_attendance_get_student_records_access',function(){return true;});

		$args = array('offset'=>($paged-1)*$per_page,'activity_id_not_in'=>$activity_id_not_in,'per_page'=>$per_page);	

		

		if(!empty($date_start) && !empty($date_end)){
			$args['date_start'] = $date_start;
			$args['date_end'] = $date_end;
		}

		

		if(!empty($course_id)){
			$args['course_ids'] = array($course_id);
		}else{
			$course_ids =array();
			global $wpdb;
			$query = apply_filters('wplms_dashboard_courses_instructors',$wpdb->prepare("
	          SELECT posts.ID as course_id
	            FROM {$wpdb->posts} AS posts
	            WHERE   posts.post_type   = 'course'
	            AND   posts.post_author  = %d
	          ",$user_id),$user_id);

			
	        $_course_ids=$wpdb->get_results($query,ARRAY_A);

	        if(!empty($_course_ids)){
	        	foreach($_course_ids as $i=>$course_id){
	        		$course_ids[$i]=$course_id['course_id'];
	        	}
	        }else{

	        	return new WP_REST_Response( $return, 200 );
	        }
	        
	        $args['course_ids']=$course_ids;
		}

		if(!empty($student_id) && is_numeric($student_id)){
			$args['user_id'] = $student_id;
		}else{
			$args['user_id']=0;
		}

		if(!empty($count)){
			
			unset($args['activity_id_not_in']);
			
			$max = wplms_attendance_get_student_records($args,true);
			$return = $max;
		}else{
			
			
			$return = wplms_attendance_get_student_records($args);
		}
		

		return new WP_REST_Response( $return, 200 );
	}


	function get_user_attendance($request){

		$user_id = $request->get_param('id');
		$count = $request->get_param('count');
		$paged = $request->get_param('page');
		$per_page = $request->get_param('per_page');	
		$course_id =$request->get_param('course_id');	
		$date_start = $request->get_param('date_start');	
		$date_end = $request->get_param('date_end');	

		$activity_id_not_in = json_decode($request->get_param('activity_ids'),true);

		if(empty($paged)){
			$paged = 1;
		}

		if(empty($activity_id_not_in)){
			$activity_id_not_in = array();
		}

		$return = array();	

		add_filter('wplms_attendance_get_student_records_access',function(){return true;});

		$args = array('user_id'=>$user_id,'offset'=>($paged-1)*$per_page,'activity_id_not_in'=>$activity_id_not_in,'per_page'=>$per_page);	

		

		if(!empty($date_start) && !empty($date_end)){
			$args['date_start'] = $date_start;
			$args['date_end'] = $date_end;
		}

		if(!empty($course_id)){
			$args['course_ids'] = array($course_id);
		}

		if(!empty($date)){
			$args['course_ids'] = array($course_id);
		}

		if(!empty($count)){
			
			unset($args['activity_id_not_in']);
			
			$max = wplms_attendance_get_student_records($args,true);

			$return = $max;
			
		}else{
			$return = wplms_attendance_get_student_records($args);
		}
		

		return new WP_REST_Response( $return, 200 );
	}


	function course_validate($request){
		return true;
	}


	function get_course_details($request){
		$course_id = $request->get_param('id');
		return new WP_REST_Response(array('id'=>$course_id,'name'=>get_the_title($course_id)), 200 );
	}

	function get_instructor_students($request){

		$user_ids = json_decode($request->get_param('students'));
		global $wpdb;

		$names = $wpdb->get_results("SELECT ID,display_name FROM {$wpdb->users} WHERE ID IN (".implode(',',$user_ids).")");

		$return = array();
		if(!empty($names)){
			foreach($names as $name){
				$return[] = array('id'=>$name->ID,'name'=>$name->display_name);
			}
		}
		return new WP_REST_Response( $return, 200 );
	}


	function search_instructor_students($request){
		$user_ids = json_decode($request->get_param('students'));
		$s = sanitize_text_field($request->get_param('s'));

		global $wpdb;

		$users = array();
		$results = $wpdb->get_results("SELECT ID, display_name FROM {$wpdb->users} WHERE user_login LIKE '%$s%' OR user_email LIKE '%$s%' OR display_name LIKE '%$s%' ");


		if(!empty($results)){
			foreach($results as $result){
				$users[]=array('id'=>$result->ID,'name'=>$result->display_name,'image'=>bp_core_fetch_avatar(array(
								'item_id' => $result->ID,
								'object'  => 'user',
								'type'=>'thumb',
								'html'	  => false
							)) );
			}
		}

		return new WP_REST_Response( $users, 200 );
	}


	public function get_user_permissions_check($request){
		$headers = vibe_getallheaders();
		if(isset($headers['Authorization'])){
			$token = $headers['Authorization'];
			$this->token = $token;
			$this->user_id = $this->get_user_from_token($token);
			if($this->user_id){
				return true;
			}
		}
		return false;
	}
	function get_user_from_token($token){
		global $wpdb;
		$user_id = $wpdb->get_var(apply_filters('wplms_usermeta_direct_query',"SELECT user_id FROM {$wpdb->usermeta} WHERE meta_key = '$token'"));
		if(is_numeric($user_id)){
			return $user_id;
		}
		return false;
	}
	function get_user_course_attendance($request){
		$course_id = $request->get_param('id');
		if(!empty($course_id) && function_exists('bp_course_is_member') && bp_course_is_member($course_id,$this->user_id)){
			// check here if attendance enable for course
			$check_att_enable = get_post_meta($course_id,'vibe_course_attendance',true);
			if($check_att_enable == 'S'){
				$attendances = get_user_meta($this->user_id,'wplms_course_attendance_'.$course_id,true);
				$all_attendance_date_format = array();
				if(!empty($attendances && is_array($attendances) && count($attendances)>=1)){
					foreach ($attendances as $key1 => $attendance) {
						$all_attendance_date_format[] = date('m/d/Y', $attendance);
					}
					array_unique($all_attendance_date_format);
				}
				// attendance available date to mark attemdance   // for all unmarked dates
				$check_att_enable_dates_str = get_post_meta($course_id,'vibe_course_attendance_data',true);
				$check_att_enable_dates_arr  = array();
				$unmarked_attendance_date_format = array();
				if($check_att_enable_dates_str){
					//convert string to array
					$check_att_enable_dates_arr = explode(",",$check_att_enable_dates_str);  
					if(is_array($check_att_enable_dates_arr) ){
						//$unmarked_attendance_date_format = $check_att_enable_dates_arr;
						$unmarked_attendance_date_format = $this->get_filtered_unmarked_attendance($check_att_enable_dates_arr,$all_attendance_date_format);
					}
				}	
				$return = array(
					'status' => 1,
					'message' => _x('Attendance found!','vibe'),
					'data' => array(
						'marked' => array(
							'date_format' => $all_attendance_date_format,
						),
						'unmarked' => array(
							'date_format' => $unmarked_attendance_date_format,
						)
					)
				);
			}else{
				$return = array(
					'status' => 0,
					'message' => _x('Attendance not enable for course','vibe'),
					'data' => null
				);
			}
		}else{
			$return = array(
				'status' => 0,
				'message' => _x('Course not Found for user!','vibe'),
				'data' => null
			);
		}
		return new WP_REST_Response($return, 200);
	}
	function get_filtered_unmarked_attendance($enable_dates_arr,$marked_dates_arr){
		$filter_arr_unmarked = array();
		foreach ($enable_dates_arr as $key => $value) {
			if (!in_array($value, $marked_dates_arr)){
				$filter_arr_unmarked[] = $value;
			}
		}
		return $filter_arr_unmarked;
	}
	function set_user_course_attendance($request){
		$post = json_decode($request->get_body());
		$course_ids = $post->course_ids;
		$user_id = $post->user_id; 
		$return = array();
		if($course_ids && is_array($course_ids) && count($course_ids)>=1 && $user_id && ($user_id == $this->user_id) ){
			$time = time();
			$date = date("m/d/Y");
			$marked_attendance_ids = array();
			$return = array(
				'status' => 1,
				'message' => _x('Attendance not marked','vibe'),
				'data' => array(
					'marked_attendance_ids' => $marked_attendance_ids,
					'date' => $date
				)
					
			);
			foreach ($course_ids as $key => $course_id) {
				// if user is course member
				if(function_exists('bp_course_is_member') && bp_course_is_member($course_id,$this->user_id)){
					// check here if attendance enable for course and for date
					//print_r('#1');
					$check_att_enable = get_post_meta($course_id,'vibe_course_attendance',true);
					$check_att_enable_dates_str = get_post_meta($course_id,'vibe_course_attendance_data',true);
					if($check_att_enable == 'S' && $check_att_enable_dates_str){
						//convert string to array
						//print_r('#2');
						$check_att_enable_dates_arr  = array();
						$check_att_enable_dates_arr = explode(",",$check_att_enable_dates_str);  
						
						if(is_array($check_att_enable_dates_arr) && in_array($date, $check_att_enable_dates_arr)){
							//get attendance for user for particular course id
							//print_r('#3');
							$attendances = get_user_meta($this->user_id,'wplms_course_attendance_'.$course_id,true);
							if(!empty($attendances) && is_array($attendances) && count($attendances)>=1){
								$all_attendance_date_format = array();
								foreach ($attendances as $key1 => $attendance) {
									$all_attendance_date_format[] = date('m/d/Y', $attendance);
								}
								//print_r('#4');
								if(is_array($all_attendance_date_format)){
									if(!in_array($date, $all_attendance_date_format)){
										//('#5');
										if(class_exists('Wplms_Attendance_Init')){
											$obj = Wplms_Attendance_Init::init();
											$t= $obj->mark_user_attendance($this->user_id,$course_id,$time);
											//print_r('#7');
											$marked_attendance_ids[] = $course_id;
											$return['message'] =  _x('Attendance Marked','vibe');
										}
									}
								}
							}else{
								if(class_exists('Wplms_Attendance_Init')){
									$obj = Wplms_Attendance_Init::init();
									$t= $obj->mark_user_attendance($this->user_id,$course_id,$time);
									//print_r('#7');
									$marked_attendance_ids[] = $course_id;
									$return['message'] =  _x('Attendance Marked','vibe');
								}
							}
						}
					}
				}
			}
			// here is marked attendance course is for current date
			$return['data'] = array(
								'marked_attendance_ids' => $marked_attendance_ids,
								'date' => $date
							);	
		}else{
			$return = array(
				'status' => 0,
				'message' => _x('Course Or User Not Found for course','vibe'),
				'data' => null
			);
		}
		return new WP_REST_Response($return, 200);
	}
}

Wplms_Attendance_API::init();
