<?php
/**
 * Initialise WPLMS Buy Batches
 *
 * @class       WPLMS_Buy_Batch_Class
 * @author      Vibethemes(H.K. Latiyan)
 * @category    Admin
 * @package     WPLMS-Batches/includes/buy_batch
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WPLMS_Buy_Batch_Class{

	public static $instance;
	public static function init(){
		
	    if ( is_null( self::$instance ) )
	        self::$instance = new WPLMS_Buy_Batch_Class();
	    return self::$instance;
	}

	private function __construct(){
		
		if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) || (function_exists('is_plugin_active') && is_plugin_active( 'woocommerce/woocommerce.php')) ){

			//Check LMS Setting status.

			if(class_exists('WPLMS_tips')){
				$tips = WPLMS_tips::init();
				if(isset($tips) && isset($tips->settings) && isset($tips->settings['enable_wplms_buy_batch_shortcode'])){
					add_action('wp_footer',array($this,'load_style_scripts'));
					add_shortcode('wplms_buy_batch', array($this,'wplms_buy_batch_shortcode'));
				}
			}
		}

	} // END public function __construct

	function wplms_buy_batch_shortcode($atts,$content = null){

		$defaults = array(
			'status'=>'private',
			'courses'=>array(),
			'seats'=>0,
			'buy_batch'=>1,
			'name_label'=> __('Name: ','wplms-batches'),
			'courses_label'=>__('Select Courses','wplms-batches'),
			'all_courses_label'=>__('Select All Courses Available','wplms-batches'),
			'seats_label'=>__('Batch Seats','wplms-batches'),
			'button_label'=>__('Buy Batch','wplms-batches'),
		);

		/* merge atts and defaults */
		$atts = wp_parse_args($atts,$defaults);
		
		ob_start();
		/* create buy batch form */
		?>
		<div class="wplms_buy_batch_form">
			<form method="post">
				<ul>
			<!-- Batch Name -->
					<li>
						<label><?php echo $atts['name_label']; ?></label>
						<input type="text" name="batch_name" class="batch_name form_field" placeholder="<?php echo $atts['name_label']; ?>">
					</li>
			<!-- Batch Courses -->
				<?php
				if(empty($atts['courses'])){
					?>
					<li>
						<label class="course_label"><?php echo $atts['courses_label']; ?></label>
						<select name="batch_course[]" class="batch_courses form_field chosen" multiple>
		                    <?php
		                        global $wpdb;
								$courses = $wpdb->get_results("SELECT m.post_id as id,p.post_title as title,m.meta_value as seat_price FROM {$wpdb->postmeta} as m LEFT JOIN {$wpdb->posts} as p ON p.id = m.post_id WHERE m.meta_key = 'wplms_price_per_batch_seat' and m.meta_value > 0");

								if(!empty($courses)){
								  foreach ($courses as $course){
								  	echo '<option class="price" data-id="'.$course->seat_price.'" value="' . $course->id . '"> '.$course->title . '</option>';
								  }
								}
		                    ?>
						</select>
						<input type="checkbox" name="all_courses" class="all_courses" id="all_courses" value="1" /><label for="all_courses"><?php echo $atts['all_courses_label']; ?></label>
						
					</li>
					<?php
				}else{
					?>
					<li><label class="course_label"><?php echo $atts['courses_label']; ?></label>
						<select name="batch_course[]" class="batch_courses form_field chosen" multiple>
		                    <?php
			                    if(!is_array($atts['courses'])){
			                    	$atts['courses'] = explode(',',$atts['courses']);
			                    }

		                    	foreach ($atts['courses'] as $course){
									$price = get_post_meta($course,'wplms_price_per_batch_seat',true);
									if(!empty($price)){
										echo '<option class="price" data-id="'.$price.'" value="' . $course . '"> '.get_the_title($course) . '</option>';
									}
								}
		                    ?>
						</select>
						<input type="checkbox" name="all_courses" class="all_courses" id="all_courses" value="1" /><label for="all_courses"><?php echo $atts['all_courses_label']; ?></label>
						
					</li>
					<?php
				}
				
				?>
				
			<!-- Batch Seats -->	
				<?php
				if($atts['seats'] == 0){
					?>
					<li><label><?php echo $atts['seats_label']; ?></label>
						<input type="number" name="batch_seats" class="batch_seats" placeholder="<?php _e('Batch seats','wplms-batches'); ?>">
					</li>
					<?php
				}else{
					?>
					<li><label><?php echo $atts['seats_label']; ?></label>
						<span  class="batch_seats" data-seats="<?php echo $atts['seats']; ?>"><?php echo $atts['seats']; ?></span>
					</li>
					<?php
				}
				?>
				</ul>
			<!-- Check Woocommerce Currency Symbol -->
				<?php
				global  $woocommerce;
				if(function_exists('get_woocommerce_currency_symbol'))
   					$currency_symbol = get_woocommerce_currency_symbol();
   				echo '<div class="currency_symbol" style="display:none;">'.$currency_symbol.'</div>';
				?>
				
			<!-- Buy Batch Button -->
				<?php echo '<a class="button-primary button disabled wplms_buy_batch">'.$atts['button_label'].'</a>'; ?>

				<input type="hidden" class="batch_status" data-status="<?php echo $atts['status']; ?>">
				<input type="hidden" class="buy_batch" data-batch="<?php echo $atts['buy_batch']; ?>">
			</form> <!-- End of Buy Batch Form -->
		</div>
		<?php

		$return = ob_get_clean();
		return $return;
	}

	function load_style_scripts(){
		?>
		<style>
			.wplms_buy_batch_form ul{list-style:none;}
			.wplms_buy_batch_form ul li{display:inline-block;width: 100%;}
			.wplms_buy_batch_form ul li+li{margin: 10px 0;}
			.wplms_buy_batch_form .select2-container,
			.wplms_buy_batch_form select,.wplms_buy_batch_form input:not(.all_courses){display:block;width:100%;}
			.wplms_buy_batch_form label:not(for="all_courses") {width:100%;display:block;}
			label[for="all_courses"]{margin-left:5px;}
			.batch_price_li {list-style:none;color:#70c989;
				font-size: 18px;font-weight: 700;}
			.wplms_buy_batch_form .button.disabled {opacity:0.6;}
		</style>
		<script>
			jQuery(document).ready(function($){

		      	$(".all_courses").on('click',function(){
		      		var $this = $(this);
		      		if($this.is(':checked') ){
				        $this.closest('.wplms_buy_batch_form').find(".batch_courses > option").prop("selected","selected");// Select All Options
				        $this.closest('.wplms_buy_batch_form').find(".batch_courses").trigger("change");// Trigger change to select 2
				    }else{
				        $this.closest('.wplms_buy_batch_form').find(".batch_courses > option").removeAttr("selected");// Deselect All Options
				        $this.closest('.wplms_buy_batch_form').find(".batch_courses").trigger("change");// Trigger change to select 2
				    }
		      	});

		      	$('.batch_seats,.batch_courses').on('change',function(){
		            var $this = $(this);
		            var batch_seats = $this.closest('.wplms_buy_batch_form').find('.batch_seats').val();
		            if(!batch_seats){
		            	batch_seats = $this.closest('.wplms_buy_batch_form').find('span.batch_seats').data('seats');
		            }
		            if(!batch_seats){
		            	batch_seats = 0;
		            }
		            var batch_courses = $this.closest('.wplms_buy_batch_form').find('.batch_courses').val();
		            var batch_price = 0;

		            // Calculate Pricee
		            $.each(batch_courses,function(k,v){
		                 var course_price = $this.closest('.wplms_buy_batch_form').find('.batch_courses .price[value="'+v+'"]').attr('data-id');
		                 var price = batch_seats * course_price;
		                 batch_price += price;
		            });

		            // Remove class disabled if the price is not 0
		            if(batch_price != 0){
		                  $this.closest('.wplms_buy_batch_form').find('.wplms_buy_batch').removeClass('disabled');
		            }else{
		                  $this.closest('.wplms_buy_batch_form').find('.wplms_buy_batch').addClass('disabled');
		            }
		            
		            // Display Price
		            var currency_symbol = $this.closest('.wplms_buy_batch_form').find('.currency_symbol').text();
		            if( !$this.closest('.wplms_buy_batch_form').find('.batch_price').length ){
		                  $this.closest('.wplms_buy_batch_form').find('.batch_seats').after('<li class="batch_price_li"><label>Total Price</label><span class="batch_price"></span></li>');
		            }
		            $this.closest('.wplms_buy_batch_form').find('.batch_price').text(currency_symbol+batch_price);
		      	});

		      	$('.wplms_buy_batch').on('click',function(){
		      		var $this = $(this);
		            // Return if class is disabled
		            if($this.hasClass('disabled')){
		                  return;
		            }
		            
		            // Change Button Text
		            $(this).text('.....');

		            // Define Variables
					var batch_name = $this.closest('.wplms_buy_batch_form').find('.batch_name').val();
					var batch_courses = $this.closest('.wplms_buy_batch_form').find('.batch_courses').val();
			            var batch_seats = $this.closest('.wplms_buy_batch_form').find('.batch_seats').val();
					if(batch_seats == 'undefined'){
		                  batch_seats = $this.closest('.wplms_buy_batch_form').find('.batch_seats').attr('data-seats');
		            }
		            var batch_status = $this.closest('.wplms_buy_batch_form').find('.batch_status').attr('data-status');
		            var buy_batch = $this.closest('.wplms_buy_batch_form').find('.buy_batch').attr('data-batch');

		            // Ajax Call
		            $.ajax({
						type: "POST",
						url: ajaxurl,
						data: { action: 'buy_wplms_batch',
								batch_name: batch_name,
								batch_courses: batch_courses,
								batch_seats: batch_seats,
						        batch_status: batch_status,
						        buy_batch: buy_batch,
						    },
						cache: false,
						success: function (html) {
						    window.location.href = html;
						}
		            });
				});

			});

		</script>
		<?php
	}

} // End of class WPLMS_Buy_Batch_Class

add_action('plugins_loaded',function(){WPLMS_Buy_Batch_Class::init();},99);
