<?php
/**
 * Initialise WPLMS Buy Batches
 *
 * @class       Wplms_Buy_Batch_Filters
 * @author      Vibethemes(H.K. Latiyan)
 * @category    Admin
 * @package     WPLMS-Batches/includes/buy_batch
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_Buy_Batch_Filters{

	public static $instance;
	public static function init(){

	    if ( is_null( self::$instance ) )
	        self::$instance = new Wplms_Buy_Batch_Filters();
	    return self::$instance;
	}

	private function __construct(){

		//Check LMS Setting status.

		if(class_exists('WPLMS_tips')){
			$tips = WPLMS_tips::init();
			if(isset($tips) && isset($tips->settings) && isset($tips->settings['enable_wplms_buy_batch_shortcode'])){
				
				add_filter('wplms_course_metabox',array($this,'add_price_per_batch_seat_backend'));
				add_filter('wplms_course_creation_tabs',array($this,'add_price_per_batch_seat_frontend'));
				add_filter('wplms_batches_settings_array',array($this,'hide_batch_course_seats_for_purchased_batch'),10,2);
			}
		}

	} // END public function __construct

	function add_price_per_batch_seat_backend($settings){
		global  $woocommerce;
		if(function_exists('get_woocommerce_currency_symbol')){
			$currency_symbol = get_woocommerce_currency_symbol();
			
			$settings['wplms_price_per_batch_seat']=array( // Text Input
			      'label' => __('Price Per Batch Seats','wplms-batches').' ( '.$currency_symbol.' )', // <label>
			      'desc'  => '', // description
			      'id'  => 'wplms_price_per_batch_seat', // field id and name
			      'type'  => 'number', // type of field
			      'std' => 0
			    );
		}
		return $settings;
	}

	function add_price_per_batch_seat_frontend($settings){
		$fields = $settings['course_settings']['fields'];
		global  $woocommerce;
		if(function_exists('get_woocommerce_currency_symbol'))
			$currency_symbol = get_woocommerce_currency_symbol();
		$arr = array(array(
				'label'=> __('Batch Seat Price','wplms-batches' ).' ( '.$currency_symbol.' )',
				'text'=>__('Price Per Batch Seats','wplms-batches' ),
				'type'=> 'number',
				'style'=>'',
				'id' => 'wplms_price_per_batch_seat',
				'from'=> 'meta',
				'default'=>0,
			));
		array_splice($fields, (count($fields)-5), 0,$arr );
		$settings['course_settings']['fields'] = $fields;
	    return $settings;
	}

	function hide_batch_course_seats_for_purchased_batch($settings,$group_id){

		if( !current_user_can('manage_options') ){
			$batch_course_seats = groups_get_groupmeta($group_id,'buy_batch_course_seats',true);
			if(!empty($batch_course_seats) && $batch_course_seats == 1){

				unset($settings['batch_course']);
				unset($settings['enable_seats']);
				unset($settings['batch_seats']);
			}
		}

		return $settings;
	}

} // End of class Wplms_Buy_Batch_Filters

add_action('plugins_loaded',function(){Wplms_Buy_Batch_Filters::init();},99);
