<?php
/**
 * Initialise WPLMS Batches
 *
 * @class       Wplms_Batches_Actions
 * @author      VibeThemes
 * @category    Admin
 * @package     WPLMS-Batches/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class Wplms_Batches_Actions{


	public static $instance;
	
	public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Batches_Actions();
        return self::$instance;
    }

	private function __construct(){
		add_action('groups_membership_accepted',array($this,'add_user_to_course_from_join_request'),10,2);
		add_action('groups_join_group',array($this,'add_user_to_course'),10,2);
		add_action('groups_leave_group',array($this,'remove_from_batch_course'),10,2);

		add_action('wp_ajax_enroll_user_to_course_batch',array($this,'enroll_user_to_batch'));
		add_action('wplms_course_before_front_main',array($this,'enroll_error_message'));

		//Start Course errors
		add_action('wplms_before_start_course',array($this,'enroll_error'));

		add_action('wplms_tips_default',array($this,'enable_batch_course_visibility_switch'),10,1);

		add_action('bp_activity_post_form_options',array($this,'batch_news'));
		add_action( 'bp_groups_posted_update', array($this,'group_news'),10,4);

		add_action('wplms_the_course_button',array($this,'force_enroll_when_enabled'),10,2);

		/* === FRONT END CURRICULUM === */
		add_action('wplms_course_curriculum_front_end_generator',array($this,'load_save_batch_curriculum'),10,1);
        add_Action('wp_ajax_get_course_batches',array($this,'get_course_batches'));
        add_action('wp_footer',array($this,'batch_curriculum_js'));
        add_action('wp_ajax_save_course_batch_curriculum',array($this,'save_course_batch_curriculum'));
        add_action('wp_ajax_load_course_batch_curriculum',array($this,'load_course_batch_curriculum'));
    	
        add_action('woocommerce_add_order_item_meta',array($this,'add_batch_id_to_order_item_meta'),1,2);

  		add_action('wplms_course_product_puchased',array($this,'enable_batch_access'),10,6);
  		add_action('woocommerce_before_cart_item_quantity_zero',array($this,'remove_batch_from_cart'),1,1);

  		//Add members to groups
  		add_action('bp_after_group_manage_members_admin',array($this,'add_group_members_front_end'));
  		add_action('wp_ajax_select_users_for_group',array($this,'select_users_for_group'));
  		add_action('wp_ajax_add_members_in_group',array($this,'add_members_in_group'));
	}

	function add_group_members_front_end(){
		global $bp;

		if(!is_user_logged_in())
			return;

		if( !current_user_can('manage_options') ){
			$user_id = get_current_user_id();
			$admins = bp_group_admin_ids();

			if(is_numeric($admins)){
				$admins = array($admins);
			}else{
				$admins = explode(',',$admins);
			}

			if( !in_array($user_id,$admins) )
				return;

			$group_id = $bp->groups->current_group->id;
			$enable_seats  = groups_get_groupmeta( $group_id, 'enable_seats' );	
			$seats  = (int)groups_get_groupmeta( $group_id, 'batch_seats' );
			if(!empty($enable_seats) && !empty($seats)){
				$total_member_count = (int)groups_get_groupmeta( $group_id, 'total_member_count' );
				if($total_member_count >= $seats)
					return;
			}
	
		}

		

		
		?>
		<div class="add_members_in_group">
		<h3 class="heading"><span><?php _e('Add member','wplms-batches'); ?></span></h3>
		<style>.select2-search__field{width:240px !important;}#group_search_students{width:300px;}</style>
			<select id="group_search_students" data-placeholder="<?php _e('Enter student name to add','wplms-batches'); ?>" data-id="<?php echo $bp->groups->current_group->id; ?>" multiple>
			</select>
			<a id="add_group_searched_students" class="button"><?php _e('Add Member','wplms-batches'); ?></a>
		</div>
		<script>
			jQuery(document).ready(function($){
				var data_id = [];
				$('#group_search_students').select2({
					minimumInputLength: 4,
			        placeholder: $(this).attr('data-placeholder'),
			        closeOnSelect: true,
			        language: {
			          inputTooShort: function() {
			            return vibe_course_module_strings.enter_more_characters;
			          }
			        },
			        ajax: {
			            url: ajaxurl,
			            type: "POST",
			            dataType: 'json',
			            delay: 250,
			            data: function(term){ 
			                    return  {   action: 'select_users_for_group', 
			                                security: '<?php echo wp_create_nonce($bp->groups->current_group->id); ?>',
			                                group: <?php echo $bp->groups->current_group->id;?>,
			                                q: term,
			                            }
			            },
			            processResults: function (data) {
			                return {
			                    results: data
			                };
			            },       
			            cache:true  
			        },
			        templateResult: function(data){
			            return '<img width="32" src="'+data.image+'">'+data.text;
			        },
			        templateSelection: function(data){
						if(!data_id.includes(data.id)){
							data_id.push(data.id);
						}
			            return '<img width="32" src="'+data.image+'">'+data.text;
			        },
			        escapeMarkup: function (m) {
			            return m;
			        }
			    }).on('select2:open',function(){
			      if($('.select2-container .select2-dropdown').hasClass('select2-dropdown--below')){
			        var topmargin = 45;
			        $('.select2-container:not(.select2)').css('top', '+='+ topmargin +'px');
			      }
			    });

			    $('#add_group_searched_students').on('click',function(){
			    	$(this).text('.....');
			    	$.ajax({
						type: "POST",
						url: ajaxurl,
						data: { action: 'add_members_in_group',
								security: '<?php echo wp_create_nonce($bp->groups->current_group->id); ?>',
			                    group: <?php echo $bp->groups->current_group->id;?>,
			                    user_id: data_id,
						    },
						cache: false,
						success: function (data) {
						    if(!data.error) location.reload(true);
						}
		            });
			    });

			});
		</script>
		<?php

	}


	function load_save_batch_curriculum(){

        if(!class_exists('wplms_miscellaneous_settings'))
            return;

        $settings = wplms_miscellaneous_settings::init();
        if(empty($settings) || empty($settings->settings) || empty($settings->settings['general']['enable_batch_curriculums']))
            return;
        
        ?>
        <select id="wplms_course_batch_curriculum" data-placecolder="<?php _e('Search for course batches','wplms-batches'); ?>">
        </select>
        <a id="load_batch_curriculum" class="button"><?php _e('Load Curriculum','wplms-batches') ?></a><a id="save_batch_curriculum" class="button"><?php _e('Save Curriculum','wplms-batches') ?></a>
        <?php
    }

    function get_course_batches(){
        $course_id = $_POST['course_id'];
        if ( ! wp_verify_nonce( $_POST['security'], 'security' )) {
            _e('Security check failed !','wplms-batches');
            die();
        } 

        $batches = wplms_get_course_batches($course_id);
        $return =array();

        if(!empty($batches)){
            foreach($batches as $batch_id){
                $group = groups_get_group( array('group_id' => $batch_id,'populate_extras' => false,'update_meta_cache' => false) );
                $return[] = array('id'=>$group->id,'text'=>$group->name);
            }
        }
        print_r(json_encode($return));
        die();
    }


    function batch_curriculum_js(){
        if(function_exists('vibe_get_option')){
            $edit_course = vibe_get_option('create_course');
            if( function_exists('icl_object_id') ){
            	$edit_course = icl_object_id($edit_course,'page',true);
            }
            if(!empty($edit_course) && is_page($edit_course)){
                wp_enqueue_script( 'wplms_batches_front_end', plugins_url('../assets/js/front_end.js',__FILE__),array('jquery','jquery-ui-sortable'),WPLMS_BATCHES_VERSION);
            }
        }
    }

    function load_course_batch_curriculum(){
        $user_id= get_current_user_id();
        $course_id =$_POST['course_id'];
        $batch_id = $_POST['batch_id'];
        if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'security')  || !current_user_can('edit_posts') || !function_exists('groups_get_groupmeta')){
             _e('Security check Failed. Contact Administrator.','wplms-front-end');
             die();
        }
         if(!is_numeric($course_id) || get_post_type($course_id) != 'course'){
            _e('Invalid Course id, please edit a course','wplms-front-end');
             die();
        }
        $course_author = get_post_field('post_author',$course_id);
        if($course_author != $user_id && !current_user_can('manage_options')){
            _e('Invalid Course Instructor','wplms-front-end');
             die();
        }
        $curriculum = groups_get_groupmeta($batch_id,'vibe_course_curriculum_'.$course_id,true);
        if(isset($curriculum) && is_array($curriculum)){
            foreach($curriculum as $kid){
                if(is_numeric($kid)){
                    if(get_post_type($kid) == 'unit'){
                        echo '<li><strong class="title" data-id="'.$kid.'"><i class="icon-file"></i> '.get_the_title($kid).'</strong>
                                <ul class="data_links">
                                    <li><a class="edit" title="'.__('Edit Unit','wplms-front-end' ).'"><span class="dashicons dashicons-edit"></span></a></li>
                                    <li><a class="preview" title="'.__('Preview','wplms-front-end' ).'" target="_blank"><span class="dashicons dashicons-visibility"></span></a></li>
                                    <li><a class="remove" title="'.__('Remove','wplms-front-end' ).'"><span class="dashicons dashicons-no-alt"></span></a></li>
                                    <li><a class="delete" title="'.__('Delete','wplms-front-end' ).'"><span class="dashicons dashicons-trash"></span></a></li>
                                </ul>
                            </li>';
                    }else{
                        echo '<li><strong class="title" data-id="'.$kid.'"><i class="icon-task"></i> '.get_the_title($kid).'</strong>
                                <ul class="data_links">
                                    <li><a class="edit" title="'.__('Edit Quiz','wplms-front-end' ).'"><span class="dashicons dashicons-edit"></span></a></li>
                                    <li><a class="preview" title="'.__('Preview','wplms-front-end' ).'" target="_blank"><span class="dashicons dashicons-visibility"></span></a></li>
                                    <li><a class="remove" title="'.__('Remove','wplms-front-end' ).'"><span class="dashicons dashicons-no-alt"></span></a></li>
                                    <li><a class="delete" title="'.__('Delete','wplms-front-end' ).'"><span class="dashicons dashicons-trash"></span></a></li>
                                </ul>
                              </li>';
                    }
                }else{
                    echo '<li class="new_section"><strong>'.$kid.'</strong>
                            <ul class="data_links">
                                <li>
                                    <a class="remove" title="'.__('Remove','wplms-front-end' ).'"><span class="dashicons dashicons-no-alt"></span></a>
                                </li>
                            </ul>
                        </li>';
                }
            }
        }else{
            echo 1;
        }
        die();
    }

    function save_course_batch_curriculum(){
        $user_id= get_current_user_id();
        $course_id =$_POST['course_id'];
        $batch_id = $_POST['batch_id'];
        if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'security')  || !current_user_can('edit_posts')){
             _e('Security check Failed. Contact Administrator.','wplms-front-end');
             die();
        }
         if(!is_numeric($course_id) || get_post_type($course_id) != 'course'){
            _e('Invalid Course id, please edit a course','wplms-front-end');
             die();
        }
        $course_author = get_post_field('post_author',$course_id);
        if($course_author != $user_id && !current_user_can('manage_options')){
            _e('Invalid Course Instructor','wplms-front-end');
             die();
        }
        $objcurriculum = json_decode(stripslashes($_POST['curriculum']));
        if(is_array($objcurriculum) && isset($objcurriculum))
        foreach($objcurriculum as $c){
            $curriculum[]=$c->id;
        }
        
       // $curriculum=array(serialize($curriculum)); // Backend Compatiblity
        groups_update_groupmeta( $batch_id, 'vibe_course_curriculum_'.$course_id,$curriculum);
        echo $course_id;
        //do_action('wplms_course_curriculum_updated',$course_id,$curriculum);
        
        die();
    }

    /* === END BATCH  CURRICULUM === */
	function force_enroll_when_enabled($course_id,$user_id){
		
		$force_batch_enrolment = get_post_meta($course_id,'vibe_force_batch_enrolment',true);
		if(!empty($force_batch_enrolment) && $force_batch_enrolment == 'S'){
			
			$taken = bp_course_get_user_course_status($user_id,$course_id);
			if(!empty($taken))
				return;

			$batch_ids = wplms_get_course_batches($course_id);
			if(!empty($batch_ids)){
				foreach($batch_ids as $batch_id){
					if(groups_is_user_member( $user_id, $batch_id)){
						bp_course_add_user_to_course($user_id,$course_id);
						return;
					}
				}
			}
		}
	}


	function group_news($content, $user_id, $group_id, $activity_id ){
		if(is_wplms_batch($group_id)){
			$str = _e('News','wplms-batches').' :';
			if(strpos($content,$str) !== false){
				bp_activity_update_meta($activity_id,'group_news',1);	
			}
		}
	}

	function batch_news(){
		if(!is_user_logged_in())
			return;

		$user_id  = get_current_user_ID();
		global $bp;
		
		if(bp_is_group_activity() && is_wplms_batch($bp->groups->current_group->id) &&  groups_is_user_mod($user_id,$bp->groups->current_group->id) ||groups_is_user_admin($user_id,$bp->groups->current_group->id)){
			?>
			<div class="checkbox" style="display:inline-block;">
				<input type="checkbox" id="group_news" />
				<label for="group_news"><?php _e('News','wplms-batches'); ?></label>
			</div>
			<script>
			jQuery(document).ready(function($){
				$('#group_news').on('click',function(){
					var val = $('#whats-new').val();
					var str = $.trim($(this).parent().text());
					var patt = new RegExp(str+' :');
					if (patt.test(val)){
						str=str+' :';
					 	val = val.replace(str,'');
					}else{
						val = str+' :'+val;
					}
					$('#whats-new').val(val);
				});
				$('#whats-new').on('mouseup',function(){
					var val = $('#whats-new').val();
					var str = $.trim($(this).parent().text());
					var patt = new RegExp(str+' :');
					if (patt.test(val)){
					 	$('#group_news').prop('checked', true);
					}else{
						$('#group_news').prop('checked', false);
					}
				});
			});
			</script>
			<?php
		}
	}
	function enable_batch_course_visibility_switch($key){
		if($key == 'enable_batch_course_visibility_switch'){
			add_filter('wplms_batches_enable_batch_course_visibility_switch',array($this,'wplms_batches_enable_batch_course_visibility_switch'));
		}
		if($key == 'enable_batch_course_auto_susbcribe'){
			add_filter('wplms_auto_subscribe',array($this,'batch_course_auto_subscribe'),10,2);
			add_action('bp_group_admin_edit_after',array($this,'wplms_batches_add_member_to_course'));
		}
	}

	function wplms_batches_enable_batch_course_visibility_switch($flag){
		return 1;
	}

	function batch_course_auto_subscribe($auto_subscribe,$course_id){
		if(!is_user_logged_in())
			return $auto_subscribe;

		$user_id = get_current_user_id();
		$coursetaken = bp_course_get_user_expiry_time($user_id,$course_id);
		if(empty($coursetaken)){
			$batches = wplms_get_course_batches($course_id);
			if(!empty($batches)){
				$batch_list = implode(',',$batches);
				global $wpdb,$bp;
				$result = $wpdb->get_var("SELECT count(group_id) FROM {$bp->groups->table_name_members} WHERE user_id = $user_id AND group_id IN ($batch_list)");
				if(!empty($result)){ // Means user is subscribed to at least one course batch
					return 1;
				}
			}
		}

		return $auto_subscribe;
	}

	function enroll_error(){
		if(isset($_POST['course_id'])){
		    $course_id=$_POST['course_id'];
		}else if(isset($_COOKIE['course'])){
		      $course_id=$_COOKIE['course'];
		}
		if(!empty($course_id)){
			$user_id = get_current_user_ID();

			$batches = wplms_get_course_batches($course_id);
			if(!empty($batches)){
				$user_id = get_current_user_ID();
				foreach($batches as $batch_id){

					if(wplms_batch_is_user_member( $user_id, $batch_id )){
						$return = wplms_batch_timing_accessibility($batch_id,$course_id);
						if(!$return['return']){
							$link = get_permalink($course_id).'?error=unavailable&batch='.$batch_id.'&message='.urlencode($return['message']);
							wp_redirect($link);
							exit;
						}
						break;
					}
				}
			}
			
		}

	}
	function enroll_error_message(){
		if(isset($_REQUEST['error']) && $_REQUEST['error'] == 'enroll'){ 
			 echo '<div id="message" class="notice"><p>'.__('Enroll in a batch to start the course. ','wplms-batches').'</p></div>';
 		}
 		if(isset($_REQUEST['error']) && $_REQUEST['error'] == 'unavailable'){ 
 			global $post;

 			if(is_numeric($_GET['batch']) && wplms_course_batch_verify($post->ID,$_GET['batch']))
			 echo '<div id="message" class="notice"><p>'.urldecode($_GET['message']).'</p></div>';
 		}
	}

	function enroll_user_to_batch(){

      	$course_id = $_POST['course_id'];
      	$batch_id = $_POST['batch'];
      	$user_id = get_current_user_ID();


      	if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'enroll_now'.$batch_id.$user_id) || !is_numeric($course_id) || !is_numeric($batch_id)){

         	_e('Security check Failed. Contact Administrator.','vibe');
         	die();
      	}

      	if(bp_course_is_member($course_id,$user_id) && wplms_course_batch_verify($course_id,$batch_id)){
      		groups_join_group($batch_id, $user_id ); 
      		echo $batch_id; //Numeric value
      	}else{
      		_e('User not in course','wplms-batches');
      	}

      	die();
	}

	function add_user_to_course_from_join_request($user_id,$group_id){
		$auto_assign = apply_filters('wplms_batches_auto_assign_to_course',0);
		if($auto_assign){
			$this->add_user_to_course($group_id,$user_id);	
		}
		

	}

	function add_user_to_course($group_id,$user_id){
		
		if(is_wplms_batch($group_id)){
			$course_ids = get_batch_courses($group_id);
			if(!empty($course_ids)){
				$enable_batch_duration = groups_get_groupmeta($group_id,'enable_batch_duration');
			
				if(!empty($enable_batch_duration)){
					$batch_duration = groups_get_groupmeta($group_id,'batch_duration');
					$batch_duration_parameter = groups_get_groupmeta($group_id,'batch_duration_parameter');
					$batch_duration_parameter = wplms_process_batch_duration($batch_duration_parameter);
					$t = $batch_duration*$batch_duration_parameter;
				}
				
				foreach($course_ids as $course_id){
					if(function_exists('bp_course_is_member') && !bp_course_is_member($course_id,$user_id)){
						if(!empty($enable_batch_duration)){
							bp_course_add_user_to_course($user_id,$course_id,$t,1);
						}else{
							bp_course_add_user_to_course($user_id,$course_id);
						}
						
					}
				}
			}
		}
	}

	function remove_from_batch_course($group_id,$user_id){

		if(is_wplms_batch($group_id)){
			$course_ids = get_batch_courses($group_id);
			if(!empty($course_ids)){
				foreach($course_ids as $course_id){
					if(function_exists('bp_course_is_member') && bp_course_is_member($course_id,$user_id)){
						bp_course_remove_user_from_course($user_id,$course_id);
					}
				}
			}
		}
	}

	function add_batch_id_to_order_item_meta($item_id, $values){

        global $woocommerce,$wpdb;
        $user_custom_values = $values['batch_id'];
        if(!empty($user_custom_values)){
            wc_add_order_item_meta($item_id,'batch_id',$user_custom_values);  
        }
  	}

  	function enable_batch_access($course_id,$user_id,$a,$b,$product_id,$item_id=null){	
		
		if(empty($item_id)){return;}
		
		$batch_id = wc_get_order_item_meta($item_id,'batch_id',true);
		if(is_numeric($batch_id) && bp_course_is_member($course_id,$user_id) && wplms_course_batch_verify($course_id,$batch_id)){
			groups_join_group($batch_id, $user_id );
		}

	}

    function remove_batch_from_cart($cart_item_key){

        global $woocommerce;
        $cart = $woocommerce->cart->get_cart();
        foreach( $cart as $key => $values){
        	if ( $values['batch_id'] == $cart_item_key ){
            	unset( $woocommerce->cart->cart_contents[ $key ] );
        	}
        }
    }

    function wplms_batches_add_member_to_course($group_id){

    	$user_names = array();
		if ( ! empty( $_POST['bp-groups-new-members'] ) ) {
			$user_names = array_merge( $user_names, explode( ',', $_POST['bp-groups-new-members'] ) );
		}

		if(!empty($user_names)){
			foreach( array_values( $user_names ) as $user_name ) {
				$un = trim( $user_name );
				$user = get_user_by( 'slug', $un );
				$user_id = $user->ID;
			}

			if(class_exists('WPLMS_Batch_Functions')){
	    		$batch = WPLMS_Batch_Functions::init();
	    		if( isset($batch) ){
					if(is_wplms_batch($group_id)){
						$courses = get_batch_courses($group_id);
						foreach ($courses as $course_id){
							if(function_exists('bp_course_add_user_to_course')){
								bp_course_add_user_to_course($user_id,$course_id);
							}
						}
					}
				}
	    	}
		}
    }


    function select_users_for_group(){
    	$group_id = $_POST['group'];
    	if(!isset($_POST['security']) || !wp_verify_nonce($_POST['security'],$group_id)){
    		_e('Security check failed !','wplms-batches');
            die();
    	}

    	global $wpdb,$bp;
	    $term = $_POST['q']['term'];
	    
	    $q = "
	        SELECT ID, display_name FROM {$wpdb->users} 
	        WHERE (
	          user_login LIKE '%$term%'
	          OR user_nicename LIKE '%$term%'
	          OR user_email LIKE '%$term%' 
	          OR user_url LIKE '%$term%'
	          OR display_name LIKE '%$term%'
	          ) AND ID NOT IN ( 
	            SELECT user_id 
	            FROM {$bp->groups->table_name_members} 
	            WHERE group_id = $group_id
	           )";

	    
	      $users = $wpdb->get_results($q);

	      $user_list = array();
	      // Check for results
	      if (!empty($users)) {
	          foreach($users as $user){
	            $user_list[] = array(
	              'id'=>$user->ID,
	              'image'=>bp_core_fetch_avatar(array('item_id' => $user->ID, 'type' => 'thumb', 'width' => 32, 'height' => 32, 'html'=>false)),
	              'text'=>$user->display_name
	            );
	          }
	          echo json_encode($user_list);
	      } else {
	          echo json_encode(array('id'=>'','text'=>_x('No Users found !','No users found in Course - admin - add users area','vibe')));
	      }
	      die();
    }

    function add_members_in_group(){
    	$group_id = $_POST['group'];
    	if(!isset($_POST['security']) || !wp_verify_nonce($_POST['security'],$group_id)){
    		_e('Security check failed !','wplms-batches');
            die();
    	}

    	if(!isset($_POST['user_id']) && empty($_POST['user_id'])){
    		_e('Security check failed !','wplms-batches');
    		die();
    	}
    	
    	$user_ids = $_POST['user_id'];
    	
    	if( !current_user_can('manage_options') ){
    		$enable_seats  = groups_get_groupmeta( $group_id, 'enable_seats' );	
			$seats  = (int)groups_get_groupmeta( $group_id, 'batch_seats' );
			if(!empty($enable_seats) && !empty($seats)){
				$total_member_count = (int)groups_get_groupmeta( $group_id, 'total_member_count' );
				$total_member_count += count($user_ids);
				if($total_member_count > $seats){
					_e('Members add limit reached,No user added!','wplms-batches');
	    			die();
				}
			}
    	}
    	

    	foreach ($user_ids as $user_id) {
    		groups_join_group($group_id, $user_id );
    		$auto_assign = apply_filters('wplms_batches_auto_assign_to_course',0);
			if($auto_assign && !did_action('groups_join_group')){
				if(class_exists('WPLMS_Batch_Functions')){
		    		$batch = WPLMS_Batch_Functions::init();
		    		if( isset($batch) ){
		    			$t = 0;
						if(is_wplms_batch($group_id)){
							$enable_batch_duration = groups_get_groupmeta($group_id,'enable_batch_duration');
							$courses = get_batch_courses($group_id);
							if(!empty($enable_batch_duration)){
								$batch_duration = groups_get_groupmeta($group_id,'batch_duration');
								$batch_duration_parameter = groups_get_groupmeta($group_id,'batch_duration_parameter');
								$batch_duration_parameter = wplms_process_batch_duration($batch_duration_parameter);
								$t = $batch_duration*$batch_duration_parameter;
								
							}
							foreach ($courses as $course_id){
								if(function_exists('bp_course_add_user_to_course')){
									if(!empty($t) ){
										bp_course_add_user_to_course($user_id,$course_id,$t,1);
									}else{
										bp_course_add_user_to_course($user_id,$course_id);
									}
									
								}
							}
						}
					}
		    	}	
			}
    	}

    	die();
    }
}

Wplms_Batches_Actions::init();
