<?php
/**
 * Initialise WPLMS Batches API
 *
 * @class       Wplms_Batches_Api
 * @author      VibeThemes
 * @category    Admin
 * @package     WPLMS-Batches/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if ( ! defined( 'BP_COURSE_API_NAMESPACE' ) )
   define( 'BP_COURSE_API_NAMESPACE', 'wplms/v1' );

class Wplms_Batches_Api  extends WP_REST_Controller{


	public static $instance;
	
	public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Batches_Api();
        return self::$instance;
    }

	private function __construct(){
		add_filter('bp_course_api_get_user_profile_tab_courses_after',array($this,'course_access_batches_filter'),10,4);
		add_filter('bp_course_api_get_user_profile_tabs',array($this,'add_batch_in_profile'));
		add_filter('bp_course_api_get_user_profile_tab_batch',array($this,'profile_batches'),10,4);
		add_filter('wplms_fetch_course_api_loggedin',array($this,'course_batch_tab'),10,4);
		add_action( 'rest_api_init', array( $this,'register_routes' ));
		


		add_action('wplms_batches_after_batches_settings_save',array($this,'update_group_tracker'),10,2);
		add_filter('wplms_course_batch_added',array($this,'add_course_to_tracker'),10,2);
		add_filter('wplms_course_batch_removed',array($this,'add_course_to_tracker'),10,2);
		add_action('groups_group_create_complete', 		 array($this,'update_all_groups'),10,1);
		add_action( 'groups_group_deleted',              array($this,'update_all_groups'),10,1);
		add_action( 'groups_group_deleted',              array($this,'add_to_tracker'),10,1);
		add_action( 'groups_settings_updated',           array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_details_updated',            array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_group_avatar_updated',       array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_join_group',    array($this,'add_to_tracker_group_course'), 10, 2 );
		add_action( 'groups_leave_group',   array($this,'add_to_tracker_group_course'), 10, 2 );

	}

	// restricted for batch only
	function add_to_tracker_group_course($group_id,$user_id){

		if($this->is_group_a_batch($group_id)){
			$tracker = BP_Course_Rest_Tracker_Controller::init();
			$tracker->get_tracker();
			if(empty($tracker->tracker['groups'])){
	    		$tracker->tracker['groups'] = array($group_id=>time());
	    	}else{
	    		$tracker->tracker['groups'][$group_id] = time();	
	    	}

		    $course_ids=groups_get_groupmeta($group_id,'batch_course',false);
		    if(!empty($course_ids)){
			    foreach ($course_ids as $course_id) {
				    if(empty($tracker->tracker['courses'])){
			    		$tracker->tracker['courses'] = array($course_id=>time());
			    	}else{
			    		$tracker->tracker['courses'][$course_id] = time();	
			    	}
			    }
		    }
	    	$tracker->update_tracker();
		}
	}

	function update_group_tracker($group_id,$post){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
	    $course_ids=groups_get_groupmeta($group_id,'batch_course',false);
	    if(!empty($course_ids)){
		    foreach ($course_ids as $course_id) {
			    if(empty($tracker->tracker['courses'])){
		    		$tracker->tracker['courses'] = array($course_id=>time());
		    	}else{
		    		$tracker->tracker['courses'][$course_id] = time();	
		    	}
		    }
	    }
    	$tracker->update_tracker();
	}


	function add_course_to_tracker($group_id,$course_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();

		if(empty($tracker->tracker['courses'])){
    		$tracker->tracker['courses'] = array($course_id=>time());
    	}else{
    		$tracker->tracker['courses'][$course_id] = time();	
    	}
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
    	$tracker->update_tracker();
	}

	function update_all_groups($group_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		$tracker->tracker['allgroups'] = (int)groups_get_total_group_count();
		$tracker->update_tracker();
	}


	function add_to_tracker($group_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
		$tracker->update_tracker();
	}


	/* Add course batch access for user ['batch']{'return':true, 'message': 'yor accessible' */
	function course_access_batches_filter($data,$user_id,$per_view,$paged){
		$this->user_id=$user_id;
		if(is_array($data)){
			foreach ($data as $key=>$value) {
				$course_id=$value['id'];
				
				if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
		    		$wplms_batches = WPLMS_Batch_Functions::init();
		    		$course_batches=$wplms_batches->get_course_batches($course_id);
		    		$batches_course_access=$this->access_course_batches($course_batches,$course_id);
		    		
		    		$data[$key]['batch']['batches_course_access']=$batches_course_access;
		    	}	
			}
		}
		return $data;
	}

	function course_batch_tab($data,$course,$request,$user_id){
		$this->user_id=$user_id;
		$data['batch']=$this->get_course_batch_api($course->ID);
		return $data;
	}

	function add_batch_in_profile($tabs){
		$newtabs = array();;
		foreach($tabs as $i=>$tab){
			if($tab['key'] == 'results'){
				$newtabs[]= array(
					'key'=>'batch',
					'type'=> 'tab',
					'label'=>_x('Batch','api','wplms-batches'),
					'value'=>'md-calendar',
				);
			}
			$newtabs[]=$tab;
		}
		
		return $newtabs;
	}

	function profile_batches($data,$user_id,$per_view,$paged){
		$this->user_id=$user_id;
		if(bp_is_active('groups')  && class_exists('WPLMS_Batch_Functions')){
	    		$wplms_batches = WPLMS_Batch_Functions::init();
	    		$batch_ids=$wplms_batches->get_user_batches($user_id);  // specify here user_id
	    		
	    		if($batch_ids){ /* if batch_ids has array length */
	    			foreach ($batch_ids as $key => $batch_id) {
							$data[]=$this->get_batch_template($batch_id);	
	    			}
	    			$return=array(
		    			'status'=>1,
		    			'message'=>_x('My batches exist you can run api','vibe'),
		    			'data'=>$data
	    			);
	    			
	    		}else{/* if batch_ids is false */
	    			$return=array(
		    			'status'=>0,
		    			'message'=>_x('You are not enrolled in any batch','vibe')
	    			);
	    		}

	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}

	    	return $return;
	}

	/**
	 * Register the routes for the objects of the controller.
	 *
	 * @since 3.0.0
	 */
	public function register_routes() {
		$this->namespace=BP_COURSE_API_NAMESPACE;
		$this->type='groups';

		register_rest_route( $this->namespace, '/'. $this->type .'/course-batches', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_course_batches' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );


		register_rest_route( $this->namespace, '/'. $this->type .'/getAllgroups', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_all_groups_for_directory' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );


		register_rest_route( $this->namespace, '/'. $this->type .'/getAllgroups-nonlog', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_all_groups_for_directory_nonlog' ),
			// 'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/getBatchById', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'getBatchById' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/getbatchtabvalue', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'getbatchtabvalue' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/join-batch', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'join_remove_batch' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

	}



		public function get_user_permissions_check($request){
			
			
			//$headers = $request->get_headers();
			$headers = vibe_getallheaders();
			if(isset($headers['Authorization'])){
				$token = $headers['Authorization'];
				$this->token = $token;
				$this->user_id = $this->get_user_from_token($token);
				if($this->user_id){
					return true;
				}
			}

			return false;
		}

		public function get_user($request){

			$token = $this->token;

			//$user = $this->get_user_from_token($token);
			
			/**
			 * Filter the response.
			 *
			 * @since 3.0.0
			 *
			 * @param array $element_data
			 * @param WP_REST_Request $request
			 */
			
			if(isset($request['full'])){
				$user_data = apply_filters( 'bp_course_api_get_user',$this->fetch_user($this->user_id), $request );
			}else{
				$user_data = apply_filters( 'bp_course_api_get_user', $this->user_id, $request );
			}
			

			return new WP_REST_Response( $user_data, 200 );
		}


		function get_user_from_token($token){

			global $wpdb;
			$user_id = $wpdb->get_var(apply_filters('wplms_usermeta_direct_query',"SELECT user_id FROM {$wpdb->usermeta} WHERE meta_key = '$token'"));

			if(is_numeric($user_id)){
				return $user_id;
			}

			return false;
			
		}


		function fetch_user($user_id){
			$user = array();
			$field = 'Location';
			if(function_exists('vibe_get_option')){$field = vibe_get_option('student_about');}
		
			$sub = bp_get_profile_field_data('field='.$field.'&user_id='.$user_id);

			$u = get_userdata($user_id);
			$user['id']	  = $user_id;
			$user['name'] = bp_core_get_user_displayname($user_id);
			$user['sub']  = ($sub?$sub:'');
			$user['email']= $u->user_email;
			$user['avatar'] = bp_core_fetch_avatar(array(
								'item_id' => $user_id,
								'object'  => 'user',
								'html'	  => false
							));

			return $user;
		}


	    function get_batch_settings($batch_id,$flag=0){
    		global $wpdb,$bp;
			$group_meta_table = $bp->groups->table_name_groupmeta;
	    	$this->batch_settings = [];
	    	// if flag set get all values from group meta else specific value
	    	if($flag){
	    		$settings_results = $wpdb->get_results($wpdb->prepare("SELECT meta_key,meta_value FROM {$group_meta_table} WHERE group_id = %d",$batch_id),ARRAY_A);
	    	}else{
		    	$meta_keys=apply_filters('batch_meta_key_setting',array(
		    		'total_member_count',
		    		'enable_batch_date',
		    		'start_batch_date',
		    		'end_batch_date',		
		    		'enable_seats',
		    		'batch_seats'
		    	));
		    	$string_keys="('".implode("','",$meta_keys)."')";
				$settings_results = $wpdb->get_results($wpdb->prepare("SELECT meta_key,meta_value FROM {$group_meta_table} WHERE group_id = %d AND meta_key IN {$string_keys}",$batch_id),ARRAY_A);
	    	}


			$settings_results=$this->create_array_for_same_meta_keys($settings_results);
			$settings_array = [];
			if(!empty($settings_results)){
				foreach ($settings_results  as $k => $r) {
					if(is_serialized($r['meta_value'])){
						
						$settings_array[$r['meta_key']] =unserialize($r['meta_value']);
					}else{
						$settings_array[$r['meta_key']] = $r['meta_value'];
					}

				}
			}
			return apply_filters('get_batch_settings_api',$settings_array);

	    }

	    function create_array_for_same_meta_keys($settings_results){
	    	$new_array=[];
	    	$_new_array = [];
	    	foreach ($settings_results  as $settings_result) {
	    		$new_array[$settings_result['meta_key']][] = $settings_result['meta_value'];
	    	}
	    	foreach ($new_array as $key => $value) {
	    		if(!(is_array($value) && count($value) > 1)){
	    			$value = $value[0];
	    		}
	    		$_new_array[] = array(
	    			'meta_key' =>  $key,
	    			'meta_value' => $value
	    		);
	    	}
	    	return $_new_array;
	    }

	    function getBatchById($request){
	    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
		    	$batch_id=$_POST['batch_id'];
		    	$data=$this->get_batch_template($batch_id);
		    	$tabs=array(
						array('key'=>'description','placeholder'=>_x('Description','vibe')),
						array('key'=>'members','placeholder'=>_x('Members','vibe')),
						array('key'=>'activity','placeholder'=>_x('Activity','vibe')),
						array('key'=>'news','placeholder'=>_x('News','vibe'))
					);
		    	$data['tabs']=apply_filters('getBatchByIdTabs',$tabs,$batch_id);
		    	
		    	$data['admins']= $this->fetch_group_admins($batch_id);
		    	$data['mods']= $this->fetch_group_mods($batch_id);
		    	
		    	$data['settings']=$this->get_batch_settings($batch_id,1);

				$return=array(
	    			'status'=>1,
	    			'message'=>_x('My batches exist','vibe'),
	    			'data'=>$data
    			);
	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}
	    	$return=apply_filters('getBatchByIdReturn',$return);
	    	return new WP_REST_Response($return, 200);
	    }

	    function getbatchtabvalue($request){
	    	if(bp_is_active('groups') && bp_is_active('members') && bp_is_active('activity') && class_exists('WPLMS_Batch_Functions')){
		    	$batch_id=$_POST['batch_id'];
		    	$batch_tab=$_POST['batch_tab'];
		    	if(isset($_POST['per_page'])){
		    		$per_page=$_POST['per_page'];
		    	}
		    	if(isset($_POST['offset'])){
		    		$offset=$_POST['offset'];
		    	}
		    	$data=array();
		    	switch ($batch_tab) {
		    		case 'description':
		    				// do nothing or make hit for description
		    			break;
		    		case 'members':
	    				$return=array(
			    			'status'=>1,
			    			'message'=>_x('Batch Members','vibe'),
			    			'data'=>$this->fetch_batch_members($batch_id,$per_page,$offset)
		    			);
		    			break;
		    		case 'activity':
		    			$return=array(
			    			'status'=>1,
			    			'message'=>_x('Batch Activities','vibe'),
			    			'data'=>$this->fetch_batch_activity($batch_id,$per_page,$offset)
		    			);
		    			break;
		    		case 'news':
		    			$return=array(
			    			'status'=>1,
			    			'message'=>_x('Batch News','vibe'),
			    			'data'=>$this->fetch_batch_news($batch_id,$per_page,$offset)
		    			);
		    			break;   				    			
		    		default:
		    				$return=[];
		    			break;
		    	}
		    }else{
		    	$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group,members or activity not active or wplms-batch plugin not activated','vibe')
	    		);
		    }
		    $return=apply_filters('getbatchtabvalue',$return);
	    	return new WP_REST_Response($return, 200);
	    }

	    function fetch_group_admins($group_id){
    		$admins=groups_get_group_admins( $group_id );
	    	if(!empty($admins)){
		    	foreach ($admins as $key => $value) {
	    			$admins[$key]->nickname=bp_core_get_user_displayname($value->user_id);
					$admins[$key]->image=get_avatar_url($value->user_id);
		    	}
		    	return $admins;
	    	}else{
	    		return array();
	    	}
	    }

	    function fetch_group_mods($group_id){
	    	$mods=groups_get_group_mods( $group_id );
	    	if(!empty($mods)){
		    	foreach ($mods as $key => $value) {
	    			$mods[$key]->nickname=bp_core_get_user_displayname($value->user_id);
					$mods[$key]->image=get_avatar_url($value->user_id);
		    	}
		    	return $mods;
	    	}else{
	    		return array();
	    	}
	    }

	    function fetch_batch_members($batch_id,$per_page,$offset){
	    	$data=array();
	    	global $wpdb,$bp;
			$table_name_members = $bp->groups->table_name_members;	
			$prepare_data=$wpdb->get_results("SELECT `user_id` FROM {$table_name_members} WHERE `group_id`={$batch_id} LIMIT {$per_page} OFFSET {$offset}");
				if(!empty($prepare_data)){
					foreach ($prepare_data as $key => $value) {
						$user_id=$value->user_id;
						$data[$key]['nickname']=bp_core_get_user_displayname($user_id);
						$data[$key]['image']=get_avatar_url($value);
					}
				}
	    	return $data;
	    }

	    function fetch_batch_activity($batch_id,$per_page,$offset){
	    	$data=array();
	    	$page=$offset;
	    	$data=$this->get_group_activity($batch_id,$per_page,$page);
	    	return $data;
	    }

	    function get_group_activity($group_id,$per_page,$page){
			$data = array();
			if ( bp_has_activities(bp_ajax_querystring( 'activity' ) ."&primary_id=".$group_id."&per_page=".$per_page."&page=".$page."&sort=DESC") ){
				while ( bp_activities() ) {
					bp_the_activity();	
					$user=array(
						'user_id'=>bp_get_activity_user_id(),
						'nickname'=>get_user_meta(bp_get_activity_user_id(),'nickname',true),
						'image'=>get_avatar_url(bp_get_activity_user_id())
					);
					$id=bp_get_activity_id();
					$type=bp_get_activity_type();
					$data[] = array(
						'id'	=> $id,
						'type'	=> $type,
						'user'=>$user,
						'date'	=> strtotime(bp_get_activity_feed_item_date()),
						'content'=> ($type=='joined_group' )? $user['nickname'].' '._x('has joined the group','vibe'):wp_strip_all_tags(bp_get_activity_content_body())
					);
				} 
			}
			return $data;
		}

	    function fetch_batch_news($batch_id,$per_page,$offset){
	    	$data = array();
	    	global $wpdb,$bp;
			$table_name_activity = $bp->activity->table_name;	
			$prepare_data=$wpdb->get_results("SELECT *FROM {$table_name_activity} WHERE `item_id`={$batch_id} AND `type`='activity_update' ORDER BY `id` DESC LIMIT {$per_page} OFFSET {$offset}");
			if(!empty($prepare_data)){
				foreach ($prepare_data as $key => $value) {
					
					$prepare_data[$key]->content=str_replace("News : ","",$value->content);  // removing 'News : ' string from content
					$prepare_data[$key]->date_recorded=strtotime($value->date_recorded);  // providing timestamp

					$user_id=$value->user_id;
					unset($prepare_data[$key]->user_id);
					$prepare_data[$key]->user=array(
						'user_id'=>$user_id,
						'nickname'=>get_user_meta($user_id,'nickname',true),
						'image'=>get_avatar_url($user_id)
					);
					$data[]=$prepare_data[$key];
				}
			}else{
				return [];
			}
			return $data;	
	    }

	    function is_group_a_batch($group_id){
	    	$is_batch = groups_get_groupmeta((int)$group_id,'course_batch');
	    	if($is_batch){
	    		return true;
	    	}else{
	    		return false;
	    	}
	    }

	    function fetch_group_image($group_id){

			$avatar  = bp_core_fetch_avatar( array(
				'item_id'    => $group_id,
				'object'     => 'group',
				'type'       => 'full',
				'avatar_dir' => 'group-avatars',
				'width'      => '460',
				'height'     => '460',
				'html'	     => false
			) );
	    	return $avatar;
	    }	

	    // function is_member($user_id,$group_id){

	    // }

	    /* It has pagination to give all group */
	    function get_all_groups_for_directory($request){

			$post = json_decode(file_get_contents('php://input'));

	    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
	    		$args=array(
	    			'offset'=>$post->offset,
	    			'limit'=>$post->limit,
	    			'args'=>$post->args
	    		);
	    		$return=$this->get_all_groups($args,$this->user_id);
	    	
	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}
	    	return new WP_REST_Response($return, 200);
	    }

	    /* It has pagination to give all group */
	    function get_all_groups_for_directory_nonlog($request){

	    	$post = json_decode(file_get_contents('php://input'));
			if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
	    		$args=array(
	    			'offset'=>$post->offset,
	    			'limit'=>$post->limit,
	    			'args'=>$post->args
	    		);
	    		$return=$this->get_all_groups($args,null);
	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}
	    	return new WP_REST_Response($return, 200);	   
	    }

	    function get_all_groups($args,$user_id=null){
			

			$defaults = array(
				'offset'=>0,
				'limit'=>10
			);
			
			$args = array_merge($defaults,$args);

    		global $wpdb,$bp;
			
			$group_table = $bp->groups->table_name;	
			$group_meta_table = $bp->groups->table_name_groupmeta;
			
			$query = "SELECT groups.* FROM {$bp->groups->table_name} as groups";

			$join = " ";
			$where = " ";
			$orderby = "";
			$limit = " LIMIT {$args['limit']} OFFSET {$args['offset']} ";

			if(!empty($args) && !empty($args['args'])){

				if($args['args']->type=='batches'){
					$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta ON groups.id = meta.group_id";
					$where .='WHERE meta.meta_key = "course_batch"';
				}else if($args['args']->type=='groups'){
					$where .= "WHERE status != 'hidden' AND groups.id NOT IN (SELECT group_id FROM {$bp->groups->table_name_groupmeta} WHERE meta_key='course_batch') ";
				}

				if($args['args']->search){
					if(empty($where)){
						$where .= " WHERE groups.name LIKE '%{$args['args']->search}%' ";
					}else{
						$where .= " AND groups.name LIKE '%{$args['args']->search}%' ";
					}
					
				}


				if($args['args']->filterby){
					if($args['args']->filterby=='newlyCreated'){
						$orderby .= ' ORDER BY groups.date_created DESC ';
					}
					if($args['args']->filterby=='alphabetical'){
						 $orderby .= 'ORDER BY groups.name ASC ';
					}


					if($args['args']->filterby=='lastActive'){
						$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
						if(empty($where)){
							$where .= ' WHERE meta2.meta_key = "last_activity" '; 
						}else{
							$where .= ' AND meta2.meta_key = "last_activity" '; 
						}
						$orderby .=' ORDER BY meta2.meta_value DESC ';
					}

					if($args['args']->filterby=='mostMembers'){
						$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
						if(empty($where)){
							$where .= ' WHERE meta2.meta_key = "total_member_count" '; 
						}else{
							$where .= ' AND meta2.meta_key = "total_member_count" '; 
						}
						
						$orderby .=' ORDER BY meta2.meta_value DESC ';
					}

					if($args['args']->filterby=='upComing'){
						$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
						$date=date('Y-m-d');
						if(empty($where)){
							$where .= " WHERE meta2.meta_key = 'start_batch_date' AND meta2.meta_value > {$date}"; 
						}else{
							$where .= " AND meta.meta_key = 'start_batch_date' AND meta2.meta_value > {$date}"; 
						}
						
					}	
				}
			}


			$query = $query . $join . $where . $orderby .$limit;

    		$all_groups = $wpdb->get_results($query,ARRAY_A);
    		if($all_groups){
	    		foreach ($all_groups as $key => $value) {
	    			unset($all_groups[$key]['description']);  // description removed
	    			$value['id'] = (int)$value['id'];

	    			$is_batch = $this->is_group_a_batch($value['id']);
			    	if($is_batch){
			    		$all_groups[$key]['is_batch']=true;
			    		$all_groups[$key]['price']=$this->get_batch_pricing($value['id']);
			    	}else{
			    		$all_groups[$key]['is_batch']=false;
			    	}
			    	$all_groups[$key]['image']=$this->fetch_group_image($value['id']);
			    	if($user_id){
			    		$all_groups[$key]['is_member']=groups_is_user_member($this->user_id,$value['id']);
			    	}else{
			    		$all_groups[$key]['is_member']=false;
			    	}
			    	
			    	$all_groups[$key]['settings']=$this->get_batch_settings($value['id']);
		    		$return=array(
		    			'status'=>1,
		    			'message'=>_x('My batches exist you can run api','vibe'),
		    			'data'=>$all_groups
		    		);
	    		}
    		}else{
				$return=array(
	    			'status'=>1,
	    			'message'=>_x('No more groups','vibe'),
	    			'data'=>[]
	    		);
    		}
    		return $return;
	    }

	    function get_batch_template($group_id){
	    	global $wpdb,$bp;
			$group_table = $bp->groups->table_name;	
			
			$prepare_data=$wpdb->get_results("SELECT * FROM {$group_table} WHERE id={$group_id} ",ARRAY_A);

			if(!empty($prepare_data)){
				$prepare_data=$prepare_data[0];
				$prepare_data['image']=$this->fetch_group_image($group_id);
		    	$prepare_data['total_member_count']=groups_get_groupmeta($group_id,'total_member_count',true);

		    	if($this->is_group_a_batch($group_id)){
		    		$prepare_data['is_batch']=true;
		    		$prepare_data['batch_seats']=groups_get_groupmeta($group_id,'batch_seats',true);
		    		$prepare_data['price']=$this->get_batch_pricing($group_id);
		    	}else{
		    		$prepare_data['is_batch']=false;
		    		
		    	}
		    	if($this->user_id){
		    		$prepare_data['is_member']=groups_is_user_member($this->user_id,$group_id);
		    	}else{
		    		$prepare_data['is_member']=false;
		    	}
			}
	    	
	    	return $prepare_data;
	    }



	    function get_batch_pricing($batch_id){
	    	global $wpdb,$bp;
    		$variations = $wpdb->get_results($wpdb->prepare("SELECT post_id FROM {$wpdb->postmeta} WHERE meta_value = %d AND meta_key=%s",$batch_id,'vibe_course_batches'),ARRAY_A);
			$price=array(
				'regular_price'=>0,
				'sale_price'=>0,
				'variation_price'=>0,
				'price_html'=>0
			);
			foreach ($variations as  $variation) {
				$variation_id=$variation['post_id'];
				$variable_product= new WC_Product_Variation( $variation_id );
				$variation_price = 0;
				$regular_price = (int)$variable_product->get_regular_price();
				$price_html = $variable_product->get_price_html();
				$sale_price = (int)$variable_product->get_sale_price();

				if($sales_price){
					$variation_price = $regular_price;
				}else{
					$variation_price = $sale_price;
				}
				$price=array(
					'regular_price'=>$regular_price,
					'sale_price'=>$sale_price,
					'variation_price'=>$variation_price,
					'price_html'=>$price_html
				);
			}
			return $price;
	    }

	    function get_course_batches($request){
	    	
	    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){

	    		$course_id=$_POST['course_id'];
	    		
	    		if(!is_numeric($course_id)){
	    			$return=array(
		    			'status'=>0,
		    			'message'=>_x('Invalid Course !','vibe'),
		    			'data'=>$data
		    		);
	    		}

	    		$wplms_batches = WPLMS_Batch_Functions::init();
	    		$course_batches=$wplms_batches->get_course_batches($course_id);
	    		$data=[];
	    		foreach ($course_batches as $key => $value) {
	    			$data[]=$this->get_batch_template($value->group_id);
	    		}
	    		$return=array(
	    			'status'=>1,
	    			'message'=>_x('Batches exist you can run api','vibe'),
	    			'data'=>$data
	    		);


	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}

	    	return new WP_REST_Response($return, 200);
	    }

	   	function get_course_batch_api($course_id){

	    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
	    		
	    		$wplms_batches = WPLMS_Batch_Functions::init();
	    		$course_batches=$wplms_batches->get_course_batches($course_id);
	    		$data=[];
	    		foreach ($course_batches as $key => $value) {
	    			$data[]=$this->get_batch_template($value->group_id);
	    		}

	    		$batches_course_access=$this->access_course_batches($course_batches,$course_id);

	    		$return=array(
	    			'status'=>1,
	    			'message'=>_x('My batches exist you can run api','vibe'),
	    			'batches_course_access'=>$batches_course_access,
	    			'data'=>$data
	    		);


	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}

	    	return $return;
	    }

	    /* fetch user batches and course batches to return access or not if course_batches already we have*/
	    function access_course_batches($course_batches,$course_id){
	    		$wplms_batches = WPLMS_Batch_Functions::init();
	    		$return =array(
		    			'return'=>false,
		    			'message'=>_x('Not access to course','vibe')
		    		);
	    		if($course_batches && is_array($course_batches)){
			    	if($this->user_id){
			    		$vibe_force_batch_enrolment=get_post_meta($course_id,'vibe_force_batch_enrolment',true);
			    		if($vibe_force_batch_enrolment=='S'){
			    			$count=wplms_user_in_course_batches($this->user_id,$course_id);
			    			if(!$count){
			    				$return =array(
					    			'return'=>false,
					    			'message'=>_x('Enroll in a batch','vibe')
					    		);
					    		return $return;
			    			}
			    		}
			    	}else{
			    		return $return;
			    	}
		    		$accesses=array();
		    		foreach ($course_batches as $key => $value) {
						if(wplms_batch_is_user_member( $this->user_id, $value->group_id )){
							$accesses[]=$wplms_batches->batch_timing_accessbility($value->group_id,$course_id,1);
						}
		    		}
		    		$bit=true;// if any false retur n from batch then no access to course 
		    		foreach ($accesses as $access) {
		    			$bit=$bit & $access['return'];
		    			
		    		}
		    		if($bit){
		    			$return=array(
							'return'=>true,
							'message'=>_x('Course is accessible with batches','vibe')
						);
		    		}else{
						$return=array(
							'return'=>false,
							'message'=>$access['message']
						);
		    		}
	    		}else{
	    			$return=array(
	    				'return'=>true,
	    				'message'=>'No batches in course'
	    			);
	    		}
	    		return $return;
	    }

	    function join_remove_batch($request){

	    	$batch_id=$_POST['batch_id'];
	    	$type=$_POST['type'];
	    	if($type=='add'){
	    		if(!empty(groups_get_groupmeta($batch_id,'enable_seats',true))){
	    			$batch_seats=groups_get_groupmeta($batch_id,'batch_seats',true);
	    			$current_members=groups_get_groupmeta($batch_id,'total_member_count',true);
	    			if($current_members<$batch_seats){
	    				if(!groups_is_user_member($this->user_id,$batch_id)){
				    		$data=groups_join_group($batch_id, $this->user_id);
					    	if($data){
					    		$return=array(
					    			'status'=>1,
					    			'message'=>_x('You have joined the batch','vibe'),
					    			'data'=>$data
					    		);
					    	}else{
					    		$return=array(
					    			'status'=>0,
					    			'message'=>_x('You have not joined the batch','vibe'),
					    			'data'=>$data
					    		);
					    	}
	    				}else{
	    					$return=array(
				    			'status'=>0,
				    			'message'=>_x('You are already member of this batch','vibe'),
				    			'data'=>$data
				    		);
	    				}
	    			}else{
	    				$return=array(
				    			'status'=>0,
				    			'message'=>_x('Member limit reached in batch','vibe'),
				    			'data'=>$data
				    	);
	    			}
	    		}else{
	    			if(!groups_is_user_member($this->user_id,$batch_id)){
		    			$data=groups_join_group($batch_id, $this->user_id);
				    	if($data){
				    		$return=array(
				    			'status'=>1,
				    			'message'=>_x('You have joined the batch','vibe'),
				    			'data'=>$data
				    		);
				    	}else{
				    		$return=array(
				    			'status'=>0,
				    			'message'=>_x('You have not joined the batch','vibe'),
				    			'data'=>$data
				    		);
				    	}
	    			}else{
	    				$return=array(
			    			'status'=>0,
			    			'message'=>_x('You are already member of this batch','vibe'),
			    			'data'=>$data
				    	);
	    			}
	    		}
		    	
	    	}else if($type=='remove'){
	    		$data=groups_leave_group($batch_id, $this->user_id);
	    		if($data){
		    		$return=array(
		    			'status'=>1,
		    			'message'=>_x('You have been removed from batch','vibe'),
		    			'data'=>$data
		    		);
		    	}else{
		    		$return=array(
		    			'status'=>0,
		    			'message'=>_x('You have been not removed from batch','vibe'),
		    			'data'=>$data
		    		);
		    	}
	    	}
	    	return new WP_REST_Response($return, 200);
	    }

	    function get_default_batch_settings($settings){
	    	$this->batch_settings = $settings;
	    }
}
Wplms_Batches_Api::init();

		
			

			
