<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Main Elementor Test Extension Class
 *
 * The main class that initiates and runs the plugin.
 *
 * @since 1.0.0
 */
final class Elementor_WPLMS_Batches_Extension {
	const VERSION = '1.0.0';

	
	const MINIMUM_ELEMENTOR_VERSION = '2.0.0';

	const MINIMUM_PHP_VERSION = '5.6';

	private static $_instance = null;

	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;

	}

	public function __construct() {

		add_action( 'init', [ $this, 'i18n' ] );
		add_action( 'plugins_loaded', [ $this, 'init' ] );
		//add_action('template_redirect',array($this,'record_page'));
	}

	public function i18n() {

		load_plugin_textdomain( 'wplms-batches' );

	}

	public function init() {

		// Check if Elementor installed and activated
		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
			return;
		}

		// Check for required Elementor version
		if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
			return;
		}

		// Check for required PHP version
		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
			return;
		}

		// Add Plugin actions
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'init_widgets' ] );
		//add_action( 'elementor/controls/controls_registered', [ $this, 'init_controls' ] );

		
	}

	public function admin_notice_missing_main_plugin() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'wplms-batches' ),
			'<strong>' . esc_html__( 'Elementor Test Extension', 'wplms-batches' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'wplms-batches' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	public function admin_notice_minimum_elementor_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'wplms-batches' ),
			'<strong>' . esc_html__( 'Elementor Batches Extension', 'wplms-batches' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'wplms-batches' ) . '</strong>',
			 self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	public function admin_notice_minimum_php_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'wplms-batches' ),
			'<strong>' . esc_html__( 'Elementor Test Extension', 'wplms-batches' ) . '</strong>',
			'<strong>' . esc_html__( 'PHP', 'wplms-batches' ) . '</strong>',
			 self::MINIMUM_PHP_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	
	public function init_widgets() {
		global $post;
		if($post->post_type === 'course-layout' || $post->post_type === 'course-card'){

			require_once( __DIR__ . '/widgets/course_batches.php' );
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Wplms_Course_Batches() );
		}
		if($post->post_type == 'member-profile' || $post->post_type == 'member-card'){

			require_once( __DIR__ . '/widgets/profile_batches.php' );
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Wplms_Profile_Batches() ); 
		}

		if($post->post_type == 'group-layout' || $post->post_type == 'group-card'){
			require_once( __DIR__ . '/widgets/batch_courses.php' );
			\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new \Wplms_Batch_Courses() ); 
		}
	}
}


Elementor_WPLMS_Batches_Extension::instance();
