<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class Wplms_Course_Batches extends \Elementor\Widget_Base  // We'll use this just to avoid function name conflicts 
{


    public function get_name() {
		return 'course_batches';
	}

	public function get_title() {
		return __( 'Course Batches', 'wplms-batches' );
	}

	public function get_icon() {
		return 'vicon vicon-layout-media-center';
	}

	public function get_categories() {
		return [ 'wplms'];
	}

	protected function _register_controls() {

		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Controls', 'wplms-batches'),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'style',
			[
				'label' => __( 'Style', 'wplms-batches'),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'list',
				'options' => [
					'list'=>__('List','wplms'),
					'block'=>__('Block','wplms'),
					'card'=>__('Card','wplms')
				],
			]
		);

		$this->add_control(
			'block_width',
			[
				'label' => __( 'Block Size', 'wplms-batches'),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'rem' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default' => [
					'unit' => 'rem',
					'size' => 1,
				],
				'selectors' => [
					'{{WRAPPER}} .course_instructor_data' => 'font-size: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .course_instructor_data img' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'enrol_button',
			[
				'label' => __( 'Show Enrol Button', 'wplms-batches'),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => __( 'Show', 'wplms-batches' ),
				'label_off' => __( 'Hide', 'wplms-batches' ),
				'return_value' => 'yes',
				'default' => 'yes',
			]
		);


		$this->end_controls_section();

	}

	protected function render() {

		$settings = $this->get_settings_for_display();
		//style , enrol_button , block_width
	 	$id = apply_filters('elementor_course_block_id','');
        if(empty($id))
            return;
		
		$batches = wplms_get_course_batches($id);
		if(empty($batches))
			return;
		
		$init=Wplms_Batches_Init::init();
		if(!empty($settings['enrol_button'])){
			$init->show_enroll_button = 1;
		}
		if($settings['style'] == 'list'){
			
			
			?>
			<div class="wplms_batches_wrapper list">
				<ul>
				<?php
					
					foreach($batches as $batch_id){
						$init->batch_list_style($batch_id);
					}
				?>
				</ul>
			</div><style>
			.wplms_batches_wrapper .batch_list{display:flex;justify-content:space-between;align-items:center}.wplms_batches_wrapper .batch_list h4{font-size:1rem;margin:0}.wplms_batches_wrapper .batch_list .batch_admin img{width:32px;border-radius:50%;height:32px}.wplms_batches_wrapper{border:1px solid var(--border)}.wplms_batches_wrapper li{padding:.5rem;border-bottom:1px solid var(--border)}.wplms_batches_wrapper .labels>span{padding:5px;font-size:11px;text-transform:uppercase;background:var(--primary);color:var(--primarycolor);border-radius:2px}
			</style>
			<?php
		}

		if($settings['style'] == 'block'){

			
			?>
			<div class="wplms_batches_wrapper block">
				<ul>
				<?php
					foreach($batches as $batch_id){
						$init->batch_grid_style('custom',$batch_id);
					}
				?>
				</ul>
			</div><style>.wplms_batches_wrapper ul{display:grid;grid-template-columns:repeat(auto-fit,minmax(240px,1fr));grid-gap:1rem}.wplms_batches_wrapper .batch_snapshot{display:flex;flex-direction:row;border:1px solid var(--border);padding:.5rem;position:relative;flex-wrap:wrap;align-items: flex-start}.batch_snapshot .labels{position:absolute;top:.5rem;left:.5rem;z-index:9;background:var(--primary);color:var(--primarycolor);padding:2px 5px;border-radius:2px;display:flex;flex-wrap:wrap}.batch_snapshot .labels>span{text-transform:uppercase;font-size:11px;letter-spacing:1px}.batch_snapshot .batch_image img{border-radius:3px}.batch_image{position:relative;margin:.5rem;flex:1 0 120px}.batch_admins{position:absolute;bottom:.5rem;right:.5rem}.batch_admins img{width:32px;height:32px;border-radius:50%}.batch_content{margin:.5rem;flex:3 0 120px}</style>
			<?php
		}

		?><script>
			if(typeof localforage != 'undefined'){
				localforage.getItem('bp_login_token').then(function(token){
					
					function sendEnrollBatchRequest(batch_id){

						var xh = new XMLHttpRequest();
						xh.open('POST', ajaxurl);
						xh.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
						xh.onload = function() {
							if (xh.status === 200) {
	                            let data = JSON.parse(xh.responseText);
	                            if(typeof data == 'object' && data.status){
	                            	document.querySelectorAll('.enrol_in_batch').forEach(function(el){
	                            		el.parentNode.removeChild(el);
	                            	});
	                            	var span = document.createElement('span');
									span.innerHTML = "<?php _e('Enrolled','wplms-batches');?>";
	                    			document.querySelector('#batch'+batch_id+' .labels').appendChild(span);
	                            }
							}else if (xhr.status !== 200) {
                            console.log('Something went wrong.Request failed.' + xhr.status);
                        	}
						}
						xh.send(encodeURI('action=enrol_in_batch&batch_id='+batch_id+'&course_id=<?php echo $id; ?>+&token=' + token));
					}

					if(token){
						var xhr = new XMLHttpRequest();
						xhr.open('POST', ajaxurl);
						xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
						xhr.onload = function() {
	                        if (xhr.status === 200) {
	                            let data = JSON.parse(xhr.responseText);
	                            if(typeof data == 'object' && data.status){
	                            	if(data.enrolled && data.enrolled.length){
	                            		data.enrolled.map(function(batch_id){
	                            			var span = document.createElement('span');
											span.innerHTML = "<?php _e('Enrolled','wplms-batches');?>";
	                            			document.querySelector('#batch'+batch_id+' .labels').appendChild(span);
	                            		});
	                            	}else{
	                            		if(data.can_enrol){
		                        			document.querySelectorAll('.batch_content').forEach(function(el){
		                        				var btn = document.createElement('a');
												btn.innerHTML = "<?php _e('Enrol Now','wplms-batches');?>";
												btn.setAttribute('class','enrol_in_batch button is-primary small');
												btn.addEventListener('click',function(){
													btn.innerHTML='...';
													document.querySelectorAll('.enrol_in_batch').forEach(function(el){
					                            		if(btn != el){
					                            			el.style.display='none';
					                            		}
					                            	});
													sendEnrollBatchRequest(el.getAttribute('data-id'));
												});
												if(el.querySelector('.enroll_now_button')){
													el.querySelector('.enroll_now_button').remove();
												}
												


		                        				el.appendChild(btn);
		                        			});
		                        		}
	                            	}
	                            }
	                            
	                        }
	                        else if (xhr.status !== 200) {
	                            console.log('Something went wrong.Request failed.' + xhr.status);
	                            
	                        }
	                    };

	                    xhr.send(encodeURI('action=get_enrolled_batches&course_id=<?php echo $id; ?>&batches='+JSON.stringify(<?php echo json_encode($batches); ?>)+'&token=' + token) );
                    }

				});
			}
		</script><?php
	}
}