<?php
/**
 * Initialise WPLMS Batches API
 *
 * @class       Wplms_Batches_Api
 * @author      VibeThemes
 * @category    Admin
 * @package     WPLMS-Batches/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if ( ! defined( 'WPLMS_API_NAMESPACE' ) )
   define( 'WPLMS_API_NAMESPACE', 'wplms/v1' );

class Wplms_Batches_Api  extends WP_REST_Controller{


	public static $instance;
	
	public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Batches_Api();
        return self::$instance;
    }

	private function __construct(){
		$this->batch_settings_array = [];
		add_filter('bp_course_api_get_user_profile_tab_courses_after',array($this,'course_access_batches_filter'),10,4);
		add_filter('bp_course_api_get_user_profile_tabs',array($this,'add_batch_in_profile'));
		add_filter('bp_course_api_get_user_profile_tab_batch',array($this,'profile_batches'),10,4);
		add_filter('wplms_fetch_course_api_loggedin',array($this,'course_batch_tab'),10,4);
		add_action( 'rest_api_init', array( $this,'register_routes' ));
		


		add_action('wplms_batches_after_batches_settings_save',array($this,'update_group_tracker'),10,2);
		add_filter('wplms_course_batch_added',array($this,'add_course_to_tracker'),10,2);
		add_filter('wplms_course_batch_removed',array($this,'add_course_to_tracker'),10,2);
		add_action('groups_group_create_complete', 		 array($this,'update_all_groups'),10,1);
		add_action( 'groups_group_deleted',              array($this,'update_all_groups'),10,1);
		add_action( 'groups_group_deleted',              array($this,'add_to_tracker'),10,1);
		add_action( 'groups_settings_updated',           array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_details_updated',            array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_group_avatar_updated',       array($this,'add_to_tracker'),10,1 );
		add_action( 'groups_join_group',    array($this,'add_to_tracker_group_course'), 10, 2 );
		add_action( 'groups_leave_group',   array($this,'add_to_tracker_group_course'), 10, 2 );

		add_filter('wplms_batches_settings_array',function($settings){
			$this->batch_settings_array = $settings;
			return $settings;
		},999);

	}

	// restricted for batch only
	function add_to_tracker_group_course($group_id,$user_id){

		if($this->is_group_a_batch($group_id)){
			$tracker = BP_Course_Rest_Tracker_Controller::init();
			$tracker->get_tracker();
			if(empty($tracker->tracker['groups'])){
	    		$tracker->tracker['groups'] = array($group_id=>time());
	    	}else{
	    		$tracker->tracker['groups'][$group_id] = time();	
	    	}

		    $course_ids=groups_get_groupmeta($group_id,'batch_course',false);
		    if(!empty($course_ids)){
			    foreach ($course_ids as $course_id) {
				    if(empty($tracker->tracker['courses'])){
			    		$tracker->tracker['courses'] = array($course_id=>time());
			    	}else{
			    		$tracker->tracker['courses'][$course_id] = time();	
			    	}
			    }
		    }
	    	$tracker->update_tracker();
		}
	}

	function update_group_tracker($group_id,$post){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
	    $course_ids=groups_get_groupmeta($group_id,'batch_course',false);
	    if(!empty($course_ids)){
		    foreach ($course_ids as $course_id) {
			    if(empty($tracker->tracker['courses'])){
		    		$tracker->tracker['courses'] = array($course_id=>time());
		    	}else{
		    		$tracker->tracker['courses'][$course_id] = time();	
		    	}
		    }
	    }
    	$tracker->update_tracker();
	}


	function add_course_to_tracker($group_id,$course_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();

		if(empty($tracker->tracker['courses'])){
    		$tracker->tracker['courses'] = array($course_id=>time());
    	}else{
    		$tracker->tracker['courses'][$course_id] = time();	
    	}
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
    	$tracker->update_tracker();
	}

	function update_all_groups($group_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		$tracker->tracker['allgroups'] = (int)groups_get_total_group_count();
		$tracker->update_tracker();
	}


	function add_to_tracker($group_id){
		$tracker = BP_Course_Rest_Tracker_Controller::init();
		$tracker->get_tracker();
		if(empty($tracker->tracker['groups'])){
    		$tracker->tracker['groups'] = array($group_id=>time());
    	}else{
    		$tracker->tracker['groups'][$group_id] = time();	
    	}
		$tracker->update_tracker();
	}


	/* Add course batch access for user ['batch']{'return':true, 'message': 'yor accessible' */
	function course_access_batches_filter($data,$user_id,$per_view,$paged){
		$this->user_id=$user_id;
		if(is_array($data)){
			foreach ($data as $key=>$value) {
				$course_id=$value['id'];
				
				if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
		    		$wplms_batches = WPLMS_Batch_Functions::init();
		    		$course_batches=$wplms_batches->get_course_batches($course_id);
		    		$batches_course_access=$this->access_course_batches($course_batches,$course_id);
		    		
		    		$data[$key]['batch']['batches_course_access']=$batches_course_access;
		    	}	
			}
		}
		return $data;
	}

	function course_batch_tab($data,$course,$request,$user_id){
		$this->user_id=$user_id;
		$data['batch']=$this->get_course_batch_api($course->ID);
		return $data;
	}

	function add_batch_in_profile($tabs){
		$newtabs = array();;
		foreach($tabs as $i=>$tab){
			if($tab['key'] == 'results'){
				$newtabs[]= array(
					'key'=>'batch',
					'type'=> 'tab',
					'label'=>_x('Batch','api','wplms-batches'),
					'value'=>'md-calendar',
				);
			}
			$newtabs[]=$tab;
		}
		
		return $newtabs;
	}

	function profile_batches($data,$user_id,$per_view,$paged){
		$this->user_id=$user_id;
		if(bp_is_active('groups')  && class_exists('WPLMS_Batch_Functions')){
	    		$wplms_batches = WPLMS_Batch_Functions::init();
	    		$batch_ids=$wplms_batches->get_user_batches($user_id);  // specify here user_id
	    		
	    		if($batch_ids){ /* if batch_ids has array length */
	    			foreach ($batch_ids as $key => $batch_id) {
							$data[]=$this->get_batch_template($batch_id);	
	    			}
	    			$return=array(
		    			'status'=>1,
		    			'message'=>_x('My batches exist you can run api','vibe'),
		    			'data'=>$data
	    			);
	    			
	    		}else{/* if batch_ids is false */
	    			$return=array(
		    			'status'=>0,
		    			'message'=>_x('You are not enrolled in any batch','vibe')
	    			);
	    		}

	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
	    		);
	    	}

	    	return $return;
	}

	/**
	 * Register the routes for the objects of the controller.
	 *
	 * @since 3.0.0
	 */
	public function register_routes() {
		$this->namespace=BP_COURSE_API_NAMESPACE;
		$this->type='groups';

		register_rest_route( $this->namespace, '/'. $this->type .'/course-batches', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_course_batches' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		

		register_rest_route( $this->namespace, '/'. $this->type .'/getAllgroups', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_all_groups_for_directory' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );


		register_rest_route( $this->namespace, '/'. $this->type .'/getAllgroups-nonlog', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'get_all_groups_for_directory_nonlog' ),
			// 'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/getBatchById', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'getBatchById' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		
		register_rest_route( $this->namespace, '/'. $this->type .'/savebatch', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'savebatch' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/calculatestats', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'calculatestats' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/loadcoursestats', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'loadcoursestats' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			
		) );

		

		register_rest_route( $this->namespace, '/'. $this->type .'/getbatchtabvalue', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'getbatchtabvalue' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/'. $this->type .'/join-batch', array(
			'methods'                   =>   'POST',
			'callback'                  =>  array( $this, 'join_remove_batch' ),
			'permission_callback' => array( $this, 'get_user_permissions_check' ),
			'args'                     	=>  array(
				'id'                       	=>  array(
					'validate_callback'     =>  function( $param, $request, $key ) {
												return is_numeric( $param );
											}
				),
			),
		) );

		register_rest_route( $this->namespace, '/create/batch', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'create_component' ),
                'permission_callback'       => array( $this, 'get_user_permissions_check' ),
                'args'                      =>  array(
                'cpt'                        =>  array(
                    'validate_callback'     =>  function( $param, $request, $key ) {
                                                return !empty( $param );
                                            }
                    ),
                ),
            ),
        ));

		register_rest_route( $this->namespace, '/load_batch_curriculum', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'load_batch_curriculum' ),
                'permission_callback'       => array( $this, 'get_user_permissions_check' ),
            ),
        ));

        register_rest_route( $this->namespace, '/save_batch_curriculum', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'save_batch_curriculum' ),
                'permission_callback'       => array( $this, 'get_user_permissions_check' ),
            ),
        ));
		
	}

	function create_component($request){
        $id = 0;
        $post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);
        
       
        $return = array();
        $name=$body['name'];
        if(!empty($name ) && !empty($this->user->id) && is_numeric($this->user->id)){
            

            $group_settings = array(
                'creator_id' => $this->user->id,
                'name' => $name,
                'description' => $body['description'],
                'status' => $body['privacy'],
                'date_created' => current_time('mysql')
            );

            $group_settings = apply_filters('wplms_front_end_group_vars',$group_settings);

            if(!empty($body['id'])){
            	$group_settings['group_id'] = $body['id'];
            }
        	
        	$new_group_id = groups_create_group( $group_settings);	
            
            

            if(is_numeric($new_group_id)){
            	if(empty($body['id'])){
            		groups_update_groupmeta( $new_group_id, 'total_member_count', 1 );	
            	}
                
                groups_update_groupmeta( $new_group_id, 'last_activity', gmdate( "Y-m-d H:i:s" ) );
            	groups_update_groupmeta( $new_group_id, 'course_batch',1);
            	$group = groups_get_group(array('group_id'=>$new_group_id,'populate_extras'=>true));

            	$d = array(
            		'link' => bp_core_get_user_domain($this->user->id).'#component=groups&action=view&id='.$new_group_id,
            		'permalink'=> bp_get_group_permalink($group),
            		'id'=>$new_group_id,
            		'text'=>$group->name
            	);
                $return = array('status'=>true,'data'=>$d);
            }else{
                $return = array('status'=>false,'message'=>_x('Batch could not be created.','api','wplms'));
            }
        }else{
        	$return = array('status'=>false,'message'=>_x('Group could not be created.','api','wplms'));
        }

        return new WP_REST_Response( $return, 200 );
    }

	public function get_user_permissions_check($request){
		$body = json_decode($request->get_body(),true);
        
        if (empty($body['token'])){
            $client_id = $request->get_param('client_id');
            if($client_id == vibebp_get_setting('client_id')){
                return true;
            }
        }else{
            $token = $body['token'];
        }

        if(!empty($body['token'])){
            
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
            	$this->user_id = $this->user->id;
                return true;
            }
        }

		//$headers = $request->get_headers();
		$headers = vibe_getallheaders();
		if(isset($headers['Authorization'])){
			$token = $headers['Authorization'];
			$this->user_id = $this->get_user_from_token($token);
			if($this->user_id){
				return true;
			}
		}
		return false;
	}

	public function get_user($request){

		$token = $this->token;

		//$user = $this->get_user_from_token($token);
		
		/**
		 * Filter the response.
		 *
		 * @since 3.0.0
		 *
		 * @param array $element_data
		 * @param WP_REST_Request $request
		 */
		
		if(isset($request['full'])){
			$user_data = apply_filters( 'bp_course_api_get_user',$this->fetch_user($this->user_id), $request );
		}else{
			$user_data = apply_filters( 'bp_course_api_get_user', $this->user_id, $request );
		}
		

		return new WP_REST_Response( $user_data, 200 );
	}


	function get_user_from_token($token){

		global $wpdb;
		$user_id = $wpdb->get_var(apply_filters('wplms_usermeta_direct_query',"SELECT user_id FROM {$wpdb->usermeta} WHERE meta_key = '$token'"));

		if(is_numeric($user_id)){
			return $user_id;
		}

		return false;
		
	}


	function fetch_user($user_id){
		$user = array();
		$field = 'Location';
		if(function_exists('vibe_get_option')){$field = vibe_get_option('student_about');}
	
		$sub = bp_get_profile_field_data('field='.$field.'&user_id='.$user_id);

		$u = get_userdata($user_id);
		$user['id']	  = $user_id;
		$user['name'] = bp_core_get_user_displayname($user_id);
		$user['sub']  = ($sub?$sub:'');
		$user['email']= $u->user_email;
		$user['avatar'] = bp_core_fetch_avatar(array(
							'item_id' => $user_id,
							'object'  => 'user',
							'html'	  => false
						));

		return $user;
	}


    function get_batch_settings($batch_id,$flag=0){
		global $wpdb,$bp;
		$group_meta_table = $bp->groups->table_name_groupmeta;
    	$this->batch_settings = [];
    	// if flag set get all values from group meta else specific value
    	if($flag){
    		$settings_results = $wpdb->get_results($wpdb->prepare("SELECT meta_key,meta_value FROM {$group_meta_table} WHERE group_id = %d",$batch_id),ARRAY_A);
    	}else{
	    	$meta_keys=apply_filters('batch_meta_key_setting',array(
	    		'total_member_count',
	    		'enable_batch_date',
	    		'start_batch_date',
	    		'end_batch_date',		
	    		'enable_seats',
	    		'batch_seats'
	    	));
	    	$string_keys="('".implode("','",$meta_keys)."')";
			$settings_results = $wpdb->get_results($wpdb->prepare("SELECT meta_key,meta_value FROM {$group_meta_table} WHERE group_id = %d AND meta_key IN {$string_keys}",$batch_id),ARRAY_A);
    	}


		$settings_results=$this->create_array_for_same_meta_keys($settings_results);
		$settings_array = [];
		if(!empty($settings_results)){
			foreach ($settings_results  as $k => $r) {
				if(is_serialized($r['meta_value'])){
					
					$settings_array[$r['meta_key']] =unserialize($r['meta_value']);
				}else{
					$settings_array[$r['meta_key']] = $r['meta_value'];
				}

			}
		}
		return apply_filters('get_batch_settings_api',$settings_array);

    }

    function create_array_for_same_meta_keys($settings_results){
    	$new_array=[];
    	$_new_array = [];
    	foreach ($settings_results  as $settings_result) {
    		$new_array[$settings_result['meta_key']][] = $settings_result['meta_value'];
    	}
    	foreach ($new_array as $key => $value) {
    		if(!(is_array($value) && count($value) > 1)){
    			$value = $value[0];
    		}
    		$_new_array[] = array(
    			'meta_key' =>  $key,
    			'meta_value' => $value
    		);
    	}
    	return $_new_array;
    }

    function save_batch_settings($setting,$batch_id){
    	if(!empty($setting) && $setting['type']){
    		switch ($setting['type']) {
	    		case 'duration':
		    		if(!empty($setting['value'])){
		    			groups_update_groupmeta($batch_id,'batch_duration',$setting['value']['value']);
		    			groups_update_groupmeta($batch_id,'batch_duration_parameter',$setting['value']['parameter']);
		    		}
	    			
	    			break;
	    		case 'courses':	
	    			if(!empty($setting['value'])){


	    				if(is_array($setting['value'])){
								
							groups_delete_groupmeta($batch_id,'batch_course',false);

							$course_ids = array_unique($setting['value']);
							$cids = groups_get_groupmeta($batch_id,'batch_course',false);

							foreach($course_ids as $course_id){
								if(is_array($cids) && !in_array($course_id,$cids)){
									groups_add_groupmeta($batch_id,'batch_course',$course_id);	
								}else{
									groups_add_groupmeta($batch_id,'batch_course',$course_id);
								}
							}
						}else{
							groups_update_groupmeta($batch_id,'batch_course',$setting['value']);
						}
		    			
		    		}
	    		break;
	    		case 'time':
	    			if($setting['id']=='course_timings'){
	    				if(!empty($setting['value'])){
	    					foreach ($setting['value'] as $key => $value) {
		    					groups_update_groupmeta($batch_id,$key,$value);
		    				}
	    				}
	    				
	    			}else{
	    				if(isset($setting['value'])){
	    					groups_update_groupmeta($batch_id,$setting['id'],$setting['value']);
	    				}
	    			}
	    			
	    			break;
	    		
	    		default:
	    			if(isset($setting['value'])){
	    				groups_update_groupmeta($batch_id,$setting['id'],$setting['value']);
	    			}else{
	    				groups_delete_groupmeta($batch_id,$setting['id']);
	    			}
	    			break;
	    	}
    	}
    	
    }

    function savebatch($request){
    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		
        	$post = json_decode(file_get_contents('php://input'));
	        $body = $request->get_body();
	        $body = json_decode($body,true);
	        $batch_id=$body['batch_id'];
	        $return = array('status'=>false);
	        $settings = $body['batch'];
	        if(!empty($settings) && !empty($batch_id)){
	        	foreach ($settings as $key => $setting) {
	        		$this->save_batch_settings($setting,$batch_id);
	        	}
	        	$return = array('status'=>true);

	        	do_action('save_batch_settings',$settings,$batch_id);
	        }

	        return new WP_REST_Response($return, 200);
	    }
    }

    function getBatchById($request){
    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		
	        if(empty($_POST['batch_id'])){
	        	$post = json_decode(file_get_contents('php://input'));
		        $body = $request->get_body();
		        $body = json_decode($body,true);
		        $batch_id=$body['batch_id'];
	        }else{
	        	$batch_id=$_POST['batch_id'];
	        }
	       
	        $return = array();

	    	$data=$this->get_batch_template($batch_id);
	    	$tabs=array(
					array('key'=>'courses','placeholder'=>_x('Courses','vibe')),
					array('key'=>'news','placeholder'=>_x('News','vibe')),
					array('key'=>'stats','placeholder'=>_x('Statistics','vibe'))

				);
	    	
	    	
	    	$data['admins']= $this->fetch_group_admins($batch_id);
	    	$data['mods']= $this->fetch_group_mods($batch_id);
	    	if(!empty($data['admins']) || !empty($data['mods'])){
	    		$flag = 0;
	    		foreach ($data['admins'] as $key => $admin) {
	    			if($admin->user_id==$this->user_id){
	    				$flag = 1;
	    				break;
	    			}
	    		}
	    		foreach ($data['mods'] as $key => $admin) {
	    			if($admin->user_id==$this->user_id){
	    				$flag = 1;
	    				break;
	    			}
	    		}
	    		if($flag){
	    			$tabs[] = array('key'=>'manage','placeholder'=>_x('Manage','vibe'));
	    		}
	    		
	    	}

	    	$data['tabs']=apply_filters('getBatchByIdTabs',$tabs,$batch_id);

	    	$data['settings']=$this->get_batch_settings($batch_id,1);

			$return=array(
    			'status'=>1,
    			'message'=>_x('My batches exist','vibe'),
    			'data'=>$data
			);
    	}else{
    		$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
    		);
    	}
    	$return=apply_filters('getBatchByIdReturn',$return);
    	return new WP_REST_Response($return, 200);
    }

    function get_batch_courses($batch_id){
    	$course_ids = get_batch_courses($batch_id);
    	$courses = [];
    	if(!empty($course_ids)){
    		foreach ($course_ids as $key => $course_id) {
    			$title = get_the_title( $course_id);
    			$courses[] = array(
    				'title' => $title,
    				'text' =>$title,
    				'link'=> get_permalink( $course_id),
    				'id'=> $course_id,
    				'featured_image' => get_the_post_thumbnail_url($course_id),
    			);
    		}
    	}
    	return $courses;
    }

    function get_batch_fields($batch_id){
    	if(!class_exists('WPLMS_Batch_Template'))
    		return array();
    	$b = WPLMS_Batch_Template::init();
		$weekly_off_schedule = array(
				array('value'=>1,'label'=>__('Monday','wplms-batches'),),
				array('value'=>2,'label'=>__('Tuesday','wplms-batches'),),
				array('value'=>3,'label'=>__('Wednesday','wplms-batches'),),
				array('value'=>4,'label'=>__('Thursday','wplms-batches'),),
				array('value'=>5,'label'=>__('Friday','wplms-batches'),),
				array('value'=>6,'label'=>__('Saturday','wplms-batches'),),
				array('value'=>7,'label'=>__('Sunday','wplms-batches'),),
			);

				
		
		$monthly_off_schedule = array();
		for($i=1;$i<=31;$i++){
			$monthly_off_schedule[] = array(
				'value'=>$i,'label'=>$i
			);
			
		}
		
		$months = array(
				1=>__('January','wplms-batches'),
				2=>__('February','wplms-batches'),
				3=>__('March','wplms-batches'),
				4=>__('April','wplms-batches'),
				5=>__('May','wplms-batches'),
				6=>__('June','wplms-batches'),
				7=>__('July','wplms-batches'),
				8=>__('August','wplms-batches'),
				9=>__('Sepetember','wplms-batches'),
				10=>__('October','wplms-batches'),
				11=>__('November','wplms-batches'),
				12=>__('December','wplms-batches'),
			);
		$annual_off_schedule = array();
		for($i=1;$i<=12;$i++){
			for($j=1;$j<=31;$j++){
				$annual_off_schedule[] = array(
					'value'=>$i.'-'.$j,'label'=>$j.' '.$months[$i]
				);
				
			}
		}
        $batch = $b->get($batch_id);

        $batch_courses = $this->get_batch_courses($batch_id);
        $_batch_courses = [];
        if(!empty($batch_courses)){
        	foreach ($batch_courses as $key => $c) {
        		$_batch_courses[] = $c['id'];
        	}
        }
        $batch_start_time = $batch_end_time = $batch_repeatable = $batch_repeatable_parameter = $batch_course_timings=[];
        $batch_start_time = groups_get_groupmeta($batch_id,'batch_start_time',true);
        $batch_end_time = groups_get_groupmeta($batch_id,'batch_end_time',true);
        $batch_repeatable = groups_get_groupmeta($batch_id,'batch_repeatable',true);
        $batch_repeatable_parameter = groups_get_groupmeta($batch_id,'batch_repeatable_parameter',true);

        if(empty($batch_start_time)){
        	$batch_start_time = new stdClass();;
        }
        if(empty($batch_end_time)){
        	$batch_end_time = new stdClass();;
        }
        if(empty($batch_repeatable)){
        	$batch_repeatable = new stdClass();;
        }
        if(empty($batch_repeatable_parameter)){
        	$batch_repeatable_parameter = new stdClass();;
        }
        $batch_course_timings = array(
        	'batch_start_time'=>$batch_start_time,
        	'batch_end_time'=>$batch_end_time,
        	'batch_repeatable'=>$batch_repeatable,
        	'batch_repeatable_parameter'=>$batch_repeatable_parameter,
        );

    	$settings_array  = apply_filters('wplms_batches_settings_array',array(
    		'course_batch'=>array(
				'id'=> 'course_batch',
				'label'=> __('Enable Batch','wplms-batches'),
				'description'=> __('Enable batch for this group', 'wplms-batches' ),
				'type'=> 'checkbox',
				'check'=> $batch->enable,
				'hide_nodes'=> array('batch_course','batch_course_visibility','enable_batch_duration','batch_duration','enable_seats','batch_seats','batch_leaderboard','enable_batch_date','start_batch_date','end_batch_date','access_within_dates','enable_off_schedule','weekly_off_schedule','monthly_off_schedule','annual_off_schedule','batch_timings','course_timings','batch_events','batch_stats_visibility','batch_exclusivity'),
				),
			'batch_course'=>array(
				'id'=> 'batch_course[]',
				'label'=>__('Select Courses', 'wplms-batches' ),
				'type'=> 'courses',
				'cpt' => 'course',
				'check' => $_batch_courses,
				'show_value' => $batch_courses,
				),
			'batch_course_visibility'=>array(
				'id'=> 'batch_course_visibility',
				'label'=> __('Students in Batch can only view courses connected to this batch', 'wplms-batches' ),
				'type'=>'checkbox',
				'check'=>$batch->batch_course_visibility,
				'visbility'=>$visibility_flag
				),
			'enable_batch_duration'=>array(
				'id' => 'enable_batch_duration',
				'label'=> __('Enable Batch Duration', 'wplms-batches' ),
				'description'=> __('Overrides course duration for batch students', 'wplms-batches' ),
				'type'=>'checkbox',
				'check'=> $batch->enable_batch_duration,
				'hide_nodes'=> array('batch_duration'),
				),
			'batch_duration'=>array(
				'id'=> 'batch_duration',
				'label'=> __('Batch Duration ','wplms-batches'),
				'name'=> 'batch_duration_parameter',
				'duration_parameter'=> $batch->batch_duration_parameter,
				'type'=> 'duration',
				'check' => (!empty($batch->batch_duration))?array('value'=>$batch->batch_duration,'parameter'=>$batch->batch_duration_parameter):null,
				),
			'enable_seats'=>array(
				'id'=> 'enable_seats',
				'label'=> __('Enable Batch Seats ','wplms-batches'),
				'description'=> __('Override course seats for batch students', 'wplms-batches' ),
				'type'=> 'checkbox',
				'check'=> $batch->enable_seats,
				'hide_nodes'=> array('batch_seats'),
				),
			'batch_seats'=>array(
				'id'=> 'batch_seats',
				'label'=> __('Number of Seats','wplms-batches'),
				'type'=> 'number',
				'check' => $batch->batch_seats
				),
			'batch_leaderboard'=>array(
				'id'=> 'batch_leaderboard',
				'label'=> __('Enable Batch Leaderboard visibility to students','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->batch_leaderboard,
				),
			'enable_batch_date'=>array(
				'id'=> 'enable_batch_date',
				'label'=> __('Enable Batch Start/End Date','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->enable_batch_date,
				'hide_nodes'=> array('start_batch_date','end_batch_date'),
				),
			'start_batch_date'=>array(
				'id'=> 'start_batch_date',
				'label'=> __('Start Date','wplms-batches'),
				'type'=> 'date',
				'check'=> $batch->start_batch_date,
				'start'=> 'start',
				),
			'end_batch_date'=>array(
				'id'=> 'end_batch_date',
				'label'=> __('End Date','wplms-batches'),
				'type'=> 'date',
				'check'=> $batch->end_batch_date,
				'end'=> 'end',
				),
			'access_within_dates'=>array(
				'id'=> 'access_within_dates',
				'label'=> __('Enable Course access within Start/End Dates','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->access_within_dates,
				
				),
			'enable_off_schedule'=>array(
				'id'=> 'enable_off_schedule',
				'label'=> __('Enable Off Schedule','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->enable_off_schedule,
				'hide_nodes'=> array('weekly_off_schedule','monthly_off_schedule','annual_off_schedule'),
				),
			'weekly_off_schedule'=>array(
				'id'=> 'weekly_off_schedule',
				'label'=> __('Set Weekly off days','wplms-batches'),
				'type'=> 'off_schedule',
				'start'=> 'start',
				'options' => $weekly_off_schedule,
				'check' => $batch->weekly_off_schedule,
				),
			'monthly_off_schedule'=>array(
				'id'=> 'monthly_off_schedule',
				'label'=> __('Set Monthly off days','wplms-batches'),
				'type'=> 'off_schedule',
				'options' => $monthly_off_schedule,
				'check' => $batch->monthly_off_schedule,
				),
			'annual_off_schedule'=>array(
				'id'=> 'annual_off_schedule',
				'label'=> __('Set specific dates','wplms-batches'),
				'type'=> 'off_schedule',
				'end'=> 'end',
				'options' => $annual_off_schedule,
				'check' => $batch->annual_off_schedule,
				),
			'batch_timings'=>array(
				'id'=> 'batch_timings',
				'label'=> __('Enable Batch Timings','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->timings,
				'hide_nodes'=>array('course_timings'),
				),
			'course_timings'=>array(
				'id'=> 'course_timings',
				'type'=> 'time',
				'courses' => $batch_courses,
				'options' => array(
					array('value'=>null,'label'=>_x('Select','','wplms-batches')),
					array('value' =>'day','label'=>_x('Day','','wplms-batches') ),
					array('value' =>'week','label'=>_x('Week','','wplms-batches') ),
					array('value' =>'month','label'=>_x('Month','','wplms-batches') ),
					array('value' =>'year','label'=>_x('Year','','wplms-batches') ),
				),
				'check' => $batch_course_timings
				),
			'batch_events'=>array(
				'id'=> 'batch_events',
				'label'=> __('Enable Events for this batch','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->batch_events,
				),
			'batch_stats_visibility'=>array(
				'id'=> 'batch_stats_visibility',
				'label'=> __('Batch Statistics Visibility','wplms-batches'),
				'options' => array(
					array(
						'value'=>'all','label' => _x('All','','wplms-batches'),
					),
					array(
						'value'=>'mods','label' => _x('Mods','','wplms-batches'),
					),
					array(
						'value'=>'admins','label' => _x('Admins','','wplms-batches'),
					),
				),
				'type'=> 'select',
				'check'=>$batch->batch_stats_visibility
				),
			'batch_exclusivity'=>array(
				'id'=> 'batch_exclusivity',
				'label'=> __('Hide "Join/Request" buttons, useful for Batch exclusivity.','wplms-batches'),
				'type'=> 'checkbox',
				'check'=> $batch->batch_exclusivity,
				),
		),$batch_id);
		$settings = [];
		foreach ($settings_array as $key => $ss) {
			$ss['id'] = $key;
			if(!empty($ss['check']) && empty($ss['value'])){
				$ss['value'] = $ss['check'];
			}
			$settings[] =  $ss;
		}
    	return apply_filters('wplms_batch_api_settings',$settings);
    }

    function getbatchtabvalue($request){

    	if(bp_is_active('groups') && bp_is_active('members') && bp_is_active('activity') && class_exists('WPLMS_Batch_Functions')){

    		
	        if(empty($_POST['batch_id'])){
	        	$post = json_decode(file_get_contents('php://input'));
		        $body = $request->get_body();
		        $body = json_decode($body,true);

		        $batch_id=$body['batch_id'];
		    	$batch_tab=$body['batch_tab'];
		    	if(isset($body['per_page'])){
		    		$per_page=$body['per_page'];
		    	}
		    	if(isset($body['offset'])){
		    		$offset=$body['offset'];
		    	}

	        }else{
	        	$batch_id=$_POST['batch_id'];
		    	$batch_tab=$_POST['batch_tab'];
		    	if(isset($_POST['per_page'])){
		    		$per_page=$_POST['per_page'];
		    	}
		    	if(isset($_POST['offset'])){
		    		$offset=$_POST['offset'];
		    	}
	        }

	    	
	    	$data=array();
	    	switch ($batch_tab) {
	    		case 'courses':
	    				$return=array(
			    			'status'=>1,
			    			'message'=>_x('Batch Courses','vibe'),
			    			'data'=>$this->get_batch_courses($batch_id),
		    			);
	    			break;
	    		case 'manage':
    				$return=array(
		    			'status'=>1,
		    			'message'=>_x('Batch Members','vibe'),
		    			'data'=>$this->get_batch_fields($batch_id)
	    			);
	    			break;
	    		case 'activity':
	    			$return=array(
		    			'status'=>1,
		    			'message'=>_x('Batch Activities','vibe'),
		    			'data'=>$this->fetch_batch_activity($batch_id,$per_page,$offset)
	    			);
	    			break;
	    		case 'news':
	    			$return=array(
		    			'status'=>1,
		    			'message'=>_x('Batch News','vibe'),
		    			'data'=>$this->fetch_batch_news($batch_id,$per_page,$offset)
	    			);
	    			break;
	    		case 'stats':
	    			$return=array(
		    			'status'=>1,
		    			'message'=>_x('Batch Stats','vibe'),
		    			'data'=>$this->fetch_batch_stats($batch_id)
	    			);
	    			break;   				    			
	    		default:
	    				$return=[];
	    			break;
	    	}
	    }else{
	    	$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group,members or activity not active or wplms-batch plugin not activated','vibe')
    		);
	    }
	    $return=apply_filters('getbatchtabvalue',$return);
    	return new WP_REST_Response($return, 200);
    }

    function loadcoursestats($request){
    	$post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);
        $return = array('status'=>1);
        $course_id=$body['course_id'];
        $batch_id=$body['batch_id'];
        $user_id = $this->user_id;

        ob_start();
        echo '<h5>'.sprintf(__('Statistics for Course %s','wplms-batches'),'<a href="'.get_permalink($course_id).'" target="_blank">'.get_the_title($course_id)).'</a></h5>';
        if(function_exists('bp_course_get_curriculum_quizes')){
            $quiz_ids = bp_course_get_curriculum_quizes($course_id);    
        }
        if(function_exists('wplms_course_get_course_assignments')){
            $assignments = wplms_course_get_course_assignments($course_id);    
        }
        $flag = 0;
        if(!empty($quiz_ids)){
            $flag++;
        }
        if(!empty($assignments)){
            $flag++;
        }
        ?>
        <table class="course_leaderboard table table-hover table-bordered">
            <thead>
                <tr>
                    <th rowspan="<?php echo (!empty($flag)?2:1); ?>">#</th>
                    <th rowspan="<?php echo (!empty($flag)?2:1); ?>"><?php _e('Student','wplms-batches'); ?></th>
                    <th rowspan="<?php echo (!empty($flag)?2:1); ?>"><?php _e('Progress','wplms-batches'); ?></th>
                    <?php
                        if($flag >= 1){
                            echo '<th rowspan="1" colspan="'.count($quiz_ids).'"><span style="width: 100%;text-align:center;display: inline-block;">'.__('Quizzes','wplms-batches').'</span></th>';
                        }
                        if($flag >= 2){
                            echo '<th rowspan="1" colspan="'.count($assignments).'"><span style="width: 100%;text-align:center;display: inline-block;">'.__('Assignments','wplms-batches').'</span></th>';
                        }
                    ?>
                    <th rowspan="<?php echo (!empty($flag)?2:1); ?>"><?php _e('Cumulative','wplms-batches'); ?></th>
                </tr>
                <tr>
                    <?php
                    if($flag >= 1){
                        foreach($quiz_ids as $quiz_id){
                            echo '<th><a href="'.get_permalink($quiz_id).'" target="_blank">'.get_the_title($quiz_id).'</a></th>';
                        }
                    }
                    if($flag >= 2){
                        foreach($assignments as $assignment){
                            echo '<th><a href="'.get_permalink($assignment->post_id).'" target="_blank">'.$assignment->title.'</a></th>';
                        }
                    }
                    ?>
                </tr>
            </thead>
            <tbody>
            <?php
            $members = BP_Groups_Member::get_group_member_ids($batch_id);
            $i=1;
            foreach($members as $member_id){
                echo '<tr>
                <td>'.$i.'</td>';
                echo '<td><a href="'.bp_core_get_user_domain($member_id).'" class="batch_member">
                            '. bp_core_fetch_avatar( array(
                                    'item_id' => $member_id,
                                     'type' => 'thumb',
                                     'width'      => '64',
                                     'height'     => '64',
                                     )).'<span>'.bp_core_get_user_displayname($member_id).'</span></a></td>';
                $progress = bp_course_get_user_progress($member_id,$course_id);
                echo '<td>'.(empty($progress)?0:$progress).'</td>';
                if($flag >= 1){
                    foreach($quiz_ids as $quiz_id){
                        echo '<td>'.get_post_meta($quiz_id,$member_id,true).'</td>';
                    }
                }
                if($flag >= 2){
                    foreach($assignments as $assignment){
                        echo '<td>'.get_post_meta($assignment->post_id,$member_id,true).'</td>';
                    }
                }            
                $sum=0;           
                $status = bp_course_get_user_course_status($member_id,$course_id);
                if($status > 3){
                    $marks = get_post_meta($course_id,$member_id,true);
                    $sum += $marks;
                    echo '<td>'.$marks.'</td>';    
                }else{
                    echo '<td>'._x('N.A','Marks not available in batch statisitcs leaderboard','wplms-batches').'</td>';
                }
                echo '</tr>';
                $i++;
            }
            ?>
            </tbody>
        </table>
        <?php
        $html = ob_get_clean();

        $return = array('status'=>1,'data'=>$html);
        return new WP_REST_Response($return, 200);
    }

    function calculatestats($request){
    	$post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);
        $return = array('status'=>1);
        $group_id=$body['batch_id'];
        $courses = groups_get_groupmeta($group_id,'batch_course',false);
        
        $members = BP_Groups_Member::get_group_member_ids($group_id);
        $batch_badge_count = $batch_certificate_count = 0;
        $total_marks = 0;$total_denom = 1;
        foreach($members as $member_id){
            foreach($courses as $course_id){
                $status = bp_course_get_user_course_status($member_id,$course_id);
                if($status > 3){
                    $marks = get_post_meta($course_id,$member_id,true);
                    $total_marks += $marks;
                    $total_denom++;
                }
            }
            $certificates = get_user_meta($member_id,'certificates',true);
            $badges = get_user_meta($member_id,'badges',true);
            if(!empty($certificates)){
                foreach($certificates as $certificate_course_id){
                    if(in_array($certificate_course_id,$courses)){
                        $batch_certificate_count++;
                    }
                }    
            }
            if(!empty($badges)){
                foreach($badges as $badges_course_id){
                    if(in_array($badges_course_id,$courses)){
                        $batch_badge_count++;
                    }
                }    
            }
        }
        if($total_denom > 1){
            $total_denom--;
        }
        $batch_average = round($total_marks/$total_denom,2);
        if(empty($batch_average))$batch_average=0;
        if(empty($batch_badge_count))$batch_badge_count=0;
        if(empty($batch_certificate_count))$batch_certificate_count=0;
        groups_update_groupmeta($group_id,'batch_average',$batch_average);
        groups_update_groupmeta($group_id,'batch_badge_count',$batch_badge_count);
        groups_update_groupmeta($group_id,'batch_certificate_count',$batch_certificate_count);
        $return['data'] = array('batch_average'=>$batch_average);
    	return new WP_REST_Response($return, 200);
    }

    function fetch_batch_stats($group_id){
    	global $wpdb,$bp;
    	
        $members = BP_Groups_Member::get_group_member_ids($group_id);
        $admins = groups_get_group_admins( $group_id );
        $mods = groups_get_group_mods( $group_id );

        
        $batch_average = $batch_badge_count = $batch_certificate_count = 0;
        $batch_average = groups_get_groupmeta($group_id,'batch_average',true);
        $batch_badge_count = groups_get_groupmeta($group_id,'batch_badge_count',true);
        $batch_certificate_count = groups_get_groupmeta($group_id,'batch_certificate_count',true);
        $return =array();
        $return['general']= array();


        $return['general']['batch_average'] = array('label'=> _x('Average Course percentage in Batch','','wplms-batches'),'value'=>$batch_average);

        $return['general']['batch_badge_count'] = array('label'=>_x('Students who got excellence Badge','','wplms-batches'),'value'=>$batch_badge_count);

        $return['general']['batch_certificate_count'] = array('label'=>_x('Students who earned certificate','','wplms-batches') ,'value'=>$batch_certificate_count);

        $return['general']['students'] = array('label'=> _x('Total Students in Batch','','wplms-batches'),'value'=>(count($members)-count($admins)-count($mods)));
        $return['general']['members'] = array('value'=>count($members),'label'=> _x('Total members in Batch','','wplms-batches'));

        $batch_stats_visibility = groups_get_groupmeta($group_id,'batch_stats_visibility');
        $stats_flag = $leaderboard_flag = 0;
        if(!empty($admins) || !empty($mods)){
    		
			foreach ($admins as $key => $admin) {
    			if($admin->user_id==$this->user_id){
    				$stats_flag = 1;
    				$leaderboard_flag = 1;
    				break;
    			}
    		}
    		if($batch_stats_visibility=='mods'){
    			foreach ($mods as $key => $admin) {
	    			if($admin->user_id==$this->user_id){
	    				$stats_flag = 1;
	    				$leaderboard_flag = 1;
	    				break;
	    			}
	    		}
    		}
    		
    		
    	}
    	if($batch_stats_visibility=='all'){
			$stats_flag = 1;
		}
		$course_ids  = [] ;
        if($stats_flag){
        	$course_ids = groups_get_groupmeta($group_id,'batch_course',false);
	    	$courses = [];
	        if(!empty($course_ids)){
	            foreach ($course_ids as $key => $course_id) {
	    			$title = get_the_title( $course_id);
	    			$courses[] = array(
	    				'label' => $title,
	    				'text' =>$title,
	    				
	    				'value'=> $course_id,
	    				
	    			);
	    		}
	        }
        	$return['courses'] = $courses;
        }

        $leaderboard = groups_get_groupmeta($group_id,'batch_leaderboard');
        if(!empty($leaderboard)){
        	$leaderboard_flag = 1;
        }
        if($leaderboard_flag){
        	if(empty($course_ids)){
        		$course_ids = groups_get_groupmeta($group_id,'batch_course',false);
        	}
        	$member_ids = implode(',',$members);
            $_course_ids = implode(',',$course_ids);
            $course_status_string = '';
            foreach($course_ids as $c){
                $course_status_string .= "'course_status".$c."',";
            }
            $course_status_string =trim($course_status_string, ",");
            global $wpdb,$bp;
            $results = $wpdb->get_results($wpdb->prepare("SELECT p.meta_key as uid,sum(p.meta_value) as sum 
                FROM {$wpdb->postmeta} as p
                LEFT JOIN {$wpdb->usermeta} as u
                ON p.meta_key = u.user_id
                WHERE p.meta_key IN ($member_ids) 
                AND p.post_id IN ($_course_ids) 
                AND u.meta_key IN ($course_status_string)
                AND u.meta_value > %d
                GROUP BY uid ORDER BY sum DESC",3));
           	if(empty($results)){
           		$html = '<div class="message notice">'._x('No stats found','','wplms-batches').'</div>';
           	}else{
	            ob_start();
	            ?>
	            <table id="batch_leaderboard" class="table table-hover table-bordered">
	                <thead>
	                    <tr>
	                        <th rowspan="2"><?php _e('Rank','wplms-batches'); ?></th>
	                        <th rowspan="2"><?php _e('Student','wplms-batches'); ?></th>
	                        <th rowspan="1" colspan="<?php echo count($course_ids); ?>"><span style="width: 100%;text-align:center;display: inline-block;"><?php _e('Course','wplms-batches'); ?></span></th>
	                        <th rowspan="2"><?php _e('Cumulative','wplms-batches'); ?></th>
	                    </tr>
	                    <tr>
	                        <?php
	                        foreach($course_ids as $course_id){
	                            echo '<th><a href="'.get_permalink($course_id).'" target="_blank">'.get_the_title($course_id).'<a/></th>';
	                        }
	                        ?>
	                    </tr>
	                </thead>
	                <tbody>
	                    <?php
	                    $i=1;
	                    foreach($results as $member){
	                        echo '<tr>
	                        <td>'.$i.'</td>';
	                        echo '<td><a href="'.bp_core_get_user_domain($member->uid).'" class="batch_member">
	                                    '. bp_core_fetch_avatar( array(
	                                            'item_id' => $member->uid,
	                                             'type' => 'thumb',
	                                             'width'      => '64',
	                                             'height'     => '64',
	                                             )).'<span>'.bp_core_get_user_displayname($member->uid).'</span></a></td>';
	                        $sum=0;           
	                        foreach($course_ids as $course_id){
	                            $status = bp_course_get_user_course_status($member->uid,$course_id);
	                            if($status > 3){
	                                $marks = bp_course_get_marks($member->uid,$course_id);
	                                $sum += $marks;
	                                echo '<td>'.$marks.'</td>';    
	                            }else{
	                                echo '<td>'._x('N.A','Marks not available in batch statisitcs leaderboard','wplms-batches').'</td>';
	                            }

	                            do_action('wplms_batches_statistics',$member->uid,$course_id);
	                        }
	                        echo '<td>'.$sum.'</td>';
	                        echo '</tr>';
	                        $i++;
	                    }
	                    ?>
	                </tbody>
	                <tfooter>
	                </tfooter>
	            </table>
	            <?php
	            $html = ob_get_clean();
        	}
            $return['leaderboard'] = $html;
        }
    	return $return;
    }

    function fetch_group_admins($group_id){
		$admins=groups_get_group_admins( $group_id );
    	if(!empty($admins)){
	    	foreach ($admins as $key => $value) {
    			$admins[$key]->nickname=bp_core_get_user_displayname($value->user_id);
				$admins[$key]->image=get_avatar_url($value->user_id);
	    	}
	    	return $admins;
    	}else{
    		return array();
    	}
    }

    function fetch_group_mods($group_id){
    	$mods=groups_get_group_mods( $group_id );
    	if(!empty($mods)){
	    	foreach ($mods as $key => $value) {
    			$mods[$key]->nickname=bp_core_get_user_displayname($value->user_id);
				$mods[$key]->image=get_avatar_url($value->user_id);
	    	}
	    	return $mods;
    	}else{
    		return array();
    	}
    }

    function fetch_batch_members($batch_id,$per_page,$offset){
    	$data=array();
    	global $wpdb,$bp;
		$table_name_members = $bp->groups->table_name_members;	
		$prepare_data=$wpdb->get_results("SELECT `user_id` FROM {$table_name_members} WHERE `group_id`={$batch_id} LIMIT {$per_page} OFFSET {$offset}");
			if(!empty($prepare_data)){
				foreach ($prepare_data as $key => $value) {
					$user_id=$value->user_id;
					$data[$key]['nickname']=bp_core_get_user_displayname($user_id);
					$data[$key]['image']=get_avatar_url($value);
				}
			}
    	return $data;
    }

    function fetch_batch_activity($batch_id,$per_page,$offset){
    	$data=array();
    	$page=$offset;
    	$data=$this->get_group_activity($batch_id,$per_page,$page);
    	return $data;
    }

    function get_group_activity($group_id,$per_page,$page){
		$data = array();
		if ( bp_has_activities(bp_ajax_querystring( 'activity' ) ."&primary_id=".$group_id."&per_page=".$per_page."&page=".$page."&sort=DESC") ){
			while ( bp_activities() ) {
				bp_the_activity();	
				$user=array(
					'user_id'=>bp_get_activity_user_id(),
					'nickname'=>get_user_meta(bp_get_activity_user_id(),'nickname',true),
					'image'=>get_avatar_url(bp_get_activity_user_id())
				);
				$id=bp_get_activity_id();
				$type=bp_get_activity_type();
				$data[] = array(
					'id'	=> $id,
					'type'	=> $type,
					'user'=>$user,
					'date'	=> strtotime(bp_get_activity_feed_item_date()),
					'content'=> ($type=='joined_group' )? $user['nickname'].' '._x('has joined the group','vibe'):wp_strip_all_tags(bp_get_activity_content_body())
				);
			} 
		}
		return $data;
	}

    function fetch_batch_news($batch_id,$per_page,$offset){
    	$data = array();
    	global $wpdb,$bp;
		$table_name_activity = $bp->activity->table_name;	
		$prepare_data=$wpdb->get_results("SELECT *FROM {$table_name_activity} WHERE `item_id`={$batch_id} AND `type`='activity_update' ORDER BY `id` DESC LIMIT {$per_page} OFFSET {$offset}");
		if(!empty($prepare_data)){
			foreach ($prepare_data as $key => $value) {
				
				$prepare_data[$key]->content=str_replace("News : ","",$value->content);  // removing 'News : ' string from content
				$prepare_data[$key]->date_recorded=strtotime($value->date_recorded);  // providing timestamp

				$user_id=$value->user_id;
				unset($prepare_data[$key]->user_id);
				$prepare_data[$key]->user=array(
					'user_id'=>$user_id,
					'nickname'=>get_user_meta($user_id,'nickname',true),
					'image'=>get_avatar_url($user_id)
				);
				$data[]=$prepare_data[$key];
			}
		}else{
			return [];
		}
		return $data;	
    }

    function is_group_a_batch($group_id){
    	$is_batch = groups_get_groupmeta((int)$group_id,'course_batch');
    	if($is_batch){
    		return true;
    	}else{
    		return false;
    	}
    }

    function fetch_group_image($group_id){

		$avatar  = bp_core_fetch_avatar( array(
			'item_id'    => $group_id,
			'object'     => 'group',
			'type'       => 'full',
			'avatar_dir' => 'group-avatars',
			'width'      => '460',
			'height'     => '460',
			'html'	     => false
		) );
    	return $avatar;
    }	

    // function is_member($user_id,$group_id){

    // }

    /* It has pagination to give all group */
    function get_all_groups_for_directory($request){

		$post = json_decode(file_get_contents('php://input'));

    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		$args=array(
    			'offset'=>$post->offset,
    			'limit'=>$post->limit,
    			'args'=>$post->args
    		);
    		$return=$this->get_all_groups($args,$this->user_id);
    	
    	}else{
    		$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
    		);
    	}
    	return new WP_REST_Response($return, 200);
    }

    /* It has pagination to give all group */
    function get_all_groups_for_directory_nonlog($request){

    	$post = json_decode(file_get_contents('php://input'));
		if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		$args=array(
    			'offset'=>$post->offset,
    			'limit'=>$post->limit,
    			'args'=>$post->args
    		);
    		$return=$this->get_all_groups($args,null);
    	}else{
    		$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
    		);
    	}
    	return new WP_REST_Response($return, 200);	   
    }

    function get_all_groups($args,$user_id=null){
		

		$defaults = array(
			'offset'=>0,
			'limit'=>10
		);
		
		$args = array_merge($defaults,$args);

		global $wpdb,$bp;
		
		$group_table = $bp->groups->table_name;	
		$group_meta_table = $bp->groups->table_name_groupmeta;
		
		$query = "SELECT groups.* FROM {$bp->groups->table_name} as groups";

		$join = " ";
		$where = " ";
		$orderby = "";
		$limit = " LIMIT {$args['limit']} OFFSET {$args['offset']} ";

		if(!empty($args) && !empty($args['args'])){

			if($args['args']->type=='batches'){
				$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta ON groups.id = meta.group_id";
				$where .='WHERE meta.meta_key = "course_batch"';
			}else if($args['args']->type=='groups'){
				$where .= "WHERE status != 'hidden' AND groups.id NOT IN (SELECT group_id FROM {$bp->groups->table_name_groupmeta} WHERE meta_key='course_batch') ";
			}

			if($args['args']->search){
				if(empty($where)){
					$where .= " WHERE groups.name LIKE '%{$args['args']->search}%' ";
				}else{
					$where .= " AND groups.name LIKE '%{$args['args']->search}%' ";
				}
				
			}


			if($args['args']->filterby){
				if($args['args']->filterby=='newlyCreated'){
					$orderby .= ' ORDER BY groups.date_created DESC ';
				}
				if($args['args']->filterby=='alphabetical'){
					 $orderby .= 'ORDER BY groups.name ASC ';
				}


				if($args['args']->filterby=='lastActive'){
					$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
					if(empty($where)){
						$where .= ' WHERE meta2.meta_key = "last_activity" '; 
					}else{
						$where .= ' AND meta2.meta_key = "last_activity" '; 
					}
					$orderby .=' ORDER BY meta2.meta_value DESC ';
				}

				if($args['args']->filterby=='mostMembers'){
					$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
					if(empty($where)){
						$where .= ' WHERE meta2.meta_key = "total_member_count" '; 
					}else{
						$where .= ' AND meta2.meta_key = "total_member_count" '; 
					}
					
					$orderby .=' ORDER BY meta2.meta_value DESC ';
				}

				if($args['args']->filterby=='upComing'){
					$join .= " INNER JOIN {$bp->groups->table_name_groupmeta} as meta2 ON groups.id = meta2.group_id";
					$date=date('Y-m-d');
					if(empty($where)){
						$where .= " WHERE meta2.meta_key = 'start_batch_date' AND meta2.meta_value > {$date}"; 
					}else{
						$where .= " AND meta.meta_key = 'start_batch_date' AND meta2.meta_value > {$date}"; 
					}
					
				}	
			}
		}


		$query = $query . $join . $where . $orderby .$limit;

		$all_groups = $wpdb->get_results($query,ARRAY_A);
		if($all_groups){
    		foreach ($all_groups as $key => $value) {
    			unset($all_groups[$key]['description']);  // description removed
    			$value['id'] = (int)$value['id'];

    			$is_batch = $this->is_group_a_batch($value['id']);
		    	if($is_batch){
		    		$all_groups[$key]['is_batch']=true;
		    		$all_groups[$key]['price']=$this->get_batch_pricing($value['id']);
		    	}else{
		    		$all_groups[$key]['is_batch']=false;
		    	}
		    	$all_groups[$key]['image']=$this->fetch_group_image($value['id']);
		    	if($user_id){
		    		$all_groups[$key]['is_member']=groups_is_user_member($this->user_id,$value['id']);
		    	}else{
		    		$all_groups[$key]['is_member']=false;
		    	}
		    	
		    	$all_groups[$key]['settings']=$this->get_batch_settings($value['id']);
	    		$return=array(
	    			'status'=>1,
	    			'message'=>_x('My batches exist you can run api','vibe'),
	    			'data'=>$all_groups
	    		);
    		}
		}else{
			$return=array(
    			'status'=>1,
    			'message'=>_x('No more groups','vibe'),
    			'data'=>[]
    		);
		}
		return $return;
    }

    function get_batch_template($group_id){
    	global $wpdb,$bp;
		$group_table = $bp->groups->table_name;	
		
		$prepare_data=$wpdb->get_results("SELECT * FROM {$group_table} WHERE id={$group_id} ",ARRAY_A);

		if(!empty($prepare_data)){
			$prepare_data=$prepare_data[0];
			$prepare_data['image']=$this->fetch_group_image($group_id);
	    	$prepare_data['total_member_count']=groups_get_groupmeta($group_id,'total_member_count',true);

	    	if($this->is_group_a_batch($group_id)){
	    		$prepare_data['is_batch']=true;
	    		$prepare_data['batch_seats']=groups_get_groupmeta($group_id,'batch_seats',true);
	    		$prepare_data['price']=$this->get_batch_pricing($group_id);
	    	}else{
	    		$prepare_data['is_batch']=false;
	    		
	    	}
	    	if($this->user_id){
	    		$prepare_data['is_member']=groups_is_user_member($this->user_id,$group_id);
	    	}else{
	    		$prepare_data['is_member']=false;
	    	}
		}
    	
    	return $prepare_data;
    }



    function get_batch_pricing($batch_id){
    	global $wpdb,$bp;
		$variations = $wpdb->get_results($wpdb->prepare("SELECT post_id FROM {$wpdb->postmeta} WHERE meta_value = %d AND meta_key=%s",$batch_id,'vibe_course_batches'),ARRAY_A);
		$price=array(
			'regular_price'=>0,
			'sale_price'=>0,
			'variation_price'=>0,
			'price_html'=>0
		);
		foreach ($variations as  $variation) {
			$variation_id=$variation['post_id'];
			$variable_product= new WC_Product_Variation( $variation_id );
			$variation_price = 0;
			$regular_price = (int)$variable_product->get_regular_price();
			$price_html = $variable_product->get_price_html();
			$sale_price = (int)$variable_product->get_sale_price();

			if($sales_price){
				$variation_price = $regular_price;
			}else{
				$variation_price = $sale_price;
			}
			$price=array(
				'regular_price'=>$regular_price,
				'sale_price'=>$sale_price,
				'variation_price'=>$variation_price,
				'price_html'=>$price_html
			);
		}
		return $price;
    }

    function get_course_batches($request){
    	
    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		
	        if(empty($_POST['course_id'])){
	        	$post = json_decode(file_get_contents('php://input'));
		        $body = $request->get_body();
		        $body = json_decode($body,true);
	        	$course_id=$body['course_id'];
	        }else{
	        	$course_id=$_POST['course_id'];
	        }
    		
    		
    		if(!is_numeric($course_id)){
    			$return=array(
	    			'status'=>0,
	    			'message'=>_x('Invalid Course !','vibe'),
	    			'data'=>$data
	    		);
    		}
    		$is_course_member=false;
    		if(function_exists('bp_course_is_member') && bp_course_is_member($course_id,$this->user->id)){
				$is_course_member = true;
			}
			$product_id = 0;
			if(class_exists('WooCommerce')){
				$product_id = $product_id = get_post_meta($course_id,'vibe_product',true);
				if( is_array($product_id) ){
					$product_id = $product_id[0];
				}

			}
			


    		$wplms_batches = WPLMS_Batch_Functions::init();
    		$course_batches=$wplms_batches->get_course_batches($course_id);
    		$data=[];
    		foreach ($course_batches as $key => $value) {
    			$d = $this->get_batch_template($value->group_id);
    		
				$d['is_course_member'] = $is_course_member;

				if( is_numeric($product_id) && !empty($product_id)){
					$cart_url = wc_get_cart_url();
					$_cart_url = $cart_url = $cart_url.'?add-to-cart='.$product_id.'&batch_id='.$value->group_id;

					$cart_url  = apply_filters('wplms_batches_add_to_cart_link',$cart_url,$product_id,$value->group_id);
					
					if(!$is_course_member){
						$d['cart_link'] = $cart_url;
					}else{
						if($_cart_url != $cart_url){
							$d['cart_link'] = $cart_url;
						}
					}
				}

				
    			$data[]=$d;
    		}
    		$return=array(
    			'status'=>1,
    			'message'=>_x('Batches exist you can run api','vibe'),
    			'data'=>$data
    		);


    	}else{
    		$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
    		);
    	}

    	return new WP_REST_Response($return, 200);
    }

   	function get_course_batch_api($course_id){

    	if(bp_is_active('groups') && class_exists('WPLMS_Batch_Functions')){
    		
    		$wplms_batches = WPLMS_Batch_Functions::init();
    		$course_batches=$wplms_batches->get_course_batches($course_id);
    		$data=[];
    		foreach ($course_batches as $key => $value) {
    			$data[]=$this->get_batch_template($value->group_id);
    		}

    		$batches_course_access=$this->access_course_batches($course_batches,$course_id);

    		$return=array(
    			'status'=>1,
    			'message'=>_x('My batches exist you can run api','vibe'),
    			'batches_course_access'=>$batches_course_access,
    			'data'=>$data
    		);


    	}else{
    		$return=array(
    			'status'=>0,
    			'message'=>_x('Buddypress group not active or wplms-batch plugin not activated','vibe')
    		);
    	}

    	return $return;
    }

    /* fetch user batches and course batches to return access or not if course_batches already we have*/
    function access_course_batches($course_batches,$course_id){
    		$wplms_batches = WPLMS_Batch_Functions::init();
    		$return =array(
	    			'return'=>false,
	    			'message'=>_x('Not access to course','vibe')
	    		);
    		if($course_batches && is_array($course_batches)){
		    	if($this->user_id){
		    		$vibe_force_batch_enrolment=get_post_meta($course_id,'vibe_force_batch_enrolment',true);
		    		if($vibe_force_batch_enrolment=='S'){
		    			$count=wplms_user_in_course_batches($this->user_id,$course_id);
		    			if(!$count){
		    				$return =array(
				    			'return'=>false,
				    			'message'=>_x('Enroll in a batch','vibe')
				    		);
				    		return $return;
		    			}
		    		}
		    	}else{
		    		return $return;
		    	}
	    		$accesses=array();
	    		foreach ($course_batches as $key => $value) {
					if(wplms_batch_is_user_member( $this->user_id, $value->group_id )){
						$accesses[]=$wplms_batches->batch_timing_accessbility($value->group_id,$course_id,1);
					}
	    		}
	    		$bit=true;// if any false retur n from batch then no access to course 
	    		foreach ($accesses as $access) {
	    			$bit=$bit & $access['return'];
	    			
	    		}
	    		if($bit){
	    			$return=array(
						'return'=>true,
						'message'=>_x('Course is accessible with batches','vibe')
					);
	    		}else{
					$return=array(
						'return'=>false,
						'message'=>$access['message']
					);
	    		}
    		}else{
    			$return=array(
    				'return'=>true,
    				'message'=>'No batches in course'
    			);
    		}
    		return $return;
    }

    function join_remove_batch($request){
    	$post = json_decode(file_get_contents('php://input'));
        $body = $request->get_body();
        $body = json_decode($body,true);

        if(empty($_POST['batch_id'])){
        	$post = json_decode(file_get_contents('php://input'));
	        $body = $request->get_body();
	        $body = json_decode($body,true);
	        $batch_id=$body['batch_id'];
	        $type=$body['type'];
        }else{
        	$batch_id=$_POST['batch_id'];
        	$type=$_POST['type'];
        }
    	
    	if($type=='add'){
    		if(!empty(groups_get_groupmeta($batch_id,'enable_seats',true))){
    			$batch_seats=groups_get_groupmeta($batch_id,'batch_seats',true);
    			$current_members=groups_get_groupmeta($batch_id,'total_member_count',true);
    			if($current_members<$batch_seats){
    				if(!groups_is_user_member($this->user_id,$batch_id)){
			    		$data=groups_join_group($batch_id, $this->user_id);
				    	if($data){
				    		$return=array(
				    			'status'=>1,
				    			'message'=>_x('You have joined the batch','vibe'),
				    			'data'=>$data
				    		);
				    	}else{
				    		$return=array(
				    			'status'=>0,
				    			'message'=>_x('You have not joined the batch','vibe'),
				    			'data'=>$data
				    		);
				    	}
    				}else{
    					$return=array(
			    			'status'=>0,
			    			'message'=>_x('You are already member of this batch','vibe'),
			    			'data'=>$data
			    		);
    				}
    			}else{
    				$return=array(
			    			'status'=>0,
			    			'message'=>_x('Member limit reached in batch','vibe'),
			    			'data'=>$data
			    	);
    			}
    		}else{
    			if(!groups_is_user_member($this->user_id,$batch_id)){
	    			$data=groups_join_group($batch_id, $this->user_id);
			    	if($data){
			    		$return=array(
			    			'status'=>1,
			    			'message'=>_x('You have joined the batch','vibe'),
			    			'data'=>$data
			    		);
			    	}else{
			    		$return=array(
			    			'status'=>0,
			    			'message'=>_x('You have not joined the batch','vibe'),
			    			'data'=>$data
			    		);
			    	}
    			}else{
    				$return=array(
		    			'status'=>0,
		    			'message'=>_x('You are already member of this batch','vibe'),
		    			'data'=>$data
			    	);
    			}
    		}
	    	
    	}else if($type=='remove'){
    		$data=groups_leave_group($batch_id, $this->user_id);
    		if($data){
	    		$return=array(
	    			'status'=>1,
	    			'message'=>_x('You have been removed from batch','vibe'),
	    			'data'=>$data
	    		);
	    	}else{
	    		$return=array(
	    			'status'=>0,
	    			'message'=>_x('You have been not removed from batch','vibe'),
	    			'data'=>$data
	    		);
	    	}
    	}
    	return new WP_REST_Response($return, 200);
    }

    function get_default_batch_settings($settings){
    	$this->batch_settings = $settings;
    }

    function load_batch_curriculum($request){

        $body = json_decode($request->get_body(),true);

        $value = groups_get_groupmeta($body['batch_id'],'vibe_course_curriculum_'.$body['course_id'],true);
        if(empty($value)){
        	$return = array(
	        	'status'=>0,
	        	'message'=>__('No curriculum found','wplms-batches')
	        );
        }else{
        	$curriculum = wplms_process_curriculum($value);
        	$return = array(
	        	'status'=>1,
	        	'curriculum'=>$curriculum,
	        	'value'=>$value,
	        );
        }
        
    	return new WP_REST_Response($return, 200);
    }

    function save_batch_curriculum($request){

    	$body = json_decode($request->get_body(),true);

        groups_update_groupmeta($body['batch_id'],'vibe_course_curriculum_'.$body['course_id'],$body['curriculum']);
        
    	$return = array(
        	'status'=>1,
        	'message'=>__('Batch Curriculum saved','wplms-batches')
        );
    	return new WP_REST_Response($return, 200);
    }
}
Wplms_Batches_Api::init();
			
