<?php
/**
 * Initialise WPLMS Batches
 *
 * @class       Bp_Wplms_Bactches_Init
 * @author      wplms-batchesThemes
 * @category    Admin
 * @package     WPLMS-Batches/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
if(!class_exists('Vibebp_Wplms_Filters')){
	class Vibebp_Wplms_Filters{


		public static $instance;
		
		public static function init(){

	        if ( is_null( self::$instance ) )
	            self::$instance = new Vibebp_Wplms_Filters();
	        return self::$instance;
	    }

		private function __construct(){

			add_filter('vibebp_group_tabs',array($this,'add_batch_tab'),10,3);
			add_filter('wplms_front_end_generate_fields_default',array($this,'create_batches'),9999,3);


			add_filter('wplms_before_course_status_api',array($this,'wplms_before_course_status_api'),10,3);

			add_filter('selectcpt_wplms_groups',array($this,'batch_search'),10,3);
			add_filter('wplms_get_course_tabs',array($this,'wplms_get_course_tabs'),10,3);

			add_filter('vibebp_groups_api_can_add_members',array($this,'check_if_seats'),10,3);
			add_filter('vibebp_groups_api_can_invite',array($this,'check_if_seats'),10,3);
			
			add_filter('wplms_create_course_settings',array($this,'curriculum_icons'));
			add_filter('bp_course_wplms_filters',array($this,'batch_course_visibility'),10,2);
		}

		function batch_course_visibility($args,$user_id = null){
			if(!empty($user_id) && !user_can($user_id,'manage_options')){
				
				$batch_ids = wplms_get_user_batches($user_id);
				foreach($batch_ids as $batch_id){
					$batch_course_visibility = groups_get_groupmeta($batch_id,'batch_course_visibility',true);	
					if(!empty($batch_course_visibility)){
						$course_ids = get_batch_courses($batch_id);
						
						if(!empty($course_ids)){
							if(!empty($args['post__in'])){
								$args['post__in'] = array_merge($args['post__in'],$course_ids);
							}else{
								$args['post__in'] = $course_ids;
							}
						}
					}
				}
			}
			return $args;
		}

		function curriculum_icons($settings){
			if(empty($settings['settings']['curriculum_icons'])){
				$settings['settings']['curriculum_icons']=[array('key'=>'batch_curriculums vicon vicon-briefcase','label'=>__('Batch Curriculums','wplms-batches'))];
			}else{
				$settings['settings']['curriculum_icons'][]=array('key'=>'batch_curriculums vicon vicon-briefcase','label'=>__('Batch Curriculums','wplms-batches'));
			}

			
			return $settings;
		}
		function check_if_seats($flag,$group_id,$user_id){
			

			if(!user_can($user_id,'manage_options')){
				$enable_seats  = groups_get_groupmeta( $group_id, 'enable_seats' );	
				$seats  = (int)groups_get_groupmeta( $group_id, 'batch_seats' );
				if(!empty($enable_seats) && !empty($seats)){
					$total_member_count = (int)groups_get_groupmeta( $group_id, 'total_member_count' );
					
					if($total_member_count >= $seats)
						return false;
				}
			}

			return $flag;
		}

		function wplms_batch_courses($args,$user_id =NULL){

			$visibility_flag = apply_filters('wplms_batches_enable_batch_course_visibility_switch',0);
			if(empty($visibility_flag) || current_user_can('edit_posts'))
				return $args;

		  	if($args['post_type'] == 'course'){
		  		if(empty($user_id)){
		    		$user_id=get_current_user_id();
		  		}
		    	
		    	if(isset($args['meta_query']) && is_array($args['meta_query']) && is_user_logged_in()){
		       		foreach($args['meta_query'] as $query){
		         		if(isset($query) && is_array($query)){
		            		if($query['key'] == $user_id){
		              			return $args;
		            		}
		         		}
		       		}
		    	}
		    	
		    	global $wpdb,$bp;
				$batches = $wpdb->get_results(
					$wpdb->prepare("SELECT gm2.meta_value as course_id
							FROM {$bp->groups->table_name_members} as g 
							LEFT JOIN {$bp->groups->table_name_groupmeta} as gm
							ON g.group_id = gm.group_id
							LEFT JOIN {$bp->groups->table_name_groupmeta} as gm2
							ON g.group_id = gm2.group_id
							WHERE g.user_id = %d 
							AND gm.meta_key = %s
							AND gm.meta_value = %d 
							AND gm2.meta_key = %s ",
							$user_id,'batch_course_visibility',1,'batch_course'));

		    	if(empty($batches)){
		    		return $args;
		    	}
		    	$courses = array();
		    	foreach($batches as $batch){
		    		$courses[$batch->course_id]=$batch->course_id;
		    	}

		    	
		      	if(!empty($courses) && is_array($courses) && !isset($args['author'])){
	        		if(!empty($args['post__in'])){
		          		$args['post__in'] = array_merge($args['post__in'], $courses);
		        	}else{
		          		$args['post__in'] = $courses;  
		        	}
		      	} 
		  	}
		  	
		  	return $args;    
		}
		
		function wplms_get_course_tabs($tabs,$course_id,$user_id){
			$style = get_post_meta($course_id,'vibe_batch_display',true);



			if(empty($style) || (!empty($style) && $style != 'hide')){
				$batches = wplms_get_course_batches($course_id);
				if(!empty($batches)){
					$tabs['course_batches'] = _x('Batches','','wplms-batches');
				}

				
			}
			return $tabs;
		}

		function wplms_before_course_status_api($stop,$course_id,$user_id){

			if(!empty($course_id)){
				

				$lock = get_post_meta($course_id,'vibe_force_batch_enrolment',true);
				if(!empty($lock) && (function_exists('vibe_validate') && vibe_validate($lock))){
					if(!wplms_user_in_course_batches($user_id,$course_id)){ 
						$stop = array('icon'=>'vicon-lock','error_code'=>'enroll','error_message'=>__('Enroll in Batch','wplms-batches'));
					}
				}
				
				$batches = wplms_get_course_batches($course_id);
				if(!empty($batches)){
					foreach($batches as $batch_id){
						
						if(wplms_batch_is_user_member( $user_id, $batch_id )){

							$accessibile = wplms_batch_timing_accessibility($batch_id,$course_id);
							if(!$accessibile['return']){
								$stop = array('icon'=>'vicon-lock','error_code'=>'unavailable','error_message'=>$accessibile['message']);
								break;
								
							}
							


							$batch_timing_course_accessibility = groups_get_groupmeta($batch_id,'batch_timing_course_accessibility');
							if(!empty($batch_timing_course_accessibility[$course_id])){
								$return = wplms_batch_timing_accessibility($batch_id,$course_id);
								if(!$return['return']){
									
									$stop = array('icon'=>'vicon-lock','error_code'=>'unavailable','error_message'=>$return['message']);
									break;
									
								}
								
							}
						}
					}
				}
				
			}


	    	return $stop;
		}

		function create_batches($field,$course_id,$user_id=null){
			
			if($field['id']=='vibe_batches' && class_exists('WPLMS_Batch_Functions')){
				$wplms_batches = WPLMS_Batch_Functions::init();
				$batches=$wplms_batches->get_course_batches($course_id);
				$field['value'] = array();
				$field['show_value'] = array();
				foreach($batches as $gg){
					if(!in_array($gg->group_id,$field['value'])){
						$group = groups_get_group(array('group_id'=>$gg->group_id,'populate_extras'=>true));

						$field['show_value'][] = array(
							'permalink' => bp_get_group_permalink($group),
							'id'=>$gg->group_id,
							'text'=>$group->name,
							'link'=>bp_core_get_user_domain($user_id).'#component=groups&action=view&id='.$group->id,
						);
						$field['value'][] =  $gg->group_id;
					}
				}
				
			}

			return $field;
		}
		function add_batch_tab($tabs,$group_id,$user_id){
			$batch = groups_get_groupmeta($group_id,'course_batch',true);
			$admins = bp_group_admin_ids();

			if(is_numeric($admins)){
				$admins = array($admins);
			}else{
				$admins = explode(',',$admins);
			}
			if($batch || (in_array($user_id,$admins) || user_can($user_id,'manage_options'))){
				$tabs['wplms_batch'] =  _x('Batch','tab batch','wplms-batches');
			}
			return $tabs;
		}
		function batch_search($args,$user,$request){

			$batch = $request->get_param('batch');
			if(!empty($batch)){
				if(empty($args['meta_query'])){
					$args['meta_query']=array('relation'=>'AND');
				}
				$args['meta_query'][]=array(
					'key'   => 'course_batch',
					'value' => 1
				);
			}
			
			return $args;
		}
	}

	
}


add_action('plugins_loaded',function(){
	
	if(defined('WPLMS_PLUGIN_VERSION')){

		Vibebp_Wplms_Filters::init();
	}
});
