<?php
/**
 * WplmsGiftCourse_Api API
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	WplmsGiftCourse/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WplmsGiftCourse_Api{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new WplmsGiftCourse_Api();
        return self::$instance;
    }

	private function __construct(){
		add_action('rest_api_init',array($this,'register_api_endpoints'));
	}


	function register_api_endpoints(){

        register_rest_route( WPLMS_GIFT_COURSE_API_NAMESPACE, '/giftsent', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_gifts_sent' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));


        register_rest_route( WPLMS_GIFT_COURSE_API_NAMESPACE, '/giftaccepted', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_accepted_gifts' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));
        

    }

    function user_permissions_check($request){
        //return true;
        
    	$body = json_decode($request->get_body(),true);
        if(!empty($body['token'])){
            global $wpdb;
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
                return true;
            }
        }
        return false;
        
    }


    function get_gifts_sent($request){
        $body = json_decode($request->get_body(),true);
        $user_id = $this->user->id;
        global $wpdb, $bp;

        //Get user's gift sent activity
        $activities = $wpdb->get_results( $wpdb->prepare( "
            SELECT SQL_CALC_FOUND_ROWS id,item_id 
            FROM {$bp->activity->table_name} 
            WHERE user_id = %d AND type = %s ", $user_id, 'wplms_gift_sent') );

        //print_r($activities);

        if(!empty($activities)){
            foreach ($activities as $activity){
                //Get gift email for each gifts sent
                $gift = bp_activity_get_meta($activity->id,'wplms_gift_sent',true);

                //Check if meta exists
                if(!empty($gift)){
                    //Add course name with its link in the table
                    //$course = '<a href="'.get_permalink($activity->item_id).'">'.get_the_title($activity->item_id).'</a>';
                    //$course = array();
                    $course = get_the_title($activity->item_id);

                    $course_link = get_permalink($activity->item_id);

                    //Check if user exists
                    $user_id = email_exists($gift['gift_email']);
                    if(!empty($user_id)){
                        //$name = bp_core_get_userlink($user_id).'('.$gift['gift_email'].')';
                        $name = $gift['gift_email'];

                        //Get user's accepted activity
                        $check_status = $wpdb->get_row( $wpdb->prepare( "SELECT id FROM {$bp->activity->table_name} WHERE user_id = %d AND item_id = %d AND type = %s", $user_id, $activity->item_id, 'wplms_gift_accepted' ) );

                        //Show user's gift status
                        if(empty($check_status)){
                            //$status = '<span class="gift_pending">'.__('PENDING','wplms-gift').'</span>';
                            $status = 'PENDING';
                            $status_class = 'gift_pending';
                        }else{
                            //$status = '<span class="gift_accepted">'.__('Accepted','wplms-gift').'</span>';
                            $status = 'ACCEPTED';
                            $status_class = 'gift_accepted';
                        }

                    }else{
                        $name = $gift['gift_email'];
                        $status = 'PENDING';
                        $status_class = 'gift_pending';
                    }

                     $data = array(
                        'status' => 1,
                        'message' => __('Gift Sent Found','wplms-gift'),
                        'sentgift' => array(
                                    'course_name' => $course,
                                    'course_link' => $course_link,
                                    'user_name' => $name,
                                    'order_status' => $status,
                                    'order_status_class' => $status_class,
                                    )
                    );

                    //print_R($data['sentgift']);
                   
                }
            }
        }else{
             $data = array(
                'status' => 0,
                'message' => __('No Gifts Sent','wplms-gift'),
            );
        }
        $data = apply_filters('wplms_gift_sent_data',$data,$request);
        return new WP_REST_Response($data, 200);
    }


    function get_accepted_gifts($request){
        $body = json_decode($request->get_body(),true);
        $user_id = $this->user->id;
        global $wpdb, $bp;

        //Get user's gift accepted activity
        $activities = $wpdb->get_results( $wpdb->prepare( "
            SELECT SQL_CALC_FOUND_ROWS id,item_id 
            FROM {$bp->activity->table_name} 
            WHERE user_id = %d AND type = %s", $user_id, 'wplms_gift_accepted') );

        if(!empty($activities)){
            foreach ($activities as $activity){

                //Get user's email who sent the gift
                $email = bp_activity_get_meta($activity->id,'wplms_gift_accepted',true);

                if(!empty($email)){
                    //Add course name with its link in the table
                    //$course = '<a href="'.get_permalink($activity->item_id).'">'.get_the_title($activity->item_id).'</a>';

                    $course = get_the_title($activity->item_id);

                    $course_link = get_permalink($activity->item_id);

                    //Check if user exists
                    $user_id = email_exists($email);
                    if(!empty($user_id)){
                        $name = bp_core_get_userlink($user_id).'('.$email.')';
                    }else{
                        $name = $email;
                    }

                    $data = array(
                        'status' => 1,
                        'message' => __('Gift Accepted Found','wplms-gift'),
                        'acceptedgift' => array(
                                    'course_name' => $course,
                                    'course_link' => $course_link,
                                    'user_name' => $name,
                                    )
                    );

                }
            }
        }else{
             $data = array(
                'status' => 0,
                'message' => __('No Gifts Accepted','wplms-gift'),
            );
        }
        $data = apply_filters('wplms_gift_accepted_data',$data,$request);
        return new WP_REST_Response($data, 200);

    }


}

WplmsGiftCourse_Api::init();