<?php
/**
 *
 * @class       Wplms_Grade_Book_Init
 * @author      VibeThemes (H.K.)
 * @category    Admin
 * @package     Wplms_Grade_Book/includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_Grade_Book_Init{

	public static $instance;
	public static function Init(){

	    if ( is_null( self::$instance ) )
	        self::$instance = new Wplms_Grade_Book_Init();
	    return self::$instance;
	}

	public function __construct(){
		// Include script file
		add_action('wp_enqueue_scripts',array($this,'include_wplms_grade_book_scripts'));
		add_action('admin_enqueue_scripts',array($this,'include_wplms_grade_book_scripts_admin'));

		//Ajax function
		add_action('wp_ajax_wplms_grade_book_search',array($this,'wplms_grade_book_search'));
		add_action('wp_ajax_wplms_update_user_marks_from_grade_book',array($this,'wplms_update_user_marks_from_grade_book'));

	} // END public function __construct

	function include_wplms_grade_book_scripts(){
		global $bp;

		$flag = 0;
		if($bp->current_component == 'dashboard'){
			if( is_active_widget( false, false, 'wplms_instructor_grade_book_stats', true ) ){
				$flag = 1;
			}
		}

		if( $flag ){
			//Include CSS
			wp_enqueue_style( 'wplms-grade-book-js', plugins_url('/wplms-grade-book/assets/gradebook.css'));

			//Include JS
			wp_enqueue_script( 'wplms-grade-book-js', plugins_url( '/wplms-grade-book/assets/gradebook.js' ),array('jquery'),WPLMS_GRADEBOOK_VERSION);
		}
	}

	function include_wplms_grade_book_scripts_admin(){
		if( $_GET['page'] == 'lms-stats' && $_GET['tab'] == 'wplms_grade_book' ){
			//Include CSS
			wp_enqueue_style( 'wplms-grade-book-js', plugins_url('/wplms-grade-book/assets/gradebook.css'));

			//Include JS
			wp_enqueue_script( 'wplms-grade-book-js', plugins_url( '/wplms-grade-book/assets/gradebook.js' ),array('jquery'),WPLMS_GRADEBOOK_VERSION);
		}
	}

	function wplms_grade_book_search(){
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'wplms_grade_book_search_security') || !is_user_logged_in() || !isset($_POST['post_type']) ){
			_e('Security check Failed. Contact Administrator.','wplms-gb');
			die();
		}

		//Define Variables
		$search 	= $_POST['search'];
		$post_type  = $_POST['post_type'];

		global $wpdb;
		if( empty($search) ){
			$current_user_id 	= get_current_user_id();
			$all_users 			= array();
			$all_courses_string = array();

			// Pagination
			$limit = vibe_get_option('loop_number');
			if(empty($limit)) $limit = 5;
			$paged = $_POST['page'];

			if( empty($paged) ){
				$paged  = 1;
				$offset = 0;
			}else{
				$offset = ( $paged - 1 ) * $limit;
			}

			$courses = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = 'course'");
			if( !empty($courses) ){
				foreach ($courses as $c) {
					$all_courses_string[] = '"course_status'.$c->ID.'"';
				}

				$sql_ready_implode = implode(',', $all_courses_string);
				$sql_ready = '('.$sql_ready_implode.')';
				$user_query = "SELECT DISTINCT user_id FROM {$wpdb->usermeta} WHERE meta_key IN {$sql_ready} LIMIT $offset,$limit";
				$users = $wpdb->get_results($user_query);
				if ( !empty($users) ){
					foreach ($users as $user){
						$all_users[] = $user->user_id;
					}
				}
			}

			$posts = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = '$post_type'");

			if( $post_type == 'course' ){
				$class = 'all_courses_grade_book_table';
			}elseif ($post_type == 'wplms-assignment'){
				$class = 'all_assignments_grade_book_table';
			}elseif ($post_type == 'quiz'){
				$class = 'all_quizzes_grade_book_table';
			}

			echo '<table class="'.$class.'">';
			echo '<tr>';
			echo '<th>'.__('Students','wplms-gb').'</th>';
			if( !empty($posts) ){
				foreach ($posts as $post) {
					echo '<th>'.get_the_title($post->ID).'</th>';
				}
			}
			echo '</tr>';
			if( !empty($all_users) ){
				foreach ($all_users as $user_id) {
					echo '<tr>';
					echo '<td>'.bp_core_get_user_displayname( $user_id ).'</td>';
					if( !empty($posts) ){
						foreach ($posts as $post) {
							$marks = get_post_meta( $post->ID,$user_id,true );
							if( empty($marks) ){
								echo '<td><span class="marks" data-id="'.$post->ID.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.__('N.A.','wplms-gb').'</span></td>';
							}else{
								echo '<td><span class="marks" data-id="'.$post->ID.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.$marks.'</span></td>';
							}
						}
					}
					echo '</tr>';
				}
			}
			echo '</table>';
			die();
		}

		$user = get_user_by( 'login', $search );
		if( $user ){
			$user_id = $user->ID;
		}else{
			$user = get_user_by( 'email', $search );
			if( $user ){
				$user_id = $user->ID;
			}else{
				echo '<div class="message">'.__('No user found !','wplms-gb').'</div>';
				die();
			}
		}

		$current_user_id = get_current_user_id();
		$posts = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = '$post_type'");

		if( $post_type == 'course' ){
			$class = 'all_courses_grade_book_table';
		}elseif ($post_type == 'wplms-assignment'){
			$class = 'all_assignments_grade_book_table';
		}elseif ($post_type == 'quiz'){
			$class = 'all_quizzes_grade_book_table';
		}

		echo '<table class="'.$class.'">';
		echo '<tr>';
		echo '<th>'.__('Students','wplms-gb').'</th>';
		if( !empty($posts) ){
			foreach ($posts as $post) {
				echo '<th>'.get_the_title($post->ID).'</th>';
			}
		}
		echo '</tr>';
		echo '<tr>';
		echo '<td>'.bp_core_get_user_displayname( $user_id ).'</td>';
		if( !empty($posts) ){
			foreach ($posts as $post) {
				$marks = get_post_meta( $post->ID,$user_id,true );
				if( empty($marks) ){
					echo '<td><span class="marks" data-id="'.$post->ID.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.__('N.A.','wplms-gb').'</span></td>';
				}else{
					echo '<td><span class="marks" data-id="'.$post->ID.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.$marks.'</span></td>';
				}
			}
		}
		echo '</tr>';
		echo '</table>';

		die();
	}

	function wplms_update_user_marks_from_grade_book(){
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'wplms_grade_book_search_security') || !is_user_logged_in() || !isset($_POST['post_id']) || !isset($_POST['user_id']) || !isset($_POST['marks']) ){
			_e('Security check Failed. Contact Administrator.','wplms-gb');
			die();
		}

		update_post_meta( $_POST['post_id'],$_POST['user_id'],$_POST['marks'] );

		if(get_post_type($_POST['post_id']) != 'course')
			die();

		$id = $_POST['post_id'];
		$user_id = $_POST['user_id'];

		do_action('bp_get_course_check_course_complete',$id,$user_id);
		$stop_finish_course_check = apply_filters('bp_get_course_check_course_complete_stop',0,$id,$user_id);

		if($stop_finish_course_check)
			die();

		$course_id = $id;
		$auto_eval = get_post_meta($id,'vibe_course_auto_eval',true);
		

		if(function_exists('vibe_validate') && vibe_validate($auto_eval)){


			$marks = $_POST['marks'];


			$badge_per = get_post_meta($id,'vibe_course_badge_percentage',true);

			$passing_cert = get_post_meta($id,'vibe_course_certificate',true); // Certificate Enable
			$passing_per = get_post_meta($id,'vibe_course_passing_percentage',true); // Certificate Passing Percentage

			//finish bit for student 1.8.4
			update_user_meta($user_id,'course_status'.$id,3);
			//end finish bit
			
			
			
			$badge_filter = 0;
			if(isset($badge_per) && $badge_per && $marks >= $badge_per)
					$badge_filter = 1;

			$badge_filter = apply_filters('wplms_course_student_badge_check',$badge_filter,$course_id,$user_id,$marks,$badge_per);
		    if($badge_filter){  
		        $badges = array();
		        $badges= vibe_sanitize(get_user_meta($user_id,'badges',false));

		        if(isset($badges) && is_array($badges)){
		        	if(!in_array($id,$badges)){
		        		$badges[]=$id;
		        	}
		        }else{
		        	$badges=array($id);
		        }

		        update_user_meta($user_id,'badges',$badges);

		        do_action('wplms_badge_earned',$id,$badges,$user_id,$badge_filter);
		    }
		    $passing_filter =0;
		    if(vibe_validate($passing_cert) && isset($passing_per) && $passing_per && $marks >= $passing_per)
		    	$passing_filter = 1;

		    $passing_filter = apply_filters('wplms_course_student_certificate_check',$passing_filter,$course_id,$user_id,$marks,$passing_per);
		    
		    if($passing_filter){
		        $pass = array();
		        $pass=vibe_sanitize(get_user_meta($user_id,'certificates',false));
		        
		        if(isset($pass) && is_array($pass)){
		        	if(!in_array($id,$pass)){
		        		$pass[]=$id;
		        	}
		        }else{
		        	$pass=array($id);
		        }

		        update_user_meta($user_id,'certificates',$pass);
		        do_action('wplms_certificate_earned',$id,$pass,$user_id,$passing_filter);
		        
		        
		    }

		    $course_end_status = apply_filters('wplms_course_status',4);  
			update_user_meta( $user_id,'course_status'.$id,$course_end_status);//EXCEPTION	

		    
		    
		    do_action('wplms_evaluate_course',$id,$marks,$user_id,1);
		    
		    do_action('wplms_submit_course',$post->ID,$user_id);

		}else{
			
			bp_course_update_user_course_status($user_id,$id,2);
		}

	}


} // END class Wplms_Grade_Book_Init

Wplms_Grade_Book_Init::Init();
