<?php

if (!defined('ABSPATH')) { exit; }

if (!class_exists('WPLMS_Grade_Book_Stats')){
	class WPLMS_Grade_Book_Stats{

		public static $instance;
		public static function init(){
			if ( is_null( self::$instance ) )
				self::$instance = new WPLMS_Grade_Book_Stats();
			return self::$instance;
		}

		public function __construct(){
			$this->all_data = array(); //<- caching variable

			add_filter('lms_stats_tabs',array($this,'add_tab'));
			add_filter('lms_stats_wplms_grade_book_subtabs',array($this,'subtabs'));
			add_action('lms_stats_tab_wplms_grade_book_output',array($this,'wplms_grade_book_stats_courses'));
			add_action('lms_stats_tab_wplms_grade_book_subtab_courses_output',array($this,'wplms_grade_book_stats_courses'));
			add_action('lms_stats_tab_wplms_grade_book_subtab_assignments_output',array($this,'wplms_grade_book_stats_assignments'));
			add_action('lms_stats_tab_wplms_grade_book_subtab_quizzes_output',array($this,'wplms_grade_book_stats_quizzes'));
		}

		function get_data(){
			if( !empty($this->all_data) ){
				return $this->all_data;
			}

			global $wpdb;
			$this->all_data = array(
								'all_courses' 	  => array(),
								'all_assignments' => array(),
								'all_quizzes' 	  => array(),
								'total_students'  => 0,
								'sql_meta_keys'   => '',
							);

			if(current_user_can('manage_options')){
				//All Posts
				$courses = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_type = 'course'");
				$assignments = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_type = 'wplms-assignment'");
				$quizzes = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_type = 'quiz'");

			}else if(current_user_can('edit_posts')){
				$current_user_id = get_current_user_id();
				//All Posts
				$courses = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = 'course'");
				$assignments = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = 'wplms-assignment'");
				$quizzes = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_author = $current_user_id AND post_type = 'quiz'");
			}

			// All Courses
			if( !empty($courses) ){
				$all_courses_string = array();
				foreach ($courses as $c) {
					$this->all_data['all_courses'][] = $c->ID;
					$all_courses_string[] = '"course_status'.$c->ID.'"';
				}

				$sql_ready_implode = implode(',', $all_courses_string);
				$sql_ready = '('.$sql_ready_implode.')';
				$this->all_data['sql_meta_keys'] = $sql_ready;

				$total_query = "SELECT COUNT(DISTINCT user_id) FROM {$wpdb->usermeta} WHERE meta_key IN {$sql_ready} ";

				$this->all_data['total_students'] = $wpdb->get_var($total_query);

			}

			// All Assignments
			if( !empty($assignments) ){
				foreach ($assignments as $a) {
					$this->all_data['all_assignments'][] = $a->ID;
				}
			}

			// All Quizzes
			if( !empty($quizzes) ){
				foreach ($quizzes as $q) {
					$this->all_data['all_quizzes'][] = $q->ID;
				}
			}

			return $this->all_data;
		}

		function add_tab($tabs){
			$tabs['wplms_grade_book'] = _x('Grade Book','Tab name in LMS stats','wplms-gb');
			return $tabs;
		}

		function subtabs($subtabs){
			echo '<h2>'._x('Grade Book Statistics','LMS statistics','wplms-gb').'</h2>';
			$subtabs['wplms_grade_book'] = array(
				'overview' 		=> __('Overview','wplms-gb'),
				'courses' 		=> __('Courses','wplms-gb'),
				'assignments' 	=> __('Assignments','wplms-gb'),
				'quizzes' 		=> __('Quizzes','wplms-gb'),
			);

			return $subtabs;
		}

		function wplms_grade_book_stats_courses(){
			global $wpdb;
			$data = $this->get_data();
			extract($data);

			// Pagination
			$num = 5;
			$paged = 0;
			if($_REQUEST['paged'] && is_numeric($_REQUEST['paged'])){
				$paged = $_REQUEST['paged'];
			}
			$page_num = 0;
			if(isset($_GET['paged']) && is_numeric($_GET['paged']) && $_GET['paged'])
					$page_num=($_GET['paged'])*$num;

			$all_users = array();
			//user query
			$user_query = "SELECT DISTINCT user_id FROM {$wpdb->usermeta} WHERE meta_key IN {$sql_meta_keys} LIMIT $page_num,$num";

			//Users
			$users = $wpdb->get_results($user_query);
			if ( !empty($users) ){
				foreach ($users as $user) {
					$all_users[] = $user->user_id;
				}
			}

			?>
			<div class="vibe-reports-wrap">
				<div class="vibe-reports-sidebar">
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Courses','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_courses); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Assignments','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_assignments); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Quizzes','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_quizzes); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Students','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo $total_students; ?></p>
						</div>
					</div>
				</div>

				<div class="vibe-reports-main">
					<div class="postbox course_info">
						<?php
						if( !empty($all_courses) ){
							//Create table for the all courses data.
							echo '<div class="grade_book_table admin">';
							echo '<table class="all_courses_grade_book_table">';
							echo '<tr>';
							echo '<th>'.__('Students','wplms-gb').'</th>';
							foreach ($all_courses as $course_id) {
								echo '<th>'.get_the_title($course_id).'</th>';
							}
							echo '</tr>';
							if( !empty($all_users) ){
								foreach ($all_users as $user_id) {
									echo '<tr>';
									echo '<td>'.bp_core_get_user_displayname( $user_id ).'</td>';
									foreach ($all_courses as $course_id) {
										$marks = get_post_meta( $course_id,$user_id,true );
										if( empty($marks) ){
											echo '<td><span class="marks" data-id="'.$course_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.__('N.A.','wplms-gb').'</span></td>';
										}else{
											echo '<td><span class="marks" data-id="'.$course_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.$marks.'</span></td>';
										}
									}
									echo '</tr>';
								}
							}
							echo '</table>';
							echo '</div>';
						}

						//Pagination
						if(isset($_GET['paged']) && $_GET['paged']){
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($_GET['paged']-1).'" class="button">&lsaquo; '.__('Prev','wplms-gb').'</a>';
						}
						$total = count($all_users);
						if($total == $num){
							if(isset($_GET['paged']) && $_GET['paged']){
								$paged =$_GET['paged'];
								echo '&nbsp;&nbsp;';
							}else{
								$paged = 0;
							}
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($paged+1).'" class="button">'.__('Next','wplms-gb').' &rsaquo;</a>';
						}
						?>
					</div>
				</div>

			</div>
			<?php
		}

		function wplms_grade_book_stats_assignments(){
			global $wpdb;
			$data = $this->get_data();
			extract($data);

			// Pagination
			$num = 5;
			$paged = 0;
			if($_REQUEST['paged'] && is_numeric($_REQUEST['paged'])){
				$paged = $_REQUEST['paged'];
			}
			$page_num = 0;
			if(isset($_GET['paged']) && is_numeric($_GET['paged']) && $_GET['paged'])
					$page_num=($_GET['paged'])*$num;

			$all_users = array();
			//user query
			$user_query = "SELECT DISTINCT user_id FROM {$wpdb->usermeta} WHERE meta_key IN {$sql_meta_keys} LIMIT $page_num,$num";

			//Users
			$users = $wpdb->get_results($user_query);
			if ( !empty($users) ){
				foreach ($users as $user) {
					$all_users[] = $user->user_id;
				}
			}

			?>
			<div class="vibe-reports-wrap">
				<div class="vibe-reports-sidebar">
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Courses','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_courses); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Assignments','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_assignments); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Quizzes','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_quizzes); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Students','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo $total_students; ?></p>
						</div>
					</div>
				</div>

				<div class="vibe-reports-main">
					<div class="postbox course_info">
						<?php
						if( !empty($all_assignments) ){
							//Create table for the all courses data.
							echo '<div class="grade_book_table admin">';
							echo '<table class="all_assignments_grade_book_table">';
							echo '<tr>';
							echo '<th>'.__('Students','wplms-gb').'</th>';
							foreach ($all_assignments as $assignment_id) {
								echo '<th>'.get_the_title($assignment_id).'</th>';
							}
							echo '</tr>';
							if( !empty($all_users) ){
								foreach ($all_users as $user_id) {
									echo '<tr>';
									echo '<td>'.bp_core_get_user_displayname( $user_id ).'</td>';
									foreach ($all_assignments as $assignment_id) {
										$marks = get_post_meta( $assignment_id,$user_id,true );
										if( empty($marks) ){
											echo '<td><span class="marks" data-id="'.$assignment_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.__('N.A.','wplms-gb').'</span></td>';
										}else{
											echo '<td><span class="marks" data-id="'.$assignment_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.$marks.'</span></td>';
										}
									}
									echo '</tr>';
								}
							}
							echo '</table>';
							echo '</div>';
						}

						//Pagination
						if(isset($_GET['paged']) && $_GET['paged']){
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($_GET['paged']-1).'" class="button">&lsaquo; '.__('Prev','wplms-gb').'</a>';
						}
						$total = count($all_users);
						if($total == $num){
							if(isset($_GET['paged']) && $_GET['paged']){
								$paged =$_GET['paged'];
								echo '&nbsp;&nbsp;';
							}else{
								$paged = 0;
							}
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($paged+1).'" class="button">'.__('Next','wplms-gb').' &rsaquo;</a>';
						}
						?>
					</div>
				</div>

			</div>
			<?php
		}

		function wplms_grade_book_stats_quizzes(){
			global $wpdb;
			$data = $this->get_data();
			extract($data);

			// Pagination
			$num = 5;
			$paged = 0;
			if($_REQUEST['paged'] && is_numeric($_REQUEST['paged'])){
				$paged = $_REQUEST['paged'];
			}
			$page_num = 0;
			if(isset($_GET['paged']) && is_numeric($_GET['paged']) && $_GET['paged'])
					$page_num=($_GET['paged'])*$num;

			$all_users = array();
			//user query
			$user_query = "SELECT DISTINCT user_id FROM {$wpdb->usermeta} WHERE meta_key IN {$sql_meta_keys} LIMIT $page_num,$num";

			//Users
			$users = $wpdb->get_results($user_query);
			if ( !empty($users) ){
				foreach ($users as $user) {
					$all_users[] = $user->user_id;
				}
			}

			?>
			<div class="vibe-reports-wrap">
				<div class="vibe-reports-sidebar">
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Courses','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_courses); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Assignments','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_assignments); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Quizzes','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo count($all_quizzes); ?></p>
						</div>
					</div>
					<div class="postbox">
						<div class="inside">
							<h3><span><?php _ex('Total Students','LMS Stats','wplms-gb'); ?></span></h3>
							<p class="stat"><?php echo $total_students; ?></p>
						</div>
					</div>
				</div>

				<div class="vibe-reports-main">
					<div class="postbox course_info">
						<?php
						if( !empty($all_quizzes) ){
							//Create table for the all courses data.
							echo '<div class="grade_book_table admin">';
							echo '<table class="all_quizzes_grade_book_table">';
							echo '<tr>';
							echo '<th>'.__('Students','wplms-gb').'</th>';
							foreach ($all_quizzes as $quiz_id) {
								echo '<th>'.get_the_title($quiz_id).'</th>';
							}
							echo '</tr>';
							if( !empty($all_users) ){
								foreach ($all_users as $user_id) {
									echo '<tr>';
									echo '<td>'.bp_core_get_user_displayname( $user_id ).'</td>';
									foreach ($all_quizzes as $quiz_id) {
										$marks = get_post_meta( $quiz_id,$user_id,true );
										if( empty($marks) ){
											echo '<td><span class="marks" data-id="'.$quiz_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.__('N.A.','wplms-gb').'</span></td>';
										}else{
											echo '<td><span class="marks" data-id="'.$quiz_id.'" data-user="'.$user_id.'" data-update="'.__('Update','wplms-gb').'" data-cancle="'.__('Cancle','wplms-gb').'">'.$marks.'</span></td>';
										}
									}
									echo '</tr>';
								}
							}
							echo '</table>';
							echo '</div>';
						}

						//Pagination
						if(isset($_GET['paged']) && $_GET['paged']){
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($_GET['paged']-1).'" class="button">&lsaquo; '.__('Prev','wplms-gb').'</a>';
						}
						$total = count($all_users);
						if($total == $num){
							if(isset($_GET['paged']) && $_GET['paged']){
								$paged =$_GET['paged'];
								echo '&nbsp;&nbsp;';
							}else{
								$paged = 0;
							}
							echo '<a href="?page=lms-stats&tab=wplms_grade_book&subtab='.$_GET['subtab'].'&paged='.($paged+1).'" class="button">'.__('Next','wplms-gb').' &rsaquo;</a>';
						}
						?>
					</div>
				</div>

			</div>
			<?php
		}
	}
}

add_action('admin_init',function(){
	WPLMS_Grade_Book_Stats::init();
});
