<?php
/**
 * Wplms SMS Notifications Init
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	wplms-sms-notifications/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_SMS_Notifications_Init{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_SMS_Notifications_Init();
        return self::$instance;
    }

	private function __construct(){

		add_filter('xprofile_group_fields',array($this,'fields'));

		// add_action('bp_after_register_page',array($this,'add_twilio_verification'),999);
		// add_action('bp_after_profile_edit_content',array($this,'add_twilio_verification'),999);

		add_action('bp_signup_validate', array($this,'validate_twilio_number'),999);
		add_action( 'xprofile_data_before_save',array($this, 'validate_edit_twilio_number'),999 );

		add_action('bp_notification_after_save',array($this,'wplms_send_pn'),10,1);


		add_action('wp_ajax_get_all_verified_numbers',array($this,'get_all_verified_numbers'));
		add_action('wp_ajax_nopriv_get_all_verified_numbers',array($this,'get_all_verified_numbers'));
	}
	
	function fields($fields){

		$twilio_settings = $this->get_settings();
		if(empty($twilio_settings['phonenumber_field']))
			return $fields;

		$field_ids = wp_list_pluck($fields,'id');


		if(!in_array($twilio_settings['phonenumber_field'],$field_ids)){
			return $fields;
		}
		$this->add_twilio_verification();

		return $fields;
	}

	function get_settings(){
		
		if(empty($this->settings)){
			$this->settings = get_option(WPLMS_SMS_NOTIFICATION_OPTION);		
		}
		return $this->settings;
	}

	function add_twilio_verification(){

		
		$twilio_settings = $this->get_settings();

		if(!$twilio_settings['sms_notifications'] || empty($twilio_settings['twilio_acc_sid']) || empty($twilio_settings['twilio_auth_token']) || empty($twilio_settings['twilio_number']) || empty($twilio_settings['phonenumber_field'])){
			return;
		}
		$field_id = 'field_'.$twilio_settings['phonenumber_field'];
		$desc_id = 'field-visibility-settings-'.$twilio_settings['phonenumber_field'];
		do_action( 'bp_twilio_verified_errors' );

		wp_nonce_field('Wplms_SMS_Notifications_Settings','Wplms_SMS_Notifications_Settings');

		wp_enqueue_script('wplms-sms-phone-input-utils',plugins_url('../assets/utils.js',__FILE__),array('jquery'),WPLMS_SMS_NOTIFICATION_VERSION,true);
		wp_enqueue_script('wplms-sms-phone-input',plugins_url('../assets/intlTelInput.min.js',__FILE__),array('wplms-sms-phone-input-utils'),WPLMS_SMS_NOTIFICATION_VERSION,true);
		wp_enqueue_style('wplms-sms-phone-input',plugins_url('../assets/css/intlTelInput.min.css',__FILE__),array(),WPLMS_SMS_NOTIFICATION_VERSION);
		?>
		<style>
		#buddypress .standard-form input[type=tel]#<?php echo $field_id; ?>{padding-left:48px;}
		.invalid_phone_number{border:1px solid #ed6b6b;}
			.twilio_verify_div, .twilio_verify_btn,.wp-core-ui .button.twilio_verify_btn {
			    display: none;
			}
			.twilio_verify_otp {
				max-width: 172px;
			    margin: 0px 25px 6px 0px;
			}
		</style>
		<script>
			var iti;
			jQuery(document).ready(function($){

				var verified_numbers = [];

				let x = new Promise(function(resolve){
					$.ajax({
						type: "POST",
						url: ajaxurl,
						dataType:'json',
						data: { action: 'get_all_verified_numbers', 
								security: $('#Wplms_SMS_Notifications_Settings').val(),
						},
						cache: false,
						success:function(resp){
							resolve();
							if(resp.status){
								verified_numbers = resp.numbers;
							}
						}
					});
				}).then(function(){



					var content = '<div class = "twilio_verify_div">\
						<input type="text" class= "twilio_verify_otp" name="twilio_otp">\
						<p class="twilio_verify_description"><?php _e('Please Enter OTP','wplms-sms-notifications'); ?></p>\
					</div>\
					<button class="twilio_verify_btn button" name="verify"><?php _e('Verify Number','wplms-sms-notifications'); ?></button>\
					<input type="hidden" name="twilio_verified" value=0 >\
					<input type="hidden" name="service_sid" >';

					$("#<?php echo $desc_id; ?>").after(content);
					iti = window.intlTelInput(document.querySelector("#<?php echo $field_id; ?>"),{
					  initialCountry: "auto",
					  separateDialCode:true,
					  geoIpLookup: function(success, failure) {
					  	if(!$("#<?php echo $field_id; ?>").val().length){
						    $.get("https://ipinfo.io", function(e){console.log(e);}, "jsonp")
						    .always(function(resp) {
						    	var countryCode = (resp && resp.country) ? resp.country : "";
					      		success(countryCode);
					      		failure('us');
						  	});
						}
					  },
					});

					if(!iti.isValidNumber()){
						$("#<?php echo $field_id; ?>").parent().addClass('invalid_phone_number');
					}else{
						console.log(iti.getNumber(intlTelInputUtils.numberFormat.E164).indexOf(verified_numbers));
						if(iti.getNumber(intlTelInputUtils.numberFormat.E164).indexOf(verified_numbers) === -1){
							$(".twilio_verify_btn").show();
						}else{
							$("input[name=twilio_verified]").val(1);
						}
					}
					document.querySelector("#<?php echo $field_id; ?>").addEventListener("countrychange", function() {
					   $("#<?php echo $field_id; ?>").val(iti.getNumber(intlTelInputUtils.numberFormat.E164));
					});
					$("#<?php echo $field_id; ?>").on('change',function(){

						var $this = $(this);

						if(iti.isValidNumber()){
							if(verified_numbers.indexOf(iti.getNumber(intlTelInputUtils.numberFormat.E164)) === -1){
								$(".twilio_verify_btn").show();
							}else{
								$("input[name=twilio_verified]").val(1);
							}
							$this.parent().removeClass('invalid_phone_number');
						} else{
							$(".twilio_verify_btn").hide();
							$this.parent().addClass('invalid_phone_number');
						}
					});

					$(".twilio_verify_btn").on('click',function(e){
						e.preventDefault();

						var phone_no = iti.getNumber(intlTelInputUtils.numberFormat.E164);
						
						

						if(verified_numbers.indexOf(iti.getNumber(intlTelInputUtils.numberFormat.E164)) !== -1){
							$("input[name=twilio_verified]").val(1);
							$(this).hide();
							return;
						}

						var service_sid = $("input[name=service_sid]").val();
						var code = $(".twilio_verify_otp").val();

						if(code.length){

							$.ajax({
								type: "POST",
								url: ajaxurl,
								data: { action: 'check_verification_code', 
										mobile:phone_no,
										code:code,
										service_sid:service_sid,
										security: $('#Wplms_SMS_Notifications_Settings').val(),
								},
								cache: false,
								success: function(response){
									obj = JSON.parse(response);
									$(".twilio_verify_div").html(obj.text);
									$(".twilio_verify_btn").hide();
									$("#<?php echo $field_id; ?>").val(iti.getNumber(intlTelInputUtils.numberFormat.E164));
									verified_numbers.push(phone_no);
									if(obj.status) {
										$("input[name=twilio_verified]").val(1);
									}
								}
							});
						}else{

							$.ajax({
								type: "POST",
								url: ajaxurl,
								data: { action: 'sending_verification_code', 
										mobile:phone_no,
										security: $('#Wplms_SMS_Notifications_Settings').val(),
								},
								cache: false,
								success: function(response){
									obj = JSON.parse(response);
									if(obj.status){
										service_sid = obj.data.service_sid;
										$(".twilio_verify_div").show();
										$("input[name=service_sid]").val(service_sid);
										$(".twilio_verify_btn").attr("value","<?php _e('Verify OTP','wplms-sms-notifications'); ?>");
										$(".twilio_verify_btn").addClass("submit_twilio_number");
									} else{
										return false;
									}
								}
							});
						}
						return false;
					});
				});
			});
		</script>

		<?php
	}

	function validate_twilio_number(){
		global $bp;
		$twilio_settings = $this->get_settings();

		if(!$twilio_settings['sms_notifications'] || empty($twilio_settings['twilio_acc_sid']) || empty($twilio_settings['twilio_auth_token']) || empty($twilio_settings['twilio_number']) || empty($twilio_settings['phonenumber_field'])){
			return;
		}
		$twilio_verified = $_POST['twilio_verified'];
		if(!$twilio_verified || empty($twilio_verified)) {
			$bp->signup->errors['twilio_verified'] = __('Mobile Number verification failed, Please verify mobile number to proceeed', 'wplms-sms-notifications' );
		}
        return;
	}

	function validate_edit_twilio_number(){
		if(!function_exists('bp_displayed_user_id'))
		   return;
		$userdomain = bp_displayed_user_id();
		if(empty($userdomain))
		   return;

		$twilio_settings = $this->get_settings();

		if(!$twilio_settings['sms_notifications'] || empty($twilio_settings['twilio_acc_sid']) || empty($twilio_settings['twilio_auth_token']) || empty($twilio_settings['twilio_number']) || empty($twilio_settings['phonenumber_field'])){
			return;
		}
		$twilio_verified = $_POST['twilio_verified'];
		$message = __('Mobile Number verification failed, Please verify mobile number to proceeed', 'wplms-sms-notifications' );
		if(!$twilio_verified){
           $redirect_url = trailingslashit( bp_displayed_user_domain() . bp_get_profile_slug() . '/edit/group/' . bp_action_variable( 1 ) );
           bp_core_add_message( $message, 'error' );
           bp_core_redirect( $redirect_url );
           die;
       }
       return;

	}

	function wplms_send_pn($args){
		
		$twilio_settings = $this->get_settings();
		
		if($twilio_settings['sms_notifications']) {
			
			if(!class_exists('WPLMS_tips'))
			   return;


			$tips = WPLMS_tips::init();
			if(isset($tips) && isset($tips->lms_settings) && isset($tips->lms_settings['touch'])){
			   
			   

			   if(strpos($args->component_action,'instructor_') !== false){
			        preg_match("/instructor_(.*)/", $args->component_action, $action);
			        $action_name = $action[1];
			        if(!empty($tips->lms_settings['touch'][$action_name]['instructor']['notification'])){
			           $this->maybe_send_pn($args);
			        }
			   }elseif(strpos($args->component_action,'_admin') !== false){
			   		preg_match("/admin(.*)", $args->component_action, $action);
			        $action_name = $action[0];
			        if(!empty($tips->lms_settings['touch'][$action_name]['admin']['notification'])){
			           $this->maybe_send_pn($args);
			        }
			   } else{

			       
					if($args->component_action == 'subscribe_course'){
						$args->component_action = 'course_subscribed';
					}
					$action_name = $args->component_action;
					
			       	if(!empty($tips->lms_settings['touch'][$action_name]['student']['notification'])){
			           $this->maybe_send_pn($args);
			       	}
			   }
			}
		}
	}

	function maybe_send_pn($args){

		if(empty($this->user_number[$args->user_id])){
			$this->user_number[$args->user_id] = strip_tags(bp_get_profile_field_data(array('user_id'=>$args->user_id,'field'=>$this->settings['phonenumber_field'])));
		}

		$user_number =  $this->user_number[$args->user_id];

    	if(!empty($user_number)){
    		$message = '';
			if(function_exists('bp_course_format_notifications')){
				if($args->component_action == 'course_subscribed'){
					$args->component_action = 'subscribe_course';
				}
					
				$message =  bp_course_format_notifications($args->component_action, $args->item_id, $args->secondary_item_id, '', 'string' );
			}

			$message = strip_tags( $message);

			$sms_api = Wplms_Twilio_SMS_Api::init();
			$sms_api->send_sms($user_number, $message);

    	}
    	return;
	}


	function get_all_verified_numbers(){
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'Wplms_SMS_Notifications_Settings') ){
         _e('Security check Failed. Contact Administrator.','wplms-sms-notifications');
         die();
        }

        global $wpdb;
        $vn = $wpdb->get_results("SELECT meta_value FROM {$wpdb->usermeta} WHERE meta_key = 'twilio_verified_number'");
        if(!empty($vn)){
        	foreach($vn as $n){
        		$verified_numbers[]=$n->meta_value;
        	}
        	print_r(json_encode(array('status'=>1,'numbers'=>$verified_numbers)));
        }else{
        	print_r(json_encode(array('status'=>0)));
        }
        die();

	}

}

Wplms_SMS_Notifications_Init::init();
