<?php 
/**
 * Settings in Admin
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	wplms-sms-notifications/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Wplms_SMS_Notifications_Settings{


	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_SMS_Notifications_Settings();
        return self::$instance;
    }

	private function __construct(){

		add_filter('wplms_lms_settings_tabs',array($this,'setting_tab'));
		add_filter('lms_settings_tab',array($this,'tab'));

	}


	function setting_tab($tabs){
		$tabs['wplms-sms-notifications'] = __('Twilio','wplms-sms-notifications');
		return $tabs;
	}

	function tab($name){
		if($name == 'wplms-sms-notifications')
			return 'Wplms_SMS_Notifications_Settings';
		
		return $name; 
	}

	function get_settings(){
 			
		return apply_filters('Wplms_SMS_Notifications_Settings',array(
			array(
				'label' => __('Enable SMS Notification','wplms-sms-notifications'),
				'name' =>'sms_notifications',
				'type' => 'select',
				'options'=>array(
					'0' => __('No','wplms-sms-notifications'),
					'1' => __('Yes','wplms-sms-notifications'),
				),
				'desc' => __('Enable SMS Notification','wplms-sms-notifications')
			),
			array(
				'label' => __('ACCOUNT SID','wplms-sms-notifications'),
				'name' => 'twilio_acc_sid',
				'type' => 'text',
				'desc' => sprintf(__('Set your Twilio account Sid, create a new app from %s','wplms-sms-notifications'),'<a href="https://www.twilio.com/">https://www.twilio.com/</a>'),
			),
			array(
				'label' => __('AUTH TOKEN','wplms-sms-notifications'),
				'name' => 'twilio_auth_token',
				'type' => 'text',
				'desc' => __('Enter Twilio Auth Token','wplms-sms-notifications'),
			),
			array(
				'label' => __('Twilio Number','wplms-sms-notifications'),
				'name' => 'twilio_number',
				'type' => 'text',
				'desc' => __('Enter Twilio Number','wplms-sms-notifications'),
			),
			array(
					'label' => __('Map Phone number to Field','wplms-sms-notifications'),
					'name' => 'phonenumber_field',
					'fields' => $this->fields,
					'type' => 'bp_fields',
					'desc' => __('Map Phone number to Field','wplms-sms-notifications')				
				),
			
		));	
	}
	function settings(){

		echo '<form method="post">';
		wp_nonce_field('Wplms_SMS_Notifications_Settings');   
		echo '<table class="form-table">
				<tbody>';

		$settings = $this->get_settings();
		$settings = apply_filters('twilio_phone_number_field',$settings);		

		$this->generate_form($settings);
		echo '<tr valign="top"><th colspan="2"><input type="submit" name="save_Wplms_SMS_Notifications_Settings" class="button button-primary" value="'.__('Save Settings','wplms-sms-notifications').'" /></th>';
		echo '</tbody></table></form>';	
	}

	function generate_form($settings){
		
		$init = Wplms_SMS_Notifications_Init::init();

		$this->settings = $init->get_settings();

		foreach($settings as $setting ){
			echo '<tr valign="top">';
			switch($setting['type']){
				case 'textarea':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><textarea name="'.$setting['name'].'" style="max-width: 560px; height: 240px;border:1px solid #DDD;">'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:'').'</textarea>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'heading':
					echo '<th scope="row" class="titledesc"><h2>'.$setting['label'].'</h2><span>'.$setting['desc'].'</span></th>';
				break;
				case 'select':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><select name="'.$setting['name'].'">';
					foreach($setting['options'] as $key=>$option){
						echo '<option value="'.$key.'" '.(isset($this->settings[$setting['name']])?selected($key,$this->settings[$setting['name']]):'').'>'.$option.'</option>';
					}
					echo '</select>';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'checkbox':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="checkbox" name="'.$setting['name'].'" '.(isset($this->settings[$setting['name']])?'CHECKED':'').' />';
					echo '<span>'.$setting['desc'].'</span>';
				break;
				case 'number':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="number" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'text':
					echo '<th scope="row" class="titledesc"><label>'.$setting['label'].'</label></th>';
					echo '<td class="forminp"><input type="text" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:$setting['std']).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
				case 'bp_fields':
					echo '<th scope="row" class="titledesc">'.$setting['label'].'</th>';
					echo '<td class="forminp">';

					global $bp,$wpdb;
					$bp_fields = $wpdb->get_results("SELECT DISTINCT id,name FROM {$bp->profile->table_name_fields} WHERE type = 'telephone'",ARRAY_A);

					
					echo '<select name="'.$setting['name'].'">';
					foreach($bp_fields as $v){
						echo '<option value="'.$v['id'].'" '.selected($v['id'],$this->settings[$setting['name']]).'>'.$v['name'].'</option>';
					}
					echo '</select>';
					echo '</td>';
				break;
				default:
					echo '<th scope="row" class="titledesc">'.$setting['label'].'</th>';
					echo '<td class="forminp"><input type="text" name="'.$setting['name'].'" value="'.(isset($this->settings[$setting['name']])?$this->settings[$setting['name']]:(isset($setting['std'])?$setting['std']:'')).'" />';
					echo '<span>'.$setting['desc'].'</span></td>';
				break;
			}
		}	
	}
	function save(){
		

		if(!isset($_POST['save_Wplms_SMS_Notifications_Settings']))
			return;

		if ( !isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'],'Wplms_SMS_Notifications_Settings') ){
		     echo '<div class="error notice is-dismissible"><p>'.__('Security check Failed. Contact Administrator.','wplms-sms-notifications').'</p></div>';
		}

		$settings = $this->get_settings();
		foreach($settings as $setting){
			if(isset($_POST[$setting['name']])){
				$this->settings[$setting['name']] = $_POST[$setting['name']];
			}else if($setting['type'] == 'checkbox' && isset($this->settings[$setting['name']])){
				unset($this->settings[$setting['name']]);
			}
		}

		update_option(WPLMS_SMS_NOTIFICATION_OPTION,$this->settings);
		echo '<div class="updated notice is-dismissible"><p>'.__('Settings Saved.','wplms-sms-notifications').'</p></div>';
	}
}

Wplms_SMS_Notifications_Settings::init();


function Wplms_SMS_Notifications_Settings(){
	
	$wplms_vimeo = Wplms_SMS_Notifications_Settings::init();
	$wplms_vimeo->save();
	$wplms_vimeo->settings();
}