<?php 
/**
 * Twilio API
 *
 * @author 		VibeThemes
 * @category 	Admin
 * @package 	wplms-sms-notifications/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define('WPLMS_SMS_TWILIO_ENDPOINT','https://api.twilio.com/2010-04-01/Accounts/');

class Wplms_Twilio_SMS_Api{

	

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new Wplms_Twilio_SMS_Api();
        return self::$instance;
    }

	private function __construct(){
		add_action('wp_ajax_sending_verification_code', array($this,'sending_verification_code'),10);
		add_action('wp_ajax_nopriv_sending_verification_code', array($this,'sending_verification_code'),10);
		add_action('wp_ajax_check_verification_code', array($this,'check_verification_code'),10);
		add_action('wp_ajax_nopriv_check_verification_code', array($this,'check_verification_code'),10);
	}

	function sending_verification_code(){

	
		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'Wplms_SMS_Notifications_Settings') ){
         _e('Security check Failed. Contact Administrator.','wplms-sms-notifications');
         die();
        }
       
		$mobile = $_POST['mobile'];

		if(strpos($mobile,'+') === false){
			_e('Invalid Phone number','wplms-sms-notifications');
			die();
		}
		$return = Array();

//TWILIO API CALL

	    //--------------------
	    $init = Wplms_SMS_Notifications_Init::init();
		$init->get_settings();

		if(!$init->settings['sms_notifications'] || empty($init->settings['twilio_acc_sid']) || empty($init->settings['twilio_auth_token']) || empty($init->settings['twilio_number']) || empty($init->settings['phonenumber_field'])){
			return 'error';
		}

		if(empty($mobile)){
			$phonenumber =  strip_tags(bp_get_profile_field_data(array('user_id'=>$user_id,'field'=>$init->settings['phonenumber_field'])));
		}else{
			$phonenumber = $mobile;
		}
		

		if(empty($phonenumber))
			return 'no phone number';

		$id = $init->settings['twilio_acc_sid'];
		$token = $init->settings['twilio_auth_token'];
		$url = "https://verify.twilio.com/v2/Services";
		$from = $init->settings['twilio_number'];
		$return = Array();
		$return['status'] = 0;
	    $return['data'] = Array();

		$data = array(
			'FriendlyName' => __('Verify Phone Number','wplms-sms-notifications')
		);
		$headers = array(
			'Authorization' => 'Basic ' . base64_encode($id . ':' . $token)
		);
		$services_response = wp_remote_post($url, array(
			'body' => $data,
			'headers' => $headers
		));
		$verify = wp_remote_retrieve_body($services_response);
		$services = json_decode($verify,true);



//After getting service ID 
		$s_sid = $services['sid']; //VA88d8697c2962c423963c018b565bafcb

	
		if(!empty($s_sid)) {
			$url = "https://verify.twilio.com/v2/Services/".$s_sid."/Verifications";
			$data = array(
				'To' => $mobile,
				'Channel' =>'sms',
			);
			$verification_response = wp_remote_post($url, array(
				'body' => $data,
				'headers' => $headers
			));

		
			$verify = wp_remote_retrieve_body($verification_response);
			$verification = json_decode($verify,true);
			
	        if($verification['sid']) {
	        	//update_user_meta($user_id, 'twilio_verified_number', $mobile);
	        	$return['status'] = 1;
	        	$return['data'] = $verification;
	        	//print_r($verification);
	        }

		}
		print_r(json_encode($return)); 
		die();
	}

	function check_verification_code() {

		if ( !isset($_POST['security']) || !wp_verify_nonce($_POST['security'],'Wplms_SMS_Notifications_Settings') ){
         _e('Security check Failed. Contact Administrator.','wplms-sms-notifications');
         die();
        }
		
		$init = Wplms_SMS_Notifications_Init::init();
		$init->get_settings();

		if(empty($_POST['mobile'])){
			if(is_user_logged_in()){
				$mobile =  strip_tags(bp_get_profile_field_data(array('user_id'=>$user_id,'field'=>$init->settings['phonenumber_field'])));
				if(empty($mobile)){
					_e('Empty Phone number','wplms-sms-notifications');
					die();
				}
    		}else{
    			_e('No Phone number to verify','wplms-sms-notifications');
    			die();
    		}
		}else{
			$mobile = $_POST['mobile'];
		}
    	
		
		$code = $_POST['code'];
		$service_sid = $_POST['service_sid'];
		$return = Array();
		$return['status'] = 0;
	    $return['text'] = sprintf(__('Your Mobile number %s has not been verified, Please input a valid number.','wplms-sms-notifications'),$mobile);

		


		if(!$init->settings['sms_notifications'] || empty($init->settings['twilio_acc_sid']) || empty($init->settings['twilio_auth_token']) || empty($init->settings['twilio_number']) || empty($init->settings['phonenumber_field'])){
			return;
		}


//TWILIO API CALL	
		$id = $init->settings['twilio_acc_sid'];
		$token = $init->settings['twilio_auth_token'];
		$url = "https://verify.twilio.com/v2/Services/".$service_sid."/VerificationCheck";

		$data = array(
			'To' => $mobile,
			'Code' => $code,
		);
		$headers = array(
			'Authorization' => 'Basic ' . base64_encode($id . ':' . $token)
		);
		$veri_check_response = wp_remote_post($url, array(
			'body' => $data,
			'headers' => $headers
		));

		$verify = wp_remote_retrieve_body($veri_check_response);
		$verification_check = json_decode($verify,true);
	    $return['data'] = $verification_check;

        if($verification_check['status'] == "approved"){
        	update_user_meta($user_id, 'twilio_verified_number', $mobile);
        	$return['status'] = 1;
        	$return['text'] = sprintf(__('Your Mobile number %s has been verified, Please save the changes.','wplms-sms-notifications'),$mobile);
        }

        print_r(json_encode($return));
        die();
	}

	function send_sms($user_number,$message){
		
		$init = Wplms_SMS_Notifications_Init::init();
		$init->get_settings();


		if(!$init->settings['sms_notifications'] || empty($init->settings['twilio_acc_sid']) || empty($init->settings['twilio_auth_token']) || empty($init->settings['twilio_number']) || empty($init->settings['phonenumber_field']) || !function_exists('bp_get_profile_field_data')){
			return;
		}

		//$phonenumber =  strip_tags(bp_get_profile_field_data(array('user_id'=>$user_id,'field'=>$init->settings['phonenumber_field'])));
		$phonenumber =  $user_number;

		if(empty($phonenumber))
			return;

		$id = $init->settings['twilio_acc_sid'];
		$token = $init->settings['twilio_auth_token'];
		$from = $init->settings['twilio_number'];
		$to = $phonenumber; // twilio trial verified number

		$url = "https://api.twilio.com/2010-04-01/Accounts/".$id."/SMS/Messages";
		$data = array(
			'From' => $from,
			'To' => $to,
			'Body' => $message
		);
		$headers = array(
			'Authorization' => 'Basic ' . base64_encode($id . ':' . $token)
		);
		$response = wp_remote_post($url, array(
			'body' => $data,
			'headers' => $headers
		));

		$body = wp_remote_retrieve_body($response);
		if ( is_wp_error( $response ) ) {
		   $error_message = $response->get_error_message();
		   return $error_message;
		}

		return true;
	}

}

Wplms_Twilio_SMS_Api::init();